package DyanaParsing;

#   Parses a Dyana Constraint file into a data structure
#
#   Author:     Gurmukh Sahota 07/15/2000
#   Copyright:  Gurmukh Sahota, 2000. All rights reserved.



require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw();
@EXPORT_OK = qw( parseDyanaUplFile parseDyanaCcoFile parseDyanaAcoFile writeDyanaUplFile);
%EXPORT_TAGS = ( ALL => [@EXPORT_OK] );                                                   

use strict;

##################################### parseDyanafile ##############################################
# Input   : dyana filename to parse into below described datastructure                            # 
# Output  : A reference to a cmap hash datastructure                                              #
#                $dyana_hlist - reference to a hash                                               #
#               $$dyana_hlist{"filename"}             - dyana filename                            #
#               $$dyana_hlist{"comments"}             - comments in the dyana file                #
#               $$dyana_hlist{"residue"}{$res_num}    - residue name (AA) of the residue with     #
#                                                       this number.                              #
#                                                                                                 #
#               $NOE = $$dyana_hlist{"NOE"}{$from_res_num}{$from_at}{$to_resnum}{$to_at}          #
#                              $$NOE{"distance"}      - numerical value of NOE distance           #
#                              $$NOE{"comment"}       - comment associated with NOE peak          #
#                                                                                                 #
# Purpose : To parse the input cmapfile and return the described data structure                   #
###################################################################################################

sub parseDyanaUplFile
   { #===== begin parseDyanaFile =====#
     
     # input dyana file
     my $dyana_infile = shift @_;
     
     my @input = ();
     my $dyana_hlist = {};

     # empty the comments element hash
     $$dyana_hlist{"comments"} = '';

     # open the input cmap file or die
     local *IN;
     if ($dyana_infile =~ /^\-$/)
     {
	 *IN = *STDIN;
     }
     else
     {
	 open(IN, "<$dyana_infile") || die("Unable to open input file: " . $dyana_infile) ;
     }

     # While you still have lines.  Either push onto the input hash or the comment element of the hash
     while (my $line = <IN>)
     	{ 
	  if ($line =~ /^\#/)
	    {$$dyana_hlist{"comments"} .= $line;}
	  elsif ($line =~ /^\s*$/)
	    {}
	  else
	    {push (@input, $line);} 
	}
     # close the dyana file or die
     close(IN)  || die("Unable to close input file: " . $dyana_infile);
     
     # save the filename in the hash
     $$dyana_hlist{"filename"} = $dyana_infile;
     
     # set the defaults
     my $current_residueNum = -1;
     my $fromResidueNum = 1;
     my $fromAT = 0;
     my $toResidueNum = 1;
     my $toResidueName = 2;
     my $toAT = 3;
     my $distance = 4;
     my $comments = 5;
     
     # while you have input lines in your input array do
     while (@input > 0)
       {
	 # get the next line
	 my $line = shift @input;
	 
	 # if the line starts with  # and AA, this will happen before the elsif
	 # we may also overwrite the current entry in the residue, but as they should
	 # be the same, this is not an issue.
	 if ($line =~ /^\s*(\d+)\s+?(\w+)\s*?$/)
	   {
	      $$dyana_hlist{"residue"}{$1} = $2;
	      $current_residueNum = $1;
	   }
	 # otherwise if the line is not just spaces or empty then do ...
	 elsif ($line =~ /^\s*(\d+)\s+?(\w+)\s+?(\w+)\s+?(\d+)\s+?(\w+)\s+?(\w+)\s+?(\d+\.\d+)\s*\#?$/)
	   {
	   my $res_name = $2;
               $$dyana_hlist{"residue"}{$1} = $2;
               $current_residueNum = $1;

	       $line =~ s/^\s*//;
	       $line =~ s/\s*\#?$//;

	       my @elem = split /\s+/, $line;
	       shift @elem;
               shift @elem;
	#       print "NOE >> ", $current_residueNum, "\t", $elem[$fromAT], "\tto\t", $elem[$toResidueNum], "\t", $elem[$toAT], " \t = \t", $elem[$distance], "\n";
	       # insert the NOE peak
	       $$dyana_hlist{"NOE"}{$current_residueNum}{$elem[$fromAT]}{$elem[$toResidueNum]}{$elem[$toAT]}{"distance"} = $elem[$distance];

	       # see if the residue it goes to is defined yet.  May be overwritten, see initial definition above.
	       if (! (exists $$dyana_hlist{"residue"}{$elem[$toResidueNum]}))
		   {
		       $$dyana_hlist{"residue"}{$elem[$toResidueNum]} = $elem[$toResidueName];
		   }

	       # put the comment of the peak in as well
	       $$dyana_hlist{"NOE"}{$current_residueNum}{$elem[$fromAT]}{$elem[$toResidueNum]}{$elem[$toAT]}{"comments"} = join ("\t", splice(@elem, $comments));
	       
	   }
	 # otherwise if the line is not just spaces or empty then do ...
	 elsif (($line !~ /^\s*$/) && ($current_residueNum > 0))
	   {
	       $line =~ s/^\s*//;
	       $line =~ s/\s*$//;
	       
	       my @elem = split /\s+/, $line;
	       
	#       print "NOE >> ", $current_residueNum, "\t", $elem[$fromAT], "\tto\t", $elem[$toResidueNum], "\t", $elem[$toAT], " \t = \t", $elem[$distance], "\n";
	       # insert the NOE peak
	       $$dyana_hlist{"NOE"}{$current_residueNum}{$elem[$fromAT]}{$elem[$toResidueNum]}{$elem[$toAT]}{"distance"} = $elem[$distance];

	       # see if the residue it goes to is defined yet.  May be overwritten, see initial definition above.
	       if (! (exists $$dyana_hlist{"residue"}{$elem[$toResidueNum]}))
		   {
		       $$dyana_hlist{"residue"}{$elem[$toResidueNum]} = $elem[$toResidueName];
		   }

	       # put the comment of the peak in as well
	       $$dyana_hlist{"NOE"}{$current_residueNum}{$elem[$fromAT]}{$elem[$toResidueNum]}{$elem[$toAT]}{"comments"} = join ("\t", splice(@elem, $comments));
	       
	   }
       }
     return $dyana_hlist;
   }
#################################### parseDyanaAcofile ############################################
# Input   : dyana filename to parse into below described datastructure                            # 
# Output  : A reference to a cmap hash datastructure                                              #
#                $dyana_hlist - reference to a hash                                               #
#               $$dyana_hlist{"filename"}             - dyana filename                            #
#               $$dyana_hlist{"comments"}             - comments in the dyana file                #
#               $$dyana_hlist{"residue"}{$res_num}    - residue name (AA) of the residue with     #
#                                                       this number.                              #
#                                                                                                 #
#               $NOE = $$dyana_hlist{"NOE"}{$from_res_num}{$from_at}{$to_resnum}{$to_at}          #
#                              $$NOE{"distance"}      - numerical value of NOE distance           #
#                              $$NOE{"comment"}       - comment associated with NOE peak          #
#                                                                                                 #
# Purpose : To parse the input cmapfile and return the described data structure                   #
###################################################################################################

sub parseDyanaCcoFile
   { #===== begin parseDyanaFile =====#
     
     # input dyana file
     my $dyana_infile = shift @_;
     
     my @input = ();
     my $dyana_hlist = {};

     # empty the comments element hash
     $$dyana_hlist{"comments"} = '';

     # open the input cmap file or die
     local *IN;
     if ($dyana_infile =~ /^\-$/)
     {
	 *IN = *STDIN;
     }
     else
     {
	 open(IN, "<$dyana_infile") || die("Unable to open input file: " . $dyana_infile) ;
     }

     # While you still have lines.  Either push onto the input hash or the comment element of the hash
     while (my $line = <IN>)
     	{ 
	  if ($line =~ /^\#/)
	    {$$dyana_hlist{"comments"} .= $line;}
	  elsif ($line =~ /^\s*$/)
	    {}
	  else
	    {push (@input, $line);} 
	}
     # close the dyana file or die
     close(IN)  || die("Unable to close input file: " . $dyana_infile);
     
     # save the filename in the hash
     $$dyana_hlist{"filename"} = $dyana_infile;
     
     # set the defaults
     my $fromResidueNum = 0;
     my $toResidueNum = 0;
     my $ResidueName = 1;
     my $fromAT = 2;
     my $toAT = 3;
     my $jval = 4;
     my $jerror = 5;
     my $jWeight = 6;
     
     # while you have input lines in your input array do
     while (@input > 0)
       {
           my $line = shift @input;
           $line =~ s/^\s*//;
           $line =~ s/\s*$//;
	   $line =~ s/\#(.*)$//;
	   my $comment = "#" . $1;
           # get the next line
           my @line = split /\s+/, $line;	       
	   $$dyana_hlist{"residue"}{$line[$fromResidueNum]} = $line[$ResidueName];
           $$dyana_hlist{"CCO"}{$line[$fromResidueNum]}{$line[$fromAT]}{$line[$toResidueNum]}{$line[$toAT]}{"comment"} = $comment;
           $$dyana_hlist{"CCO"}{$line[$fromResidueNum]}{$line[$fromAT]}{$line[$toResidueNum]}{$line[$toAT]}{"jval"} = $line[$jval];
           $$dyana_hlist{"CCO"}{$line[$fromResidueNum]}{$line[$fromAT]}{$line[$toResidueNum]}{$line[$toAT]}{"jerror"} = $line[$jerror]          if(@line > $jerror );
           $$dyana_hlist{"CCO"}{$line[$fromResidueNum]}{$line[$fromAT]}{$line[$toResidueNum]}{$line[$toAT]}{"weight"} = $line[$jWeight]          if(@line > $jWeight );
       }
     return $dyana_hlist;
   }

#################################### parseDyanaCcofile ############################################
# Input   : dyana filename to parse into below described datastructure                            # 
# Output  : A reference to a cmap hash datastructure                                              #
#                $dyana_hlist - reference to a hash                                               #
#               $$dyana_hlist{"filename"}             - dyana filename                            #
#               $$dyana_hlist{"comments"}             - comments in the dyana file                #
#               $$dyana_hlist{"residue"}{$res_num}    - residue name (AA) of the residue with     #
#                                                       this number.                              #
#                                                                                                 #
#               $NOE = $$dyana_hlist{"NOE"}{$from_res_num}{$from_at}{$to_resnum}{$to_at}          #
#                              $$NOE{"distance"}      - numerical value of NOE distance           #
#                              $$NOE{"comment"}       - comment associated with NOE peak          #
#                                                                                                 #
# Purpose : To parse the input cmapfile and return the described data structure                   #
###################################################################################################

sub parseDyanaAcoFile
   { #===== begin parseDyanaFile =====#
     
     # input dyana file
     my $dyana_infile = shift @_;
     
     my @input = ();
     my $dyana_hlist = {};

     # empty the comments element hash
     $$dyana_hlist{"comments"} = '';

     # open the input cmap file or die
     local *IN;
     if ($dyana_infile =~ /^\-$/)
     {
	 *IN = *STDIN;
     }
     else
     {
	 open(IN, "<$dyana_infile") || die("Unable to open input file: " . $dyana_infile) ;
     }

     # While you still have lines.  Either push onto the input hash or the comment element of the hash
     while (my $line = <IN>)
     	{ 
	  if ($line =~ /^\#/)
	    {$$dyana_hlist{"comments"} .= $line;}
	  elsif ($line =~ /^\s*$/)
	    {}
	  else
	    {push (@input, $line);} 
	}
     # close the dyana file or die
     close(IN)  || die("Unable to close input file: " . $dyana_infile);
     
     # save the filename in the hash
     $$dyana_hlist{"filename"} = $dyana_infile;
     
     # set the defaults
     my $residueNum = 0;
     my $residueName = 1;
     my $angleType = 2;
     my $angleStart = 3;
     my $angleEnd = 4;
     my $angleWeight =5;
     
     # while you have input lines in your input array do
     while (@input > 0)
       {
           my $line = shift @input;
           $line =~ s/^\s*//;
           $line =~ s/\s*$//;
           $line =~ s/\#(.*)$//;
	   my $comment = "#" . $1;
           # get the next line
           my @line = split /\s+/, $line;
           $$dyana_hlist{"ACO"}{$line[$residueNum]}{$line[$residueName]}{$line[$angleType]}{"comment"} = $comment;
           $$dyana_hlist{"ACO"}{$line[$residueNum]}{$line[$residueName]}{$line[$angleType]}{"start"} = $line[$angleStart];
           $$dyana_hlist{"ACO"}{$line[$residueNum]}{$line[$residueName]}{$line[$angleType]}{"end"} = $line[$angleEnd];
           $$dyana_hlist{"ACO"}{$line[$residueNum]}{$line[$residueName]}{$line[$angleType]}{"weight"} = $line[$angleWeight]          if(@line > $angleWeight);
       }
     return $dyana_hlist;
   }


##################################### writeDyanafile ##############################################
# Input   : dyana filename to parse into below described datastructure                            # 
# Output  : A reference to a cmap hash datastructure                                              #
#                $dyana_hlist - reference to a hash                                               #
#               $$dyana_hlist{"filename"}             - dyana filename                            #
#               $$dyana_hlist{"comments"}             - comments in the dyana file                #
#               $$dyana_hlist{"residue"}{$res_num}    - residue name (AA) of the residue with     #
#                                                       this number.                              #
#                                                                                                 #
#               $NOE = $$dyana_hlist{"NOE"}{$from_res_num}{$from_at}{$to_resnum}{$to_at}          #
#                              $$NOE{"distance"}      - numerical value of NOE distance           #
#                              $$NOE{"comment"}       - comment associated with NOE peak          #
#                                                                                                 #
# Purpose : To parse the input cmapfile and return the described data structure                   #
###################################################################################################

sub writeDyanaUplFile
   { #===== begin parseDyanaFile =====#
     
     # input dyana file
     my $dyana_hlist = shift @_;
     my $dyana_outfile = shift @_;
     
     # open the input cmap file or die
     local *OUT;
     if ($dyana_outfile =~ /^\-$/)
     {
	 *OUT = *STDOUT;
     }
     else
     {
	 open(OUT, ">$dyana_outfile") || die("Unable to open output file: " . $dyana_outfile) ;
     }

     my $date =  `date`;
     chomp $date;
     print OUT "#Created by DyanaParsing.pm $date Copyright(C) 2001\n\n";  
     foreach my $fromRes (sort {$a <=> $b} keys %{$$dyana_hlist{"NOE"}})
     {
#         print "FromRES >> $fromRes\n";
         foreach my $fromAT ( keys %{$$dyana_hlist{"NOE"}{$fromRes}} )
         {
#             print "\tFromAT >> $fromAT", "\n";
             foreach my $toRes (keys %{$$dyana_hlist{"NOE"}{$fromRes}{$fromAT}})
             {
#                 print "\t\tToRES >> $toRes\n";
                 foreach my $toAT (keys %{$$dyana_hlist{"NOE"}{$fromRes}{$fromAT}{$toRes}})
                 {
#                     print "\t\t\ttoAT >> $toAT\n";
                     printf OUT "%3d %3s  %-4s %4d %3s  %-4s    %2.2f %15s\n",  $fromRes, $$dyana_hlist{"residue"}{$fromRes}, $fromAT, $toRes, $$dyana_hlist{"residue"}{$toRes}, $toAT, $$dyana_hlist{"NOE"}{$fromRes}{$fromAT}{$toRes}{$toAT}{"distance"},  $$dyana_hlist{"NOE"}{$fromRes}{$fromAT}{$toRes}{$toAT}{"comments"};
                 }
             }
         }
     }
     close OUT;
     return;
   }
1;
