#!/usr/bin/perl
#!/usr/bin/perl
#!/usr/local/bin/perl
# average_duplicates.pl
#	Average duplicate peaks on given dimensions.
# 
#	Copyright Hunter Moseley, 2000. All rights reserved.
#	Written by Hunter Moseley 8/18/2000
#       Updated by Gurmukh Sahota 4/12/2001 (as noted)
#
#   usage: average_duplicates.pl in_peak_file new_workbook_name out_peak_file [options] tol1 tol2 ...
#
# 	peak_file 	- name of peak file or "-" for standard input/output.
# 	workbook_name	- string to use as workbook name.
# 	tol1 tol2 ... 	- dimension tolerances for comparing peaks.  Must have a value for
#			  every dimension.
#       option (requires either -min or -max)
#	    -min count	- minimum number of peaks needed to average.
#	    -max count	- maximum number of peaks needed to average.
#           -abs        - take the absolute values of the intensities
# 
use strict;

# Print usage info
if ((@ARGV < 6) || ($ARGV[0] =~ /^-h/))
  { &print_help; }

#GSS 4/12/2001   since we shift @ARGV, this keeps track of the initial command
my $command = join(" ", @ARGV);


# Read in command line parameters
my $input_peak_file = shift @ARGV;
my $workbook_name = shift @ARGV;
my $out_peak_file = shift @ARGV;
my $min_dup_count = 0;
my $max_dup_count = 0;
my $absolute_value = 0;

#GSS 4/12/2001   added the option tags
while (($ARGV[0] =~ /^\-/) && (@ARGV > 1))
  {
    my $switch = shift @ARGV;
    if ($switch =~ /^\-max/)
      { $max_dup_count = shift @ARGV; }
    elsif ($switch =~ /^\-min/)
      { $min_dup_count = shift @ARGV; }
    elsif ($switch =~ /^\-abs/)
      { $absolute_value = 1; }
   }

#GSS 4/12/2001   checks to see that either min or max has been set to a non zero value
if ((! $max_dup_count) &&  (! $min_dup_count))
{
    &print_help;
} 

#GSS 4/12/2001   tolerences can now be accessed like this
my @tol_alist = @ARGV;


# Read Peak file
my ($peak_alist, $end_comments_alist) = &read_peak_file($input_peak_file);
my $num_cols = @{$$peak_alist[0]};
if (($num_cols-3) < @tol_alist)
  { &print_help; }

# Open Output file
local *OUTPUT_FILE;
if ($out_peak_file eq "-")
  { *OUTPUT_FILE = *STDOUT; }
else
  { open(OUTPUT_FILE, ">$out_peak_file") || die ("open " . $out_peak_file . " failed."); }

# Print Header
print OUTPUT_FILE "#Index	";
my $dim_total = scalar(grep { $_; } @tol_alist);
for (my $x=1; $x <= $dim_total; $x++)
  { print OUTPUT_FILE $x, "Dim	"; }
print OUTPUT_FILE "	Intensity	Workbook\n";


# Group peaks into globs using tolerances
my @peak_groups;
foreach my $peak (@$peak_alist)
  {
  if (@peak_groups)
    {
    # find peak_groups that the peak should be added to
    my @add_list = grep { grep { &within_tolerances($_, $peak, \@tol_alist); } @{$peak_groups[$_]}; } 0..$#peak_groups;

    if (! @add_list) # create new peak_group because no suitable group was found.
      {
      push @peak_groups, [ $peak ];
      next;
      }
    elsif (@add_list > 1) # if there are more than one peak_group, then combine them.
      {
      my @hold_list;
      @add_list = sort { $b <=> $a; } @add_list; 
      foreach my $index (@add_list)
	{ push @hold_list, @{splice @peak_groups, $index, 1}; }

      push @peak_groups, [ @hold_list ];
      @add_list = ($#peak_groups);
      }

    push @{$peak_groups[$add_list[0]]}, $peak;
    }
  else
    { $peak_groups[0] = [ $peak ]; }
  }

# chop out peak groups not having the minimum count.
@peak_groups = grep { @$_ >= $min_dup_count; } @peak_groups;


# chop out peak groups having more than the maximum count.
if ($max_dup_count > 0)
  { @peak_groups = grep { @$_ <= $max_dup_count; } @peak_groups; }


# Find/Print average peaks 
my $index = 1;
my $close_peak_alist;
foreach my $peak_group (@peak_groups)
  {
  my $peak = &create_peak($index++, $peak_group); 
  print OUTPUT_FILE join("\t",@$peak), "\n";
  }


# Print Comments
print OUTPUT_FILE "*\n";

my $line;
my $history_found = 0;
foreach $line (@$end_comments_alist)
  {
  if (! $history_found && ($line =~ /^File History:/))
    {
    print OUTPUT_FILE "File History: $out_peak_file \n\n";
    print OUTPUT_FILE $0, " ", join(' ',@ARGV),"\n";
    $history_found = 1;
    next;
    }
  
  print OUTPUT_FILE $line;  
  }

if (! $history_found)
  {
  print OUTPUT_FILE "\nFile History: $out_peak_file \n\n";
  print OUTPUT_FILE $0, " ", $command,"\n";
  }

close OUTPUT_FILE;




#
#  Subroutines
#
sub read_peak_file 
  {
  my $filename = shift @_;
    
  local *PEAKFILE;
  if ($filename eq "-")
    { *PEAKFILE = *STDIN; }
  else
    { open(PEAKFILE, $filename) || die ("open " . $filename . " failed."); }

  my $peak_alist = [];
  my $line;
  while ($line = <PEAKFILE>) 
    {
    next if ($line =~ /^\#/); # skip comment lines
    last if ($line =~ /^[*]/); # end loop if end_of_file marker is encountered.
    
    ($line) = ($line =~/^\s*(.*)/); # skip leading white space.
    chomp $line;
    push @$peak_alist, [ split(/\s+/, $line) ];
    }

  my $comment_alist = [];
  while ($line = <PEAKFILE>)
    { push @$comment_alist, $line; }

  close PEAKFILE;
  return wantarray ? ($peak_alist, $comment_alist) : $peak_alist;
  }


sub print_help
  {
  print "\nError: not enough parameters given (or given incorrectly).\n" if ($ARGV[0] !~ /^-h/); 
  print "\n";
  print " average_duplicates.pl\n";
  print "	Average duplicate peaks on given dimensions.\n";
  print " \n";
  print "   usage: average_duplicates.pl in_peak_file new_workbook_name out_peak_file [options] tol1 tol2 ...\n";
  print "\n";
  print " 	peak_file 	- name of peak file or \"-\" for standard input/output.\n";
  print " 	workbook_name	- string to use as workbook name.\n";
  print " 	tol1 tol2 ... 	- dimension tolerances for comparing peaks.  Must have a value for\n";
  print "			  every dimension.\n"; 
  print "       options (requires either -min or -max):\n";
  print "	     -min count	- minimum number of peaks needed to average.\n";
  print "	     -max count	- maximum number of peaks needed to average.\n";
  print "	     -abs	- take the absolute value of the intensities.\n";

  exit(1);
  }

sub within_tolerances
  {
  my $peak = shift @_;
  my $peak2 = shift @_;
  my $tolerance_alist = shift @_;

  my $x;
  for($x=0; $x < @$tolerance_alist; $x++)
    {
    if (($$tolerance_alist[$x] != 0.0) && (abs($$peak[$x+1] - $$peak2[$x+1]) >= $$tolerance_alist[$x]))
      { return 0; }
    }

  return 1;
  }



sub create_peak
  {
  my $peak_index = shift @_;
  my $peak_group = shift @_;
  
  my $new_peak = [];

  # add peak index
  push @$new_peak, $peak_index;

  # add peak dimensions
  for (my $dim=0; $dim < @tol_alist; $dim++)
    {
    if ($tol_alist[$dim])
      {
      my $dim_value = 0;
      foreach my $peak (@$peak_group) { $dim_value += $$peak[$dim+1]; }
      $dim_value /= @$peak_group;
      $dim_value = int($dim_value * 1000)/ 1000;
      push @$new_peak, $dim_value;
      }
    }

  # add intensity
  my $intensity_value = 0;

  #GSS 4/12/2001   checks for the absolute value tag (add intensities as abs or not)
  if ($absolute_value)
  {
      foreach my $peak (@$peak_group) { $intensity_value += abs($$peak[$num_cols-2]); }
  }
  else
  {
      foreach my $peak (@$peak_group) { $intensity_value += $$peak[$num_cols-2]; }
  }
  $intensity_value /= @$peak_group;  
  push @$new_peak, ("\t" . $intensity_value);

  # add workbook
  push @$new_peak, (("\t" x (length($$new_peak[@$new_peak - 1]) < 9)) . $workbook_name); 

  return $new_peak;
  }
