#!/usr/bin/perl
#!/usr/bin/perl
#  bmrb2XeasyRL.pl
#	Converts bmrb format to Xeasy prot format.
#
#  usage: bmrb2XeasyRL.pl input_bmrb_file output_xeasy_prot [options]
#
#		bmrb_file - name of bmrb file or "-" to read from standard input.
#		xeasy_prot - name of xeasy prot file or "-" to write to standard output.
#		options:
#			-cyana - use cyana pseudo atoms for ambiguous assignments
#			-restrict shift_list - restrict conversion to the given quoted list of shift names.
#			-entity idnum - entity to use.
#
#  Copyright Hunter Moseley, 2006. All rights reserved.
#  Written by Hunter Moseley 6/13/2006
#
use strict;

# allows perl to fine local modules.
use FindBin;
use lib $FindBin::Bin;

# contains subroutines for reading and writing bmrb files.
use BMRBParsing qw(:ALL);

# print help info
if ((@ARGV < 2) || ($ARGV[0] =~ /^\-h/))
  {
  print STDERR "Error: not enough parameters given.\n\n" if (@ARGV < 2);
  print STDERR "bmrb2XeasyRL.pl\n";
  print STDERR "	Converts bmrb assigned chemical shift save frames to Xeasy prot format.\n";
  print STDERR "\n";
  print STDERR "  usage: bmrb2XeasyRL.pl input_bmrb_file output_xeasy_prot [options]\n";
  print STDERR "\n";
  print STDERR "		bmrb_file - name of bmrb file or \"-\" to read from standard input.\n";
  print STDERR "		xeasy_prot - name of xeasy prot file or \"-\" to write to standard output.\n";
  print STDERR "		options:\n";
  print STDERR "			-cyana - use cyana pseudo atoms for ambiguous assignments\n";
  print STDERR "			-restrict shift_list - restrict conversion to the given quoted list of shift names.\n";
  print STDERR "			-entity idnum - entity to use.\n";
  exit(1);
  }

my $cyana_ambiguity = {
  "A" => [ 
	   [ "C", [] ],
	   [ "O" , [] ],
	   [ "N" , [] ],
	   [ "H" , [] ],
	   [ "CA" , [] ],
	   [ "HA" , [] ],
	   [ "CB" , [] ],
	   [ "HB" , [ [ 0, "QB", [ "HB1", "HB2", "HB3" ] ] ] ],
	   [ "QB" , [] ],
	   [ "HB1" , [] ],
	   [ "HB2" , [] ],
	   [ "HB3" , [] ]
      ],
  "C" => [
	  [ "C" , [] ],
	  [ "O" , [] ],
	  [ "N" , [] ],
	  [ "H" , [] ],
	  [ "CA" , [] ],
	  [ "HA" , [] ],
	  [ "CB" , [] ],
	  [ "HB2" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	  [ "HB3" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	  [ "QB" , [] ],
	  [ "SG" , [] ],
	  [ "HG" , [] ]
    ],
  "D" => [
	 [ "C" , [] ],
	 [ "O" , [] ],
	 [ "N" , [] ],
	 [ "H" , [] ],
	 [ "CA" , [] ],
	 [ "HA" , [] ],
	 [ "CB" , [] ],
	 [ "HB2" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	 [ "HB3" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	 [ "QB" , [] ],
	 [ "CG" , [] ],
	 [ "OD1" , [] ],
	 [ "OD2" , [] ]
    ],
  "E" => [
	 [ "C" , [] ],
	 [ "O" , [] ],
	 [ "N" , [] ],
	 [ "H" , [] ],
	 [ "CA" , [] ],
	 [ "HA" , [] ],
	 [ "CB" , [] ],
	 [ "HB2" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	 [ "HB3" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	 [ "QB" , [] ],
	 [ "CG" , [] ],
	 [ "HG2" , [ [ 1, "QG", [ "HG2", "HG3" ] ] ] ],
	 [ "HG3" , [ [ 1, "QG", [ "HG2", "HG3" ] ] ] ],
	 [ "QG" , [] ],
	 [ "CD" , [] ],
	 [ "OE1" , [] ],
	 [ "OE2" , [] ]
    ],
  "F" => [
	 [ "C" , [] ],
	 [ "O" , [] ],
	 [ "N" , [] ],
	 [ "H" , [] ],
	 [ "CA" , [] ],
	 [ "HA" , [] ],
	 [ "CB" , [] ],
	 [ "HB2" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	 [ "HB3" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	 [ "QB" , [] ],
	 [ "CG" , [] ],
	 [ "HD1" , [ [ 1, "QR", [ "QD", "QE", "HD1", "HD2", "HE1", "HE2" ] ], [ 1, "QD", [ "HD1", "HD2" ] ] ] ],
	 [ "HD2" , [ [ 1, "QR", [ "QD", "QE", "HD1", "HD2", "HE1", "HE2" ] ], [ 1, "QD", [ "HD1", "HD2" ] ] ] ],
	 [ "HE1" , [ [ 1, "QR", [ "QD", "QE", "HD1", "HD2", "HE1", "HE2" ] ], [ 1, "QE", [ "HE1", "HE2" ] ] ] ],
	 [ "HE2" , [ [ 1, "QR", [ "QD", "QE", "HD1", "HD2", "HE1", "HE2" ] ], [ 1, "QE", [ "HE1", "HE2" ] ] ] ],
	 [ "QD" , [] ],
	 [ "QE" , [] ],
	 [ "QR" , [] ],
	 [ "CZ" , [] ],
	 [ "HZ" , [] ]
    ],
  "G" => [
	  [ "C" , [] ],
	  [ "O" , [] ],
	  [ "N" , [] ],
	  [ "H" , [] ],
	  [ "CA" , [] ],
	  [ "HA2" , [ [ 1, "QA", [ "HA2", "HA3" ] ] ] ],
	  [ "HA3" , [ [ 1, "QA", [ "HA2", "HA3" ] ] ] ],
	  [ "QA" , [] ]
    ],
  "H" => [
	  [ "C" , [] ],
	  [ "O" , [] ],
	  [ "N" , [] ],
	  [ "H" , [] ],
	  [ "CA" , [] ],
	  [ "HA" , [] ],
	  [ "CB" , [] ],
	  [ "HB2" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	  [ "HB3" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	  [ "QB" , [] ],
	  [ "CG" , [] ],
	  [ "ND1" , [] ],
	  [ "CD2" , [] ],
	  [ "HD1" , [] ],
	  [ "CE2" , [] ],
	  [ "NE2" , [] ],
	  [ "HD2" , [] ],
	  [ "HE1" , [] ]
    ],
  "I" => [
	  [ "C" , [] ],
	  [ "O" , [] ],
	  [ "N" , [] ],
	  [ "H" , [] ],
	  [ "CA" , [] ],
	  [ "HA" , [] ],
	  [ "CB" , [] ],
	  [ "HB" , [] ],
	  [ "CB" , [] ],
	  [ "CG2" , [] ],
	  [ "HG2" , [ [ 0, "QG2", [ "HG21", "HG22", "HG23" ] ] ] ],
	  [ "QG2" , [] ],
	  [ "HG21" , [] ],
	  [ "HG22" , [] ],
	  [ "HG23" , [] ],
	  [ "CG1" , [] ],
	  [ "HG2" , [ [ 1, "QG1", [ "HG12", "HG13" ] ] ] ],
	  [ "HG3" , [ [ 1, "QG1", [ "HG12", "HG13" ] ] ] ],
	  [ "QG1" , [] ],
	  [ "CD1" , [] ],
	  [ "HD1" , [ [ 0, "QD1", [ "HD11", "HD12", "HD13" ] ] ] ],
	  [ "QD1" , [] ],
	  [ "HD11" , [] ],
	  [ "HD12" , [] ],
	  [ "HD13" , [] ]
    ],
  "K" => [
	  [ "C" , [] ],
	  [ "O" , [] ],
	  [ "N" , [] ],
	  [ "H" , [] ],
	  [ "CA" , [] ],
	  [ "HA" , [] ],
	  [ "CB" , [] ],
	  [ "HB2" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	  [ "HB3" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	  [ "QB" , [] ],
	  [ "CG" , [] ],
	  [ "HG2" , [ [ 1, "QG", [ "HG2", "HG3" ] ] ] ],
	  [ "HG3" , [ [ 1, "QG", [ "HG2", "HG3" ] ] ] ],
	  [ "QG" , [] ],
	  [ "CD" , [] ],
	  [ "HD2" , [ [ 1, "QD", [ "HD2", "HD3" ] ] ] ],
	  [ "HD3" , [ [ 1, "QD", [ "HD2", "HD3" ] ] ] ],
	  [ "QD" , [] ],
	  [ "CE" , [] ],
	  [ "HE2" , [ [ 1, "QE", [ "HE2", "HE3" ] ] ] ],
	  [ "HE3" , [ [ 1, "QE", [ "HE2", "HE3" ] ] ] ],
	  [ "QE" , [] ],
	  [ "NZ" , [] ],
	  [ "HZ" , [ [ 0, "QZ", [ "HZ1", "HZ2", "HZ3" ] ] ] ],
	  [ "QZ" , [] ],
	  [ "HZ1" , [] ],
	  [ "HZ2" , [] ],
	  [ "HZ3" , [] ]
    ],
  "L" => [
	  [ "C" , [] ],
	  [ "O" , [] ],
	  [ "N" , [] ],
	  [ "H" , [] ],
	  [ "CA" , [] ],
	  [ "HA" , [] ],
	  [ "CB" , [] ],
	  [ "HB2" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	  [ "HB3" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	  [ "QB" , [] ],
	  [ "CG" , [] ],
	  [ "HG" , [] ],
	  [ "CD1" , [] ],
	  [ "CD2" , [] ],
	  [ "HD1" , [ [ 1, "QQD", [ "QD1", "HD11", "HD12", "HD13", "QD2", "HD21", "HD22", "HD23" ] ], [ -1, "QD1", [ "HD11", "HD12", "HD13" ] ] ] ],
	  [ "HD2" , [ [ 1, "QQD", [ "QD1", "HD11", "HD12", "HD13", "QD2", "HD21", "HD22", "HD23" ] ], [ -1, "QD2", [ "HD21", "HD22", "HD23" ] ] ] ],
	  [ "QD1" , [] ],
	  [ "QD2" , [] ],
	  [ "QQD" , [] ],
	  [ "HD11" , [] ],
	  [ "HD12" , [] ],
	  [ "HD13" , [] ],
	  [ "HD21" , [] ],
	  [ "HD22" , [] ],
	  [ "HD23" , [] ]
    ],
  "M" => [
	  [ "C" , [] ],
	  [ "O" , [] ],
	  [ "N" , [] ],
	  [ "H" , [] ],
	  [ "CA" , [] ],
	  [ "HA" , [] ],
	  [ "CB" , [] ],
	  [ "HB2" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	  [ "HB3" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	  [ "QB" , [] ],
	  [ "CG" , [] ],
	  [ "HG2" , [ [ 1, "QG", [ "HG2", "HG3" ] ] ] ],
	  [ "HG3" , [ [ 1, "QG", [ "HG2", "HG3" ] ] ] ],
	  [ "QG" , [] ],
	  [ "SD" , [] ],
	  [ "CE" , [] ],
	  [ "HE" , [ [ 0, "QE", [ "HE1", "HE2", "HE3" ] ] ] ],
	  [ "QE" , [] ],
	  [ "HE1" , [] ],
	  [ "HE2" , [] ],
	  [ "HE3" , [] ]
    ],
  "N" => [
	  [ "C" , [] ],
	  [ "O" , [] ],
	  [ "N" , [] ],
	  [ "H" , [] ],
	  [ "CA" , [] ],
	  [ "HA" , [] ],
	  [ "CB" , [] ],
	  [ "HB2" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	  [ "HB3" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	  [ "QB" , [] ],
	  [ "CG" , [] ],
	  [ "OD1" , [] ],
	  [ "ND2" , [] ],
	  [ "HD21" , [ [ 1, "QD2", [ "HD21", "HD22" ] ] ] ],
	  [ "HD22" , [ [ 1, "QD2", [ "HD21", "HD22" ] ] ] ],
	  [ "QD2" , [] ]
    ],
  "P" => [
	  [ "C" , [] ],
	  [ "O" , [] ],
	  [ "N" , [] ],
	  [ "CA" , [] ],
	  [ "HA" , [] ],
	  [ "CB" , [] ],
	  [ "HB2" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	  [ "HB3" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	  [ "QB" , [] ],
	  [ "CG" , [] ],
	  [ "HG2" , [ [ 1, "QG", [ "HG2", "HG3" ] ] ] ],
	  [ "HG3" , [ [ 1, "QG", [ "HG2", "HG3" ] ] ] ],
	  [ "QG" , [] ],
	  [ "CD" , [] ],
	  [ "HD2" , [ [ 1, "QD", [ "HD2", "HD3" ] ] ] ],
	  [ "HD3" , [ [ 1, "QD", [ "HD2", "HD3" ] ] ] ],
	  [ "QD" , [] ]
    ],
  "Q" => [
	  [ "C" , [] ],
	  [ "O" , [] ],
	  [ "N" , [] ],
	  [ "H" , [] ],
	  [ "CA" , [] ],
	  [ "HA" , [] ],
	  [ "CB" , [] ],
	  [ "HB2" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	  [ "HB3" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	  [ "QB" , [] ],
	  [ "CG" , [] ],
	  [ "HG2" , [ [ 1, "QG", [ "HG2", "HG3" ] ] ] ],
	  [ "HG3" , [ [ 1, "QG", [ "HG2", "HG3" ] ] ] ],
	  [ "QG" , [] ],
	  [ "CD" , [] ],
	  [ "OE1" , [] ],
	  [ "NE2" , [] ],
	  [ "HE21" , [ [ 1, "QE2", [ "HE21", "HE22" ] ] ] ],
	  [ "HE22" , [ [ 1, "QE2", [ "HE21", "HE22" ] ] ] ],
	  [ "QE2" , [] ]
    ],
  "R" => [
	  [ "C" , [] ],
	  [ "O" , [] ],
	  [ "N" , [] ],
	  [ "H" , [] ],
	  [ "CA" , [] ],
	  [ "HA" , [] ],
	  [ "CB" , [] ],
	  [ "HB2" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	  [ "HB3" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	  [ "QB" , [] ],
	  [ "CG" , [] ],
	  [ "HG2" , [ [ 1, "QG", [ "HG2", "HG3" ] ] ] ],
	  [ "HG3" , [ [ 1, "QG", [ "HG2", "HG3" ] ] ] ],
	  [ "QG" , [] ],
	  [ "CD" , [] ],
	  [ "HD2" , [ [ 1, "QD", [ "HD2", "HD3" ] ] ] ],
	  [ "HD3" , [ [ 1, "QD", [ "HD2", "HD3" ] ] ] ],
	  [ "QD" , [] ],
	  [ "NE" , [] ],
	  [ "HE" , [] ],
	  [ "CZ" , [] ],
	  [ "NH1" , [] ],
	  [ "HH11" , [ [ 1, "QH1", [ "HH11", "HH12" ] ] ] ],
	  [ "HH12" , [ [ 1, "QH1", [ "HH11", "HH12" ] ] ] ],
	  [ "QH1" , [] ],
	  [ "NH2" , [] ],
	  [ "HH21" , [ [ 1, "QH2", [ "HH21", "HH22" ] ] ] ],
	  [ "HH22" , [ [ 1, "QH2", [ "HH21", "HH22" ] ] ] ],
	  [ "QH2" , [] ]
    ],
  "S" => [
	  [ "C" , [] ],
	  [ "O" , [] ],
	  [ "N" , [] ],
	  [ "H" , [] ],
	  [ "CA" , [] ],
	  [ "HA" , [] ],
	  [ "CB" , [] ],
	  [ "HB2" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	  [ "HB3" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	  [ "QB" , [] ],
	  [ "OG" , [] ],
	  [ "HG" , [] ]
    ],
  "T" => [
	  [ "C" , [] ],
	  [ "O" , [] ],
	  [ "N" , [] ],
	  [ "H" , [] ],
	  [ "CA" , [] ],
	  [ "HA" , [] ],
	  [ "CB" , [] ],
	  [ "HB" , [] ],
	  [ "OG1" , [] ],
	  [ "HG1" , [] ],
	  [ "CG2" , [] ],
	  [ "HG2" , [ [ 0, "QG2", [ "HG21", "HG22", "HG23" ] ] ] ],
	  [ "QG2" , [] ],
	  [ "HG21" , [] ],
	  [ "HG22" , [] ],
	  [ "HG23" , [] ]
    ],
  "V" => [
	  [ "C" , [] ],
	  [ "O" , [] ],
	  [ "N" , [] ],
	  [ "H" , [] ],
	  [ "CA" , [] ],
	  [ "HA" , [] ],
	  [ "CB" , [] ],
	  [ "HB" , [] ],
	  [ "CG1" , [] ],
	  [ "CG2" , [] ],
	  [ "HG1" , [ [ 1, "QQG", [ "QG1", "HG11", "HG12", "HG13", "QG2", "HG21", "HG22", "HG23" ] ], [ -1, "QG1", [ "HG11", "HG12", "HG13" ] ] ] ],
	  [ "HG2" , [ [ 1, "QQG", [ "QG1", "HG11", "HG12", "HG13", "QG2", "HG21", "HG22", "HG23" ] ], [ -1, "QG2", [ "HG21", "HG22", "HG23" ] ] ] ],
	  [ "QG1" , [] ],
	  [ "QG2" , [] ],
	  [ "QQG" , [] ],
	  [ "HG11" , [] ],
	  [ "HG12" , [] ],
	  [ "HG13" , [] ],
	  [ "HG21" , [] ],
	  [ "HG22" , [] ],
	  [ "HG23" , [] ]
    ],
  "W" => [
	  [ "C" , [] ],
	  [ "O" , [] ],
	  [ "N" , [] ],
	  [ "H" , [] ],
	  [ "CA" , [] ],
	  [ "HA" , [] ],
	  [ "CB" , [] ],
	  [ "HB2" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	  [ "HB3" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	  [ "QB" , [] ],
	  [ "CG" , [] ],
	  [ "CD1" , [] ],
	  [ "CD2" , [] ],
	  [ "CE3" , [] ],
	  [ "NE1" , [] ],
	  [ "HD1" , [] ],
	  [ "HE3" , [] ],
	  [ "CZ3" , [] ],
	  [ "CZ2" , [] ],
	  [ "HE1" , [] ],
	  [ "HZ3" , [] ],
	  [ "CH2" , [] ],
	  [ "HZ2" , [] ],
	  [ "HH2" , [] ]
    ],
  "Y" => [
	  [ "C" , [] ],
          [ "O" , [] ],
          [ "N" , [] ],
	  [ "H" , [] ],
	  [ "CA" , [] ],
	  [ "HA" , [] ],
	  [ "CB" , [] ],
	  [ "HB2" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	  [ "HB3" , [ [ 1, "QB", [ "HB2", "HB3" ] ] ] ],
	  [ "QB" , [] ],
          [ "CG" , [] ],
	  [ "HD1" , [ [ 1, "QR", [ "QD", "QE", "HD1", "HD2", "HE1", "HE2" ] ], [ 1, "QD", [ "HD1", "HD2" ] ] ] ],
	  [ "HD2" , [ [ 1, "QR", [ "QD", "QE", "HD1", "HD2", "HE1", "HE2" ] ], [ 1, "QD", [ "HD1", "HD2" ] ] ] ],
	  [ "HE1" , [ [ 1, "QR", [ "QD", "QE", "HD1", "HD2", "HE1", "HE2" ] ], [ 1, "QE", [ "HE1", "HE2" ] ] ] ],
	  [ "HE2" , [ [ 1, "QR", [ "QD", "QE", "HD1", "HD2", "HE1", "HE2" ] ], [ 1, "QE", [ "HE1", "HE2" ] ] ] ],
	  [ "QD" , [] ],
	  [ "QE" , [] ],
	  [ "QR" , [] ],
	  [ "CZ" , [] ],
	  [ "OH" , [] ],
	  [ "HH" , [] ]
    ]
  };

# read command line options.
my $input_bmrb_filename = shift @ARGV;
my $output_xeasy_prot_filename = shift @ARGV;
my $entity_id = 0;
my @shift_name_list;
my $cyana = 0;

while (@ARGV && ($ARGV[0] =~ /^\-/))
  {
  my $switch = shift @ARGV;

  if ($switch =~ /^\-e/)
    { $entity_id = shift @ARGV; }
  elsif ($switch =~ /^\-r/)
    { @shift_name_list = split(",",shift @ARGV); }
  elsif ($switch =~ /^\-c/)
    { $cyana = 1; }
  else
    { 
    print STDERR "INVALID OPTION: $switch\n\nExiting Program ...\n\n";  
    exit(1);
    }
  }


# read the bmrb file
my $bmrb_hlist = &read_bmrb_file($input_bmrb_filename, {});

# open Xeasy prot file
local *XEASY_FILE;
if ($output_xeasy_prot_filename eq "-")
  { *XEASY_FILE = *STDOUT; }
else
  { open(XEASY_FILE, ">$output_xeasy_prot_filename") || die ("open " . $output_xeasy_prot_filename . " failed."); }


# cycle through each residue
if (! $cyana)
  {
  my $count = 1;
  foreach my $entity (values %{$$bmrb_hlist{"entity_list"}})
    {
    next if ($entity_id && $entity_id != $$entity{"entity_id"});
    
    foreach my $residue_name (@{$$entity{"name_array"}})
      {      
      foreach my $shift_name (keys %{$$entity{"rlist"}{$residue_name}{"shifts"}})
	{
	if (!scalar(@shift_name_list) || scalar(grep { $_ eq $shift_name; } @shift_name_list))
	  {
	  foreach my $shift_value (@{$$entity{"rlist"}{$residue_name}{"shifts"}{$shift_name}{"list"}})
	    { print XEASY_FILE " ", $count++, "  ",$shift_value, "  0.000 ",$shift_name," \t",$$entity{"rlist"}{$residue_name}{"index"},"\n"; }
	  }
	}
      }
    }
  }
else
  {
  my $count = 1;
  foreach my $entity (values %{$$bmrb_hlist{"entity_list"}})
    {
    next if ($entity_id && $entity_id != $$entity{"entity_id"});
    
    foreach my $residue_name (@{$$entity{"name_array"}})
      {
      my $used = {};
      foreach my $shift_test (@{$$cyana_ambiguity{$$entity{"rlist"}{$residue_name}{"aa"}}})
	{
	my $shift_name = $$shift_test[0];
	if (!scalar(@shift_name_list) || scalar(grep { $_ eq $shift_name; } @shift_name_list))
	  {
	  if (! exists $$used{$shift_name})
	    {
	    if (! exists $$entity{"rlist"}{$residue_name}{"shifts"}{$shift_name})
	      { 
	      print XEASY_FILE $count++, " \t 999.000 \t 0.000 \t ",$shift_name," \t ",$$entity{"rlist"}{$residue_name}{"index"},"\n"; 
	      $$used{$shift_name} = 1;
	      }
	    else
	      {
	      my $shift_value = $$entity{"rlist"}{$residue_name}{"shifts"}{$shift_name}{"list"}[0];
	      my $shift_name2 = $shift_name;
	      my $add_alist = 0;
	      if (@{$$shift_test[1]})
		{
		foreach my $test (@{$$shift_test[1]})
		  {
		  if ($$test[0] == 0 || 
		      ($$test[0] == -1 && scalar(grep { exists $$entity{"rlist"}{$residue_name}{"shifts"}{$_} && ($$entity{"rlist"}{$residue_name}{"shifts"}{$_}{"list"}[0] != $shift_value); } (@{$$test[2]})) ) ||
		      ($$test[0] == 1 && ! scalar(grep { exists $$entity{"rlist"}{$residue_name}{"shifts"}{$_} && ($$entity{"rlist"}{$residue_name}{"shifts"}{$_}{"list"}[0] != $shift_value); } (@{$$test[2]})) ) )
		    {
		    $shift_name2 = $$test[1];
		    $add_alist = $$test[2];
		    last;
		    }
		  }
		}

	      if (! exists $$used{$shift_name2})
		{
		print XEASY_FILE $count++, " \t  ",$shift_value, " \t 0.000 \t ",$shift_name2," \t ",$$entity{"rlist"}{$residue_name}{"index"},"\n"; 
		$$used{$shift_name2} = 1;
		}

	      if ($add_alist)
		{
		foreach my $shift_name (@$add_alist)
		  {
		  if (! exists $$used{$shift_name})
		    {
		    print XEASY_FILE $count++, " \t 999.000 \t 0.000 \t ",$shift_name," \t ",$$entity{"rlist"}{$residue_name}{"index"},"\n"; 
		    $$used{$shift_name} = 1;
		    }
		  }
		}
	      }
	    }
	  }	  
	}
      }
    }
  }

close XEASY_FILE;
