#!/usr/bin/perl
#!/usr/bin/perl
#  bmrbACS2SparkyPks.pl
#	Converts bmrb assigned chemical shift save frame to Assigned Sparky Peak List format.
#
#  usage: bmrbACS2SparkyPks.pl input_bmrb_file output_peak_file dim1 dim2 ...
#
#		input_bmrb_file - bmrb file to read ("-" to read from standard input).
#		output_peak_file - output file to write to ("-" to write to standard output).
#		dim1 dim2 ...
#			dim - quoted list of atom names for a dimension.
#
#  Hint: Sparky reads them in using the rp command.
#
#  Copyright Hunter Moseley, 2001. All rights reserved.
#  Written by Hunter Moseley 10/4/2001
#  Used bmrb2pks.pl as a template, its copyrights apply.
#
#
#  Updated to use new BMRBParsing/CMapParsing modules.
#  Copyright Hunter Moseley 2005. All rights reserved.
#

# These modules allow me to use modules rather than hardcoding everything into one script
use FindBin;
use lib $FindBin::Bin;

# Parser module parses in/out the cmap data and the peak file
use BMRBParsing qw(:ALL);

# Use the strict module: This is off because of silly bareword problems with the initialize_matirices
 use strict;

if ((@ARGV < 3) || ($ARGV[0] =~ /^-h/))
  {
  print STDERR "Error: not enough parameters given.\n\n" if (@ARGV < 3);
  print STDERR "  bmrbACS2SparkyPks.pl\n";
  print STDERR "	Converts bmrb assigned chemical shift save frame to Assigned Sparky Peak List format.\n";
  print STDERR "\n";
  print STDERR "  usage: bmrbACS2SparkyPks.pl input_bmrb_file output_peak_file dim1 dim2 ...\n";
  print STDERR "\n";
  print STDERR "		input_bmrb_file - bmrb file to read (\"-\" to read from standard input).\n";
  print STDERR "		output_peak_file - output file to write to (\"-\" to write to standard output).\n";
  print STDERR "		dim1 dim2 ...\n";
  print STDERR "			dim - quoted list of atom names for a dimension.\n";
  print STDERR "\n";
  print STDERR "  Hint: Sparky reads them in using the rp command.\n";
  print STDERR "\n";
 
  exit(1);
  }

# read command line options
my $infile = shift @ARGV;
my $outfile = shift @ARGV;
my @atom_input = @ARGV;

my @filter_atoms;
# split the atoms into an array of arrays of atoms
foreach my $atoms (@atom_input)
  {
  # take off the "   
  $atoms =~ s/\"//g;  #"
  
  # push the array of array of atoms     
  my  @split_atoms = split(" ", $atoms);
  push @filter_atoms, [@split_atoms];
  }


# read bmrb file
my $bmrb_hlist = &read_bmrb_file($infile,{});


#  Here we pass in [H], [N], [CA,CB] and get [H,N,CA] and [H,N,CB]
my @peakdefinitions = recombine(@filter_atoms);
my $index = 1;


# open Sparky Assigned Peak list file
local *SPARKY_FILE;
if ($outfile eq "-")
  { *SPARKY_FILE = *STDOUT; }
else
  { open(SPARKY_FILE, ">$outfile") || die ("open " . $outfile . " failed."); }


# loop through all residues looking for peaks
foreach my $entity (values %{$$bmrb_hlist{"entity_list"}})
  {
  foreach my $residue ( @{$$entity{"name_array"}} )
    {	
    foreach my $peakdef (@peakdefinitions)
      {
      my @shift_ahlist = ();
      
      # check for proper shift lists
      # this comment means that we make sure that every atomType (H,N,CA) exists for the residue so that it
      # is actually a peak.
      
      # iterate through each element in the peak definition (H,N,CA)
      for (my $count = 0; $count < @$peakdef; $count++)
	{
	# set your current atomType    
	my $atomType = $$peakdef[$count];
	
	# GSS 04/14/2001 add $ at end of regexp of atomType so that HA-12 will be a stupid and incorrect name,
	# but will be a current residue's atomType name
	
	# if you have a previous tag on the atomType and the previous residue is defined ...
	if (($atomType =~ /-1$/) && (defined $$entity{"rlist"}{$residue}{"prev"}))
	  {
	  my $prevresidue = $$entity{"rlist"}{$residue}{"prev"};
	  
	  # GSS 04/14/2001 deleted [A-Z]+ and replaced with \w+ because HG1-1 should be a valid atomType in CMAP naming scheme
	  # we get the name of the atomType without the previous tag and we should beable to read as pure BMRB ...
	  my ($true_atomType) = ($atomType =~ /^(\w+)\-1$/);
	  
	  # and see if it is defined as a standard atomType on the previous residue
	  if ( (exists $$entity{"rlist"}{$prevresidue}{"shifts"}{$true_atomType}{"list"}) && (@{$$entity{"rlist"}{$prevresidue}{"shifts"}{$true_atomType}{"list"}}))
	    { 
	    $shift_ahlist[$count] = { "atomType" => $true_atomType, "list" => $$entity{"rlist"}{$prevresidue}{"shifts"}{$true_atomType}{"list"}, "curr" => 0, "residue" => $prevresidue, "name" => $prevresidue };
	    if (exists $$entity{"rlist"}{$prevresidue}{"author_name"})
	      { $shift_ahlist[$count]{"name"} = $$entity{"rlist"}{$prevresidue}{"author_name"}; }
	    }
	  else
	    { last; }
	  }
	
	# otherwise, you are working with an atomType for the current residue.
	else
	  {
	  # you see if the atomType is defined for the current residue
	  if ( (exists $$entity{"rlist"}{$residue}{"shifts"}{$atomType}{"list"}) && (@{$$entity{"rlist"}{$residue}{"shifts"}{$atomType}{"list"}}))
	    { 
	    $shift_ahlist[$count] = { "atomType" => $atomType, "list" => $$entity{"rlist"}{$residue}{"shifts"}{$atomType}{"list"}, "curr" => 0, "residue" => $residue, "name" => $residue };
	    if (exists $$entity{"rlist"}{$residue}{"author_name"})
	      { $shift_ahlist[$count]{"name"} = $$entity{"rlist"}{$residue}{"author_name"}; }	      
	    }
	  else
	    { last; }
	  }
	}
      
      # start creating peaks if the proper shift lists are there.
      if (@shift_ahlist == @$peakdef)
	{
	while($shift_ahlist[$#shift_ahlist]{"curr"} < @{$shift_ahlist[$#shift_ahlist]{"list"}})
	  {
	  # write out peak
	  my $curr_residue = $shift_ahlist[0]{"residue"};
	  print SPARKY_FILE $shift_ahlist[0]{"name"},$shift_ahlist[0]{"atomType"};
	  for(my $count=1; $count < @shift_ahlist; $count++)
	    {
	    print SPARKY_FILE "-";
	    if ($curr_residue ne $shift_ahlist[$count]{"residue"})
	      { 
	      $curr_residue = $shift_ahlist[$count]{"residue"};
	      print SPARKY_FILE $shift_ahlist[$count]{"name"};
	      }
	    print SPARKY_FILE $shift_ahlist[$count]{"atomType"}; 
	    }
	  
	  print SPARKY_FILE " ";
	  for(my $count=0; $count < @shift_ahlist; $count++)
	    { print SPARKY_FILE "\t",$shift_ahlist[$count]{"list"}[$shift_ahlist[$count]{"curr"}]; }
	  
	  print SPARKY_FILE "\n";
	  
	  # increment curr counters in shift_ahlist.
	  # the curr counter tags allow us to have a list of resonances for each peak type and then 
	  # using the same logic as the recombine subroutine, we iterate until we have permuted through
	  # the entire list.
	  $shift_ahlist[0]{"curr"}++;
	  for (my $count = 0; $count < @$peakdef; $count++)
	    {
	    if (($shift_ahlist[$count]{"curr"} >= @{$shift_ahlist[$count]{"list"}}) && ($count+1 < @$peakdef))
	      { 
	      $shift_ahlist[$count]{"curr"} = 0;
	      $shift_ahlist[$count+1]{"curr"}++;
	      }
	    }
	  }
	}      
      }
    }
  }

#---------------------------------------------------------------------------------------------------------------#
# sub recombine("array of array of character strings" @strings )                                                #
#                                                                                                               #
#       Input   : array of array of character elements.                                                         #
#       Output  : an array of an array of recombined elements. One from each of the outer array.                #
#       Purpose : To recombine each of the inner array elements with the elements of the others.                #
#                      One from each outer array. Makes atom filters in this script.                            #
#                                                                                                               #
#       How     : an array of current, an array of maxes.  You increment the last element until it maxes out.   #
#                      Then you run though the counter in reverse.  If the element is maxed.  Add one to the    #
#                      previous element.  If the first element is maxed then exit the sub because you are done  #
#---------------------------------------------------------------------------------------------------------------#

sub recombine
  {
  # the array of the strings
  my @strings = @_;

  # initialize the variables
  my  $last_string = $#strings;     
  my @max_elements;
  my @current;
  my @recombination;
  my @element_of_recombination;
  my $count;
  my $element_count;
  
  # initalize the maximum number of elements for the inner arrays and sets the current to 0.
  foreach my $elements_of_string (@strings)
    {
    push @max_elements, $#$elements_of_string;
    push @current, 0;
    }
  
  # an infinite while loop which exits when the first inner array's counter maxes out.
  while (1)
    {
    # Run through each element of the last string  
    for ($element_count=0; $element_count<=$max_elements[$last_string];$element_count++)
      {
      # Go through each element of the outer array and build a list of the recombined elements
      my $temp_string_count = 0;
      foreach my $elements_of_string (@strings)
	{
	push @element_of_recombination, $$elements_of_string[$current[$temp_string_count]];  
	$temp_string_count++;
	}
      # push the list of recombined elements onto your return array of arrays.  
      push @recombination, [@element_of_recombination];
      @element_of_recombination = ();
      $current[$last_string]++;
      }
    # Do the "carries" of the current elements 
    for ($count=$last_string; $count>0; $count--)
      {
      # if the current count is greater than the max number of elements,
      #     add one to prev and set current to 0.  
      if ($current[$count] > $max_elements[$count])
	{
	$current[$count] = 0;
	$current[$count-1] += 1;
	}
      # Once you hit an element that does not need to be carried exit.
      else
	{last;}
      }
    # if your first element has maxed out then exit the script
    if ($current[0] > $max_elements[0])
      {return (@recombination);}
    }
  
  }
