#!/usr/bin/perl
#!/usr/bin/perl
#  bmrbACS2pks.pl
#	Converts BMRB assigned chemical shifts frame to an AutoAssign peak list.
#
#  usage: bmrbACS2pks.pl input_bmrb_file output_peak_file intensity workbook dim1 dim2 ...
#
#		input_bmrb_file - bmrb file to read ("-" to read from standard input).
#		output_peak_file - output file to write to ("-" to write to standard output).
#		intensity - intensity value to use for the peaks
#		workbook - workbook name for the peaks (residue name and peak definition are added by script)
#		dim1 dim2 ...
#			dim - quoted list of atom names (in CMap convention) for a dimension.
#
#
#  Copyright Gurmukh Sahota, 2000. All rights reserved.
#  Written by Gurmukh Sahota 7/31/2000
#
#  Mostly Rewritten by Hunter Moseley 2/14/2001
#  Copyright Hunter Moseley, 2001. All rights reserved.
#
#  90% more comments ( and added the peakdefinition to the workbookname ) Gurmukh Sahota 4/14/2001
#  Again Copyright Gurmukh Sahota 2001.  All rights reserved
#
#  Updated to use new BMRBParsing/CMapParsing modules.
#  Copyright Hunter Moseley 2005. All rights reserved.
#
#

# These modules allow me to use modules rather than hardcoding everything into one script
use FindBin;
use lib $FindBin::Bin;

# Parser module parses in/out the cmap data and the peak file
use CMapParsing qw(:ALL);
use PeakParsing qw(:ALL);

# Use the strict module: This is off because of silly bareword problems with the initialize_matirices
use strict;

# create history line
my $history_line = $0 . " " . join(' ',@ARGV) . "\n";

if ((@ARGV < 5) || ($ARGV[0] =~ /^-h/))
  {
  print STDERR "Error: not enough parameters given.\n\n" if (@ARGV < 5);
  print STDERR "  bmrbACS2pks.pl\n";
  print STDERR "	Converts BMRB assigned chemical shifts frame to an AutoAssign peak list.\n";
  print STDERR "\n";
  print STDERR "  usage: bmrbACS2pks.pl input_bmrb_file output_peak_file intensity workbook dim1 dim2 ...\n";
  print STDERR "\n";
  print STDERR "		input_bmrb_file - bmrb file to read (\"-\" to read from standard input).\n";
  print STDERR "		output_peak_file - output file to write to (\"-\" to write to standard output).\n";
  print STDERR "		intensity - intensity value to use for the peaks\n";
  print STDERR "		workbook - workbook name for the peaks (residue name and peak definition are added by script)\n";
  print STDERR "		dim1 dim2 ...\n";
  print STDERR "			dim - quoted list of atom names (in CMap convention) for a dimension.\n";
  print STDERR "\n";
  exit(1);
  }

# read command line options
my $infile = shift @ARGV;
my $outfile = shift @ARGV;
my $intensity = shift @ARGV;
my $workbookname = shift @ARGV;
my @atom_input = @ARGV;

my @filter_atoms;
# split the atoms into an array of arrays of atoms
foreach my $atoms (@atom_input)
  {
  # take off the "   
  $atoms =~ s/\"//g;  #"
  
  # push the array of array of atoms     
  my  @split_atoms = split(" ", $atoms);
  push @filter_atoms, [@split_atoms];
  }


# read bmrb file
my $cmap_hlist = &readBMRBasCMap($infile);

my $peakfile_hlist = {};
$$peakfile_hlist{"filename"} = $outfile;
foreach my $ats (@filter_atoms)
    { push @{$$peakfile_hlist{"dimlist"}}, join(':', @$ats); }

#  Here we pass in [H], [N], [CA,CB] and get [H,N,CA] and [H,N,CB]

my @peakdefinitions = recombine(@filter_atoms);
my $index = 1;

foreach my $residue ( @{$$cmap_hlist{"name_array"}} )
  {	
  foreach my $peakdef (@peakdefinitions)
    {
    my @shift_ahlist = ();
    
    # check for proper shift lists
    # this comment means that we make sure that every atomType (H,N,CA) exists for the residue so that it
    # is actually a peak.
    
    # iterate through each element in the peak definition (H,N,CA)
    for (my $count = 0; $count < @$peakdef; $count++)
      {
      # set your current atomType    
      my $atomType = $$peakdef[$count];
      
      # GSS 04/14/2001 add $ at end of regexp of atomType so that HA-12 will be a stupid and incorrect name,
      # but will be a current residue's atomType name
      
      # if you have a previous tag on the atomType and the previous residue is defined ...
      if (($atomType =~ /-1$/) && (defined $$cmap_hlist{"rlist"}{$residue}{"prev"}))
	{
	my $prevresidue = $$cmap_hlist{"rlist"}{$residue}{"prev"};
	
	# GSS 04/14/2001 deleted [A-Z]+ and replaced with \w+ because HG1-1 should be a valid atomType in CMAP naming scheme
	# we get the name of the atomType without the previous tag and we should beable to read as pure BMRB ...
	my ($true_atomType) = ($atomType =~ /^(\w+)\-1$/);
	
	# and see if it is defined as a standard atomType on the previous residue
	if ( (exists $$cmap_hlist{"rlist"}{$prevresidue}{"shifts"}{$true_atomType}{"list"}) && (@{$$cmap_hlist{"rlist"}{$prevresidue}{"shifts"}{$true_atomType}{"list"}}))
	  { $shift_ahlist[$count] = { "atomType" => $true_atomType, "list" => $$cmap_hlist{"rlist"}{$prevresidue}{"shifts"}{$true_atomType}{"list"}, "curr" => 0 }; }
	else
	  { last; }
	}
      
      # otherwise, you are working with an atomType for the current residue.
      else
	{
	# you see if the atomType is defined for the current residue
	if ( (exists $$cmap_hlist{"rlist"}{$residue}{"shifts"}{$atomType}{"list"}) && (@{$$cmap_hlist{"rlist"}{$residue}{"shifts"}{$atomType}{"list"}}))
	  { 
	  $shift_ahlist[$count] = { "atomType" => $atomType, "list" => $$cmap_hlist{"rlist"}{$residue}{"shifts"}{$atomType}{"list"}, "curr" => 0 }; 
	  }
	else
	  { last; }
	}
      }
    
      # start creating peaks if the proper shift lists are there.
    if (@shift_ahlist == @$peakdef)
      {
      while($shift_ahlist[$#shift_ahlist]{"curr"} < @{$shift_ahlist[$#shift_ahlist]{"list"}})
	{
	# create peak
	$$peakfile_hlist{"plist"}{$index} = &create_add_peak(\@shift_ahlist, $$peakfile_hlist{"dimlist"}, ($workbookname . "." . $residue), $intensity);
	$index++;
	
	# increment curr counters in shift_ahlist.
	# the curr counter tags allow us to have a list of resonances for each peak type and then 
	# using the same logic as the recombine subroutine, we iterate until we have permuted through
	# the entire list.
	$shift_ahlist[0]{"curr"}++;
	  for (my $count = 0; $count < @$peakdef; $count++)
	    {
	    if (($shift_ahlist[$count]{"curr"} >= @{$shift_ahlist[$count]{"list"}}) && ($count+1 < @$peakdef))
	      { 
	      $shift_ahlist[$count]{"curr"} = 0;
	      $shift_ahlist[$count+1]{"curr"}++;
	      }
	    }
	}
      }      
    }
  }

# add File History
$$peakfile_hlist{"end_comments"} = "File History: $outfile \n\n" . $history_line;


#
# write new peak list file.
#

&outputPeakfile($peakfile_hlist, $outfile);





# create_peak
#   Create and return a  peak from the given shifts
#
#   Parameters
#	shift_ahlist - ref to shift_ahlist with keys "curr" and "list".
#	dim_alist - ref to array of peak dimension names.
#	$workbook - workbook name of peak
#	$intensity - intensity value of peak
#
sub create_add_peak
  {
  my $shift_ahlist = shift @_;
  my $dim_alist = shift @_;
  my $workbook = shift @_;
  my $intensity = shift @_;
  
  my $peak = {};
  my $peak_deftag = '';
  for(my $count=0; $count < @$dim_alist; $count++)
    { $$peak{"shifts"}{$$dim_alist[$count]} = $$shift_ahlist[$count]{"list"}[$$shift_ahlist[$count]{"curr"}]; $peak_deftag .= "_" . $$shift_ahlist[$count]{"atomType"}}
  
  $$peak{"intensity"} = $intensity;
  $$peak{"workbookname"} = $workbook . "." . $peak_deftag;
  
  return $peak;  
  }

#---------------------------------------------------------------------------------------------------------------#
# sub recombine("array of array of character strings" @strings )                                                #
#                                                                                                               #
#       Input   : array of array of character elements.                                                         #
#       Output  : an array of an array of recombined elements. One from each of the outer array.                #
#       Purpose : To recombine each of the inner array elements with the elements of the others.                #
#                      One from each outer array. Makes atom filters in this script.                            #
#                                                                                                               #
#       How     : an array of current, an array of maxes.  You increment the last element until it maxes out.   #
#                      Then you run though the counter in reverse.  If the element is maxed.  Add one to the    #
#                      previous element.  If the first element is maxed then exit the sub because you are done  #
#---------------------------------------------------------------------------------------------------------------#

sub recombine
  {
  # the array of the strings
  my @strings = @_;

  # initialize the variables
  my  $last_string = $#strings;     
  my @max_elements;
  my @current;
  my @recombination;
  my @element_of_recombination;
  my $count;
  my $element_count;
  
  # initalize the maximum number of elements for the inner arrays and sets the current to 0.
  foreach my $elements_of_string (@strings)
    {
    push @max_elements, $#$elements_of_string;
    push @current, 0;
    }
  
  # an infinite while loop which exits when the first inner array's counter maxes out.
  while (1)
    {
    # Run through each element of the last string  
    for ($element_count=0; $element_count<=$max_elements[$last_string];$element_count++)
      {
      # Go through each element of the outer array and build a list of the recombined elements
      my $temp_string_count = 0;
      foreach my $elements_of_string (@strings)
	{
	push @element_of_recombination, $$elements_of_string[$current[$temp_string_count]];  
	$temp_string_count++;
	}
      # push the list of recombined elements onto your return array of arrays.  
      push @recombination, [@element_of_recombination];
      @element_of_recombination = ();
      $current[$last_string]++;
      }
    # Do the "carries" of the current elements 
    for ($count=$last_string; $count>0; $count--)
      {
      # if the current count is greater than the max number of elements,
      #     add one to prev and set current to 0.  
      if ($current[$count] > $max_elements[$count])
	{
	$current[$count] = 0;
	$current[$count-1] += 1;
	}
      # Once you hit an element that does not need to be carried exit.
      else
	{last;}
      }
    # if your first element has maxed out then exit the script
    if ($current[0] > $max_elements[0])
      {return (@recombination);}
    }
  
  }
