#!/usr/bin/perl
#!/usr/bin/perl
#  bmrbSPL2pks.pl
#	Converts BMRB spectral peak list frames to an AutoAssign peak lists.
#
#  usage: bmrbSPL2pks.pl input_bmrb_file [options]
#
#		input_bmrb_file - bmrb file to read ("-" to read from standard input).
#		options:
#			-id peak_list_id output_peak_list - use only the spectral peak list with given ID.
#
#
#  Copyright Hunter Moseley, 2002. All rights reserved.
#  Written by Hunter Moseley 3/16/2002
#
#
#  Updated to use new BMRBParsing/CMapParsing modules.
#  Copyright Hunter Moseley 2005. All rights reserved.
#

# These modules allow me to use modules rather than hardcoding everything into one script
use FindBin;
use lib $FindBin::Bin;

# Parser module parses in/out the cmap data and the peak file
use BMRBParsing qw(:ALL);
use PeakParsing qw(:ALL);

# Use the strict module: This is off because of silly bareword problems with the initialize_matirices
use strict;

if ((@ARGV < 1) || ($ARGV[0] =~ /^-h/))
  {
  print STDERR "Error: not enough parameters given.\n\n" if ($ARGV[0] !~ /^-h/);
  print STDERR "bmrbSPL2pks.pl\n";
  print STDERR "	Converts BMRB spectral peak list frames to an AutoAssign peak lists.\n";
  print STDERR "\n";
  print STDERR "  usage: bmrbSPL2pks.pl input_bmrb_file [options]\n";
  print STDERR "\n";
  print STDERR "		input_bmrb_file - bmrb file to read (\"-\" to read from standard input).\n";
  print STDERR "		options:\n";
  print STDERR "			-id peak_list_id output_peak_list - use only the spectral peak list with given ID.\n";
  print STDERR "\n";
  exit(1);
  }


# create history line
my $history_line = $0 . " " . join(' ',@ARGV) . "\n";

# read command line options
my $infile = shift @ARGV;

my $workbook_name = "";
my $peak_list_id = -1;
my $outfile = "";

while (($ARGV[0] =~ /^\-/) && @ARGV)
  {
  my $switch = shift @ARGV;
  if ($switch =~ /^\-i/)
    { 
    $peak_list_id = shift @ARGV; 
    $outfile = shift @ARGV;
    }
  else
    { 
    print STDERR "INVALID OPTION: $switch\n\nExiting Program ...\n\n";  
    exit(1);
    }
  }

# read bmrb file
my $bmrb_hlist = &read_bmrb_file($infile);


# find appropriate spectral peak list frame
my $found = 0;
foreach my $peak_list (values %{$$bmrb_hlist{"peak_lists"}})
  {
  if (($peak_list_id == -1 || ($$peak_list{"peak_list_id"} == $peak_list_id)))
    {
    $found = 1;
        
    # find workbookname if not specified
    if (exists $$peak_list{"experiment_label"})
      { 
      $workbook_name = $$peak_list{"experiment_label"}; 
      $workbook_name =~ s/\$?([^\_]*).*$/$1/;
      }
    
    if ($workbook_name eq "")
      {
      print STDERR "Error: No proper workbook name in spectral peak list frame in bmrb file $infile\n";
      next;
      }
    
    my $peakfile_hlist = {};
    if ($outfile ne "")
      { $$peakfile_hlist{"filename"} = $outfile; }
    else
      { $$peakfile_hlist{"filename"} = $workbook_name . ".pks"; }
    
    # build dimlist for peak list
    $$peakfile_hlist{"dimlist"} = [];
    foreach my $dim_description (@{$$peak_list{"dimension_descriptions"}})
      {
      my $test_name = $$dim_description{"spectral_region"};
      my $test_index = 1;
      while (grep {$_ eq $test_name;} (@{$$peakfile_hlist{"dimlist"}}))
	{ $test_name = $$dim_description{"spectral_region"} . $test_index++; }
      
      push @{$$peakfile_hlist{"dimlist"}}, $test_name;
      }
    
    # build peaks
    foreach my $peak (values %{$$peak_list{"plist"}})
      {
      my $new_peak = {};
      $$new_peak{"index"} = $$peak{"index"};
      $$new_peak{"intensity"} = $$peak{"intensity"};
      $$new_peak{"workbookname"} = $workbook_name;
      if (exists $$peak{"details"})
	{ $$new_peak{"workbookname"} .= "." . $$peak{"details"}; }
      
      for(my $x =0; $x < @{$$peakfile_hlist{"dimlist"}}; $x++)
	{
	my $dim_name = $$peakfile_hlist{"dimlist"}[$x];
	my $dim_id = $$peak_list{"dimension_descriptions"}[$x]{"dim_id"};
	
	$$new_peak{"shifts"}{$dim_name} = $$peak{"dimensions"}{$dim_id}{"shift"};
	}
      
      $$peakfile_hlist{"plist"}{$$new_peak{"index"}} = $new_peak;
      }
    
    # add File History
    $$peakfile_hlist{"end_comments"} = "File History: $outfile \n\n" . $history_line;
    if (exists $$peak_list{"details"} && ref($$peak_list{"details"}) eq "ARRAY" )
      { $$peakfile_hlist{"end_comments"} .= "\n" . join("\n",@{$$peak_list{"details"}}) . "\n"; }
    
    #
    # write new peak list file.
    #
    &outputPeakfile($peakfile_hlist, $$peakfile_hlist{"filename"});

    last if ($peak_list_id != -1);
    }
  }


if (! $found)
  {
  print STDERR "Error: No proper spectral peak list frame in bmrb file $infile\n";
  exit(1);
  }
