#!/usr/bin/perl
#!/usr/bin/perl
#
#  bmrb_assign_spl.pl
#       Assigns the spectral_peak_lists in a BMRB file.
#
#  usage: bmrb_assign_spl.pl in_bmrb_file out_bmrb_file [options] 
#
#		in_bmrb_file  - current assignments in bmrb format
#		out_bmrb_file  - current assignments in bmrb format
#		options:
#			-ambiguity num - level of ambiguity assignment to allow; 0 is none; 1 is everything within tolerance; default is 0.25 which equates to 1/4 the tolerance.
#			-override - let given atom_type tolerances override tolerances (shift_error) given in the peak lists.
#			-atolerance tol_pair ... - list of tolerances by atom_type.
#				tol_pair - atom_type tolerance (where atom_type can be H, N, HA, CA,...).
#			-ntolerance tol_pair ... - list of tolerances by nuclei_type.
#				tol_pair - nuclei_type tolerance (where nuclei_type can be H, N, C).
#
#   Author:     Gurmukh Sahota 04/27/2002
#   Copyright:  Gurmukh Sahota, 2002. All rights reserved.
#
#
#  A complete rewrite of the code by Hunter Moseley 08/18/2005
#    Rewrote and redesigned the major algorithms to allow proper reuse of peaks in assignments while fixing design flaws.  
#    Added meaningful handling of ambiguous peak assignments while preventing multiple peaks being assigned to the same chemical shift tuplet.
#    Updated to use new BMRBParsing/CMapParsing modules.
#
#  Copyright Hunter Moseley 2005. All rights reserved.
#

use FindBin;                    
use lib $FindBin::Bin;    

use BMRBParsing qw(:ALL);

use strict;

my %aa_name_3_to_1 = ( "ala" => "A", "arg" => "R", "asn" => "N", "asp" => "D",
		       "cys" => "C", "gln" => "Q", "glu" => "E", "gly" => "G",
		       "his" => "H", "ile" => "I", "leu" => "L", "lys" => "K",
		       "met" => "M", "phe" => "F", "pro" => "P", "ser" => "S",
		       "thr" => "T", "trp" => "W", "tyr" => "Y", "val" => "V",
		       "a"   => "A", "r"   => "R", "n"   => "N", "d"   => "D",
		       "c"   => "C", "q"   => "Q", "e"   => "E", "g"   => "G",
		       "h"   => "H", "i"   => "I", "l"   => "L", "k"   => "K",
		       "m"   => "M", "f"   => "F", "p"   => "P", "s"   => "S",
		       "t"   => "T", "w"   => "W", "y"   => "Y", "v"   => "V");

my %aromatic = ( F=>1, W=>1, Y=>1, H=>1 );
my %aromatic_atomTypes = ( F => {CG=>1, CD1=>1, CD2=>1,CE1=>1, CE2=>1, CZ=>1, HD1=>1, HD2=>1, HE1=>1, HE2=>1, HZ=>1}, 
                           W => {CG=>1, CD1=>1, CD2=>1, CE2=>1, CE3=>1, CZ2=>1, CZ3=>1, CH2=>1, HD1=>1, HE1=>1, HE3=>1, HZ2=>1, HZ3=>1, HH2=>1, NE1=>1}, 
			   Y => {CG=>1, CD1=>1, CD2=>1, CE1=>1, CE2=>1, CZ=>1, HD1=>1, HD2=>1, HE1=>1, HE2=>1, OH=>1, HH=>1}, 
			   H => {CG=>1, CD2=>1, CE1=>1, HD1=>1, HD2=>1, HE1=>1, HE2=>1, ND1=>1, NE2=>1} );

my %peak_list_types = ( 
			"HSQC" => [ ["H"], ["N"] ],
			"NH-HSQC" => [ ["H"], ["N"] ],
			"HNCO" => [ ["H"], ["N"], ["C-1"] ], 
			"HNcaCO" => [ ["H"], ["N"], ["C-1","C"] ], 
			"HNcoCA" => [ ["H"], ["N"], ["CA-1"] ], 
			"HNCA" => [ ["H"], ["N"], ["CA-1","CA"] ], 
			"HNcoCACB" => [ ["H"], ["N"], ["CA-1","CB-1"] ], 
			"HNCACB" => [ ["H"], ["N"], ["CA-1","CB-1","CA","CB"] ], 
			"HNcoHA" => [ ["H"], ["N"], ["HA-1","HA2-1", "HA3-1"] ], 
			"HNHA" => [ ["H"], ["N"], ["HA-1","HA2-1", "HA3-1","HA","HA2","HA3"] ], 
			"HNcoHAHB" => [ ["H"], ["N"], ["HA-1","HA2-1", "HA3-1","HB-1","HB2-1","HB3-1"] ], 
			"HNHAHB" => [ ["H"], ["N"], ["HA-1","HA2-1", "HA3-1","HB-1","HB2-1","HB3-1","HA","HA2","HA3","HB","HB2","HB3"] ], 
			"hCCcoNH" => [ ["H"], ["N"], ["CA-1","CB-1", ["CG-1","REQLKMP"], ["CG1-1","IV"], ["CG2-1","VTI"], ["CD-1","RKP"], ["CD1-1","IL"], ["CD2-1","L"], ["CE-1","K"]  ] ], 
			"hCCNH" => [ ["H"], ["N"], ["CA-1","CB-1", ["CG-1","REQLKMP"], ["CG1-1","IV"], ["CG2-1","VTI"], ["CD-1","RKP"], ["CD1-1","IL"], ["CD2-1","L"], ["CE-1","K"],
						    "CA","CB", ["CG","REQLKMP"], ["CG1","IV"], ["CG2","VTI"], ["CD","RKP"], ["CD1","IL"], ["CD2","L"], ["CE","K"] ] ],
			"HcccoNH" => [ ["H"], ["N"], ["HA-1","HA2-1", "HA3-1","HB-1","HB2-1","HB3-1", 
						      ["HG-1","L"], ["HG1-1","V"], ["HG2-1", "VREQIKMPT"], ["HG3-1", "REQKMP"], ["HG12-1", "I"], ["HG13-1", "I"],
						      ["HD1-1","IL"], ["HD2-1", "RLKP"], ["HD3-1", "RKP"], ["HE2-1", "K"], ["HE3-1", "K"] ] ], 
			"HccNH" => [ ["H"], ["N"], ["HA-1","HA2-1", "HA3-1","HB-1","HB2-1","HB3-1", 
						    ["HG-1","L"], ["HG1-1","V"], ["HG2-1", "VREQIKMPT"], ["HG3-1", "REQKMP"], ["HG12-1", "I"], ["HG13-1", "I"],
						    ["HD1-1","IL"], ["HD2-1", "RLKP"], ["HD3-1", "RKP"], ["HE2-1", "K"], ["HE3-1", "K"],
						    "HA","HA2", "HA3","HB","HB2","HB3", 
						    ["HG","L"], ["HG1","V"], ["HG2", "VREQIKMPT"], ["HG3", "REQKMP"], ["HG12", "I"], ["HG13", "I"],
						    ["HD1","IL"], ["HD2", "RLKP"], ["HD3", "RKP"], ["HE2", "K"], ["HE3", "K"] ] ]
			);

if ((@ARGV < 2) || $ARGV[0] =~ /^\-h/)
  {
  if ($ARGV[0] !~ /^\-h/)
    { print STDERR "Not enough parameters \n\n"; }
  
  print STDERR "  bmrb_assign_spl.pl\n";
  print STDERR "       Assigns the spectral_peak_lists in a BMRB file.\n";
  print STDERR "\n";
  print STDERR "  usage: bmrb_assign_spl.pl in_bmrb_file out_bmrb_file [options]\n";
  print STDERR "\n";
  print STDERR "		in_bmrb_file  - current assignments in bmrb format\n";
  print STDERR "		out_bmrb_file  - current assignments in bmrb format\n";
  print STDERR "		options:\n";
  print STDERR "			-ambiguity num - level of ambiguity assignment to allow; 0 is none; 1 is everything within tolerance; default is 0.25 which equates to 1/4 the tolerance.\n";
  print STDERR "			-override - let given atom_type tolerances override tolerances (shift_error) given in the peak lists.\n";
  print STDERR "			-atolerance tol_pair ... - list of tolerances by atom_type.\n";
  print STDERR "				tol_pair - atom_type tolerance (where atom_type can be H, N, HA, CA,...).\n";
  print STDERR "			-ntolerance tol_pair ... - list of tolerances by nuclei_type.\n";
  print STDERR "				tol_pair - nuclei_type tolerance (where nuclei_type can be H, N, C).\n";

  exit;
  }

my $bmrb_infile = shift @ARGV;
my $bmrb_outfile = shift @ARGV;
my %atom_match_tolerances;
my %nuclei_match_tolerances = ("H" => 0.03, "N" => 0.3, "C" => 0.3);
my $enable_aromatic=0;
my $ambiguity_level = 0.25;
my $override_tolerance = 0;

# Input parameters
while (@ARGV)
  {
  my $switch = shift @ARGV;
  if ($switch =~ /^\-am/)
    { $ambiguity_level = shift @ARGV; }
  elsif ($switch =~ /^\-o/)
    { $override_tolerance = 1; }
  elsif ($switch =~ /^\-at/)
    { 
    %atom_match_tolerances = (@ARGV); 
    last;
    }
  elsif ($switch =~ /^\-nt/)
    { 
    %nuclei_match_tolerances = (@ARGV); 
    last;
    }
  else
    { 
    print STDERR "INVALID_SWITCH: $switch \n\n"; 
    exit; 
    }
  }


# read the files into their data structures
my $bmrb_hlist = &read_bmrb_file($bmrb_infile);

foreach my $peak_list (values %{$$bmrb_hlist{"peak_lists"}})
  {
  my $peakfileDescriptor = $$peak_list{"experiment_label"};
  $peakfileDescriptor =~ s/\$?([^\_]*).*$/$1/;
  print "Working ... ", $peakfileDescriptor, "\n";

  if (exists $peak_list_types{$peakfileDescriptor})
    { &assignMatchingPeaks($bmrb_hlist, $peak_list, $peakfileDescriptor); }

  }

&write_bmrb_file($bmrb_hlist, $bmrb_outfile);


#
# Subroutines
#

sub assignMatchingPeaks
  {
  my $bmrb_hlist  = shift @_;
  my $peak_list   = shift @_;
  my $list_type = shift @_;
  my $assignment_alist = [];

  my $peakdef = $peak_list_types{$list_type};

  foreach my $entity (values %{$$bmrb_hlist{"entity_list"}})
    {
    foreach my $residue ( @{$$entity{"name_array"}} )
      {	
      my $peak_dim_counter = [ map { 0; } (@$peakdef) ];
      while ($$peak_dim_counter[0] < @{$$peakdef[0]})
	{
	my @shift_ahlist = ();
	
	for (my $count = 0; $count < @$peakdef; $count++)
	  {
	  my $atomType = $$peakdef[$count][$$peak_dim_counter[$count]];
	  if (ref($atomType) eq "ARRAY")
	    {
	    my $aa = $$residue{"aa"};
	    last if ($$atomType[1] !~ /$aa/);
	    $atomType = $$atomType[0]
	    }
	  
	  if (($atomType =~ /^(\w+)\-1$/) && (defined $$entity{"rlist"}{$residue}{"prev"})) # test inter peak
	    {
	    my $std_atomType = $1;
	    my $prevresidue = $$entity{"rlist"}{$residue}{"prev"};
	    
	    # and see if it is defined as a standard atomType on the previous residue
	    last if ( ! (exists $$entity{"rlist"}{$prevresidue}{"shifts"}{$std_atomType} && exists $$entity{"rlist"}{$prevresidue}{"shifts"}{$std_atomType}{"list"} && 
			 @{$$entity{"rlist"}{$prevresidue}{"shifts"}{$std_atomType}{"list"}}));

	    $std_atomType =~ /^(\w+)\d?$/;
	    $shift_ahlist[$count] = { 
	      "entity" => $entity,
	      "fromResidue" => $prevresidue,
	      "stdAtomType" => $std_atomType, 
	      "supraAtomType" => $1, 
	      "list" => $$entity{"rlist"}{$prevresidue}{"shifts"}{$std_atomType}{"list"},
	      "idlist" => $$entity{"rlist"}{$prevresidue}{"shifts"}{$std_atomType}{"idlist"},
	      "list_id_list" => $$entity{"rlist"}{$prevresidue}{"shifts"}{$std_atomType}{"list_id_list"},
	      "curr" => 0 }; 
	    }		
	  else # test intra peak
	    {
	    last if (! (exists $$entity{"rlist"}{$residue}{"shifts"}{$atomType} && exists $$entity{"rlist"}{$residue}{"shifts"}{$atomType}{"list"} && 
			@{$$entity{"rlist"}{$residue}{"shifts"}{$atomType}{"list"}}));

	    $atomType =~ /^(\w+)\d?$/;
	    $shift_ahlist[$count] = { 
	      "entity" => $entity,
	      "fromResidue" => $residue,
	      "stdAtomType" => $atomType,
	      "supraAtomType" => $1, 
	      "list" => $$entity{"rlist"}{$residue}{"shifts"}{$atomType}{"list"}, 
	      "idlist" => $$entity{"rlist"}{$residue}{"shifts"}{$atomType}{"idlist"},
	      "list_id_list" => $$entity{"rlist"}{$residue}{"shifts"}{$atomType}{"list_id_list"},
	      "curr" => 0 }; 
	    }
	  }

	# start creating peaks if the proper shift lists are there.
	if (scalar(@shift_ahlist) == scalar(@$peakdef))
	  {
	  while($shift_ahlist[0]{"curr"} < @{$shift_ahlist[0]{"list"}})
	    {
	    # match peaks from the peak list that are less than the tolerence
	    # CODING HAZARD :: see the \@$assignment_alist.  That was so that the subroutine got a pointer and could
	    #                  return values.
	    &matching_peaks(\@shift_ahlist, $peak_list, $assignment_alist);
	    
	    # increment curr counters in shift_ahlist.
	    # the curr counter tags allow us to have a list of resonances for each peak type and then 
	    # using the same logic as the recombine subroutine, we iterate until we have permuted through
	    # the entire list.
	    $shift_ahlist[$#shift_ahlist]{"curr"}++;	    
	    for (my $count = $#$peakdef; $count > 0; $count--)
	      {
	      if ($shift_ahlist[$count]{"curr"} >= @{$shift_ahlist[$count]{"list"}})
		{ 
		$shift_ahlist[$count]{"curr"} = 0;
		$shift_ahlist[$count-1]{"curr"}++;
		} 
	      }		    
	    }  
	  }

	# increment to new peak definition
	$$peak_dim_counter[$#$peakdef]++;
	for (my $count = $#$peakdef; $count > 0; $count--)
	  {
	  if ($$peak_dim_counter[$count] >= @{$$peakdef[$count]})
	    { 
	    $$peak_dim_counter[$count] = 0;
	    $$peak_dim_counter[$count-1]++;
	    }
	  }
	}
      }
    }
  
  if ($assignment_alist)  
    {  &assign_peaks($peak_list, $assignment_alist); }
  }

sub matching_peaks 
  {
  my $shift_ahlist = shift @_;
  my $peak_list = shift @_;
  my $assignment_alist = shift @_;
  
  # forevery peak in the peakfile
  foreach my $peak ( values %{$$peak_list{"plist"}} )
    {
    my $dimension_assignment_alist = [];
    my $completed=0;
    my $distance=0;
    my $name = "";

    # count through the dimensions
    foreach my $dim (keys %{$$peak{"dimensions"}})
      {
      my $dimension = $$peak{"dimensions"}{$dim};
      my $count = $dim - 1;
      
      # if the found peak and and the peak from the peak file is less than the tolerence
      my $shiftDifference = abs($$dimension{"shift"} - $$shift_ahlist[$count]{"list"}[$$shift_ahlist[$count]{"curr"}]);
      my $shiftTolerance = $nuclei_match_tolerances{substr($$shift_ahlist[$count]{"supraAtomType"},0,1)};
      if (exists $atom_match_tolerances{$$shift_ahlist[$count]{"supraAtomType"}} && ($override_tolerance || ! exists $$dimension{"shift_error"}))
	{ $shiftTolerance = $atom_match_tolerances{$$shift_ahlist[$count]{"supraAtomType"}}; }
      elsif (exists $$dimension{"shift_error"})
	{ $shiftTolerance = $$dimension{"shift_error"}; }

      if ($shiftDifference <= $shiftTolerance)
	{
	# add the distances
	$distance += ($shiftDifference/$shiftTolerance) ** 2;
	
	$name .= $$shift_ahlist[$count]{"entity"}{"entity_id"} . "_" . $$shift_ahlist[$count]{"fromResidue"} . "_" . $$shift_ahlist[$count]{"stdAtomType"} . "_" . $$shift_ahlist[$count]{"idlist"}[$$shift_ahlist[$count]{"curr"}] . "_";
	
	# if the atomType is not from the previous residue
	$$dimension_assignment_alist[$count]{"entity_id"} = $$shift_ahlist[$count]{"entity"}{"entity_id"};
	$$dimension_assignment_alist[$count]{"aa"} = $$shift_ahlist[$count]{"entity"}{"rlist"}{$$shift_ahlist[$count]{"fromResidue"}}{"aa"};
	$$dimension_assignment_alist[$count]{"index"} = $$shift_ahlist[$count]{"entity"}{"rlist"}{$$shift_ahlist[$count]{"fromResidue"}}{"index"};
	$$dimension_assignment_alist[$count]{"atom_name"} = $$shift_ahlist[$count]{"stdAtomType"};
	$$dimension_assignment_alist[$count]{"shift_id"} = $$shift_ahlist[$count]{"idlist"}[$$shift_ahlist[$count]{"curr"}];
	$$dimension_assignment_alist[$count]{"list_id"} = $$shift_ahlist[$count]{"list_id_list"}[$$shift_ahlist[$count]{"curr"}];
	$completed=1;	
	}
      else
	{$completed = 0; last;}
      }

    # if all of the AtomTypes were there and under tolerence then add the assignment_alist
    if ($completed)
      { push @$assignment_alist, { "name" => $name, "distance" => sqrt($distance), "peak_index" => $$peak{"index"}, "dimension_assignment" => $dimension_assignment_alist}; }
    }

  return;
  }


sub assign_peaks
  {  
  my $peak_list = shift @_;
  my $assignment_alist = shift @_;
  
  my %peak_seen;
  my %assignment_seen;

  foreach my $assignment (sort { $$a{"distance"} <=> $$b{"distance"}; } (@$assignment_alist))
    {
    if (! exists $assignment_seen{$$assignment{"name"}} && 
	(! exists $peak_seen{$$assignment{"peak_index"}} || 
	 (($ambiguity_level != 0) && (abs($peak_seen{$$assignment{"peak_index"}}{"distance"} - $$assignment{"distance"}) < $ambiguity_level))))
      {
      $assignment_seen{$$assignment{"name"}}++;

      my $set_id = 1;
      if (exists $peak_seen{$$assignment{"peak_index"}})
	{ $set_id = $peak_seen{$$assignment{"peak_index"}}{"set_id"} + 1; }
      $peak_seen{$$assignment{"peak_index"}} = $assignment;
      $peak_seen{$$assignment{"peak_index"}}{"set_id"} = $set_id;

      foreach my $dim (keys %{$$peak_list{"plist"}{$$assignment{"peak_index"}}{"dimensions"}})
	{
	my $dimension = $$peak_list{"plist"}{$$assignment{"peak_index"}}{"dimensions"}{$dim};
	if (! exists $$dimension{"assignments"})
	  { $$dimension{"assignments"} = []; }

	$$assignment{"dimension_assignment"}[$dim-1]{"set_id"} = $set_id;
	push @{$$dimension{"assignments"}}, $$assignment{"dimension_assignment"}[$dim-1];
	}
      }
    }
  }
