#!/usr/bin/perl
#!/usr/bin/perl
#  bmrb_calculate_shifting.pl
#	Calculate difference in same shifts between two bmrb files.
#
#  usage: bmrb_calculate_shifting.pl input_bmrb_file1 input_bmrb_file2 [options]
#
#	bmrb_file - name of bmrb file or "-" for standard input/output.
#	options:
#		-verbose - give extra information during calculation.
#		-degeneracy - treat shift degenerately.
#		-cutoff - inclusion cutoff in units of standard deviation.
# 
#
#  Copyright Hunter Moseley, 2000. All rights reserved.
#  Written by Hunter Moseley 7/31/2000
#
#
#  Updated to use new BMRBParsing/CMapParsing modules.
#  Copyright Hunter Moseley 2005. All rights reserved.
#
use strict;

# allows perl to fine local modules.
use FindBin;
use lib $FindBin::Bin;

# contains subroutines for reading and writing bmrb files.
use BMRBParsing qw(:ALL);


# print help info.
if ((@ARGV < 2) || ($ARGV[0] =~ /^\-h/))
  {
  print STDERR "Error: not enough parameters given.\n\n" if (@ARGV < 2);
  print STDERR "bmrb_calculate_shifting.pl\n";
  print STDERR "	Calculate difference in same shifts between two bmrb files.\n";
  print STDERR "\n";
  print STDERR "  usage: bmrb_calculate_shifting.pl input_bmrb_file1 input_bmrb_file2 [options]\n";
  print STDERR "\n";
  print STDERR "	bmrb_file - name of bmrb file or \"-\" for standard input/output.\n";
  print STDERR "	options:\n";
  print STDERR "		-verbose - give extra information during calculation.\n";
  print STDERR "		-degeneracy - treat shift degenerately.\n";
  print STDERR "		-cutoff - inclusion cutoff in units of standard deviation.\n";
  exit(1);
  }

# read command line arguments.
my $input_filename1 = shift @ARGV;
my $input_filename2 = shift @ARGV;

my $verbose = 0;
my $degeneracy = 0;
my $sd_cutoff = 4;

for(my $x=0; $x < @ARGV; $x++) 
  {
  if ($ARGV[$x] =~ /^-v/)
    { $verbose = 1; }
  elsif ($ARGV[$x] =~ /-d/)
    { $degeneracy = 1; }
  elsif ($ARGV[$x] =~ /-cu/)
    { $x++; $sd_cutoff = $ARGV[$x]; }
  else
    {
    print STDERR "Error: unknown option ",$ARGV[$x]," given.\n";
    exit(1);
    }
  }

my $read_options = { "convert_shift_names" => $degeneracy };
my $input_bmrb_hlist1 = &read_bmrb_file($input_filename1, $read_options );
my $input_bmrb_hlist2 = &read_bmrb_file($input_filename2, $read_options );

my %shifting_info = ( "H" => [], "C" => [], "N" => [] );
my %shifting_ave;
my %shift_count;

# cycle through each residue
foreach my $entity_id (keys %{$$input_bmrb_hlist1{"entity_list"}})
  {
  foreach my $residue_name (keys %{$$input_bmrb_hlist1{"entity_list"}{$entity_id}{"rlist"}})
    {
    # cycle through each shift
    foreach my $shift_name (keys %{$$input_bmrb_hlist1{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"shifts"}})
      {
      if (exists $$input_bmrb_hlist1{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"shifts"}{$shift_name} 
	  && exists $$input_bmrb_hlist2{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"shifts"}{$shift_name} 
	  && (scalar(@{$$input_bmrb_hlist1{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"shifts"}{$shift_name}{"list"}}) ==
	      scalar(@{$$input_bmrb_hlist2{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"shifts"}{$shift_name}{"list"}}))  )
	{
	my $shift1 = $$input_bmrb_hlist1{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"shifts"}{$shift_name};
	my $shift2 = $$input_bmrb_hlist2{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"shifts"}{$shift_name};
	my $atom_type = $$shift1{"atom_type"};
	
	my $total1 = 0.0;
	map { $total1 += $_; } @{$$shift1{"list"}};
	my $total2 = 0.0;
	map { $total2 += $_; } @{$$shift2{"list"}};
	
	my $info = {};
	my $difference = $total2 - $total1;
	$$info{"difference"} = $difference;
	$$info{"count"} = @{$$shift1{"list"}};
      	
	$shifting_ave{$atom_type} += $difference;
	$shift_count{$atom_type} += $$info{"count"};
	
	if ($verbose)
	  {
	  my $list1 = $$shift1{"list"};
	  my $list2 = $$shift2{"list"};
	  if (@$list1 == 1)
	    { $$info{"verbose_line"} = "\t$residue_name.$shift_name:\t $total1 -> $total2 \t=> $difference "; }
	  else
	    { $$info{"verbose_line"} = "\t$residue_name.$shift_name:\t $total1 -> $total2 \t=> $difference \t: ( @$list1) -> ( @$list2 )"; }
	  }
	
	push @{$shifting_info{$atom_type}},  $info;
	}
      }
    }
  }

# calculate standard deviation.
my %shifting_sd;

foreach my $atom_type (keys %shifting_ave)
  {
  $shifting_ave{$atom_type} = $shifting_ave{$atom_type} / $shift_count{$atom_type};

  my $count = 0;
  $shifting_sd{$atom_type} = 0.0;
  foreach my $info (@{$shifting_info{$atom_type}})
    {
    $shifting_sd{$atom_type} += (($$info{"difference"} / $$info{"count"})  - $shifting_ave{$atom_type}) ** 2;
    $count++;
    }

  $shifting_sd{$atom_type} = ($shifting_sd{$atom_type} / $count) ** 0.5 ;
  }


# print the results
foreach my $atom_type (keys %shifting_ave)
  {
  my $final_shifting = 0.0;
  my $final_shift_count = 0;
  my $exclude_count = 0;
  my $verbose_info = "";
  my $inclusion_range = $shifting_sd{$atom_type} * $sd_cutoff;
  foreach my $info (@{$shifting_info{$atom_type}})
    {
    if ($inclusion_range >= abs($$info{"difference"} / $$info{"count"} - $shifting_ave{$atom_type}))
      {
      $final_shifting += $$info{"difference"};
      $final_shift_count += $$info{"count"};
      $verbose_info .= $$info{"verbose_line"} . "*\n";
      }
    else
      { 
      $exclude_count++;
      $verbose_info .= $$info{"verbose_line"} . "\n"; 
      }
    }

  if ($final_shift_count)
    { $final_shifting = $final_shifting / $final_shift_count; }
  print " $atom_type shifting for $input_filename1 => $input_filename2: $final_shifting\n";
  if ($verbose)
    {
    print "\tInclusion range = ",$shifting_ave{$atom_type}," +/- ",$inclusion_range,"\n";
    print "\tIncluded $final_shift_count ; excluded $exclude_count\n";
    print "\n",$verbose_info,"\n\n"; 
    }
  }
