#!/usr/bin/perl
#  bmrb_correct_deuteration.pl
#	Corrects for deuterium shift effects on carbon and nitrogen chemical shifts for amide rooted out-and-back NMR experiments. 
#
#  usage: bmrb_correct_deuteration.pl input_filename output_filename [options]
#
#	filename - bmrb filename or "-" for standard input/output.
#	options:
#		-deuration fraction - fraction of deuteration incorporated (default is 1.0).
#		-amide fraction - fraction of labile amide hydrogens that are deuterated (default is 0.1 for 10% D2O normally present).  
#		-pyruvate fraction - fraction of protonated pyruvate incorporated at L methyls (i.e. should be roughly 0.8).
#			3/4 of the fraction is used for VI methyl protonation (I gamma 2 only).
#			1/2 of the fraction is used for A methyl protonation.
#			This usually generates 80% L, 60% VI, and 40% A protonated samples.
#			Ketobutyricacid and ketoisovaleric acid take precedence over pyruvate.
#		-ketobutyricacid fraction - fraction of protonated ketobutyric acid incorporated (i.e. should be roughly 0.95)
#			This usually generates 95% I protonated samples (on delta 1 only). 
#		-ketoisovalericacid fraction - fraction of protonated ketoisovaleric acid incorporated (i.e. should be roughly 0.95)
#			This usually generates 95% LV protonated samples. 
#			This assumes that both methyls on ketoisovaleric acid are protonated and C13 labelled. 
#		-shikimicacid fraction - fraction of protonated shikimic acid incorporated (i.e. should be 0.70)
#			This usually generates 70% FYW protonated samples.  But their HAs and HBs are 100% deuterated.
#			If shikimate auxotrophic bacteria are used, then the incorporation should be 1.0, except for the HA/HBs of course.
#		-added aa_list side_fraction ha_fraction - side_fraction of listed protonated amino acids incorporated (i.e. side_fraction can be 0.9 if well incorporated)
#			side_fraction ranges based on the bacterial strain and the expression protocol.  It can be as low as 0.5.
#			ha_fraction is the fraction of protonated HAs and is often 0.2 but can range as high as 0.7. The final HA protonation is fraction * ha_fraction.
#			Added amino acids takes precedence over added metabolic precursors.  Precursors are applied to what amino acid fraction not incorporated.
#		-fraction_table filename - deuteration fraction table to use. Whatever is not in the table is filled in by other parameters.
#		-print_fraction_table - print the deuteration fraction table.  This can be used as a starting point for making a custom table.
#			Remember that deuteration fraction is (1 - protonation_fraction).
#		-correction_table filename - deuteration correction table to use. Whatever is not in the table is filled in by other parameters.
#		-print_correction_table - print the deuteration correction table.  This can be used as a starting point for making a custom table.
#			The correction table is made from 1-bond, 2-bond, 3-bond deuterium shift effects matrix multipled by the deuteration fraction table.
#		-ILV - same as "-ketob 0.95 -ketoi 0.95".
#		-ILVFY - same as "-ketob 0.95 -ketoi 0.95 -ad FY 0.9 0.2".
#
#  Copyright Hunter Moseley, 2008. All rights reserved.
#  Written by Hunter Moseley 3/18/2008
#
#
use strict;

# allows perl to fine local modules.
use FindBin;
use lib $FindBin::Bin;

# contains subroutines for reading and writing bmrb files.
use BMRBParsing qw(:ALL);


# print help info.
if ((@ARGV < 2) || ($ARGV[0] =~ /^-h/))
  {
  print STDERR "Error: not enough parameters given.\n\n" if (@ARGV < 2);

  print STDERR "  bmrb_correct_deuteration.pl\n";
  print STDERR "	Correct for deuterium shift effects for carbon and nitrogen chemical shifts\n";
  print STDERR "\n";
  print STDERR "  usage: bmrb_correct_deuteration.pl input_filename output_filename [options]\n";
  print STDERR "\n";
  print STDERR "	filename - bmrb filename or \"-\" for standard input/output.\n";
  print STDERR "	options:\n";
  print STDERR "		-deuration fraction - fraction of deuteration incorporated (default is 1.0).\n";
  print STDERR "		-amide fraction - fraction of labile amide hydrogens that are deuterated (default is 0.1 for 10% D2O normally present).\n";
  print STDERR "		-pyruvate fraction - fraction of protonated pyruvate incorporated at L methyls (i.e. should be roughly 0.8).\n";
  print STDERR "			3/4 of the fraction is used for VI methyl protonation (I gamma 2 only).\n";
  print STDERR "			1/2 of the fraction is used for A methyl protonation.\n";
  print STDERR "			This usually generates 80% L, 60% VI, and 40% A protonated samples.\n";
  print STDERR "			Ketobutyricacid and ketoisovaleric acid take precedence over pyruvate.\n";
  print STDERR "		-ketobutyricacid fraction - fraction of protonated ketobutyric acid incorporated (i.e. should be roughly 0.95)\n";
  print STDERR "			This usually generates 95% I methyl protonated samples (on delta 1 only). \n";
  print STDERR "		-ketoisovalericacid fraction - fraction of protonated ketoisovaleric acid incorporated (i.e. should be roughly 0.95)\n";
  print STDERR "			This usually generates 95% LV methyl protonated samples. \n";
  print STDERR "			This assumes that both methyls on ketoisovaleric acid are protonated and C13 labelled. \n";
  print STDERR "		-shikimicacid fraction - fraction of protonated shikimic acid incorporated (i.e. should be 0.70)\n";
  print STDERR "			This usually generates 70% FYW protonated samples.  But their HAs and HBs are 100% deuterated.\n";
  print STDERR "			If shikimate auxotrophic bacteria are used, then the incorporation should be 1.0, except for the HA/HBs of course.\n";
  print STDERR "		-added aa_list side_fraction ha_fraction - side_fraction of listed protonated amino acids incorporated (i.e. side_fraction can be 0.9 if well incorporated).\n";
  print STDERR "			side_fraction ranges based on the bacterial strain and the expression protocol.  It can be as low as 0.5.\n";
  print STDERR "			ha_fraction is the fraction of protonated HAs and is often 0.2 but can range as high as 0.7. The final HA protonation is fraction * ha_fraction.\n";
  print STDERR "			Added amino acids takes precedence over added metabolic precursors.  Precursors are applied to what amino acid fraction not incorporated.\n";
  print STDERR "		-fraction_table filename - deuteration fraction table to use. Whatever is not in the table is filled in by other parameters.\n";
  print STDERR "		-print_fraction_table - print the deuteration fraction table.  This can be used as a starting point for making a custom table.\n";
  print STDERR "			Remember that deuteration fraction is (1 - protonation_fraction).\n";
  print STDERR "		-correction_table filename - deuteration correction table to use. Whatever is not in the table is filled in by other parameters.\n";
  print STDERR "		-print_correction_table - print the deuteration correction table.  This can be used as a starting point for making a custom table.\n";
  print STDERR "			The correction table is made from 1-bond, 2-bond, 3-bond deuterium shift effects matrix multipled by the deuteration fraction table.\n";
  print STDERR "		-ILV - same as \"-ketob 0.95 -ketoi 0.95\".\n";
  print STDERR "		-ILVFY - same as \"-ketob 0.95 -ketoi 0.95 -ad FY 0.9 0.2\".\n";

  exit(1);
  }

# read command line arguments.
my $input_filename = shift @ARGV;
my $output_filename = shift @ARGV;

my $deuteration = 1.0;
my $amide_fraction = 0.1;
my $pyruvate_fraction = 0;
my $ketobutyric_fraction = 0;
my $ketoisovaleric_fraction = 0;
my $shikimic_fraction = 0;
my $aa_fraction = {};
my $deuteration_fraction_table_filename = "";
my $print_fraction_table = 0;
my $deuteration_correction_table_filename = "";
my $print_correction_table = 0;

while (@ARGV && ($ARGV[0] =~ /^\-/))
  {
  my $switch = shift @ARGV;
  if ($switch =~ /^\-d/)
    { $deuteration = shift @ARGV; }
  elsif ($switch =~ /^\-am/)
    { $amide_fraction = shift @ARGV; }
  elsif ($switch =~ /^\-f/)
    { $deuteration_fraction_table_filename = shift @ARGV; }
  elsif ($switch =~ /^\-c/)
    { $deuteration_correction_table_filename = shift @ARGV; }
  elsif ($switch =~ /^\-print_f/)
    { $print_fraction_table = 1; }
  elsif ($switch =~ /^\-print_c/)
    { $print_correction_table = 1; }
  elsif ($switch =~ /^\-py/)
    { $pyruvate_fraction = shift @ARGV; }
  elsif ($switch =~ /^\-ketob/)
    { $ketobutyric_fraction = shift @ARGV; }
  elsif ($switch =~ /^\-ketoi/)
    { $ketoisovaleric_fraction = shift @ARGV; }
  elsif ($switch =~ /^\-s/)
    { $shikimic_fraction = shift @ARGV; }
  elsif ($switch =~ /^\-ad/)
    {
    my $aa_list = shift @ARGV;
    my $fraction = shift @ARGV;
    my $ha_fraction = shift @ARGV;
    foreach my $aa (split(//,$aa_list))
      { 
      $$aa_fraction{$aa}{"incorportation"} = $fraction; 
      $$aa_fraction{$aa}{"HA_incorporation"} = $ha_fraction * $fraction; 
      }
    }
  elsif ($switch =~ /^\-ILVFY/)
    { 
    $ketobutyric_fraction = 0.95; 
    $ketoisovaleric_fraction = 0.95;
    $$aa_fraction{"F"}{"incorporation"} = 0.9;
    $$aa_fraction{"F"}{"HA_incorporation"} = 0.2 * 0.9;
    $$aa_fraction{"Y"}{"incorporation"} = 0.9;
    $$aa_fraction{"Y"}{"HA_incorporation"} = 0.2 * 0.9;
    }
  elsif ($switch =~ /^\-ILV/)
    { 
    $ketobutyric_fraction = 0.95; 
    $ketoisovaleric_fraction = 0.95;
    }
  else
    {
    print STDERR "INVALID OPTION: $switch\n\nExiting Program ...\n\n";
    exit(1);
    }
  }


#
my $oab_deuteration_effects = &create_out_and_back_deuteration_effects_table();

my $deuteration_fraction_table = &create_deuteration_fraction_table($deuteration,$amide_fraction,$pyruvate_fraction,$ketobutyric_fraction,$ketoisovaleric_fraction,$shikimic_fraction,$aa_fraction);
if ($deuteration_fraction_table_filename ne "")
  { &read_table($deuteration_fraction_table,$deuteration_fraction_table_filename); }

if ($print_fraction_table)
  { &print_table($deuteration_fraction_table, "Deuteration Fraction Table"); }

my $deuteration_correction_table = &create_deuteration_correction_table($oab_deuteration_effects, $deuteration_fraction_table);
if ($deuteration_correction_table_filename ne "")
  { &read_table($deuteration_correction_table,$deuteration_correction_table_filename); }

if ($print_correction_table)
  { &print_table($deuteration_correction_table, "Deuteration Correction Table"); }


# read the bmrb file
my $input_bmrb_hlist = &read_bmrb_file($input_filename, { } );

# cycle through each residue
foreach my $entity (values %{$$input_bmrb_hlist{"entity_list"}})
  {
  foreach my $residue_name (keys %{$$entity{"rlist"}})
    {
    my $aa = &convert_aa3to1($$entity{"rlist"}{$residue_name}{"aa"});
    next if (! exists $$deuteration_correction_table{$aa});

    # cycle through each shift
    foreach my $shift_name (keys %{$$entity{"rlist"}{$residue_name}{"shifts"}})
      {
      next if (! exists $$deuteration_correction_table{$aa}{$shift_name});

      # cycle through each shift value.
      foreach my $value (@{$$entity{"rlist"}{$residue_name}{"shifts"}{$shift_name}{"list"}})
	{ $value += $$deuteration_correction_table{$aa}{$shift_name}; } # update the shift
      }
    }
  }

# write updated bmrb file
&write_bmrb_file($input_bmrb_hlist, $output_filename);



#
# Subroutines
#

sub print_table
  {
  my $table = shift @_;
  my $title = shift @_;

  print "\n# $title\n";
  foreach my $aa (sort { $a cmp $b } (keys %$table))
    {
    print "$aa:";
    foreach my $atom (sort { $a cmp $b } (keys %{$$table{$aa}}))
      { print " $atom ",$$table{$aa}{$atom}; }

    print "\n";
    }
  }

sub read_table
  { 
  my $table = shift @_;
  my $table_filename = shift @_;

  local *TABLEFILE;
  open(TABLEFILE, $table_filename) || die ("open " . $table_filename . " failed."); 
  my $line;
  while ($line = <TABLEFILE>)
    {
    next if ($line =~ /^\#/); # skip comment lines

    ($line) = ($line =~/^\s*(.*)/); # skip leading white space.
    chomp $line;

    if ($line =~ /^[a-zA-Z][:]/)
      {
      my ($aa,$temp) = ($line =~ /^([a-zA-Z])[:](.*)/);
      ($temp) = ($temp =~/^\s*(.*)/); # skip leading white space.
      my @fraction_list = split(/\s+/,$temp);
      while (@fraction_list > 1)
	{
	my $atom = shift @fraction_list;
	my $value = shift @fraction_list;
	if ($atom !~ /^[HCN]/)
	  { 
	  print STDERR "Invalid atom $atom in table file line \"$line\"\n"; 
	  next; 
	  }

	$$table{$aa}{$atom} = $value;
	}	
      }
    }

  close TABLEFILE;  
  }


sub create_deuteration_correction_table
  {
  my $oab_deuteration_effects = shift @_;
  my $deuteration_fraction_table = shift @_;

  my $deuteration_correction_table = {};

  foreach my $aa (keys %$oab_deuteration_effects)
    {
    foreach my $atom (keys %{$$oab_deuteration_effects{$aa}})
      {
      my $correction = 0;
      foreach my $hydrogen (%{$$oab_deuteration_effects{$aa}{$atom}})
	{ $correction += $$oab_deuteration_effects{$aa}{$atom}{$hydrogen} * $$deuteration_fraction_table{$aa}{$hydrogen}; }

      $$deuteration_correction_table{$aa}{$atom} = $correction;
      }
    }

  return $deuteration_correction_table;
  }


sub create_deuteration_fraction_table
  {
  my $deuteration = shift @_;
  my $amide_fraction = shift @_;
  my $pyruvate_fraction = shift @_;
  my $ketobutyric_fraction = shift @_;
  my $ketoisovaleric_fraction = shift @_;
  my $shikimic_fraction = shift @_;
  my $aa_fraction = shift @_;

  my $deuteration_fraction_table = {};


  # Ala - A
  $$deuteration_fraction_table{A}{H} = $amide_fraction;
  $$deuteration_fraction_table{A}{HA} = (1 - $$aa_fraction{A}{"HA_incorporation"}) * $deuteration;
  $$deuteration_fraction_table{A}{HB} = (1 - ($$aa_fraction{A}{"incorporation"} + $pyruvate_fraction * 0.5 * (1 - $$aa_fraction{A}{"incorporation"}))) * $deuteration;

  # Cys - C
  $$deuteration_fraction_table{C}{H} = $amide_fraction;
  $$deuteration_fraction_table{C}{HA} = (1 - $$aa_fraction{C}{"HA_incorporation"}) * $deuteration;
  $$deuteration_fraction_table{C}{HB} = (1 - $$aa_fraction{C}{"incorporation"}) * $deuteration;

  # Asp - D
  $$deuteration_fraction_table{D}{H} = $amide_fraction;
  $$deuteration_fraction_table{D}{HA} = (1 - $$aa_fraction{D}{"HA_incorporation"}) * $deuteration;
  $$deuteration_fraction_table{D}{HB} = (1 - $$aa_fraction{D}{"incorporation"}) * $deuteration;

  # Glu - E
  $$deuteration_fraction_table{E}{H} = $amide_fraction;
  $$deuteration_fraction_table{E}{HA} = (1 - $$aa_fraction{E}{"HA_incorporation"}) * $deuteration;
  $$deuteration_fraction_table{E}{HB} = (1 - $$aa_fraction{E}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{E}{HG} = (1 - $$aa_fraction{E}{"incorporation"}) * $deuteration;

  # Phe - F
  $$deuteration_fraction_table{F}{H} = $amide_fraction;
  $$deuteration_fraction_table{F}{HA} = (1 - $$aa_fraction{F}{"HA_incorporation"}) * $deuteration;
  $$deuteration_fraction_table{F}{HB} = (1 - $$aa_fraction{F}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{F}{HD} = (1 - ($$aa_fraction{F}{"incorporation"} + $shikimic_fraction * (1 - $$aa_fraction{F}{"incorporation"}))) * $deuteration;
  $$deuteration_fraction_table{F}{HE} = (1 - ($$aa_fraction{F}{"incorporation"} + $shikimic_fraction * (1 - $$aa_fraction{F}{"incorporation"}))) * $deuteration;
  $$deuteration_fraction_table{F}{HZ} = (1 - ($$aa_fraction{F}{"incorporation"} + $shikimic_fraction * (1 - $$aa_fraction{F}{"incorporation"}))) * $deuteration;

  # Gly - G
  $$deuteration_fraction_table{G}{H} = $amide_fraction;
  $$deuteration_fraction_table{G}{HA} = (1 - $$aa_fraction{G}{"HA_incorporation"}) * $deuteration;

  # His - H
  $$deuteration_fraction_table{H}{H} = $amide_fraction;
  $$deuteration_fraction_table{H}{HA} = (1 - $$aa_fraction{H}{"HA_incorporation"}) * $deuteration;
  $$deuteration_fraction_table{H}{HB} = (1 - $$aa_fraction{H}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{H}{HD1} = $amide_fraction;
  $$deuteration_fraction_table{H}{HD2} = (1 - $$aa_fraction{H}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{H}{HE1} = (1 - $$aa_fraction{H}{"incorporation"}) * $deuteration;

  # Ile - I
  $$deuteration_fraction_table{I}{H} = $amide_fraction;
  $$deuteration_fraction_table{I}{HA} = (1 - $$aa_fraction{I}{"HA_incorporation"}) * $deuteration;
  $$deuteration_fraction_table{I}{HB} = (1 - $$aa_fraction{I}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{I}{HG1} = (1 - $$aa_fraction{I}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{I}{HG2} = (1 - ($$aa_fraction{I}{"incorporation"} + $pyruvate_fraction * 0.75 * (1 - $$aa_fraction{I}{"incorporation"}) * (1 - $ketobutyric_fraction))) * $deuteration;
  $$deuteration_fraction_table{I}{HD1} = (1 - ($$aa_fraction{I}{"incorporation"} + $ketobutyric_fraction * (1 - $$aa_fraction{I}{"incorporation"}))) * $deuteration;

  # Lys - K
  $$deuteration_fraction_table{K}{H} = $amide_fraction;
  $$deuteration_fraction_table{K}{HA} = (1 - $$aa_fraction{K}{"HA_incorporation"}) * $deuteration;
  $$deuteration_fraction_table{K}{HB} = (1 - $$aa_fraction{K}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{K}{HG} = (1 - $$aa_fraction{K}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{K}{HD} = (1 - $$aa_fraction{K}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{K}{HE} = (1 - $$aa_fraction{K}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{K}{HZ} = $amide_fraction;

  # Leu - L
  $$deuteration_fraction_table{L}{H} = $amide_fraction;
  $$deuteration_fraction_table{L}{HA} = (1 - $$aa_fraction{L}{"HA_incorporation"}) * $deuteration;
  $$deuteration_fraction_table{L}{HB} = (1 - $$aa_fraction{L}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{L}{HG} = (1 - $$aa_fraction{L}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{L}{HD1} = (1 - ($$aa_fraction{L}{"incorporation"} + $ketoisovaleric_fraction * (1 - $$aa_fraction{L}{"incorporation"}) + $pyruvate_fraction * (1 - $$aa_fraction{L}{"incorporation"}) * (1 - $ketobutyric_fraction))) * $deuteration;
  $$deuteration_fraction_table{L}{HD2} = (1 - ($$aa_fraction{L}{"incorporation"} + $ketoisovaleric_fraction * (1 - $$aa_fraction{L}{"incorporation"}) + $pyruvate_fraction * (1 - $$aa_fraction{L}{"incorporation"}) * (1 - $ketobutyric_fraction))) * $deuteration;

  # Met - M
  $$deuteration_fraction_table{M}{H} = $amide_fraction;
  $$deuteration_fraction_table{M}{HA} = (1 - $$aa_fraction{M}{"HA_incorporation"}) * $deuteration;
  $$deuteration_fraction_table{M}{HB} = (1 - $$aa_fraction{M}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{M}{HG} = (1 - $$aa_fraction{M}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{M}{HE} = (1 - $$aa_fraction{M}{"incorporation"}) * $deuteration;
  
  # Asn - N
  $$deuteration_fraction_table{N}{H} = $amide_fraction;
  $$deuteration_fraction_table{N}{HA} = (1 - $$aa_fraction{N}{"HA_incorporation"}) * $deuteration;
  $$deuteration_fraction_table{N}{HB} = (1 - $$aa_fraction{N}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{N}{HD2} = $amide_fraction;

  # Pro - P
  $$deuteration_fraction_table{P}{HA} = (1 - $$aa_fraction{P}{"HA_incorporation"}) * $deuteration;
  $$deuteration_fraction_table{P}{HB} = (1 - $$aa_fraction{P}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{P}{HG} = (1 - $$aa_fraction{P}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{P}{HD} = (1 - $$aa_fraction{P}{"incorporation"}) * $deuteration;

  # Gln - Q
  $$deuteration_fraction_table{Q}{H} = $amide_fraction;
  $$deuteration_fraction_table{Q}{HA} = (1 - $$aa_fraction{Q}{"HA_incorporation"}) * $deuteration;
  $$deuteration_fraction_table{Q}{HB} = (1 - $$aa_fraction{Q}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{Q}{HG} = (1 - $$aa_fraction{Q}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{Q}{HE2} = $amide_fraction;

  # Arg - R
  $$deuteration_fraction_table{R}{H} = $amide_fraction;
  $$deuteration_fraction_table{R}{HA} = (1 - $$aa_fraction{R}{"HA_incorporation"}) * $deuteration;
  $$deuteration_fraction_table{R}{HB} = (1 - $$aa_fraction{R}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{R}{HG} = (1 - $$aa_fraction{R}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{R}{HD} = (1 - $$aa_fraction{R}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{R}{HE} = $amide_fraction;
  $$deuteration_fraction_table{R}{HH1} = $amide_fraction;
  $$deuteration_fraction_table{R}{HH2} = $amide_fraction;

  # Ser - S
  $$deuteration_fraction_table{S}{H} = $amide_fraction;
  $$deuteration_fraction_table{S}{HA} = (1 - $$aa_fraction{S}{"HA_incorporation"}) * $deuteration;
  $$deuteration_fraction_table{S}{HB} = (1 - $$aa_fraction{S}{"incorporation"}) * $deuteration;

  # Thr - T
  $$deuteration_fraction_table{T}{H} = $amide_fraction;
  $$deuteration_fraction_table{T}{HA} = (1 - $$aa_fraction{T}{"HA_incorporation"}) * $deuteration;
  $$deuteration_fraction_table{T}{HB} = (1 - $$aa_fraction{T}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{T}{HG} = (1 - $$aa_fraction{T}{"incorporation"}) * $deuteration;

  # Val - V
  $$deuteration_fraction_table{V}{H} = $amide_fraction;
  $$deuteration_fraction_table{V}{HA} = (1 - $$aa_fraction{V}{"HA_incorporation"}) * $deuteration;
  $$deuteration_fraction_table{V}{HB} = (1 - $$aa_fraction{V}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{V}{HG1} = (1 - ($$aa_fraction{V}{"incorporation"} + $ketoisovaleric_fraction * (1 - $$aa_fraction{V}{"incorporation"}) + $pyruvate_fraction * (1 - $$aa_fraction{V}{"incorporation"}) * (1 - $ketobutyric_fraction))) * $deuteration;
  $$deuteration_fraction_table{V}{HG2} = (1 - ($$aa_fraction{V}{"incorporation"} + $ketoisovaleric_fraction * (1 - $$aa_fraction{V}{"incorporation"}) + $pyruvate_fraction * (1 - $$aa_fraction{V}{"incorporation"}) * (1 - $ketobutyric_fraction))) * $deuteration;

  # Trp - W
  $$deuteration_fraction_table{W}{H} = $amide_fraction;
  $$deuteration_fraction_table{W}{HA} = (1 - $$aa_fraction{W}{"HA_incorporation"}) * $deuteration;
  $$deuteration_fraction_table{W}{HB} = (1 - $$aa_fraction{W}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{W}{HD1} = (1 - ($$aa_fraction{W}{"incorporation"} + $shikimic_fraction * (1 - $$aa_fraction{W}{"incorporation"}))) * $deuteration;
  $$deuteration_fraction_table{W}{HE1} = $amide_fraction;
  $$deuteration_fraction_table{W}{HE3} = (1 - ($$aa_fraction{W}{"incorporation"} + $shikimic_fraction * (1 - $$aa_fraction{W}{"incorporation"}))) * $deuteration;
  $$deuteration_fraction_table{W}{HZ2} = (1 - ($$aa_fraction{W}{"incorporation"} + $shikimic_fraction * (1 - $$aa_fraction{W}{"incorporation"}))) * $deuteration;
  $$deuteration_fraction_table{W}{HZ3} = (1 - ($$aa_fraction{W}{"incorporation"} + $shikimic_fraction * (1 - $$aa_fraction{W}{"incorporation"}))) * $deuteration;
  $$deuteration_fraction_table{W}{HH2} = (1 - ($$aa_fraction{W}{"incorporation"} + $shikimic_fraction * (1 - $$aa_fraction{W}{"incorporation"}))) * $deuteration;

  # Tyr - Y
  $$deuteration_fraction_table{Y}{H} = $amide_fraction;
  $$deuteration_fraction_table{Y}{HA} = (1 - $$aa_fraction{Y}{"HA_incorporation"}) * $deuteration;
  $$deuteration_fraction_table{Y}{HB} = (1 - $$aa_fraction{Y}{"incorporation"}) * $deuteration;
  $$deuteration_fraction_table{Y}{HD} = (1 - ($$aa_fraction{Y}{"incorporation"} + $shikimic_fraction * (1 - $$aa_fraction{Y}{"incorporation"}))) * $deuteration;
  $$deuteration_fraction_table{Y}{HE} = (1 - ($$aa_fraction{Y}{"incorporation"} + $shikimic_fraction * (1 - $$aa_fraction{Y}{"incorporation"}))) * $deuteration;

  return $deuteration_fraction_table;
  }


sub create_out_and_back_deuteration_effects_table
  {
  my $deuteration_effects = {};
  
  # Deuteration effects for HN rooted out-and-back experiments.
  #   The H is fully protonated since detection starts on this resonance.
  #   Otherwise the 1-bond deuterium effect would be 0.53 for N

  # Ala - A
  $$deuteration_effects{A}{CA}{HA} = 0.392;
  $$deuteration_effects{A}{CA}{HB} = 0.12 * 3;
  $$deuteration_effects{A}{CA}{H} = 0; # H is fully protonated
  
  $$deuteration_effects{A}{CB}{HB} = 0.285 * 3;
  $$deuteration_effects{A}{CB}{HA} = 0.077;
  $$deuteration_effects{A}{CB}{H} = 0; # H is fully protonated

  $$deuteration_effects{A}{N}{H} = 0; # H is fully protonated
  $$deuteration_effects{A}{N}{HA} = 0.16;
  $$deuteration_effects{A}{N}{HB} = 0.06 * 3;

  # Cys - C
  $$deuteration_effects{C}{CA}{HA} = 0.392;
  $$deuteration_effects{C}{CA}{HB} = 0.12 * 2;
  $$deuteration_effects{C}{CA}{H} = 0; # H is fully protonated

  $$deuteration_effects{C}{CB}{HB} = 0.38 * 2;
  $$deuteration_effects{C}{CB}{HA} = 0.077;
  $$deuteration_effects{C}{CB}{H} = 0; # H is fully protonated
    
  $$deuteration_effects{C}{N}{H} = 0; # H is fully protonated
  $$deuteration_effects{C}{N}{HA} = 0.16;
  $$deuteration_effects{C}{N}{HB} = 0.06 * 2;

  # Asp - D
  $$deuteration_effects{D}{CA}{HA} = 0.392;
  $$deuteration_effects{D}{CA}{HB} = 0.12 * 2;
  $$deuteration_effects{D}{CA}{H} = 0; # H is fully protonated

  $$deuteration_effects{D}{CB}{HB} = 0.30 * 2;
  $$deuteration_effects{D}{CB}{HA} = 0.09;
  $$deuteration_effects{D}{CB}{H} = 0; # H is fully protonated

  $$deuteration_effects{D}{N}{H} = 0; # H is fully protonated
  $$deuteration_effects{D}{N}{HA} = 0.16;
  $$deuteration_effects{D}{N}{HB} = 0.06 * 2;

  # Glu - E
  $$deuteration_effects{E}{CA}{HA} = 0.392;
  $$deuteration_effects{E}{CA}{HB} = 0.12 * 2;
  $$deuteration_effects{E}{CA}{H} = 0; # H is fully protonated
  $$deuteration_effects{E}{CA}{HG} = 0.042 * 2;

  $$deuteration_effects{E}{CB}{HB} = 0.36 * 2;
  $$deuteration_effects{E}{CB}{HA} = 0.077;
  $$deuteration_effects{E}{CB}{HG} = 0.07 * 2;
  $$deuteration_effects{E}{CB}{H} = 0; # H is fully protonated

  $$deuteration_effects{E}{CG}{HG} = 0.30 * 2;
  $$deuteration_effects{E}{CG}{HB} = 0.09 * 2;
  $$deuteration_effects{E}{CG}{HA} = 0.026;

  $$deuteration_effects{E}{N}{H} = 0; # H is fully protonated
  $$deuteration_effects{E}{N}{HA} = 0.16;
  $$deuteration_effects{E}{N}{HB} = 0.06 * 2;

  # Phe - F
  $$deuteration_effects{F}{CA}{HA} = 0.392;
  $$deuteration_effects{F}{CA}{HB} = 0.12 * 2;
  $$deuteration_effects{F}{CA}{H} = 0; # H is fully protonated

  $$deuteration_effects{F}{CB}{HB} = 0.36 * 2;
  $$deuteration_effects{F}{CB}{HA} = 0.077;
  $$deuteration_effects{F}{CB}{HD} = 0.061 * 2;
  $$deuteration_effects{F}{CB}{H} = 0; # H is fully protonated

  $$deuteration_effects{F}{CG}{HB} = 0.034 * 2;
  $$deuteration_effects{F}{CG}{HD} = 0.086 * 2;
  $$deuteration_effects{F}{CG}{HA} = 0.012;
  $$deuteration_effects{F}{CG}{HE} = 0.011 * 2;

  $$deuteration_effects{F}{CD1}{HD} = 0.312 + 0; # 1-bond + 3-bond
  $$deuteration_effects{F}{CD1}{HE} = 0.106;
  $$deuteration_effects{F}{CD1}{HB} = 0;
  $$deuteration_effects{F}{CD1}{HZ} = 0.011;

  $$deuteration_effects{F}{CD2}{HD} = 0.312 + 0; # 1-bond + 3-bond
  $$deuteration_effects{F}{CD2}{HE} = 0.106;
  $$deuteration_effects{F}{CD2}{HB} = 0;
  $$deuteration_effects{F}{CD2}{HZ} = 0.011;

  $$deuteration_effects{F}{CE1}{HE} = 0.28 + 0.007; # 1-bond + 3-bond
  $$deuteration_effects{F}{CE1}{HD} = 0.11;
  $$deuteration_effects{F}{CE1}{HZ} = 0.11;

  $$deuteration_effects{F}{CE2}{HE} = 0.28 + 0.007; # 1-bond + 3-bond
  $$deuteration_effects{F}{CE2}{HD} = 0.11;
  $$deuteration_effects{F}{CE2}{HZ} = 0.11;

  $$deuteration_effects{F}{CZ}{HZ} = 0.276; 
  $$deuteration_effects{F}{CZ}{HE} = 0.11 * 2;
  $$deuteration_effects{F}{CZ}{HD} = 0;

  $$deuteration_effects{F}{N}{H} = 0; # H is fully protonated
  $$deuteration_effects{F}{N}{HA} = 0.16;
  $$deuteration_effects{F}{N}{HB} = 0.06 * 2;

  # Gly - G
  $$deuteration_effects{G}{CA}{HA} = 0.392 * 2;

  $$deuteration_effects{G}{N}{H} = 0; # H is fully protonated
  $$deuteration_effects{G}{N}{HA} = 0.16 * 2;

  # His - H
  $$deuteration_effects{H}{CA}{HA} = 0.392;
  $$deuteration_effects{H}{CA}{HB} = 0.12 * 2;
  $$deuteration_effects{H}{CA}{H} = 0; # H is fully protonated

  $$deuteration_effects{H}{CB}{HB} = 0.36 * 2;
  $$deuteration_effects{H}{CB}{HA} = 0.077;
  $$deuteration_effects{H}{CB}{HD1} = 0.025;
  $$deuteration_effects{H}{CB}{HD2} = 0.034;
  $$deuteration_effects{H}{CB}{H} = 0; # H is fully protonated


  $$deuteration_effects{H}{CG}{HB} = 0.11 * 2; # just an estimate
  $$deuteration_effects{H}{CG}{HD1} = 0.11;
  $$deuteration_effects{H}{CG}{HD2} = 0.11;
  $$deuteration_effects{H}{CG}{HA} = 0.012;
  $$deuteration_effects{H}{CG}{HE1} = 0.02;

  $$deuteration_effects{H}{ND1}{HD1} = 0; # H is fully protonated
  $$deuteration_effects{H}{ND1}{HE1} = 0.16;
  $$deuteration_effects{H}{ND1}{HB} = 0.06 * 2;
  $$deuteration_effects{H}{ND1}{HD2} = 0.06;

  $$deuteration_effects{H}{CD2}{HD2} = 0.4; 
  $$deuteration_effects{H}{CD2}{HB} = 0.034 * 2;
  $$deuteration_effects{H}{CD2}{HD1} = 0.034;
  $$deuteration_effects{H}{CD2}{HE1} = 0.034;

  $$deuteration_effects{H}{CE1}{HE1} = 0.4; 
  $$deuteration_effects{H}{CE1}{HD1} = 0.1;
  $$deuteration_effects{H}{CE1}{HD2} = 0.034;

  $$deuteration_effects{H}{NE2}{HE1} = 0.16;
  $$deuteration_effects{H}{NE2}{HD2} = 0.16;
  $$deuteration_effects{H}{NE2}{HD1} = 0.06;

  $$deuteration_effects{H}{N}{H} = 0; # H is fully protonated
  $$deuteration_effects{H}{N}{HA} = 0.16;
  $$deuteration_effects{H}{N}{HB} = 0.06 * 2;

  # Ile - I
  $$deuteration_effects{I}{CA}{HA} = 0.392;
  $$deuteration_effects{I}{CA}{HB} = 0.12;
  $$deuteration_effects{I}{CA}{H} = 0; # H is fully protonated
  $$deuteration_effects{I}{CA}{HG1} = 0.042 * 2;
  $$deuteration_effects{I}{CA}{HG2} = 0.042 * 3;

  $$deuteration_effects{I}{CB}{HB} = 0.44;
  $$deuteration_effects{I}{CB}{HA} = 0.077;
  $$deuteration_effects{I}{CB}{HG1} = 0.055 * 2;
  $$deuteration_effects{I}{CB}{HG2} = 0.055 * 3;
  $$deuteration_effects{I}{CB}{HD1} = 0.02 * 3;
  $$deuteration_effects{I}{CB}{H} = 0; # H is fully protonated
  
  $$deuteration_effects{I}{CG1}{HG1} = 0.36 * 2;
  $$deuteration_effects{I}{CG1}{HB} = 0.1;
  $$deuteration_effects{I}{CG1}{HD1} = 0.12 * 3;
  $$deuteration_effects{I}{CG1}{HA} = 0.026;
  $$deuteration_effects{I}{CG1}{HG2} = 0.04 * 3;

  $$deuteration_effects{I}{CG2}{HG2} = 0.285 * 3;
  $$deuteration_effects{I}{CG2}{HB} = 0.1;
  $$deuteration_effects{I}{CG2}{HG1} = 0.03 * 2;
  $$deuteration_effects{I}{CG2}{HA} = 0.026;

  $$deuteration_effects{I}{CD1}{HD1} = 0.285 * 3;
  $$deuteration_effects{I}{CD1}{HG1} = 0.1 * 2;
  $$deuteration_effects{I}{CD1}{HB} = 0.03;

  $$deuteration_effects{I}{N}{H} = 0; # H is fully protonated
  $$deuteration_effects{I}{N}{HA} = 0.16;
  $$deuteration_effects{I}{N}{HB} = 0.06;

  # Lys - K
  $$deuteration_effects{K}{CA}{HA} = 0.392;
  $$deuteration_effects{K}{CA}{HB} = 0.12 * 2;
  $$deuteration_effects{K}{CA}{HG} = 0.042 * 2;
  $$deuteration_effects{K}{CA}{H} = 0; # H is fully protonated

  $$deuteration_effects{K}{CB}{HB} = 0.36 * 2;
  $$deuteration_effects{K}{CB}{HA} = 0.077;
  $$deuteration_effects{K}{CB}{HG} = 0.1 * 2;
  $$deuteration_effects{K}{CB}{HD} = 0.03 * 2;
  $$deuteration_effects{K}{CB}{H} = 0; # H is fully protonated

  $$deuteration_effects{K}{CG}{HG} = 0.36 * 2;
  $$deuteration_effects{K}{CG}{HB} = 0.1 * 2;
  $$deuteration_effects{K}{CG}{HD} = 0.1 * 2;
  $$deuteration_effects{K}{CG}{HA} = 0.026;
  $$deuteration_effects{K}{CG}{HE} = 0.03 * 2;

  $$deuteration_effects{K}{CD}{HD} = 0.36 * 2;
  $$deuteration_effects{K}{CD}{HG} = 0.1 * 2;
  $$deuteration_effects{K}{CD}{HE} = 0.1 * 2;
  $$deuteration_effects{K}{CD}{HB} = 0.03 * 2;
  $$deuteration_effects{K}{CD}{HZ} = 0.03 * 3;

  $$deuteration_effects{K}{CE}{HE} = 0.36 * 2;
  $$deuteration_effects{K}{CE}{HD} = 0.1 * 2;
  $$deuteration_effects{K}{CE}{HZ} = 0.1 * 3;
  $$deuteration_effects{K}{CE}{HG} = 0.03 * 2;

  $$deuteration_effects{K}{NZ}{HZ} = 0.35 * 2; # one HZ is fully protonated
  $$deuteration_effects{K}{NZ}{HE} = 0.16 * 2;
  $$deuteration_effects{K}{NZ}{HD} = 0.06 * 2;

  $$deuteration_effects{K}{N}{H} = 0; # H is fully protonated
  $$deuteration_effects{K}{N}{HA} = 0.16;
  $$deuteration_effects{K}{N}{HB} = 0.06 * 2;

  # Leu - L
  $$deuteration_effects{L}{CA}{HA} = 0.392;
  $$deuteration_effects{L}{CA}{HB} = 0.12 * 2;
  $$deuteration_effects{L}{CA}{HG} = 0.042;
  $$deuteration_effects{L}{CA}{H} = 0; # H is fully protonated
  
  $$deuteration_effects{L}{CB}{HB} = 0.36 * 2;
  $$deuteration_effects{L}{CB}{HA} = 0.077;
  $$deuteration_effects{L}{CB}{HG} = 0.1;
  $$deuteration_effects{L}{CB}{HD1} = 0.03 * 3;
  $$deuteration_effects{L}{CB}{HD2} = 0.03 * 3;
  $$deuteration_effects{L}{CB}{H} = 0; # H is fully protonated

  $$deuteration_effects{L}{CG}{HG} = 0.44;
  $$deuteration_effects{L}{CG}{HB} = 0.1 * 2;
  $$deuteration_effects{L}{CG}{HD1} = 0.12 * 3;
  $$deuteration_effects{L}{CG}{HD2} = 0.12 * 3;
  $$deuteration_effects{K}{CG}{HA} = 0.026;

  $$deuteration_effects{L}{CD1}{HD1} = 0.285 * 3;
  $$deuteration_effects{L}{CD1}{HG} = 0.1;
  $$deuteration_effects{L}{CD1}{HB} = 0.03 * 2;
  $$deuteration_effects{L}{CD1}{HD2} = 0.03 * 3;

  $$deuteration_effects{L}{CD2}{HD2} = 0.285 * 3;
  $$deuteration_effects{L}{CD2}{HG} = 0.1;
  $$deuteration_effects{L}{CD2}{HB} = 0.03 * 2;
  $$deuteration_effects{L}{CD2}{HD1} = 0.03 * 3;

  $$deuteration_effects{L}{N}{H} = 0; # H is fully protonated
  $$deuteration_effects{L}{N}{HA} = 0.16;
  $$deuteration_effects{L}{N}{HB} = 0.06 * 2;

  # Met - M
  $$deuteration_effects{M}{CA}{HA} = 0.392;
  $$deuteration_effects{M}{CA}{HB} = 0.12 * 2;
  $$deuteration_effects{M}{CA}{HG} = 0.042 * 2;
  $$deuteration_effects{M}{CA}{H} = 0; # H is fully protonated
  
  $$deuteration_effects{M}{CB}{HB} = 0.36 * 2;
  $$deuteration_effects{M}{CB}{HA} = 0.077;
  $$deuteration_effects{M}{CB}{HG} = 0.06 * 2;
  $$deuteration_effects{M}{CB}{H} = 0; # H is fully protonated

  $$deuteration_effects{M}{CG}{HG} = 0.38 * 2;
  $$deuteration_effects{M}{CG}{HB} = 0.1 * 2;
  $$deuteration_effects{M}{CG}{HA} = 0.026;

  $$deuteration_effects{M}{CE}{HE} = 0.285 * 3;

  $$deuteration_effects{M}{N}{H} = 0; # H is fully protonated
  $$deuteration_effects{M}{N}{HA} = 0.16;
  $$deuteration_effects{M}{N}{HB} = 0.06 * 2;
  
  # Asn - N
  $$deuteration_effects{N}{CA}{HA} = 0.392;
  $$deuteration_effects{N}{CA}{HB} = 0.12 * 2;
  $$deuteration_effects{N}{CA}{H} = 0; # H is fully protonated
  
  $$deuteration_effects{N}{CB}{HB} = 0.34 * 2;
  $$deuteration_effects{N}{CB}{HA} = 0.077;
  $$deuteration_effects{N}{CB}{H} = 0; # H is fully protonated

  $$deuteration_effects{N}{ND2}{HD2} = 0.53; # one HD2 is fully protonated
  
  $$deuteration_effects{N}{N}{H} = 0; # H is fully protonated
  $$deuteration_effects{N}{N}{HA} = 0.16;
  $$deuteration_effects{N}{N}{HB} = 0.06 * 2;
  
  # Pro - P
  $$deuteration_effects{P}{CA}{HA} = 0.392;
  $$deuteration_effects{P}{CA}{HB} = 0.12 * 2;
  $$deuteration_effects{P}{CA}{HG} = 0.042 * 2;
  $$deuteration_effects{P}{CA}{HD} = 0.042 * 2;
  
  $$deuteration_effects{P}{CB}{HB} = 0.36 * 2;
  $$deuteration_effects{P}{CB}{HA} = 0.077;
  $$deuteration_effects{P}{CB}{HG} = 0.1 * 2;
  $$deuteration_effects{P}{CB}{HD} = 0.03 * 2;

  $$deuteration_effects{P}{CG}{HG} = 0.36 * 2;
  $$deuteration_effects{P}{CG}{HB} = 0.1 * 2;
  $$deuteration_effects{P}{CG}{HD} = 0.1 * 2;
  $$deuteration_effects{P}{CG}{HA} = 0.026;

  $$deuteration_effects{P}{CD}{HD} = 0.36 * 2;
  $$deuteration_effects{P}{CD}{HG} = 0.1 * 2;
  $$deuteration_effects{P}{CD}{HB} = 0.03 * 2;
  $$deuteration_effects{P}{CD}{HA} = 0.026;

  $$deuteration_effects{P}{N}{HA} = 0.16;
  $$deuteration_effects{P}{N}{HD} = 0.16 * 2;
  $$deuteration_effects{P}{N}{HB} = 0.06 * 2;
  $$deuteration_effects{P}{N}{HG} = 0.06 * 2;

  # Gln - Q
  $$deuteration_effects{Q}{CA}{HA} = 0.392;
  $$deuteration_effects{Q}{CA}{HB} = 0.12 * 2;
  $$deuteration_effects{Q}{CA}{HG} = 0.042 * 2;
  $$deuteration_effects{Q}{CA}{H} = 0; # H is fully protonated
  
  $$deuteration_effects{Q}{CB}{HB} = 0.36 * 2;
  $$deuteration_effects{Q}{CB}{HA} = 0.077;
  $$deuteration_effects{Q}{CB}{HG} = 0.07 * 2;
  $$deuteration_effects{Q}{CB}{H} = 0; # H is fully protonated

  $$deuteration_effects{Q}{CG}{HG} = 0.34 * 2;
  $$deuteration_effects{Q}{CG}{HB} = 0.09 * 2;
  $$deuteration_effects{Q}{CG}{HA} = 0.026;

  $$deuteration_effects{Q}{NE2}{HE2} = 0.53; # one HE2 is fully protonated
  
  $$deuteration_effects{Q}{N}{H} = 0; # H is fully protonated
  $$deuteration_effects{Q}{N}{HA} = 0.16;
  $$deuteration_effects{Q}{N}{HB} = 0.06 * 2;
  
  # Arg - R
  $$deuteration_effects{R}{CA}{HA} = 0.392;
  $$deuteration_effects{R}{CA}{HB} = 0.12 * 2;
  $$deuteration_effects{R}{CA}{HG} = 0.042 * 2;
  $$deuteration_effects{R}{CA}{H} = 0; # H is fully protonated

  $$deuteration_effects{R}{CB}{HB} = 0.36 * 2;
  $$deuteration_effects{R}{CB}{HA} = 0.077;
  $$deuteration_effects{R}{CB}{HG} = 0.1 * 2;
  $$deuteration_effects{R}{CB}{HD} = 0.03 * 2;
  $$deuteration_effects{R}{CB}{H} = 0; # H is fully protonated

  $$deuteration_effects{R}{CG}{HG} = 0.36 * 2;
  $$deuteration_effects{R}{CG}{HB} = 0.1 * 2;
  $$deuteration_effects{R}{CG}{HD} = 0.1 * 2;
  $$deuteration_effects{R}{CG}{HA} = 0.026;
  $$deuteration_effects{R}{CG}{HE} = 0.03;

  $$deuteration_effects{R}{CD}{HD} = 0.36 * 2;
  $$deuteration_effects{R}{CD}{HG} = 0.1 * 2;
  $$deuteration_effects{R}{CD}{HE} = 0.1;
  $$deuteration_effects{R}{CD}{HB} = 0.03 * 2;

  $$deuteration_effects{R}{NE}{HE} = 0; # HE is fully protonated
  $$deuteration_effects{R}{NE}{HD} = 0.16 * 2;
  $$deuteration_effects{R}{NE}{HG} = 0.06 * 2;
  $$deuteration_effects{R}{NE}{HH1} = 0.06 * 2;
  $$deuteration_effects{R}{NE}{HH2} = 0.06 * 2;

  $$deuteration_effects{R}{CZ}{HE} = 0; # HE is fully protonated
  $$deuteration_effects{R}{CZ}{HH1} = 0.11; # one HH1 is fully protonated
  $$deuteration_effects{R}{CZ}{HH2} = 0.11; # one HH2 is fully protonated
  $$deuteration_effects{R}{CZ}{HG} = 0.037 * 2;

  $$deuteration_effects{R}{NH1}{HH1} = 0.275; # one HH1 is fully protonated
  $$deuteration_effects{R}{NH1}{HH2} = 0.16 * 2;
  $$deuteration_effects{R}{NH1}{HE} = 0.06;

  $$deuteration_effects{R}{NH2}{HH2} = 0.275; # one HH1 is fully protonated
  $$deuteration_effects{R}{NH2}{HH1} = 0.16 * 2;
  $$deuteration_effects{R}{NH2}{HE} = 0.06;

  $$deuteration_effects{R}{N}{H} = 0; # H is fully protonated
  $$deuteration_effects{R}{N}{HA} = 0.16;
  $$deuteration_effects{R}{N}{HB} = 0.06 * 2;
  
  # Ser - S
  $$deuteration_effects{S}{CA}{HA} = 0.392;
  $$deuteration_effects{S}{CA}{HB} = 0.12 * 2;
  $$deuteration_effects{S}{CA}{H} = 0; # H is fully protonated

  $$deuteration_effects{S}{CB}{HB} = 0.38 * 2;
  $$deuteration_effects{S}{CB}{HA} = 0.077;
  $$deuteration_effects{S}{CB}{H} = 0; # H is fully protonated

  $$deuteration_effects{S}{N}{H} = 0; # H is fully protonated
  $$deuteration_effects{S}{N}{HA} = 0.16;
  $$deuteration_effects{S}{N}{HB} = 0.06 * 2;
  
  # Thr - T
  $$deuteration_effects{T}{CA}{HA} = 0.392;
  $$deuteration_effects{T}{CA}{HB} = 0.12;
  $$deuteration_effects{T}{CA}{HG} = 0.042 * 3;
  $$deuteration_effects{T}{CA}{H} = 0; # H is fully protonated

  $$deuteration_effects{T}{CB}{HB} = 0.52;
  $$deuteration_effects{T}{CB}{HA} = 0.077;
  $$deuteration_effects{T}{CB}{HG} = 0.09 * 3;
  $$deuteration_effects{T}{CB}{H} = 0; # H is fully protonated

  $$deuteration_effects{T}{CG}{HG} = 0.285 * 3;
  $$deuteration_effects{T}{CG}{HB} = 0.06;
  $$deuteration_effects{T}{CG}{HA} = 0.026;

  $$deuteration_effects{T}{N}{H} = 0; # H is fully protonated
  $$deuteration_effects{T}{N}{HA} = 0.16;
  $$deuteration_effects{T}{N}{HB} = 0.06;
  
  # Val - V
  $$deuteration_effects{V}{CA}{HA} = 0.392;
  $$deuteration_effects{V}{CA}{HB} = 0.12;
  $$deuteration_effects{V}{CA}{HG1} = 0.042 * 3;
  $$deuteration_effects{V}{CA}{HG2} = 0.042 * 3;
  $$deuteration_effects{V}{CA}{H} = 0; # H is fully protonated

  $$deuteration_effects{V}{CB}{HB} = 0.44;
  $$deuteration_effects{V}{CB}{HA} = 0.077;
  $$deuteration_effects{V}{CB}{HG1} = 0.12 * 3;
  $$deuteration_effects{V}{CB}{HG2} = 0.12 * 3;
  $$deuteration_effects{V}{CB}{H} = 0; # H is fully protonated

  $$deuteration_effects{V}{CG1}{HG1} = 0.285 * 3;
  $$deuteration_effects{V}{CG1}{HB} = 0.09;
  $$deuteration_effects{V}{CG1}{HG2} = 0.03 * 3;
  $$deuteration_effects{V}{CG1}{HA} = 0.026;

  $$deuteration_effects{V}{CG2}{HG2} = 0.285 * 3;
  $$deuteration_effects{V}{CG2}{HB} = 0.09;
  $$deuteration_effects{V}{CG2}{HG1} = 0.03 * 3;
  $$deuteration_effects{V}{CG2}{HA} = 0.026;

  $$deuteration_effects{V}{N}{H} = 0; # H is fully protonated
  $$deuteration_effects{V}{N}{HA} = 0.16;
  $$deuteration_effects{V}{N}{HB} = 0.06;
  
  # Trp - W
  $$deuteration_effects{W}{CA}{HA} = 0.392;
  $$deuteration_effects{W}{CA}{HB} = 0.12 * 2;
  $$deuteration_effects{W}{CA}{H} = 0; # H is fully protonated

  $$deuteration_effects{W}{CB}{HB} = 0.36 * 2;
  $$deuteration_effects{W}{CB}{HA} = 0.077;
  $$deuteration_effects{W}{CB}{HD1} = 0.061;
  $$deuteration_effects{W}{CB}{H} = 0; # H is fully protonated
  
  $$deuteration_effects{W}{CG}{HB} = 0.034;
  $$deuteration_effects{W}{CG}{HA} = 0.012;
  $$deuteration_effects{W}{CG}{HD1} = 0.086;
  $$deuteration_effects{W}{CG}{HE1} = 0.011;
  $$deuteration_effects{W}{CG}{HE3} = 0.011;

  $$deuteration_effects{W}{CD1}{HD1} = 0.312;
  $$deuteration_effects{W}{CD1}{HB} = 0;
  $$deuteration_effects{W}{CD1}{HE1} = 0; # HE1 is fully protonated

  $$deuteration_effects{W}{CD2}{HB} = 0;
  $$deuteration_effects{W}{CD2}{HE3} = 0.11;
  $$deuteration_effects{W}{CD2}{HE1} = 0; # HE1 is fully protonated
  $$deuteration_effects{W}{CD2}{HZ3} = 0.011;
  $$deuteration_effects{W}{CD2}{HZ2} = 0.011;

  $$deuteration_effects{W}{NE1}{HE1} = 0; # HE1 is fully protonated
  $$deuteration_effects{W}{NE1}{HD1} = 0.16;
  $$deuteration_effects{W}{NE1}{HZ2} = 0.02;

  $$deuteration_effects{W}{CE2}{HE1} = 0; # HE1 is fully protonated
  $$deuteration_effects{W}{CE2}{HZ3} = 0.086;
  $$deuteration_effects{W}{CE2}{HZ3} = 0.011;
  $$deuteration_effects{W}{CE2}{HH2} = 0.011;

  $$deuteration_effects{W}{CE3}{HE3} = 0.312; 
  $$deuteration_effects{W}{CE3}{HZ3} = 0.106;
  $$deuteration_effects{W}{CE3}{HH2} = 0.011;

  $$deuteration_effects{W}{CZ2}{HZ2} = 0.312; 
  $$deuteration_effects{W}{CZ2}{HE1} = 0; # HE1 is fully protonated
  $$deuteration_effects{W}{CZ2}{HH2} = 0.106;
  $$deuteration_effects{W}{CZ2}{HZ3} = 0.011;

  $$deuteration_effects{W}{CZ3}{HZ3} = 0.280; 
  $$deuteration_effects{W}{CZ3}{HE3} = 0.11;
  $$deuteration_effects{W}{CZ3}{HH2} = 0.11;
  $$deuteration_effects{W}{CZ3}{HZ2} = 0.011;

  $$deuteration_effects{W}{CH2}{HH2} = 0.280; 
  $$deuteration_effects{W}{CH2}{HZ2} = 0.11;
  $$deuteration_effects{W}{CH2}{HZ3} = 0.11;
  $$deuteration_effects{W}{CH2}{HE3} = 0;

  $$deuteration_effects{W}{N}{H} = 0; # H is fully protonated
  $$deuteration_effects{W}{N}{HA} = 0.16;
  $$deuteration_effects{W}{N}{HB} = 0.06 * 2;
  
  # Tyr - Y
  $$deuteration_effects{Y}{CA}{HA} = 0.392;
  $$deuteration_effects{Y}{CA}{HB} = 0.12 * 2;
  $$deuteration_effects{Y}{CA}{H} = 0; # H is fully protonated

  $$deuteration_effects{Y}{CB}{HB} = 0.36 * 2;
  $$deuteration_effects{Y}{CB}{HA} = 0.077;
  $$deuteration_effects{Y}{CB}{HD} = 0.061 * 2;
  $$deuteration_effects{Y}{CB}{H} = 0; # H is fully protonated

  $$deuteration_effects{Y}{CG}{HB} = 0.034 * 2;
  $$deuteration_effects{Y}{CG}{HD} = 0.086 * 2;
  $$deuteration_effects{Y}{CG}{HA} = 0.012;
  $$deuteration_effects{Y}{CG}{HE} = 0.011 * 2;

  $$deuteration_effects{Y}{CD1}{HD} = 0.312 + 0; # 1-bond + 3-bond
  $$deuteration_effects{Y}{CD1}{HE} = 0.106;
  $$deuteration_effects{Y}{CD1}{HB} = 0;

  $$deuteration_effects{Y}{CD2}{HD} = 0.312 + 0; # 1-bond + 3-bond
  $$deuteration_effects{Y}{CD2}{HE} = 0.106;
  $$deuteration_effects{Y}{CD2}{HB} = 0;

  $$deuteration_effects{Y}{CE1}{HE} = 0.28 + 0.007; # 1-bond + 3-bond
  $$deuteration_effects{Y}{CE1}{HD} = 0.11;

  $$deuteration_effects{Y}{CE2}{HE} = 0.28 + 0.007; # 1-bond + 3-bond
  $$deuteration_effects{Y}{CE2}{HD} = 0.11;

  $$deuteration_effects{Y}{CZ}{HE} = 0.11 * 2;
  $$deuteration_effects{Y}{CZ}{HD} = 0;

  $$deuteration_effects{Y}{N}{H} = 0; # H is fully protonated
  $$deuteration_effects{Y}{N}{HA} = 0.16;
  $$deuteration_effects{Y}{N}{HB} = 0.06 * 2;

  return $deuteration_effects;
  }
