#!/usr/bin/perl
#!/usr/bin/perl 
#
#  calc_rdc.pl
#	Calculates RDCs from peak lists with the peak splittings for the free and aligned protein. 
# 
#  usage: calc_rdc.pl free_splits_peaklist aligned_splits_peaklist out_rdc_peaklist out_unused_peaklist workbook_name
#
#	free_splits_peaklist - peak list with splittings from free (i.e. unaligned) sample in last dimension.
#	aligned_splits_peaklist - peak list with splitting from aligned sample in last dimension.
#	out_rdc_peaklist - peak list with RDCs in last dimension.
#	out_unused_peaklist -  peaklist of used peaks because they are not present in both input lists.
#	workbook_name - string to use as workbook name.
#
# Assumptions: The peak list for free and aligned proteins have the same number and order of dimensions columns
#              (splittings in ppm in the 2nd from last *Dim, and in Hz in the last *Dim column) 
#              and the sets of peaks have the same registration.
#
# Hint: Use 'calc_resonance_splitting.pl' first to generate the list of splittings for each peak list (for each sample).
#
#
#
# Copyright Aneerban Bhattacharya, 2001. All rights reserved.
# Written by Aneerban Bhattacharya; Jun 6, 2001
# Modified by Aneerban Bhattacharya		Feb 15, 2002
# 
# Procedure for writing out file history, and interactive help format
# taken from AutoPeak scripts written by Hunter Moseley.
#

use strict;

# Print usage information
if ((scalar(@ARGV) < 5) || ($ARGV[0] =~ /^-h/)) {
	print STDERR "\nError: not enough parameters given.\n" if ($ARGV[0] !~ /^-h/);
	print STDERR "\n";
	print STDERR "calc_rdc.pl\n";
	print STDERR "	Calculates RDCs from peak lists with the peak splittings for the free and aligned protein. \n";
	print STDERR "\n";
	print STDERR "  usage: calc_rdc.pl free_splits_peaklist aligned_splits_peaklist out_rdc_peaklist out_unused_peaklist workbook_name\n";
	print STDERR "\n";
	print STDERR "	free_splits_peaklist - peak list with splittings from free (i.e. unaligned) sample in last dimension.\n";
	print STDERR "	aligned_splits_peaklist - peak list with splitting from aligned sample in last dimension.\n";
	print STDERR "	out_rdc_peaklist - peak list with RDCs in last dimension.\n";
	print STDERR "	out_unused_peaklist -  peaklist of used peaks because they are not present in both input lists.\n";
	print STDERR "	workbook_name - string to use as workbook name.\n";
	print STDERR "\n";
	print STDERR "  Assumptions: The peak list for free and aligned proteins have the same number and order of dimensions columns\n";
	print STDERR "               (splittings in ppm in the 2nd from last *Dim, and in Hz in the last *Dim column) \n";
	print STDERR "               and the sets of peaks have the same registration.\n";
	print STDERR "\n";
	print STDERR "  Hint: Use 'calc_resonance_splitting.pl' first to generate the list of splittings for each peak list (for each sample).\n";
	exit(1);
}

# read in command line arguments
my ($free_list_s, $aligned_list_s, $RDC_file_s, $unusedfile_s, $workbook_name_s) = @ARGV;

my $line_s; 
my %assigns_h; # to store the list of split peaks and difference assigned to particular residue
# structure: assigns_h -> residue_s -> { peak1 dimensions { dim 1, dim 2, ... }, peak2 dimensions { dim 1, dim 2, ...} } 
# order of dimensions for the peaks must be in the SAME, for all peaks being put into the list

# get list of peak lines, and end coment lines
my ($in_peak_lines_alist_free, $end_comments_alist_free) = &read_peak_file($free_list_s);
my ($in_peak_lines_alist_aligned, $end_comments_alist_aligned) = &read_peak_file($aligned_list_s);

my $num_cols_n = scalar(@{$$in_peak_lines_alist_free[0]}); # number of columns in peak list (same for both)

# read in the splittings for each residue
foreach my $peak_line_alist (@{$in_peak_lines_alist_free}) {
	
	$$peak_line_alist[$num_cols_n - 1] =~ /.*([A-Z]\d+).*/; # extract the residue number and type
    my $residue_s = $1;

	my @peak_dims_a; # stores each dimension of this peak
	# assuming penultimate column reserved for intensity
	for (my $colnum_n = 1; $colnum_n < ($num_cols_n - 2); $colnum_n++) {
		push(@peak_dims_a, $$peak_line_alist[$colnum_n]);
	}
	
    if (!exists ($assigns_h{$residue_s})) {
		$assigns_h{$residue_s} = [  [ @peak_dims_a ] ];
	}
	else {
		print("Error: > 2 splittings for $residue_s in free sample.\n");
	}
}


open(UF, ">$unusedfile_s") || die("Can't open $unusedfile_s : $!"); # initialise file
close(UF);

foreach my $peak_line_alist (@{$in_peak_lines_alist_aligned}) {
	$$peak_line_alist[$num_cols_n - 1] =~ /.*([A-Z]\d+).*/; # extract the residue number and type
	my $residue_s = $1;
	
	my @peak_dims_a; # stores each dimension of this peak
	# assuming penultimate column reserved for intensity
	for (my $colnum_n = 1; $colnum_n < ($num_cols_n - 2); $colnum_n++) {
		push(@peak_dims_a, $$peak_line_alist[$colnum_n]);													    	}
		
	if (!exists ($assigns_h{$residue_s})) {
		open(UF, ">>$unusedfile_s");
		print(UF "Found in file 2 but not file 1: $residue_s\n"); 
		close(UF);
		next;
	}
	elsif (scalar(@{$assigns_h{$residue_s}}) == 1) {
		push(@{$assigns_h{$residue_s}}, [ @peak_dims_a ]);
	}
	else {
		print("Error: > 2 splittings for $residue_s in aligned sample.\n");
	}
}

# write output of RDCs, for each pseud-central peak (one for each residue)
open(OF, ">$RDC_file_s") || die("Can't open $RDC_file_s : $!");
print(OF "#Last *Dim is RDC in Hz, the prev *Dim is the same in ppm\n");
print(OF "#Index\t");
for (my $dim_n = 1; $dim_n < $num_cols_n - 2; $dim_n++) { # 2 new columns being added
		    print(OF "$dim_n", "Dim\t");
}
print(OF "Intensity\tWorkbook\n");
close(OF);

my $residue_s;
my $index_n = 0;

foreach $residue_s (sort keys(%assigns_h)) {
	if (scalar(@{$assigns_h{$residue_s}}) == 1) {
		open(UF, ">>$unusedfile_s");
		print(UF "Found in file 1 but not file 2 $residue_s\n");
		close(UF);
        next;
    }
	
#	my $RDC_n = ($assigns_h{$residue_s}[1] - $assigns_h{$residue_s}[0]); 

	# calculate pseudo-central peak
	my @peak_dims_a; # stores each dimension of pseudo-central peak
	for (my $dim_n = 0; $dim_n < (scalar(@{$assigns_h{$residue_s}[0]}) - 2); $dim_n++) { # last 2 are for splittings in ppm and Hz
		$peak_dims_a[$dim_n] = ($assigns_h{$residue_s}[0][$dim_n] + $assigns_h{$residue_s}[1][$dim_n]) / 2;
		$peak_dims_a[$dim_n] = sprintf("%.3f", $peak_dims_a[$dim_n]);
	}
	for (my $dim_n = (scalar(@{$assigns_h{$residue_s}[0]}) - 2); $dim_n < scalar(@{$assigns_h{$residue_s}[0]}); $dim_n++) { # last 2 are for splittings in ppm and Hz
		$peak_dims_a[$dim_n] = $assigns_h{$residue_s}[1][$dim_n] - $assigns_h{$residue_s}[0][$dim_n];
		$peak_dims_a[$dim_n] = sprintf("%.3f", $peak_dims_a[$dim_n]); # RDC
	}
		
	$index_n++;
	open(OF, ">>$RDC_file_s") || die("Can't open $RDC_file_s : $!");
#	print(OF sprintf("%4d\t%6s\t%4.2f\t%4.2f\t%5.2f\t%4d\n", $index_n, $residue_s, $assigns_h{$residue_s}[0], $assigns_h{$residue_s}[1], $RDC_n, $resnum_n));
    print(OF "$index_n\t");
	print(OF join("\t", @peak_dims_a));
	print(OF "\t3000");
	print(OF "\t$workbook_name_s" . "\.$residue_s\n");
	close(OF);
}
	
# print comments
open(OF, ">>$RDC_file_s") || die("Can't open $RDC_file_s : $!");

# modified from AutoPeak script written by Hunter Moseley
print(OF "*\n\n");
print(OF "\nFile History: $RDC_file_s \n\n");
print(OF $0, " ", join(' ',@ARGV),"\n");
print(OF "\n");

print(OF "\n\nComments: $free_list_s\n");
print(OF "-" x 80, "\n");
foreach $line_s (@{$end_comments_alist_free}) {
	print(OF "$line_s");	
}

print(OF "\n\nComments: $aligned_list_s\n");
print(OF "-" x 80, "\n");
foreach $line_s (@{$end_comments_alist_aligned}) {
    print(OF "$line_s");
}
print("\n");

close(OF);


# Read in peaks from AutoPeak format files - modified from the subroutine written by Hunter Moseley
sub read_peak_file {
	my ($filename) = @_;
	
	local *PEAKFILE;
	if ($filename eq "-") {
		*PEAKFILE = *STDIN;
	}
	else {
		open(PEAKFILE, "<$filename") || die("Open " . $filename . " failed : $!");
	}
	
	my $peak_alist = []; # list of lines containing peak info (each line stored as an array of columns)
	my $line_s;
	while ($line_s = <PEAKFILE>) {
		next if ($line_s =~ /^\#/); # skip comment lines
		last if ($line_s =~ /^[*]/); # end loop if end_of_file marker is encountered.
		
		($line_s) = ($line_s =~/^\s*(.*)/); # skip leading white space.
		chomp($line_s);
		push(@$peak_alist, [ split(/\s+/, $line_s) ]);
	}
	my $comment_alist = []; # list of comment lines - History section
	while ($line_s = <PEAKFILE>) {
		push(@$comment_alist, $line_s);
	}
	
	close(PEAKFILE);
	
	return wantarray ? ($peak_alist, $comment_alist) : $peak_alist;
}

