#!/usr/bin/perl
#!/usr/bin/perl 
#
# calc_resonance_splitting.pl
#	Converts assigned peak doublets into pseudo-central peaks with splittings dimension in Hz. 
#
# usage: calc_resonance_splitting.pl in_peaklist out_peaklist unused_peaklist workbook_name split_dim conversion_factor
#
#	peaklist - name of peak file or "-" for standard input/output.
#	unused_peaklist -  name of peak file for unpaired split peaks.
#	workbook_name - string to use as workbook name.
#	split_dim - dimension in which peaks are split.
#	conversion_factor - ppm to Hz conversion factor (dependent on spectrometer field strength).
#
# Hint: In case you want to deal with splittings from IPAP, concatenate the assigned peak lists first.
#
#
#
# Copyright Aneerban Bhattacharya, 2001. All rights reserved.
# Written by Aneerban Bhattacharya; Jul 9, 2001
# Modified by Aneerban Bhattacharya		Feb 15, 2002
#
# Subroutine read_peak_file, procedure for writing out file history, and interactive help format
# taken from AutoPeak scripts written by Hunter Moseley.
#

use strict;

# Print usage information
if ((scalar(@ARGV) < 6) || ($ARGV[0] =~ /^-h/)) {
	print STDERR "\nError: not enough parameters given.\n" if ($ARGV[0] !~ /^-h/);
	print STDERR "\n";
        print STDERR " calc_resonance_splitting.pl\n";
        print STDERR "	Converts assigned peak doublets into pseudo-central peaks with splittings dimension in Hz. \n";
        print STDERR "\n";
        print STDERR " usage: calc_resonance_splitting.pl in_peaklist out_peaklist unused_peaklist workbook_name split_dim conversion_factor\n";
        print STDERR "\n";
        print STDERR "	peaklist - name of peak file or \"-\" for standard input/output.\n";
        print STDERR "	unused_peaklist -  name of peak file for unpaired split peaks.\n";
        print STDERR "	workbook_name - string to use as workbook name.\n";
        print STDERR "	split_dim - dimension in which peaks are split.\n";
        print STDERR "	conversion_factor - ppm to Hz conversion factor (dependent on spectrometer field strength).\n";
        print STDERR "\n";
        print STDERR " Hint: In case you want to deal with splittings from IPAP, concatenate the assigned peak lists first.\n";
	exit(1);
}

# read in command line arguments
my ($infile_s, $outfile_s, $unusedfile_s, $workbook_name_s, $split_col_n, $conv_fact_n) = @ARGV;
$split_col_n = int($split_col_n) - 1; # 1Dim is column 0 and so on

# get list of peak lines, and end coment lines
my ($in_peak_lines_alist, $end_comments_alist) = &read_peak_file($infile_s);

my $num_cols_n = scalar(@{$$in_peak_lines_alist[0]}); # number of columns in peak list
my %assigns_h; # to store the list of split peaks and difference assigned to particular residue
# structure: assigns_h -> residue_s -> { peak1 dimensions { dim 1, dim 2, ... }, peak2 dimensions { dim 1, dim 2, ...} } 
# order of dimensions for the peaks must be in the SAME, for all peaks being put into the list


# pair the split peaks for each residue
foreach	my $peak_line_alist (@$in_peak_lines_alist) {	
	$$peak_line_alist[$num_cols_n - 1] =~ /.*\.([A-Z]\d+)\S*$/; # extract the residue number and type
    my $residue_s = $1;

	# currently not being used
	my ($notes_s) = ($$peak_line_alist[$num_cols_n - 1] =~ /[^.]*[.](.*)/);

	my @peak_dims_a; # stores each dimension of this peak
	# assuming penultimate column reserved for intensity
	for (my $colnum_n = 1; $colnum_n < ($num_cols_n - 2); $colnum_n++) {
		push(@peak_dims_a, $$peak_line_alist[$colnum_n]);
	}
	
	if (!exists ($assigns_h{$residue_s})) {
		$assigns_h{$residue_s} = [  [ @peak_dims_a ] ];	
	}
	elsif (scalar(@{$assigns_h{$residue_s}}) == 1) {
		push(@{$assigns_h{$residue_s}}, [ @peak_dims_a ]);
	}
	else {
		print("Error: > 2 peaks for $residue_s\n");
	}		
}

# output the splittings for each residue
open(OF, ">$outfile_s") || die("Can't open $outfile_s : $!");
print(OF "#Last *Dim is splitting in Hz, the prev *Dim is the same in ppm\n");
print(OF "#Index\t");
for (my $dim_n = 1; $dim_n < $num_cols_n; $dim_n++) { # 2 new columns being added
	print(OF "$dim_n", "Dim\t");
}
print(OF "Intensity\tWorkbook\n");
close(OF);

open(UF, ">$unusedfile_s") || die("Can't open $unusedfile_s : $!"); # initialise file
close(UF);

my $residue_s;
my $index_n = 0;

foreach $residue_s (sort keys(%assigns_h)) {
	if (scalar(@{$assigns_h{$residue_s}}) == 1) {
		open(UF, ">>$unusedfile_s") || die("Can't open $unusedfile_s : $!");
		print(UF "Found only one peak for $residue_s\n");
		close(UF);
        next;
    }
	
#	my $difference_n = abs($assigns_h{$residue_s}[1] - $assigns_h{$residue_s}[0]) / 2; # splittnig from the central peak
	my $difference_n = abs($assigns_h{$residue_s}[1][$split_col_n] - $assigns_h{$residue_s}[0][$split_col_n]); # splitting between split peaks
	$difference_n = sprintf("%.3f", $difference_n);
	my $Hz_split_n = $conv_fact_n * $difference_n;
	$Hz_split_n = sprintf("%.3f", $Hz_split_n);

	# calculate pseudo-central peak
	my @peak_dims_a; # stores each dimension of pseudo-central peak
	for (my $dim_n = 0; $dim_n < scalar(@{$assigns_h{$residue_s}[0]}); $dim_n++) {
		$peak_dims_a[$dim_n] = ($assigns_h{$residue_s}[0][$dim_n] + $assigns_h{$residue_s}[1][$dim_n]) / 2;
		$peak_dims_a[$dim_n] = sprintf("%.3f", $peak_dims_a[$dim_n]);
	}
	
	$index_n++;
	open(OF, ">>$outfile_s") || die("Can't open $outfile_s : $!");
#	print(OF sprintf("%4d\t%4s\t%.3f\t%.3f\t%.3f\t%.2f\t%d\n", $index_n, $residue_s, $assigns_h{$residue_s}[0], $assigns_h{$residue_s}[1], $difference_n, $Hz_split_n, $resnum_n));
	my $for_print_split_col_n = $split_col_n + 1; # for print output
	print(OF "$index_n\t");
	print(OF join("\t", @peak_dims_a));
	print(OF "\t$difference_n", "\t$Hz_split_n", "\t3000");
	print(OF "\t$workbook_name_s" . "_dim" . "$for_print_split_col_n" . "\.$residue_s\n");
	close(OF);
}

# print comments
open(OF, ">>$outfile_s") || die("Can't open $outfile_s : $!");

# modified from AutoPeak script written by Hunter Moseley
print(OF "*\n\n");
print(OF "\nFile History: $outfile_s \n\n");
print(OF $0, " ", join(' ',@ARGV),"\n");
print(OF "\n");

my $line_s;
my $history_found_b = 0;
foreach $line_s (@{$end_comments_alist}) {
	if ((!$history_found_b) && ($line_s =~ /^File History:/)) {
#		print(OF "File History: $outfile_s \n\n");
#		print(OF $0, " ", join(' ', @ARGV), "\n");
		$history_found_b = 1;
		next;
	}
	
	print(OF "$line_s");
}
print("\n");

#if (!$history_found_b) {
#	print(OF "\nFile History: $outfile_s \n\n");
#	print(OF $0, " ", join(' ',@ARGV),"\n");
#}
																			
close(OF);


# Read in peaks from AutoPeak format files - modified from the subroutine written by Hunter Moseley
sub read_peak_file {
	my ($filename) = @_;

	local *PEAKFILE;
	if ($filename eq "-") {
		*PEAKFILE = *STDIN;
	}
	else {
		open(PEAKFILE, "<$filename") || die("Open " . $filename . " failed : $!");
	}

	my $peak_alist = []; # list of lines containing peak info (each line stored as an array of columns)
	my $line_s;
	while ($line_s = <PEAKFILE>) {
		next if ($line_s =~ /^\#/); # skip comment lines
		last if ($line_s =~ /^[*]/); # end loop if end_of_file marker is encountered.

		($line_s) = ($line_s =~/^\s*(.*)/); # skip leading white space.
		chomp($line_s);
		push(@$peak_alist, [ split(/\s+/, $line_s) ]);
	}
	
	my $comment_alist = []; # list of comment lines - History section
	while ($line_s = <PEAKFILE>) {
		push(@$comment_alist, $line_s);
	}
	
	close(PEAKFILE);
	
	return wantarray ? ($peak_alist, $comment_alist) : $peak_alist;
}
	



