#!/usr/bin/perl
#!/usr/bin/perl
#  cmap2bmrb.pl
#	Converts a cmap file into a BMRB file.
#
#  usage: cmap2bmrb.pl input_cmap_file output_bmrb_file sequence_start sequence
#       
#	input_cmap_file - name of cmap file or "-" for standard input.
#	output_bmrb_file - name of the output BMRB file or  "-" for standard output.
#	sequence_start - the starting residue number of the sequence.
#	sequence - the protein sequence using single letter codes.
#
#
#   Written by Hunter Moseley 02/19/2001
#   Copyright:  Hunter Moseley, 2001. All rights reserved.
#
#
#  Updated to use new BMRBParsing/CMapParsing modules.
#  Copyright Hunter Moseley 2005. All rights reserved.
#

# These modules allow me to use modules rather than hardcoding everything into one script
use FindBin;
use lib $FindBin::Bin;

# Parser module parses in/out the cmap data and the peak file
use BMRBParsing qw(:ALL);
use CMapParsing qw(:ALL);
use strict;

my %aa_name_conversion = ( "a"   => "A", "r"   => "R", "n"   => "N", "d"   => "D", 
			   "c"   => "C", "b"   => "C", "q"   => "Q", "e"   => "E", "g"   => "G",
			   "h"   => "H", "i"   => "I", "l"   => "L", "k"   => "K",
			   "m"   => "M", "f"   => "F", "p"   => "P", "s"   => "S",
			   "t"   => "T", "w"   => "W", "y"   => "Y", "v"   => "V"); 




my %cmap_shift_conversion = 
  ( "A" => { "HN" => ["H"], "HA" => ["HA"], "HB" => ["HB"], "CO" => ["C"], "CA" => ["CA"], "CB" => ["CB"], "N15" => ["N"] }, 
    "R" => { "HN" => ["H"], "HA" => ["HA"], "HB" => ["HB2", "HB3"],  "HG" => ["HG2", "HG3"], 
	     "HD" => ["HD2","HD3"],  "HE" => ["HE"], "HH" => ["HH11","HH12","HH21","HH22"], 
	     "CO" => ["C"], "CA" => ["CA"], "CB" => ["CB"], "CG" => ["CG"],
	     "CD" => ["CD"], "CZ" => ["CZ"], "N15" => ["N"], "NE" => ["NE"], "NH" => ["NH1","NH2"] },
    "N" => { "HN" => ["H"], "HA" => ["HA"], "HB" => ["HB2", "HB3"], "HD" => ["HD21","HD22"],
	     "CO" => ["C"], "CA" => ["CA"], "CB" => ["CB"], "CG" => ["CG"], "N15" => ["N"], "ND" => ["ND2"] },
    "D" => { "HN" => ["H"], "HA" => ["HA"], "HB" => ["HB2","HB3"], "CO" => ["C"], "CA" => ["CA"],
	       "CB" => ["CB"], "CG" => ["CG"], "N15" => ["N"] },
    "C" => { "HN" => ["H"], "HA" => ["HA"], "HB" => ["HB2","HB3"], "HG" => ["HG"], "CO" => ["C"],
	       "CA" => ["CA"], "CB" => ["CB"], "N15" => ["N"] },
    "B" => { "HN" => ["H"], "HA" => ["HA"], "HB" => ["HB2","HB3"], "HG" => ["HG"], "CO" => ["C"],
	       "CA" => ["CA"], "CB" => ["CB"], "N15" => ["N"] },
    "Q" => { "HN" => ["H"], "HA" => ["HA"], "HB" => ["HB2","HB3"], "HG" => ["HG2","HG3"], 
	       "HE" => ["HE21","HE22"], "CO" => ["C"], "CA" => ["CA"], "CB" => ["CB"],
	       "CG" => ["CG"], "CD" => ["CD"], "N15" => ["N"], "NE" => ["NE2"] },
    "E" => { "HN" => ["H"], "HA" => ["HA"], "HB" => ["HB2","HB3"], "HG" => ["HG2","HG3"], 
	       "CO" => ["C"], "CA" => ["CA"], "CB" => ["CB"], "CG" => ["CG"], "CD" => ["CD"], "N15" => ["N"] },
    "G" => { "HN" => ["H"], "HA" => ["HA2","HA3"], "CO" => ["C"], "CA" => ["CA"], "N15" => ["N"] },
    "H" => { "HN" => ["H"], "HA" => ["HA"], "HB" => ["HB2","HB3"], "HD" => ["HD1","HD2"],
	       "HE" => ["HE1","HE2"], "CO" => ["C"], "CA" => ["CA"], "CB" => ["CB"], "CG" => ["CG"], "CD" => ["CD2"],
	       "CE" => ["CE1"], "N15" => ["N"], "ND" => ["ND1"], "NE" => ["NE2"], "ND" => ["ND"], "NE" => ["NE"], "CD" => ["CD"] },
    "I" => { "HN" => ["H"], "HA" => ["HA"], "HB" => ["HB"], "HG" => ["HG12","HG13","HG2"], 
	       "HD" => ["HD1"], "CO" => ["C"], "CA" => ["CA"], "CB" => ["CB"], "CG" => ["CG1","CG2"],
	       "CD" => ["CD1"], "N15" => ["N"] },
    "L" => { "HN" => ["H"], "HA" => ["HA"], "HB" => ["HB2","HB3"], "HG" => ["HG"], 
	       "HD" => ["HD1","HD2"], "CO" => ["C"], "CA" => ["CA"], "CB" => ["CB"], "CG" => ["CG"], "CD" => ["CD1","CD2"], 
	     "N15" => ["N"]  },
    "K" => { "HN" => ["H"], "HA" => ["HA"], "HB" => ["HB2","HB3"], "HG" => ["HG2","HG3"], 
	       "HD" => ["HD2","HD3"], "HE" => ["HE2","HE3"],
	       "HZ" => ["HZ"], "CO" => ["C"], "CA" => ["CA"], "CB" => ["CB"], "CG" => ["CG"], "CD" => ["CD"], "CE" => ["CE"],
	       "N15" => ["N"], "NZ" => ["NZ"] },
    "M" => { "HN" => ["H"], "HA" => ["HA"], "HB" => ["HB2","HB3"], "HG" => ["HG2","HG3"], 
	       "HE" => ["HE"], "CO" => ["C"], "CA" => ["CA"], "CB" => ["CB"], "CG" => ["CG"], "CE" => ["CE"],
	       "N15" => ["N"] },
    "F" => { "HN" => ["H"], "HA" => ["HA"], "HB" => ["HB2","HB3"], "HD" => ["HD1","HD2"],
	        "HE" => ["HE1","HE2"], "HZ" => ["HZ"], "CO" => ["C"], "CA" => ["CA"], "CB" => ["CB"],
	       "CG" => ["CG"], "CD" => ["CD1","CD2"], "CE" => ["CE1","CE2"], "CZ" => ["CZ"],
	       "N15" => ["N"] },
    "P" => { "HA" => ["HA"], "HB" => ["HB2","HB3"], "HG" => ["HG2","HG3"], 
	       "HD" => ["HD2","HD3"], "CO" => ["C"], "CA" => ["CA"], "CB" => ["CB"], "CG" => ["CG"],
	       "CD" => ["CD"], "N15" => ["N"] },
    "S" => { "HN" => ["H"], "HA" => ["HA"], "HB" => ["HB2","HB3"], "HG" => ["HG"], "CO" => ["C"],
	       "CA" => ["CA"], "CB" => ["CB"], "N15" => ["N"] },
    "T" => { "HN" => ["H"], "HA" => ["HA"], "HB" => ["HB"], "HG" => ["HG1","HG2"], "CO" => ["C"], "CA" => ["CA"],
	       "CB" => ["CB"], "CG" => ["CG2"], "N15" => ["N"] },
    "W" => { "HN" => ["H"], "HA" => ["HA"], "HB" => ["HB2","HB3"], "HD" => ["HD1"], "HE" => ["HE1","HE3"],
	       "HZ" => ["HZ2","HZ3"], "HH" => ["HH2"], "CO" => ["C"], "CA" => ["CA"], "CB" => ["CB"],
	       "CG" => ["CG"], "CD" => ["CD1","CD2"], "CE" => ["CE2","CE3"], "CZ" => ["CZ2","CZ3"],
	       "CH" => ["CH2"], "N15" => ["N"], "NE" => ["NE1"],  },
    "Y" => { "HN" => ["H"], "HA" => ["HA"], "HB" => ["HB2","HB3"], "HD" => ["HD1","HD2"],
	        "HE" => ["HE1","HE2"], "HH" => ["HH"], "CO" => ["C"], "CA" => ["CA"],
	       "CB" => ["CB"], "CG" => ["CG"], "CD" => ["CD1","CD2"], "CE" => ["CE1","CE2"], "CZ" => ["CZ"], "N15" => ["N"] },
    "V" => { "HN" => ["H"], "HA" => ["HA"], "HB" => ["HB"], "HG" => ["HG1","HG2"], "CO" => ["C"],
	       "CA" => ["CA"], "CB" => ["CB"], "CG" => ["CG1","CG2"], "N15" => ["N"] } );


if ((@ARGV < 4) || ($ARGV[0] =~ /^-h/))
  {
  print "\nError: not enough parameters given.\n\n" if ($ARGV[0] !~ /^-h/);
  print "cmap2bmrb.pl\n";
  print "	Converts a cmap file into a BMRB file.\n";
  print "\n";
  print "  usage: cmap2bmrb.pl input_cmap_file output_bmrb_file sequence_start sequence\n";
  print "       \n";
  print "	input_cmap_file - name of cmap file or \"-\" for standard input.\n";
  print "	output_bmrb_file - name of the output BMRB file or  \"-\" for standard output.\n";
  print "	sequence_start - the starting residue number of the sequence.\n";
  print "	sequence - the protein sequence using single letter codes.\n";
  print " \n";
  exit(1);
  }

# Read in command line parameters
my $input_cmap_file = shift @ARGV;
my $output_bmrb_file = shift @ARGV;
my $sequence_startpos = shift @ARGV;
my $sequence_string = shift @ARGV;

die "No sequence given.\n" if ($sequence_string eq "");

my $cmap_hlist = &parseCMapfile($input_cmap_file);

my %author_mapping_hlist;
my $bmrb_hlist = {};
$$bmrb_hlist{"filename"} = $output_bmrb_file;

# create name_array and sequence fields
$$bmrb_hlist{"sequence_startpos"} = $sequence_startpos;
$sequence_string =~ s/\s+//g;
$sequence_string =~ s/[Bb]/C/g;
my @sequence = split '', $sequence_string;
$$bmrb_hlist{"sequence"} = $sequence_string;
my $sequence_pos = $sequence_startpos;

$$bmrb_hlist{"entity_list"}{1}{"name_array"} = [];
$$bmrb_hlist{"entity_list"}{1}{"entity_id"} = 1;
for(my $count = 0; $count < @sequence; $count++)
  {
  # There are no 0's in the sequence_position
  if ($sequence_pos == 0)
    { $sequence_pos++; }
  my $residue_name = $sequence[$count] . ($count + 1);
  $author_mapping_hlist{$sequence[$count] . $sequence_pos} = $residue_name;
  push @{$$bmrb_hlist{"entity_list"}{1}{"name_array"}}, $residue_name;
  $$bmrb_hlist{"entity_list"}{1}{"rlist"}{$residue_name}{"aa"} = $sequence[$count];
  $$bmrb_hlist{"entity_list"}{1}{"rlist"}{$residue_name}{"index"} = $count+1;
  $$bmrb_hlist{"entity_list"}{1}{"rlist"}{$residue_name}{"author_index"} = $sequence_pos;
  $$bmrb_hlist{"entity_list"}{1}{"rlist"}{$residue_name}{"author_name"} = $sequence[$count] . $sequence_pos;

  $sequence_pos++;  
  }



foreach my $residue (values %{$$cmap_hlist{"rlist"}})
  {
  $$residue{"author_name"} =~ s/^[Bb]/C/; # convert B's to C's.
  my $res_name = $author_mapping_hlist{$$residue{"author_name"}};

  if (exists $cmap_shift_conversion{$$residue{"aa"}})
    {
    my $aa_shift_conversion = $cmap_shift_conversion{$$residue{"aa"}};
    foreach my $cmap_atom_type (keys %$aa_shift_conversion)
      {
      my $intra_list = [];
      if (exists $$residue{"shifts"}{$cmap_atom_type})
	{ $intra_list = $$residue{"shifts"}{$cmap_atom_type}{"list"}; }

      my $inter_list = [];
      if (exists $$cmap_hlist{"rlist"}{$$residue{"next"}} && exists $$cmap_hlist{"rlist"}{$$residue{"next"}}{"shifts"}{$cmap_atom_type . "-1"})
	{ $inter_list = $$cmap_hlist{"rlist"}{$$residue{"next"}}{"shifts"}{$cmap_atom_type . "-1"}{"list"}; }

      for(my $x=0; $x < @{$$aa_shift_conversion{$cmap_atom_type}}; $x++)
	{
	if ($x < @$intra_list && $$intra_list[$x] ne "-")
	  {
	  $$bmrb_hlist{"entity_list"}{1}{"rlist"}{$res_name}{"shifts"}{$$aa_shift_conversion{$cmap_atom_type}[$x]}{"atom_type"} = ($$aa_shift_conversion{$cmap_atom_type}[$x] =~ /^([HNC])/)[0];
	  push @{$$bmrb_hlist{"entity_list"}{1}{"rlist"}{$res_name}{"shifts"}{$$aa_shift_conversion{$cmap_atom_type}[$x]}{"list"}}, $$intra_list[$x];
	  }
	elsif ($x < @$inter_list && $$inter_list[$x] ne "-")
	  {
	  $$bmrb_hlist{"entity_list"}{1}{"rlist"}{$res_name}{"shifts"}{$$aa_shift_conversion{$cmap_atom_type}[$x]}{"atom_type"} = ($$aa_shift_conversion{$cmap_atom_type}[$x] =~ /^([HNC])/)[0];
	  push @{$$bmrb_hlist{"entity_list"}{1}{"rlist"}{$res_name}{"shifts"}{$$aa_shift_conversion{$cmap_atom_type}[$x]}{"list"}}, $$inter_list[$x];
	  }
	}
      }
    }
  }



#create next and prev fields
for(my $x=0; $x < @{$$bmrb_hlist{"entity_list"}{1}{"name_array"}}; $x++)
  {
  my $resname = $$bmrb_hlist{"entity_list"}{1}{"name_array"}[$x];
  
  if ($x)
    { $$bmrb_hlist{"entity_list"}{1}{"rlist"}{$resname}{"prev"} = $$bmrb_hlist{"name_array"}[$x-1]; }
  if ($x < (@{$$bmrb_hlist{"entity_list"}{1}{"name_array"}} - 1))
    { $$bmrb_hlist{"entity_list"}{1}{"rlist"}{$resname}{"next"} = $$bmrb_hlist{"name_array"}[$x+1]; }
  }


# write bmrb file
&write_bmrb_file($bmrb_hlist, $output_bmrb_file);
