#!/usr/bin/perl
#!/usr/bin/perl
#  compare_bmrb.pl
#	Compares two bmrb files and returns how well they match.
#
#  usage: compare_bmrb.pl input_bmrb_file input_bmrb_file [options]
#
#	bmrb_file - name of bmrb file or "-" for standard input/output.
#	options:
#		-verbose - print information verbosely.
#		-master - work as if the first input_file contains master verified results.
#		-Ctol value, -Ntol value, -Htol value - set identity tolerances to value for
#		        carbons, nitrogens, or hydrogens (default values are 0.1, 0.1, and 0.01 
#		        respectively).
#               -Cshift value, -Nshift value, Hshift value - shift the second input_file shifts 
#		        before comparison.
#		-nodegenerate - do not compare shifts degenerately. 
# 
#
#  Copyright Hunter Moseley, 2000. All rights reserved.
#  Written by Hunter Moseley 7/31/2000
#
#
#  Updated to use new BMRBParsing/CMapParsing modules.
#  Copyright Hunter Moseley 2005. All rights reserved.
#
use strict;

# allows perl to fine local modules.
use FindBin;
use lib $FindBin::Bin;

# contains subroutines for reading and writing bmrb files.
use BMRBParsing qw(:ALL);

# print help info.
if ((@ARGV < 2) || ($ARGV[0] =~ /^\-h/))
  {
  print STDERR "Error: not enough parameters given.\n\n" if (@ARGV < 2);
  print STDERR "compare_bmrb.pl\n";
  print STDERR "\tCompares two bmrb files and returns how well they match.\n";
  print STDERR "\n";
  print STDERR "  usage: compare_bmrb.pl input_bmrb_file input_bmrb_file [options]\n";
  print STDERR "\n";
  print STDERR "\tbmrb_file - name of bmrb file or \"-\" for standard input/output.\n";
  print STDERR "\toptions:\n";
  print STDERR "\t\t-verbose - print information verbosely.\n";
  print STDERR "\t\t-master - work as if the first input_file contains master verified results.\n";
  print STDERR "\t\t-Ctol value, -Ntol value, -Htol value - set identity tolerances to value for\n";
  print STDERR "\t\t\tcarbons, nitrogens, or hydrogens (default values are 0.1, 0.1, and 0.01 \n";
  print STDERR "\t\t\trespectively).\n";
  print STDERR "\t\t-Cshift value, -Nshift value, Hshift value - shift the second input_file shifts \n";
  print STDERR "\t\t\tbefore comparison. \n";
  print STDERR "\t\t-nodegenerate - do not compare shifts degenerately.\n";
  exit(1);
  }

# read command line arguments.
my $input_bmrb1 = shift @ARGV;
my $input_bmrb2 = shift @ARGV;

my $verbose = 0;
my $master = 0;
my %tolerances = ( "C" => 0.1, "N" => 0.1, "H" => 0.01 );
my %shifting = ( "C" => 0, "N" => 0, "H" => 0 );
my $degenerate = 1;
for(my $x=0; $x < @ARGV; $x++) 
  {
  if ($ARGV[$x] =~ /^-v/)
    { $verbose = 1; }
  elsif ($ARGV[$x] =~ /^-n/)
    { $degenerate = 0; }
  elsif ($ARGV[$x] =~ /^-m/)
    { $master = 1; }
  elsif ($ARGV[$x] =~ /-Ct/)
    { $x++; $tolerances{"C"} = $ARGV[$x]; }
  elsif ($ARGV[$x] =~ /^-Nt/)
    { $x++; $tolerances{"N"} = $ARGV[$x]; }
  elsif ($ARGV[$x] =~ /^-Ht/)
    { $x++; $tolerances{"H"} = $ARGV[$x]; }
  elsif ($ARGV[$x] =~ /-Cs/)
    { $x++; $shifting{"C"} = $ARGV[$x]; }
  elsif ($ARGV[$x] =~ /^-Ns/)
    { $x++; $shifting{"N"} = $ARGV[$x]; }
  elsif ($ARGV[$x] =~ /^-Hs/)
    { $x++; $shifting{"H"} = $ARGV[$x]; }
  else
    {
    print STDERR "Error: unknown option ",$ARGV[$x]," given.\n";
    exit(1);
    }
  }

my %read_options = ( "convert_aa_names" => 1, "convert_shift_names" => $degenerate );
my $bmrb_hlist1 = &read_bmrb_file($input_bmrb1,\%read_options);
my $bmrb_hlist2 = &read_bmrb_file($input_bmrb2, \%read_options);


my $error_count = 0;
my $correct_count = 0;
my $unused_count = 0;

# get sorted list of residue names
foreach my $entity1 (values %{$$bmrb_hlist1{"entity_list"}})
  {
  my $entity2 = 0;
  if (exists $$bmrb_hlist2{"entity_list"}{$$entity1{"entity_id"}})
    { $entity2 = $$bmrb_hlist2{"entity_list"}{$$entity1{"entity_id"}}; }

  my @sorted_residue_names;  
  if ($entity2 && exists $$entity2{"name_array"})
    { @sorted_residue_names = @{$$entity2{"name_array"}}; }
  elsif ($entity2)
    { @sorted_residue_names = sort { substr($a,3) <=> substr($b,3); } (keys %{$$entity2{"rlist"}}); }
  
  # look for incompatible shifts
  foreach my $residue_name (@sorted_residue_names)
    {
    if (exists $$entity1{"rlist"}{$residue_name})
      {
      foreach my $shift_name (keys %{$$entity2{"rlist"}{$residue_name}{"shifts"}})
	{
	if (exists $$entity1{"rlist"}{$residue_name}{"shifts"}{$shift_name})
	  {
	  my $shift1 = $$entity1{"rlist"}{$residue_name}{"shifts"}{$shift_name};
	  my $shift2 = $$entity2{"rlist"}{$residue_name}{"shifts"}{$shift_name};
	  
	  $$shift1{"used"} = 1;
	  foreach my $shift_value (@{$$shift2{"list"}})
	    {
	    my $found = grep { abs($_ - ($shift_value + $shifting{$$shift2{"atom_type"}})) <= ($tolerances{$$shift2{"atom_type"}} + 0.00000000001);  } @{$$shift1{"list"}};
	    if (! $found)
	      {
	      if ($verbose)
		{ print "($input_bmrb2)    $residue_name "," "x(6-length($residue_name)),"- $shift_name $shift_value \t!=  ", join(",",@{$$shift1{"list"}}),"    ($input_bmrb1)\n"; }
	      
	      $error_count++;
	      }
	    else
	      { $correct_count++; }
	    }
	  }
	elsif (! $master && $entity2) # perform null check
	  {
	  my $shift2 = $$entity2{"rlist"}{$residue_name}{"shifts"}{$shift_name};
	  foreach my $shift_value (@{$$shift2{"list"}})
	    {
	    if ($verbose)
	      { print "$input_bmrb1 lacks $residue_name "," "x(6-length($residue_name)),"- $shift_name $shift_value\n"; }
	    $error_count++;
	    }
	  }
	}
      }
    elsif (! $master && $entity2) # perform null check
      {
      foreach my $shift_name (keys %{$$entity2{"rlist"}{$residue_name}{"shifts"}})
	{
	my $shift2 = $$entity2{"rlist"}{$residue_name}{"shifts"}{$shift_name};
	foreach my $shift_value (@{$$shift2{"list"}})
	  {
	  if ($verbose)
	    { print "$input_bmrb1 lacks $residue_name "," "x(6-length($residue_name)),"- $shift_name $shift_value\n"; }
	  
	  $error_count++;
	  }
	}
      }
    }
  
  # perform null check in other direction if not in master mode
  if (! $master) 
    {
    foreach my $residue_name (keys %{$$entity1{"rlist"}})
      {
      if ($entity2 && exists $$entity2{"rlist"}{$residue_name})
	{
	foreach my $shift_name (keys %{$$entity1{"rlist"}{$residue_name}{"shifts"}})
	  {
	  if ($entity2 && exists $$entity2{"rlist"}{$residue_name}{"shifts"}{$shift_name})
	    {
	    my $shift2 = $$entity2{"rlist"}{$residue_name}{"shifts"}{$shift_name};
	    my $shift1 = $$entity1{"rlist"}{$residue_name}{"shifts"}{$shift_name};
	    my $shift_loss = @{$$shift1{"list"}} - @{$$shift2{"list"}};
	    if ($shift_loss > 0)
	      {
	      if ($verbose)
		{ print "$input_bmrb2 lacks $shift_loss shifts from $residue_name "," "x(6-length($residue_name)),"- $shift_name \n"; }
	      
	      $error_count += $shift_loss;
	      }
	    }
	  else
	    {
	    my $shift1 = $$entity1{"rlist"}{$residue_name}{"shifts"}{$shift_name};
	    foreach my $shift_value (@{$$shift1{"list"}})
	      {
	      if ($verbose)
		{ print "$input_bmrb2 lacks $residue_name "," "x(6-length($residue_name)),"- $shift_name $shift_value\n"; }
	      $error_count++;
	      }
	    }
	  }
	}
      else
	{
	foreach my $shift_name (keys %{$$entity1{"rlist"}{$residue_name}{"shifts"}})
	  {
	  my $shift1 = $$entity1{"rlist"}{$residue_name}{"shifts"}{$shift_name};
	  foreach my $shift_value (@{$$shift1{"list"}})
	    {
	    if ($verbose)
	      { print "$input_bmrb2 lacks $residue_name "," "x(6-length($residue_name)),"- $shift_name $shift_value\n"; }	  
	    $error_count++;
	    }
	  }
	}
      }
    }
  
  # check for unused shifts in master list
  if ($master) 
    {
    foreach my $residue_name (keys %{$$entity1{"rlist"}})
      {
      foreach my $shift_name (keys %{$$entity1{"rlist"}{$residue_name}{"shifts"}})
	{
	if (! exists $$entity1{"rlist"}{$residue_name}{"shifts"}{$shift_name}{"used"})
	  { $unused_count++; }
	}
      }
    }
  }  

# print out final error info.
if ($verbose)
  {
  print "\nComparison of $input_bmrb1 and $input_bmrb2 has $error_count errors.\n";
  print "\nComparison of $input_bmrb1 and $input_bmrb2 has $correct_count comparable shifts.\n";
  print "\nMaster list $input_bmrb1 has $unused_count unused shifts.\n" if ($master);
  }
else
  {
  if ($master)
    { print "$error_count $correct_count $unused_count\n"; }
  else
    { print "$error_count $correct_count\n"; }
  }
