#!/usr/bin/perl
#!/usr/bin/perl
#  compare_cmap.pl
#	Compares two CMap files and returns how well they match.
#
#  usage: compare_cmap.pl old_cmap_file new_cmap_file [options]
#
#	cmap_file - name of CMap file.
#	options:
#		-Ctol value, -Ntol value, -Htol value - set match tolerances to value for
#		        carbons, nitrogens, or hydrogens (default values are 0.1, 0.1, and 0.01 
#		        respectively).
#		-tol resonance_type value - set match tolerance to value for specific resonance_type.
#		-resonance_only - test resonance differences only.
#		-output output_cmap_file - output cmap file with differences graphics row.
#		-master - treat old_cmap_file as the master file.
#
#  Copyright Hunter Moseley, 2002. All rights reserved.
#  Written by Hunter Moseley 10/8/2002
#
use strict;

# allows perl to fine local modules.
use FindBin;
use lib $FindBin::Bin;

# contains subroutines for reading and writing bmrb files.
use CMapParsing qw(:ALL);

# print help info.
if ((@ARGV < 2) || ($ARGV[0] =~ /^\-h/))
  {
  print STDERR "Error: not enough parameters given.\n\n" if (@ARGV < 2);
  print STDERR "  compare_cmap.pl\n";
  print STDERR "	Compares two CMap files and returns how well they match.\n";
  print STDERR "\n";
  print STDERR "  usage: compare_cmap.pl old_cmap_file new_cmap_file [options]\n";
  print STDERR "\n";
  print STDERR "	cmap_file - name of CMap file.\n";
  print STDERR "	options:\n";
  print STDERR "		-Ctol value, -Ntol value, -Htol value - set match tolerances to value for\n";
  print STDERR "		        carbons, nitrogens, or hydrogens (default values are 0.1, 0.1, and 0.01 \n";
  print STDERR "		        respectively).\n";
  print STDERR "		-tol resonance_type value - set match tolerance to value for specific resonance_type.\n";
  print STDERR "		-resonance_only - test resonance differences only.\n";
  print STDERR "		-output output_cmap_file - output cmap file with differences graphics row.\n";
  print STDERR "		-master - treat old_cmap_file as the master file.\n";

  exit(1);
  }

# read command line arguments.
my $old_cmap_file = shift @ARGV;
my $new_cmap_file = shift @ARGV;
my $output_cmap_file;
my $resonance_only = 0;
my $master = 0;

my %base_tolerances = ( "C" => 0.1, "N" => 0.1, "H" => 0.01 );
my %specific_tolerances;

while (@ARGV)
  {
  my $option = shift @ARGV;
  if ($option =~ /^-r/)
    { $resonance_only = 1; }
  elsif ($option =~ /^-o/)
    { $output_cmap_file = shift @ARGV; }
  elsif ($option =~ /^-Ct/)
    { $base_tolerances{"C"} = shift @ARGV; }
  elsif ($option =~ /^-Ht/)
    { $base_tolerances{"H"} = shift @ARGV; }
  elsif ($option =~ /^-Nt/)
    { $base_tolerances{"N"} = shift @ARGV; }
  elsif ($option =~ /^-m/)
    { $master = 1; }
  elsif ($option =~ /^-t/)
    { 
    my $res_type = shift @ARGV;
    $specific_tolerances{$res_type} = shift @ARGV;
    }
  else
    {
    print STDERR "Error: unknown option ",$option," given.\n";
    exit(1);
    }
  }


#
# Read CMap files
#
my $old_cmap_hlist = &parseCMapfile($old_cmap_file);
my $new_cmap_hlist = &parseCMapfile($new_cmap_file);

#
# Determine specific tolerances
#
foreach my $token (@{$$old_cmap_hlist{"parsedtokens"}})
  {
  if (($token =~ /^[C]/) && (! exists $specific_tolerances{$token}))
    { $specific_tolerances{$token} = $base_tolerances{"C"}; }
  elsif (($token =~ /^[H]/) && (! exists $specific_tolerances{$token}))
    { $specific_tolerances{$token} = $base_tolerances{"H"}; }
  elsif (($token =~ /^[N]/) && (! exists $specific_tolerances{$token}))
    { $specific_tolerances{$token} = $base_tolerances{"N"}; }
  }

foreach my $token (@{$$new_cmap_hlist{"parsedtokens"}})
  {
  if (($token =~ /^[C]/) && (! exists $specific_tolerances{$token}))
    { $specific_tolerances{$token} = $base_tolerances{"C"}; }
  elsif (($token =~ /^[H]/) && (! exists $specific_tolerances{$token}))
    { $specific_tolerances{$token} = $base_tolerances{"H"}; }
  elsif (($token =~ /^[N]/) && (! exists $specific_tolerances{$token}))
    { $specific_tolerances{$token} = $base_tolerances{"N"}; }
  }


#
# Check new_cmap against old_cmap first
#
my $error_count = 0;
my $correct_count = 0;

# look for incompatible shifts
foreach my $residue_name (@{$$new_cmap_hlist{"name_array"}})
  {
  if (exists $$old_cmap_hlist{"rlist"}{$residue_name})
    {
    foreach my $shift_name (keys %{$$new_cmap_hlist{"rlist"}{$residue_name}{"shifts"}})
      {
      next if (($shift_name !~ /^[CHN]/) && $resonance_only);

      if (exists $$old_cmap_hlist{"rlist"}{$residue_name}{"shifts"}{$shift_name} && exists $$old_cmap_hlist{"rlist"}{$residue_name}{"shifts"}{$shift_name}{"list"} && @{$$old_cmap_hlist{"rlist"}{$residue_name}{"shifts"}{$shift_name}{"list"}})
	{
	my $shift1 = $$old_cmap_hlist{"rlist"}{$residue_name}{"shifts"}{$shift_name};
	my $shift2 = $$new_cmap_hlist{"rlist"}{$residue_name}{"shifts"}{$shift_name};

	$$shift1{"used"} = 1;
	foreach my $shift_value (@{$$shift2{"list"}})
	  {
	  my $found = 0;
	  if (exists $specific_tolerances{$shift_name})
	    { $found = grep { abs($_ - $shift_value) <= ($specific_tolerances{$shift_name} + 0.00000000001);  } @{$$shift1{"list"}}; }
	  else
	    { $found = grep { $_ eq $shift_value; } @{$$shift1{"list"}}; }

	  if (! $found)
	    {
	    print "($new_cmap_file)    $residue_name"," - $shift_name "," "x(6-length($residue_name)),"\t$shift_value \t!=  ", join(",",@{$$shift1{"list"}}),"    ($old_cmap_file)\n"; 

	    $$new_cmap_hlist{"rlist"}{$residue_name}{"diff_count"}++;
	    $error_count++;
	    }
	  else
	    { $correct_count++; }
	  }
	}
      elsif (! $master) # perform null check
	{
	my $shift2 = $$new_cmap_hlist{"rlist"}{$residue_name}{"shifts"}{$shift_name};
	foreach my $shift_value (@{$$shift2{"list"}})
	  {
	  print "$old_cmap_file lacks $residue_name "," "x(6-length($residue_name)),"- $shift_name $shift_value\n"; 
	  $$new_cmap_hlist{"rlist"}{$residue_name}{"diff_count"}++;
	  $error_count++;
	  }
	}
      }
    }
  elsif (! $master) # perform null check
    {
    foreach my $shift_name (keys %{$$new_cmap_hlist{"rlist"}{$residue_name}{"shifts"}})
      {
      next if (($shift_name !~ /^[CHN]/) && $resonance_only);

      my $shift2 = $$new_cmap_hlist{"rlist"}{$residue_name}{"shifts"}{$shift_name};
      foreach my $shift_value (@{$$shift2{"list"}})
	{
	print "$old_cmap_file lacks $residue_name "," "x(6-length($residue_name)),"- $shift_name $shift_value\n"; 
	$$new_cmap_hlist{"rlist"}{$residue_name}{"diff_count"}++;
	$error_count++;
	}
      }
    }
  }

# perform null check in other direction if not in master mode
if (! $master) 
  {
  foreach my $residue_name (keys %{$$old_cmap_hlist{"rlist"}})
    {
    if (exists $$new_cmap_hlist{"rlist"}{$residue_name})
      {
      foreach my $shift_name (keys %{$$old_cmap_hlist{"rlist"}{$residue_name}{"shifts"}})
	{
	next if (($shift_name !~ /^[CHN]/) && $resonance_only);

	if (exists $$new_cmap_hlist{"rlist"}{$residue_name}{"shifts"}{$shift_name} && exists $$new_cmap_hlist{"rlist"}{$residue_name}{"shifts"}{$shift_name}{"list"} && @{$$new_cmap_hlist{"rlist"}{$residue_name}{"shifts"}{$shift_name}{"list"}})
	  {
	  my $shift2 = $$new_cmap_hlist{"rlist"}{$residue_name}{"shifts"}{$shift_name};
	  my $shift1 = $$old_cmap_hlist{"rlist"}{$residue_name}{"shifts"}{$shift_name};
	  my $shift_loss = @{$$shift1{"list"}} - @{$$shift2{"list"}};
	  if ($shift_loss > 0)
	    {
	    print "$new_cmap_file lacks $shift_loss shifts from $residue_name "," "x(6-length($residue_name)),"- $shift_name \n";   
	    $$new_cmap_hlist{"rlist"}{$residue_name}{"diff_count"} += $shift_loss;
	    $error_count += $shift_loss;
	    }
	  }
	else
	  {
	  my $shift1 = $$old_cmap_hlist{"rlist"}{$residue_name}{"shifts"}{$shift_name};
	  foreach my $shift_value (@{$$shift1{"list"}})
	    {
	    print "$new_cmap_file lacks $residue_name "," "x(6-length($residue_name)),"- $shift_name $shift_value\n"; 
	    $$new_cmap_hlist{"rlist"}{$residue_name}{"diff_count"}++;
	    $error_count++;
	    }
	  }
	}
      }
    else
      {
      foreach my $shift_name (keys %{$$old_cmap_hlist{"rlist"}{$residue_name}{"shifts"}})
	{
	next if (($shift_name !~ /^[CHN]/) && $resonance_only);

	my $shift1 = $$old_cmap_hlist{"rlist"}{$residue_name}{"shifts"}{$shift_name};
	foreach my $shift_value (@{$$shift1{"list"}})
	  {
	  print "$new_cmap_file lacks $residue_name "," "x(6-length($residue_name)),"- $shift_name $shift_value\n"; 
	  $error_count++;
	  }
	}
      }
    }
  }

# check for unused shifts in master list
my $unused_count = 0;
if ($master) 
  {
  foreach my $residue_name (keys %{$$old_cmap_hlist{"rlist"}})
    {
    foreach my $shift_name (keys %{$$old_cmap_hlist{"rlist"}{$residue_name}{"shifts"}})
      {
      next if (($shift_name !~ /^[CHN]/) && $resonance_only);
      next if ( ! @{$$old_cmap_hlist{"rlist"}{$residue_name}{"shifts"}{$shift_name}{"list"}});

      if (! exists $$old_cmap_hlist{"rlist"}{$residue_name}{"shifts"}{$shift_name}{"used"})
	{ $unused_count++; }
      }
    }
  }

# print out final error info.
print "\n\nComparison of $old_cmap_file and $new_cmap_file has $error_count errors.\n";
print "\nComparison of $old_cmap_file and $new_cmap_file has $correct_count comparable shifts.\n";
print "\nMaster list $old_cmap_file has $unused_count unused shifts.\n" if ($master);


#
# Write output cmap file if specified
#
if ($output_cmap_file ne "")
  {
  my $new_token = "GDifferences\@A";
  push @{$$new_cmap_hlist{"parsedtokens"}}, $new_token;
	 
  my $top_diff = (sort { $a <=> $b; } (map { $$new_cmap_hlist{"rlist"}{$_}{"diff_count"}; } (@{$$new_cmap_hlist{"name_array"}})))[-1];
  
  if ($top_diff)
    {
    foreach my $residue_name (@{$$new_cmap_hlist{"name_array"}})
      {
      if ($$new_cmap_hlist{"rlist"}{$residue_name}{"diff_count"})
	{
	my $diff_type = "+" x ($$new_cmap_hlist{"rlist"}{$residue_name}{"diff_count"} * 4 / $top_diff);
	if ($diff_type eq "")
	  { $diff_type = "+"; }
	
	$$new_cmap_hlist{"rlist"}{$residue_name}{"shifts"}{$new_token}{"list"}[0] = $diff_type;
	}  
      }
    }
  
  &outputCMapfile($new_cmap_hlist,$output_cmap_file);  
  }
