#!/usr/bin/perl
#!/usr/bin/perl
#  concat_bmrb_shifts.pl
#	Concatenate a set of bmrb files.
#
#  usage: concat_bmrb_shifts.pl [options] -input filename ...
#
#	-Ctol value, -Ntol value, -Htol value - set identity tolerances to value for
#	        carbons, nitrogens, or hydrogens (default values are 0.1, 0.1, and 0.01 
#	        respectively).
#	-output filename - output file ("-" for stdout).
#	-master filename - master input file to add to ("-" for stdin).
#	-input filename... - input files ("-" for stdin).
#	-inclusive - include all shifts (include only unique or consistent is default).
#	-keep_sequence - keep current sequence.
#	-limit - limit to number of possible shifts for each type.
# 
#
#  Copyright Hunter Moseley, 2000. All rights reserved.
#  Written by Hunter Moseley 7/31/2000
#
#
#  Updated to use new BMRBParsing/CMapParsing modules.
#  Copyright Hunter Moseley 2005. All rights reserved.
#
use strict;

# allows perl to fine local modules.
use FindBin;
use lib $FindBin::Bin;

# contains subroutines for reading and writing bmrb files.
use BMRBParsing qw(:ALL);


# print help info.
if ((@ARGV < 2) || ($ARGV[0] =~ /-h/))
  {
  print STDERR "Error: not enough parameters given.\n\n" if (@ARGV < 2);
  print "concat_bmrb_shifts.pl\n";
  print "  Combine a set of bmrb files.\n";
  print "\n";
  print "  usage: concat_bmrb_shifts.pl [options] -input filename ...\n";
  print "\n";
  print "\t-Ctol value, -Ntol value, -Htol value - set identity tolerances to value for\n";
  print "\t\tcarbons, nitrogens, or hydrogens (default values are 0.1, 0.1, and 0.01 \n";
  print "\t\trespectively).\n";
  print "\t-output filename - output file (\"-\" for stdout).\n";
  print "\t-master filename - master input file to add to (\"-\" for stdin).\n";
  print "\t-input filename... - input files (\"-\" for stdin).\n";
  print "\t-inclusive - include all shifts (include only unique or consistent is default).\n";
  print "\t-keep_sequence - keep current sequence.\n";
  print "\t-limit - limit to number of possible shifts for each type.\n";
  exit(1);
  }

# read command line arguments.
my %tolerances = ( "C" => 0.1, "N" => 0.1, "H" => 0.01 );
my $consistent = 1;
my $output_filename = "";
my $master_filename = "";
my @input_filenames;
my $keep_sequence = 0;
my $demultiplicate = 0;

for(my $x=0; $x < @ARGV; $x++) 
  {
  if ($ARGV[$x] =~ /-Ct/)
    { $x++; $tolerances{"C"} = $ARGV[$x]; }
  elsif ($ARGV[$x] =~ /^-Nt/)
    { $x++; $tolerances{"N"} = $ARGV[$x]; }
  elsif ($ARGV[$x] =~ /^-Ht/)
    { $x++; $tolerances{"H"} = $ARGV[$x]; }
  elsif ( $ARGV[$x] =~ /^-o/)
    { $x++; $output_filename = $ARGV[$x]; }
  elsif ( $ARGV[$x] =~ /^-m/)
    { $x++; $master_filename = $ARGV[$x]; }
  elsif ($ARGV[$x] =~ /^-k/)
    { $keep_sequence = 1; }
  elsif ($ARGV[$x] =~ /^-l/)
    { $demultiplicate = 1; }
  elsif ($ARGV[$x] =~ /^-inc/)
    { $consistent = 0; }
  elsif ( $ARGV[$x] =~ /^-inp/)
    { 
    for(my $y = $x+1; $y < @ARGV; $y++)
      {push @input_filenames, $ARGV[$y]; }
    last;
    }
  else
    {
    print STDERR "Error: unknown option ",$ARGV[$x]," given.\n";
    exit(1);
    }
  }


print STDERR "Error: no output file specified.\n" if ($output_filename eq "");
print STDERR "Error: no input files specified.\n" if (! @input_filenames);

my $master_bmrb_hlist = {};

if ($master_filename ne "")
  { $master_bmrb_hlist = &read_bmrb_file($master_filename, {}); }

if (! $consistent)
  {
  my %read_options = ("bmrb_hlist" => $master_bmrb_hlist );
  &read_bmrb_file($input_filenames[0], \%read_options);

  $read_options{"delete_save_frames"} = 1;
  for(my $x=1; $x < @input_filenames; $x++)
    { &read_bmrb_file($input_filenames[$x], \%read_options); }
  }
else
  {
  # read the input bmrb files
  my @bmrb_hlists;
  foreach my $filename (@input_filenames)
    { push @bmrb_hlists, &read_bmrb_file($filename, {}); }

  # master file overrides other input files.
  if ($master_filename ne "")
    {
    foreach my $entity_id (keys %{$$master_bmrb_hlist{"entity_list"}})
      {
      foreach my $residue_name (keys %{$$master_bmrb_hlist{"entity_list"}{$entity_id}{"rlist"}})
	{
	foreach my $shift_name (keys %{$$master_bmrb_hlist{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"shifts"}})
	  { &mark_matching_shifts(\@bmrb_hlists, $residue_name, $shift_name); }
	}
      }
    }
  elsif ($keep_sequence)
    { 
    foreach my $entity_id (keys %{$bmrb_hlists[0]{"entity_list"}})
      {
      $$master_bmrb_hlist{"entity_list"}{$entity_id}{"name_array"} = $bmrb_hlists[0]{"entity_list"}{$entity_id}{"name_array"}; 
      $$master_bmrb_hlist{"entity_list"}{$entity_id}{"sequence"} = $bmrb_hlists[0]{"entity_list"}{$entity_id}{"sequence"}; 
      $$master_bmrb_hlist{"entity_list"}{$entity_id}{"entity_id"} = $entity_id;
      }
    }
  
  for(my $x = 0; $x < @bmrb_hlists; $x++)
    {
    foreach my $entity_id (keys %{$bmrb_hlists[$x]{"entity_list"}})
      {
      foreach my $residue_name (keys %{$bmrb_hlists[$x]{"entity_list"}{$entity_id}{"rlist"}})
	{
	foreach my $shift_name (keys %{$bmrb_hlists[$x]{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"shifts"}})
	  {
	  my $shift = $bmrb_hlists[$x]{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"shifts"}{$shift_name};
	  foreach my $shift_value (@{$$shift{"list"}})
	    {
	    if (! exists $$shift{"used"}{$shift_value} && 
		&test_shift_value(\@bmrb_hlists, $bmrb_hlists[$x], $entity_id, $residue_name, $shift_name, $shift_value) )
	      {
	      $$master_bmrb_hlist{"entity_list"}{$entity_id}{"entity_id"} = $entity_id; 
	      $$master_bmrb_hlist{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"name"} = $residue_name;
	      $$master_bmrb_hlist{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"aa"} = $bmrb_hlists[$x]{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"aa"};
	      $$master_bmrb_hlist{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"index"} = $bmrb_hlists[$x]{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"index"};
	      if (exists $bmrb_hlists[$x]{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"author_index"})
		{ $$master_bmrb_hlist{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"author_index"} = $bmrb_hlists[$x]{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"author_index"}; }
	      $$master_bmrb_hlist{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"shifts"}{$shift_name}{"atom_type"} = $$shift{"atom_type"};
	      if (! exists $$master_bmrb_hlist{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"shifts"}{$shift_name}{"list"})
		{ $$master_bmrb_hlist{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"shifts"}{$shift_name}{"list"} = []; }
	      push @{$$master_bmrb_hlist{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"shifts"}{$shift_name}{"list"}}, $shift_value;
	      }
	    
	    $$shift{"used"}{$shift_value} = 1;
	    }
	  }
	}
      }
    }
  }

# delete name array so that it can be rebuilt
if (! $keep_sequence)
  { 
  foreach my $entity_id (keys %{$$master_bmrb_hlist{"entity_list"}})
    {
    delete $$master_bmrb_hlist{"entity_list"}{$entity_id}{"name_array"}; 
    delete $$master_bmrb_hlist{"entity_list"}{$entity_id}{"sequence"}; 
    }
  }

if ($demultiplicate)
  { &deMultiplicate($master_bmrb_hlist); }

# print the master bmrb file
&write_bmrb_file($master_bmrb_hlist, $output_filename);




# test_shift_value 
#   returns true if the shift_value is unique or consistent.
#
#  Parameters:
#	$residue_ahlists - array of residue_hlist.
#       $current_hlist - current residue_hlist.
#       $residue_name - current residue name.
#	$shift_name - current shift name.
#	$shift_value - current shift value.
sub test_shift_value
  {
  my $residue_ahlists = shift @_;
  my $current_hlist = shift @_;
  my $entity_id = shift @_;
  my $residue_name = shift @_;
  my $shift_name = shift @_;
  my $shift_value = shift @_;

  
  foreach my $residue_hlist (@{$residue_ahlists})
    {
    next if ($$residue_hlist{"filename"} eq $$current_hlist{"filename"});
    
    if (exists $$residue_hlist{"entity_list"}{$entity_id} && exists $$residue_hlist{"entity_list"}{$entity_id}{"rlist"}{$residue_name} && exists $$residue_hlist{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"shifts"}{$shift_name})
      {
      my $shift = $$residue_hlist{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"shifts"}{$shift_name};
      my @shift_values = grep { abs($_ - $shift_value) <= ($tolerances{$$shift{"atom_type"}} + 0.00000000001);  } @{$$shift{"list"}};
      if (@shift_values)
	{
	foreach my $value (@shift_values)
	  { $$shift{"used"}{$value} = 1; }
	}
      else 
	{ return 0; }
      }
    }

  return 1;
  }


# mark_matching_shifts 
#   Mark shift_values that have matching name in master list.
#
#  Parameters:
#	$residue_ahlists - array of residue_hlist.
#       $residue_name - current residue name.
#	$shift_name - current shift name.
sub mark_matching_shifts 
  {
  my $residue_ahlists = shift @_;
  my $entity_id = shift @_;
  my $residue_name = shift @_;
  my $shift_name = shift @_;
  
  foreach my $residue_hlist (@{$residue_ahlists})
    {
    if (exists $$residue_hlist{"entity_list"}{$entity_id} && exists $$residue_hlist{"entity_list"}{$entity_id}{"rlist"}{$residue_name} && exists $$residue_hlist{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"shifts"}{$shift_name})
      {
      my $shift = $$residue_hlist{"entity_list"}{$entity_id}{"rlist"}{$residue_name}{"shifts"}{$shift_name};
      my @shift_values = @{$$shift{"list"}};
      if (@shift_values)
	{
	foreach my $value (@shift_values)
	  { $$shift{"used"}{$value} = 1; }
	}
      }
    }
  }

