#!/usr/bin/perl
#!/usr/bin/perl
#  createSparkyNOESYPks.pl
#	Create a variety of virtual Sparky NOESY peak lists from resonance assignments in bmrb format.
#
#  usage: createSparkyNOESYPks.pl input_bmrb_file output_peak_file [options]
#
#		input_bmrb_file - bmrb file to read ("-" to read from standard input).
#		output_peak_file - output file to write to ("-" to write to standard output).
#		options:
#			-n15 - generate peak list for N15-edited NOESY.
#			-c13 - generate peak list for C13-edited NOESY.
#			-both - generate peak list for combined N15-C13-edited NOESY.
#			-4D - generate 4D version of peak list.
#			-local_connectivity - limit peaks to 2- and 3-bond local connectivity peaks.
#			-residue num - limit to i +- num residue connectivity peaks.
#			-aromatic - include aromatic resonances.
#			-diagonal - include diagonal peaks.
#			-degenerate - include peaks with degenerate resonances.
#
#  Hint: Sparky reads them in using the rp command.
#
#  Copyright Hunter Moseley, 2004. All rights reserved.
#  Written by Hunter Moseley 8/4/2004
#
#
#  Updated to use new BMRBParsing/CMapParsing modules.
#  Copyright Hunter Moseley 2005. All rights reserved.
#

# These modules allow me to use modules rather than hardcoding everything into one script
use FindBin;
use lib $FindBin::Bin;

# Parser module parses in/out the cmap data and the peak file
use BMRBParsing qw(:ALL);

# Use the strict module: This is off because of silly bareword problems with the initialize_matirices
 use strict;

if ((@ARGV < 2) || ($ARGV[0] =~ /^-h/))
  {
  print STDERR "Error: not enough parameters given.\n\n" if (@ARGV < 3);
  print STDERR "createSparkyNOESYPks.pl\n";
  print STDERR "	Create a variety of virtual Sparky NOESY peak lists from resonance assignments in bmrb format.\n";
  print STDERR "\n";
  print STDERR "  usage: createSparkyNOESYPks.pl input_bmrb_file output_peak_file [options]\n";
  print STDERR "\n";
  print STDERR "		input_bmrb_file - bmrb file to read (\"-\" to read from standard input).\n";
  print STDERR "		output_peak_file - output file to write to (\"-\" to write to standard output).\n";
  print STDERR "		options:\n";
  print STDERR "			-n15 - generate peak list for N15-edited NOESY.\n";
  print STDERR "			-c13 - generate peak list for C13-edited NOESY.\n";
  print STDERR "			-both - generate peak list for combined N15-C13-edited NOESY.\n";
  print STDERR "			-4D - generate 4D peak list.\n";
  print STDERR "			-local_connectivity - limit peaks to 2- and 3-bond local connectivity peaks.\n";
  print STDERR "			-residue num - limit to i +- num residue connectivity peaks.\n";
  print STDERR "			-aromatic - include aromatic resonances.\n";
  print STDERR "			-diagonal - include diagonal peaks.\n";
  print STDERR "			-degenerate - include peaks with degenerate resonances.\n";
  print STDERR "\n";
  print STDERR "  Hint: Sparky reads them in using the rp command.\n";
  print STDERR "\n";
 
  exit(1);
  }

# read command line options
my $infile = shift @ARGV;
my $outfile = shift @ARGV;
my $peak_list_type = "n15";
my $fourD = 0;
my $limit_local = 0;
my $aromatic = 0;
my $diagonal = 0;
my $peak_resonance_hash = {};
my $limit_residue = 0;

while (@ARGV && ($ARGV[0] =~ /^\-/))
  {
  my $switch = shift @ARGV;
  if ($switch =~ /^\-n/)
    { $peak_list_type = "n15"; }
  elsif ($switch =~ /^\-c/)
    { $peak_list_type = "c13"; }
  elsif ($switch =~ /^\-b/)
    { $peak_list_type = "both"; }
  elsif ($switch =~ /^\-4/)
    { $fourD = 1; }
  elsif ($switch =~ /^\-l/)
    { $limit_local = 1; }
  elsif ($switch =~ /^\-r/)
    { $limit_residue = shift @ARGV; }
  elsif ($switch =~ /^\-a/)
    { $aromatic = 1; }
  elsif ($switch =~ /^\-di/)
    { $diagonal = 1; }
  elsif ($switch =~ /^\-de/)
    { $peak_resonance_hash = 0; }
  else
    { 
    print STDERR "INVALID OPTION: $switch\n\nExiting Program ...\n\n";  
    exit(1);
    }
  }

# read bmrb file
my $bmrb_hlist = &read_bmrb_file($infile,{});


my $index = 1;
# open Sparky Assigned Peak list file
local *SPARKY_FILE;
if ($outfile eq "-")
  { *SPARKY_FILE = *STDOUT; }
else
  { open(SPARKY_FILE, ">$outfile") || die ("open " . $outfile . " failed."); }


# initialize connectivity lists
my ($one_bond_hlist, $local_peaks, $hydrogens) = &initialize_connectivity_lists($peak_list_type, $fourD, $aromatic, $diagonal);



# only create peaks for local NOE connectivity
if ($limit_local)
  {
  # loop through all residues looking for peaks
  foreach my $entity (values %{$$bmrb_hlist{"entity_list"}})
    {
    foreach my $residue ( @{$$entity{"name_array"}} )
      {
      foreach my $peakdef (@{$$local_peaks{$$entity{"rlist"}{$residue}{"aa"}}})
	{
	if ((grep { (exists $$entity{"rlist"}{$residue}{"shifts"}{$_}) && (@{$$entity{"rlist"}{$residue}{"shifts"}{$_}{"list"}}); } (@$peakdef)) == @$peakdef)
	  { &print_single_NOE_peak([ map { [$residue, $_, $entity]; } (@$peakdef) ], $bmrb_hlist, $peak_resonance_hash); }
	}
      }
    }
  }
# create all possible peaks
else 
  { 
  # loop through all residues looking for peaks
  foreach my $entity1 (values %{$$bmrb_hlist{"entity_list"}})
    {
    foreach my $residue1 ( @{$$entity1{"name_array"}} )
      {
      foreach my $one_bond1 (@{$$one_bond_hlist{$$entity1{"rlist"}{$residue1}{"aa"}}})
	{
	if ((grep { (exists $$entity1{"rlist"}{$residue1}{"shifts"}{$_}) && (@{$$entity1{"rlist"}{$residue1}{"shifts"}{$_}{"list"}}); } (@$one_bond1)) == @$one_bond1)
	  {
	  foreach my $entity2 (values %{$$bmrb_hlist{"entity_list"}})
	    {
	    next if ($limit_residue && $entity1 != $entity2);

	    foreach my $residue2 ( @{$$entity2{"name_array"}} )
	      {
	      next if ($limit_residue && (abs($$entity1{"rlist"}{$residue1}{"index"} - $$entity2{"rlist"}{$residue2}{"index"}) > $limit_residue));
	      
	      if ($fourD)
		{
		foreach my $one_bond2 (@{$$one_bond_hlist{$$entity2{"rlist"}{$residue2}{"aa"}}})
		  {
		  next if (! $diagonal && ($residue1 eq $residue2) && ($one_bond1 == $one_bond2) && ($entity1 == $entity2));

		  if ((grep { (exists $$entity2{"rlist"}{$residue2}{"shifts"}{$_}) && (@{$$entity2{"rlist"}{$residue2}{"shifts"}{$_}{"list"}}); } (@$one_bond2)) == @$one_bond2)
		    { &print_single_NOE_peak([ (map { [$residue1, $_, $entity1]; } (@$one_bond1)), (map { [$residue2, $_, $entity2]; } (reverse @$one_bond2)) ], $bmrb_hlist, $peak_resonance_hash); }
		  }
		}
	      else # handle 3D case
		{
		foreach my $atom (@{$$hydrogens{$$entity2{"rlist"}{$residue2}{"aa"}}})
		  {
		  next if (! $diagonal && ($residue1 eq $residue2) && ($atom eq $$one_bond1[0]) && ($entity1 == $entity2));

		  if (exists $$entity2{"rlist"}{$residue2}{"shifts"}{$atom} && @{$$entity2{"rlist"}{$residue2}{"shifts"}{$atom}{"list"}})
		    { &print_single_NOE_peak([ (map { [$residue1, $_, $entity1]; } (@$one_bond1)), [$residue2, $atom, $entity2] ], $bmrb_hlist, $peak_resonance_hash); }
		  }
		}
	      }
	    }
	  }
	}
      }
    }
  }

# close the peak list file
if ($outfile ne "-")
  {
  close(SPARKY_FILE);
  }



#
# Subroutines
#


# print_single_NOE_peak
#   Print a single NOE peak in Sparky format.  Assumes that SPARKY_FILE is defined.
#
# Parameters:
#   $shift_alist - description of peak to print.
#   $bmrb_hlist - ref to hash structure of the bmrb file.
#   $peak_resonance_hash - ref to hash of peak resonances 
#
sub print_single_NOE_peak
  {
  my $shift_alist = shift @_;
  my $bmrb_hlist = shift @_;
  my $peak_resonance_hash = shift @_;

  my @count_array = map { 0; } (@$shift_alist);
  while($count_array[$#$shift_alist] < @{$$shift_alist[$#$shift_alist][2]{"rlist"}{$$shift_alist[$#$shift_alist][0]}{"shifts"}{$$shift_alist[$#$shift_alist][1]}{"list"}})
    {
    my $peak_resonance_name;
    for(my $x=0; $x < @$shift_alist; $x++)
      { $peak_resonance_name .= "_" . $$shift_alist[$x][2]{"rlist"}{$$shift_alist[$x][0]}{"shifts"}{$$shift_alist[$x][1]}{"list"}[$count_array[$x]]; }

    # print peak if resonance degeneracy is allowed or it has not been printed before.
    if (! $peak_resonance_hash || ! $$peak_resonance_hash{$peak_resonance_name})
      {
      print SPARKY_FILE $$shift_alist[0][0], $$shift_alist[0][1];
      for(my $x=1; $x < @$shift_alist; $x++)
	{ print SPARKY_FILE "-", $$shift_alist[$x][0], $$shift_alist[$x][1]; }
      
      print SPARKY_FILE " ";
      for(my $x=0; $x < @$shift_alist; $x++)
	{ print SPARKY_FILE "\t", $$shift_alist[$x][2]{"rlist"}{$$shift_alist[$x][0]}{"shifts"}{$$shift_alist[$x][1]}{"list"}[$count_array[$x]]; }
      print SPARKY_FILE "\n";
      }

    if ($peak_resonance_hash)
      { $$peak_resonance_hash{$peak_resonance_name} = 1; }

    $count_array[0]++;
    for (my $x=0; $x < @$shift_alist - 1; $x++)
      {
      if ($count_array[$x] >= @{$$shift_alist[$x][2]{"rlist"}{$$shift_alist[$x][0]}{"shifts"}{$$shift_alist[$x][1]}{"list"}})
	{ 
	$count_array[$x] = 0;
	$count_array[$x+1]++;
	}
      }
    }

  return;
  }


# initialize_connectivity_lists
#   Creates one_bond and local NOESY connectivity lists.
#
# Parameters:
#   $peak_list_type - peak list type of the NOESY.
#   $fourD - boolean of if NOESY is 4D.
#   $aromatic - boolean of if to include aromatic resonances. 
#   $diagonal - boolean of if to include diagoanl peaks.
#
sub initialize_connectivity_lists
  {
  my $peak_list_type = shift @_;
  my $fourD = shift @_;
  my $aromatic = shift @_;
  my $diagonal = shift @_;

  my $n15_local_peaks = {};
  $$n15_local_peaks{A} =  [ [ "H", "N", "N", "H" ], [ "H", "N", "CA", "HA" ] ];
  $$n15_local_peaks{R} =  [ [ "H", "N", "N", "H" ], [ "H", "N", "CA", "HA" ], [ "HE", "NE", "NE", "HE" ], 
			   [ "HE", "NE", "CD", "HD2" ], [ "HE", "NE", "CD", "HD3" ], 
			   [ "HH11", "NH1", "NH1", "HH11" ], [ "HH11", "NH1", "NH1", "HH12" ], [ "HH12", "NH1", "NH1", "HH11" ], [ "HH12", "NH1", "NH1", "HH12" ],
			   [ "HH21", "NH2", "NH2", "HH21" ], [ "HH21", "NH2", "NH2", "HH22" ], [ "HH22", "NH2", "NH2", "HH21" ], [ "HH22", "NH2", "NH2", "HH22" ] ];
  $$n15_local_peaks{D} =  [ [ "H", "N", "N", "H" ], [ "H", "N", "CA", "HA" ] ];
  $$n15_local_peaks{N} =  [ [ "H", "N", "N", "H" ], [ "H", "N", "CA", "HA" ], 
			   [ "HD21", "ND2", "ND2", "HD21" ], [ "HD21", "ND2", "ND2", "HD22" ], [ "HD22", "ND2", "ND2", "HD21" ], [ "HD22", "ND2", "ND2", "HD22" ] ];
  $$n15_local_peaks{C} =  [ [ "H", "N", "N", "H" ], [ "H", "N", "CA", "HA" ] ];
  $$n15_local_peaks{E} =  [ [ "H", "N", "N", "H" ], [ "H", "N", "CA", "HA" ] ];
  $$n15_local_peaks{Q} =  [ [ "H", "N", "N", "H" ], [ "H", "N", "CA", "HA" ],
			   [ "HE21", "NE2", "NE2", "HE21" ], [ "HE21", "NE2", "NE2", "HE22" ], [ "HE22", "NE2", "NE2", "HE21" ], [ "HE22", "NE2", "NE2", "HE22" ] ];
  $$n15_local_peaks{G} =  [ [ "H", "N", "N", "H" ], [ "H", "N", "CA", "HA2" ], [ "H", "N", "CA", "HA3" ] ];
  $$n15_local_peaks{H} =  [ [ "H", "N", "N", "H" ], [ "H", "N", "CA", "HA" ], 
			   [ "HD1", "ND1", "ND1", "HD1" ], [ "HD1", "ND1", "CE1", "HE1" ],
			   [ "HE2", "NE2", "NE2", "HE2" ], [ "HE2", "NE2", "CE1", "HE1" ], [ "HE2", "NE2", "CD2", "HD2" ] ];
  $$n15_local_peaks{I} =  [ [ "H", "N", "N", "H" ], [ "H", "N", "CA", "HA" ] ];
  $$n15_local_peaks{L} =  [ [ "H", "N", "N", "H" ], [ "H", "N", "CA", "HA" ] ];
  $$n15_local_peaks{K} =  [ [ "H", "N", "N", "H" ], [ "H", "N", "CA", "HA" ] ];
  $$n15_local_peaks{M} =  [ [ "H", "N", "N", "H" ], [ "H", "N", "CA", "HA" ] ];
  $$n15_local_peaks{F} =  [ [ "H", "N", "N", "H" ], [ "H", "N", "CA", "HA" ] ];
  $$n15_local_peaks{P} =  [  ];
  $$n15_local_peaks{S} =  [ [ "H", "N", "N", "H" ], [ "H", "N", "CA", "HA" ] ];
  $$n15_local_peaks{T} =  [ [ "H", "N", "N", "H" ], [ "H", "N", "CA", "HA" ] ];
  $$n15_local_peaks{W} =  [ [ "H", "N", "N", "H" ], [ "H", "N", "CA", "HA" ] ];
  if ($aromatic)
    { push @{$$n15_local_peaks{W}}, ( [ "HE1", "NE1", "NE1", "HE1" ], [ "HE1", "NE1", "CD1", "HD1" ] ); }
  $$n15_local_peaks{Y} =  [ [ "H", "N", "N", "H" ], [ "H", "N", "CA", "HA" ] ];
  $$n15_local_peaks{V} =  [ [ "H", "N", "N", "H" ], [ "H", "N", "CA", "HA" ] ];


  my $c13_local_peaks = {};
  $$c13_local_peaks{A} = [ [ "HA", "CA", "CA", "HA" ], [ "HA", "CA", "CB", "HB" ], [ "HB", "CB", "CA", "HA" ], [ "HB", "CB", "CB", "HB" ] ];
  $$c13_local_peaks{R} = [ [ "HA", "CA", "CA", "HA" ], [ "HA", "CA", "CB", "HB2" ], [ "HB2", "CB", "CA", "HA" ], [ "HB2", "CB", "CB", "HB2" ],
		     [ "HA", "CA", "CB", "HB3" ], [ "HB3", "CB", "CA", "HA" ], [ "HB3", "CB", "CB", "HB3" ], [ "HB2", "CB", "CB", "HB3" ],
		     [ "HB3", "CB", "CB", "HB2" ], 
		     [ "HB2", "CB", "CG", "HG2" ], [ "HG2", "CG", "CB", "HB2" ], [ "HG2", "CG", "CG", "HG2" ], 
		     [ "HB2", "CB", "CG", "HG3" ], [ "HG3", "CG", "CB", "HB2" ], [ "HG3", "CG", "CG", "HG3" ], 
		     [ "HB3", "CB", "CG", "HG2" ], [ "HG2", "CG", "CB", "HB3" ], 
		     [ "HB3", "CB", "CG", "HG3" ], [ "HG3", "CG", "CB", "HB3" ], 
		     [ "HG2", "CG", "CG", "HG3" ], [ "HG3", "CG", "CG", "HG2" ], 
		     [ "HG2", "CG", "CD", "HD2" ], [ "HD2", "CD", "CG", "HG2" ], [ "HD2", "CD", "CD", "HD2" ], 
		     [ "HG2", "CG", "CD", "HD3" ], [ "HD3", "CD", "CG", "HG2" ], [ "HD3", "CD", "CD", "HD3" ], 
		     [ "HG3", "CG", "CD", "HD2" ], [ "HD2", "CD", "CG", "HG3" ], 
		     [ "HG3", "CG", "CD", "HD3" ], [ "HD3", "CD", "CG", "HG3" ], 
		     [ "HD2", "CD", "CD", "HD3" ], [ "HD3", "CD", "CD", "HD2" ] ];
  $$c13_local_peaks{D} = [ [ "HA", "CA", "CA", "HA" ], [ "HA", "CA", "CB", "HB2" ], [ "HB2", "CB", "CA", "HA" ], [ "HB2", "CB", "CB", "HB2" ],
		     [ "HA", "CA", "CB", "HB3" ], [ "HB3", "CB", "CA", "HA" ], [ "HB3", "CB", "CB", "HB3" ], [ "HB2", "CB", "CB", "HB3" ],
		     [ "HB3", "CB", "CB", "HB2" ] ]; 
  $$c13_local_peaks{N} = [ [ "HA", "CA", "CA", "HA" ], [ "HA", "CA", "CB", "HB2" ], [ "HB2", "CB", "CA", "HA" ], [ "HB2", "CB", "CB", "HB2" ],
		     [ "HA", "CA", "CB", "HB3" ], [ "HB3", "CB", "CA", "HA" ], [ "HB3", "CB", "CB", "HB3" ], [ "HB2", "CB", "CB", "HB3" ],
		     [ "HB3", "CB", "CB", "HB2" ] ]; 
  $$c13_local_peaks{C} = [ [ "HA", "CA", "CA", "HA" ], [ "HA", "CA", "CB", "HB2" ], [ "HB2", "CB", "CA", "HA" ], [ "HB2", "CB", "CB", "HB2" ],
		     [ "HA", "CA", "CB", "HB3" ], [ "HB3", "CB", "CA", "HA" ], [ "HB3", "CB", "CB", "HB3" ], [ "HB2", "CB", "CB", "HB3" ],
		     [ "HB3", "CB", "CB", "HB2" ] ]; 
  $$c13_local_peaks{E} = [ [ "HA", "CA", "CA", "HA" ], [ "HA", "CA", "CB", "HB2" ], [ "HB2", "CB", "CA", "HA" ], [ "HB2", "CB", "CB", "HB2" ],
		     [ "HA", "CA", "CB", "HB3" ], [ "HB3", "CB", "CA", "HA" ], [ "HB3", "CB", "CB", "HB3" ], [ "HB2", "CB", "CB", "HB3" ],
		     [ "HB3", "CB", "CB", "HB2" ], 
		     [ "HB2", "CB", "CG", "HG2" ], [ "HG2", "CG", "CB", "HB2" ], [ "HG2", "CG", "CG", "HG2" ], 
		     [ "HB2", "CB", "CG", "HG3" ], [ "HG3", "CG", "CB", "HB2" ], [ "HG3", "CG", "CG", "HG3" ], 
		     [ "HB3", "CB", "CG", "HG2" ], [ "HG2", "CG", "CB", "HB3" ], 
		     [ "HB3", "CB", "CG", "HG3" ], [ "HG3", "CG", "CB", "HB3" ], 
		     [ "HG2", "CG", "CG", "HG3" ], [ "HG3", "CG", "CG", "HG2" ] ]; 
  $$c13_local_peaks{Q} = [ [ "HA", "CA", "CA", "HA" ], [ "HA", "CA", "CB", "HB2" ], [ "HB2", "CB", "CA", "HA" ], [ "HB2", "CB", "CB", "HB2" ],
		     [ "HA", "CA", "CB", "HB3" ], [ "HB3", "CB", "CA", "HA" ], [ "HB3", "CB", "CB", "HB3" ], [ "HB2", "CB", "CB", "HB3" ],
		     [ "HB3", "CB", "CB", "HB2" ], 
		     [ "HB2", "CB", "CG", "HG2" ], [ "HG2", "CG", "CB", "HB2" ], [ "HG2", "CG", "CG", "HG2" ], 
		     [ "HB2", "CB", "CG", "HG3" ], [ "HG3", "CG", "CB", "HB2" ], [ "HG3", "CG", "CG", "HG3" ], 
		     [ "HB3", "CB", "CG", "HG2" ], [ "HG2", "CG", "CB", "HB3" ], 
		     [ "HB3", "CB", "CG", "HG3" ], [ "HG3", "CG", "CB", "HB3" ], 
		     [ "HG2", "CG", "CG", "HG3" ], [ "HG3", "CG", "CG", "HG2" ] ]; 
  $$c13_local_peaks{G} = [ [ "HA2", "CA", "CA", "HA2" ], [ "HA2", "CA", "CA", "HA3" ], [ "HA3", "CA", "CA", "HA2" ], [ "HA3", "CA", "CA", "HA3" ] ];
  $$c13_local_peaks{H} = [ [ "HA", "CA", "CA", "HA" ], [ "HA", "CA", "CB", "HB2" ], [ "HB2", "CB", "CA", "HA" ], [ "HB2", "CB", "CB", "HB2" ],
		     [ "HA", "CA", "CB", "HB3" ], [ "HB3", "CB", "CA", "HA" ], [ "HB3", "CB", "CB", "HB3" ], [ "HB2", "CB", "CB", "HB3" ],
		     [ "HB3", "CB", "CB", "HB2" ] ];
  $$c13_local_peaks{I} = [ [ "HA", "CA", "CA", "HA" ], [ "HA", "CA", "CB", "HB" ], [ "HB", "CB", "CA", "HA" ], [ "HB", "CB", "CB", "HB" ],
		     [ "HB", "CB", "CG1", "HG12" ], [ "HG12", "CG1", "CB", "HB" ], [ "HG12", "CG1", "CG1", "HG12" ], 
		     [ "HB", "CB", "CG1", "HG13" ], [ "HG13", "CG1", "CB", "HB" ], [ "HG13", "CG1", "CG1", "HG13" ], 
		     [ "HG12", "CG1", "CG1", "HG13" ], [ "HG13", "CG1", "CG1", "HG12" ],
		     [ "HB", "CB", "CG2", "HG2" ], [ "HG2", "CG2", "CB", "HB" ], [ "HG2", "CG2", "CG2", "HG2" ], 
		     [ "HG12", "CG1", "CD1", "HD1" ], [ "HD1", "CD1", "CG1", "HG12" ], [ "HD1", "CD1", "CD1", "HD1" ], 
		     [ "HG13", "CG1", "CD1", "HD1" ], [ "HD1", "CD1", "CG1", "HG13" ] ]; 
  $$c13_local_peaks{L} = [ [ "HA", "CA", "CA", "HA" ], [ "HA", "CA", "CB", "HB2" ], [ "HB2", "CB", "CA", "HA" ], [ "HB2", "CB", "CB", "HB2" ],
		     [ "HA", "CA", "CB", "HB3" ], [ "HB3", "CB", "CA", "HA" ], [ "HB3", "CB", "CB", "HB3" ], [ "HB2", "CB", "CB", "HB3" ],
		     [ "HB3", "CB", "CB", "HB2" ], 
		     [ "HB2", "CB", "CG", "HG" ], [ "HG", "CG", "CB", "HB2" ], [ "HG", "CG", "CG", "HG" ], 
		     [ "HB3", "CB", "CG", "HG" ], [ "HG", "CG", "CB", "HB3" ],
		     [ "HG", "CG", "CD1", "HD1" ], [ "HD1", "CD1", "CG", "HG" ], [ "HD1", "CD1", "CD1", "HD1" ],
		     [ "HG", "CG", "CD2", "HD2" ], [ "HD2", "CD2", "CG", "HG" ], [ "HD2", "CD2", "CD2", "HD2" ] ];
  $$c13_local_peaks{K} = [ [ "HA", "CA", "CA", "HA" ], [ "HA", "CA", "CB", "HB2" ], [ "HB2", "CB", "CA", "HA" ], [ "HB2", "CB", "CB", "HB2" ],
		     [ "HA", "CA", "CB", "HB3" ], [ "HB3", "CB", "CA", "HA" ], [ "HB3", "CB", "CB", "HB3" ], [ "HB2", "CB", "CB", "HB3" ],
		     [ "HB3", "CB", "CB", "HB2" ], 
		     [ "HB2", "CB", "CG", "HG2" ], [ "HG2", "CG", "CB", "HB2" ], [ "HG2", "CG", "CG", "HG2" ], 
		     [ "HB2", "CB", "CG", "HG3" ], [ "HG3", "CG", "CB", "HB2" ], [ "HG3", "CG", "CG", "HG3" ], 
		     [ "HB3", "CB", "CG", "HG2" ], [ "HG2", "CG", "CB", "HB3" ], 
		     [ "HB3", "CB", "CG", "HG3" ], [ "HG3", "CG", "CB", "HB3" ], 
		     [ "HG2", "CG", "CG", "HG3" ], [ "HG3", "CG", "CG", "HG2" ], 
		     [ "HG2", "CG", "CD", "HD2" ], [ "HD2", "CD", "CG", "HG2" ], [ "HD2", "CD", "CD", "HD2" ], 
		     [ "HG2", "CG", "CD", "HD3" ], [ "HD3", "CD", "CG", "HG2" ], [ "HD3", "CD", "CD", "HD3" ], 
		     [ "HG3", "CG", "CD", "HD2" ], [ "HD2", "CD", "CG", "HG3" ], 
		     [ "HG3", "CG", "CD", "HD3" ], [ "HD3", "CD", "CG", "HG3" ], 
		     [ "HD2", "CD", "CD", "HD3" ], [ "HD3", "CD", "CD", "HD2" ],
		     [ "HD2", "CD", "CE", "HE2" ], [ "HE2", "CE", "CD", "HD2" ], [ "HE2", "CE", "CE", "HE2" ], 
		     [ "HD2", "CD", "CE", "HE3" ], [ "HE3", "CE", "CD", "HD2" ], [ "HE3", "CE", "CE", "HE3" ], 
		     [ "HD3", "CD", "CE", "HE2" ], [ "HE2", "CE", "CD", "HD3" ], 
		     [ "HD3", "CD", "CE", "HE3" ], [ "HE3", "CE", "CD", "HD3" ], 
		     [ "HE2", "CE", "CE", "HE3" ], [ "HE3", "CE", "CE", "HE2" ] ];
  $$c13_local_peaks{M} = [ [ "HA", "CA", "CA", "HA" ], [ "HA", "CA", "CB", "HB2" ], [ "HB2", "CB", "CA", "HA" ], [ "HB2", "CB", "CB", "HB2" ],
		     [ "HA", "CA", "CB", "HB3" ], [ "HB3", "CB", "CA", "HA" ], [ "HB3", "CB", "CB", "HB3" ], [ "HB2", "CB", "CB", "HB3" ],
		     [ "HB3", "CB", "CB", "HB2" ], 
		     [ "HB2", "CB", "CG", "HG2" ], [ "HG2", "CG", "CB", "HB2" ], [ "HG2", "CG", "CG", "HG2" ], 
		     [ "HB2", "CB", "CG", "HG3" ], [ "HG3", "CG", "CB", "HB2" ], [ "HG3", "CG", "CG", "HG3" ], 
		     [ "HB3", "CB", "CG", "HG2" ], [ "HG2", "CG", "CB", "HB3" ], 
		     [ "HB3", "CB", "CG", "HG3" ], [ "HG3", "CG", "CB", "HB3" ], 
		     [ "HG2", "CG", "CG", "HG3" ], [ "HG3", "CG", "CG", "HG2" ] ]; 
  $$c13_local_peaks{F} = [ [ "HA", "CA", "CA", "HA" ], [ "HA", "CA", "CB", "HB2" ], [ "HB2", "CB", "CA", "HA" ], [ "HB2", "CB", "CB", "HB2" ],
		     [ "HA", "CA", "CB", "HB3" ], [ "HB3", "CB", "CA", "HA" ], [ "HB3", "CB", "CB", "HB3" ], [ "HB2", "CB", "CB", "HB3" ],
		     [ "HB3", "CB", "CB", "HB2" ] ];
  if ($aromatic)
    { 
    push @{$$c13_local_peaks{F}}, ([ "HD1", "CD1", "CE1", "HE1" ], [ "HE1", "CE1", "CD1", "HD1" ], 
				   [ "HD2", "CD2", "CE2", "HE2" ], [ "HE2", "CE2", "CD2", "HD2" ],
				   [ "HE1", "CE1", "CZ", "HZ" ], [ "HZ", "CZ", "CE1", "HE1" ],
				   [ "HE2", "CE2", "CZ", "HZ" ], [ "HZ", "CZ", "CE2", "HE2" ] ) ; 
    }
  $$c13_local_peaks{P} = [ [ "HA", "CA", "CA", "HA" ], [ "HA", "CA", "CB", "HB2" ], [ "HB2", "CB", "CA", "HA" ], [ "HB2", "CB", "CB", "HB2" ],
		     [ "HA", "CA", "CB", "HB3" ], [ "HB3", "CB", "CA", "HA" ], [ "HB3", "CB", "CB", "HB3" ], [ "HB2", "CB", "CB", "HB3" ],
		     [ "HB3", "CB", "CB", "HB2" ], 
		     [ "HB2", "CB", "CG", "HG2" ], [ "HG2", "CG", "CB", "HB2" ], [ "HG2", "CG", "CG", "HG2" ], 
		     [ "HB2", "CB", "CG", "HG3" ], [ "HG3", "CG", "CB", "HB2" ], [ "HG3", "CG", "CG", "HG3" ], 
		     [ "HB3", "CB", "CG", "HG2" ], [ "HG2", "CG", "CB", "HB3" ], 
		     [ "HB3", "CB", "CG", "HG3" ], [ "HG3", "CG", "CB", "HB3" ], 
		     [ "HG2", "CG", "CG", "HG3" ], [ "HG3", "CG", "CG", "HG2" ], 
		     [ "HG2", "CG", "CD", "HD2" ], [ "HD2", "CD", "CG", "HG2" ], [ "HD2", "CD", "CD", "HD2" ], 
		     [ "HG2", "CG", "CD", "HD3" ], [ "HD3", "CD", "CG", "HG2" ], [ "HD3", "CD", "CD", "HD3" ], 
		     [ "HG3", "CG", "CD", "HD2" ], [ "HD2", "CD", "CG", "HG3" ], 
		     [ "HG3", "CG", "CD", "HD3" ], [ "HD3", "CD", "CG", "HG3" ], 
		     [ "HD2", "CD", "CD", "HD3" ], [ "HD3", "CD", "CD", "HD2" ] ];
  $$c13_local_peaks{S} = [ [ "HA", "CA", "CA", "HA" ], [ "HA", "CA", "CB", "HB2" ], [ "HB2", "CB", "CA", "HA" ], [ "HB2", "CB", "CB", "HB2" ],
		     [ "HA", "CA", "CB", "HB3" ], [ "HB3", "CB", "CA", "HA" ], [ "HB3", "CB", "CB", "HB3" ], [ "HB2", "CB", "CB", "HB3" ],
		     [ "HB3", "CB", "CB", "HB2" ] ];
  $$c13_local_peaks{T} = [ [ "HA", "CA", "CA", "HA" ], [ "HA", "CA", "CB", "HB" ], [ "HB", "CB", "CA", "HA" ], [ "HB", "CB", "CB", "HB" ],
		     [ "HB", "CB", "CG2", "HG2" ], [ "HG2", "CG2", "CB", "HB" ], [ "HG2", "CG2", "CG2", "HG2" ] ];
  $$c13_local_peaks{W} = [ [ "HA", "CA", "CA", "HA" ], [ "HA", "CA", "CB", "HB2" ], [ "HB2", "CB", "CA", "HA" ], [ "HB2", "CB", "CB", "HB2" ],
		     [ "HA", "CA", "CB", "HB3" ], [ "HB3", "CB", "CA", "HA" ], [ "HB3", "CB", "CB", "HB3" ], [ "HB2", "CB", "CB", "HB3" ],
		     [ "HB3", "CB", "CB", "HB2" ] ];
  if ($aromatic)
    { 
    push @{$$c13_local_peaks{W}}, ([ "HE3", "CE3", "CZ3", "HZ3" ], [ "HZ3", "CZ3", "CE3", "HE3" ], 
				   [ "HZ3", "CZ3", "CH2", "HH2" ], [ "HH2", "CH2", "CZ3", "HZ3" ],
				   [ "HH2", "CH2", "CZ2", "HZ2" ], [ "HZ2", "CZ2", "CH2", "HH2" ] ) ; 
    }
  $$c13_local_peaks{Y} = [ [ "HA", "CA", "CA", "HA" ], [ "HA", "CA", "CB", "HB2" ], [ "HB2", "CB", "CA", "HA" ], [ "HB2", "CB", "CB", "HB2" ],
		     [ "HA", "CA", "CB", "HB3" ], [ "HB3", "CB", "CA", "HA" ], [ "HB3", "CB", "CB", "HB3" ], [ "HB2", "CB", "CB", "HB3" ],
		     [ "HB3", "CB", "CB", "HB2" ] ];
  if ($aromatic)
    { 
    push @{$$c13_local_peaks{Y}}, ([ "HD1", "CD1", "CE1", "HE1" ], [ "HE1", "CE1", "CD1", "HD1" ], 
				   [ "HD2", "CD2", "CE2", "HE2" ], [ "HE2", "CE2", "CD2", "HD2" ] ) ; 
    }
  $$c13_local_peaks{V} = [ [ "HA", "CA", "CA", "HA" ], [ "HA", "CA", "CB", "HB" ], [ "HB", "CB", "CA", "HA" ], [ "HB", "CB", "CB", "HB" ],
		     [ "HB", "CB", "CG1", "HG1" ], [ "HG1", "CG1", "CB", "HB" ], [ "HG1", "CG1", "CG1", "HG1" ],
		     [ "HB", "CB", "CG2", "HG2" ], [ "HG2", "CG2", "CB", "HB" ], [ "HG2", "CG2", "CG2", "HG2" ] ];
  

  my $local_peaks = {};
  if ($peak_list_type eq "n15")
    { $local_peaks = $n15_local_peaks; }
  elsif ($peak_list_type eq "c13")
    { $local_peaks = $c13_local_peaks; }
  elsif ($peak_list_type eq "both")
    {
    foreach my $aa_alist (values %$n15_local_peaks)
      {
      # duplicate n15 local peak base in reverse atom order
      my $unique_hlist = {};
      foreach my $peak (@$aa_alist)
	{ 
	$$unique_hlist{ join("",@$peak) } = $peak; 
	my $new_peak = [];

	# create reverse of the peak
	for(my $x = $#$peak; $x >=0; $x--)
	  { push @$new_peak, $$peak[$x]; }
	
	$$unique_hlist{ join("",@$new_peak) } = $new_peak; 	
	}
      $aa_alist = [ (values %$unique_hlist) ];
      }

    # combine n15 and c13 local peak bases
    $local_peaks = $c13_local_peaks;
    foreach my $aa (keys %$local_peaks)
      { push @{$$local_peaks{$aa}}, @{$$n15_local_peaks{$aa}}; }
    }

  # create one_bond list
  my $one_bond_hlist = {};
  foreach my $aa (keys %$local_peaks)
    {
    my $unique_hlist = {};
    foreach my $peak (@{$$local_peaks{$aa}})
      {
      my $one_bond = [ $$peak[0], $$peak[1] ];
      $$unique_hlist{ join("",@$one_bond) } = $one_bond;
      }
    
    $$one_bond_hlist{$aa} = [ (values %$unique_hlist) ];
    }

  # delete diagonal peaks
  if (! $diagonal)
    {
    foreach my $aa_alist (values %$local_peaks)
      {
      # delete diagonal peaks
      $aa_alist = [ grep { $$_[0] ne $$_[$#$_]; } (@$aa_alist) ];
      }
    }

  # delete 2nd middle carbon if limited to 3D
  if (! $fourD)
    {
    foreach my $aa_alist (values %$local_peaks)
      {
      foreach my $peak (@$aa_alist)
	{ splice (@$peak,2,1); }

      # delete duplicates that arise from hydrogen deletion
      my $unique_hlist = {};
      foreach my $peak (@$aa_alist)
	{ $$unique_hlist{ join("",@$peak) } = $peak; }
      $aa_alist = [ (values %$unique_hlist) ];
      }
    }


  my $hydrogens = 
    { 
    A  =>  ["H", "HA", "HB"],
    R  =>  ["H", "HA", "HB2", "HB3", "HG2", "HG3", "HD2", "HD3", "HE", "HH11", "HH12", "HH21", "HH22"],
    N  =>  ["H", "HA", "HB2", "HB3", "HD21", "HD22"],
    D  =>  ["H", "HA", "HB2", "HB3"],
    C  =>  ["H", "HA", "HB2", "HB3", "HG"],
    Q  =>  ["H", "HA", "HB2", "HB3", "HG2", "HG3", "HE21", "HE22"],
    E  =>  ["H", "HA", "HB2", "HB3", "HG2", "HG3"],
    G  =>  ["H", "HA2", "HA3"],
    H  =>  ["H", "HA", "HB2", "HB3", "HD1", "HD2", "HE1", "HE2"],
    I  =>  ["H", "HA", "HB", "HG12", "HG13", "HG2", "HD1"],
    L  =>  ["H", "HA", "HB2", "HB3", "HG", "HD1", "HD2"],
    K  =>  ["H", "HA", "HB2", "HB3", "HG2", "HG3", "HD2", "HD3", "HE2", "HE3", "HZ"],
    M  =>  ["H", "HA", "HB2", "HB3", "HG2", "HG3", "HE"],
    F  =>  ["H", "HA", "HB2", "HB3", "HD1", "HD2", "HE1", "HE2", "HZ"],
    P  =>  ["HA", "HB2", "HB3", "HG2", "HG3", "HD2", "HD3"],
    S  =>  ["H", "HA", "HB2", "HB3", "HG"],
    T  =>  ["H", "HA", "HB", "HG1", "HG2"],
    W  =>  ["H", "HA", "HB2", "HB3", "HD1", "HE1", "HE3", "HZ2", "HZ3", "HH2"],
    Y  =>  ["H", "HA", "HB2", "HB3", "HD1", "HD2", "HE1", "HE2", "HH"],
    V  =>  ["H", "HA", "HB", "HG1", "HG2"]
      };

  if (! $aromatic)
    {
    foreach my $aa ("F", "Y", "W")
      { $$hydrogens{$aa} = ["H", "HA", "HB2", "HB3"]; }
    }

  return ($one_bond_hlist, $local_peaks, $hydrogens);
  }
