#!/usr/bin/perl
#  createSparkyPeakList.pl
#	Create specific Sparky peak list from assigned chemical shifts in bmrb file.
#
#  usage: createSparkyPeakList.pl input_bmrb_file output_peak_file peak_list_type
#
#		input_bmrb_file - bmrb file to read ("-" to read from standard input).
#		output_peak_file - output file to write to ("-" to write to standard output).
#		peak_list_type - type of peak list.  Known peak list types are listed below:
#
#  Hint: Sparky reads them in using the rp command.
#
#  Copyright Hunter Moseley, 2007. All rights reserved.
#  Written by Hunter Moseley 08/21/2001
#  Used bmrbACS2SparkyPks.pl as a template, its copyrights apply.
#
#

# These modules allow me to use modules rather than hardcoding everything into one script
use FindBin;
use lib $FindBin::Bin;

# Parser module parses in/out the cmap data and the peak file
use BMRBParsing qw(:ALL);

# Use the strict module: This is off because of silly bareword problems with the initialize_matirices
 use strict;

my %peak_list_definitions;

$peak_list_definitions{"NH-HSQC"} = [ ["N", "H"], 
				     ["NE", "HE"], ["NH1", "HH11"], ["NH1", "HH12"], ["NH2", "HH21"], ["NH2", "HH22"], 
				     ["ND2", "HD21"], ["ND2", "HD22"], 
				     ["NE2", "HE21"], ["NE2", "HE22"],
				     ["ND1", "HD1"], ["NE2", "HE2"],
				     ["NE1", "HE1"] 
				     ];

$peak_list_definitions{"CH-HSQC"} = [ ["CA", "HA"], ["CA", "HA2"], ["CA", "HA3"],
				     ["CB", "HB"], ["CB", "HB2"], ["CB", "HB3"],
				     ["CG", "HG"], ["CG", "HG2"], ["CG", "HG3"], ["CG1", "HG12"], ["CG1", "HG13"], ["CG2", "HG2"], ["CG1", "HG1"],
				     ["CD", "HD2"], ["CD", "HD3"], ["CD1", "HD1"], ["CD2", "HD2"], 
				     ["CE", "HE2"], ["CE", "HE3"], ["CE1", "HE1"], ["CE2", "HE2"],
				     ["CZ", "HZ"], ["CZ2", "HZ2"], ["CZ3", "HZ3"],
				     ["CH2", "HH2"] 
				     ];

$peak_list_definitions{"CNH-HSQC"} = [ @{$peak_list_definitions{"NH-HSQC"}}, @{$peak_list_definitions{"CH-HSQC"}} ];


$peak_list_definitions{"C-NOESY"} = [ ["HA", "CA", "HA"], ["HA", "CA-1", "HA-1"], ["H", "CA-1", "HA-1"], ["HA2", "CA-1", "HA-1"], ["HA3", "CA-1", "HA-1"],
				     ["H", "CB-1", "HB3-1"], ["H", "CB-1", "HB2-1"], ["H", "CB-1", "HB-1"],
				     ["H", "CG-1", "HG2-1"], ["H", "CG-1", "HG3-1"], ["H", "CG-1", "HG-1"],
				     ["HA2", "CA", "HA2"], ["HA3", "CA", "HA3"], ["H", "CA", "HA"], ["H-1", "CA", "HA"],
				     ["HB", "CA", "HA"], ["HB2", "CA", "HA"], ["HB3", "CA", "HA"],
				     ["HG2", "CA", "HA"], ["HG3", "CA", "HA"], ["HG", "CA", "HA"],
				     ["HG12", "CA", "HA"], ["HG13", "CA", "HA"], ["HG1", "CA", "HA"],
				     ["HD1", "CA", "HA"], ["HD2", "CA", "HA"], ["HD3", "CA", "HA"],
				     ["HE2", "CA", "HA"], ["HE3", "CA", "HA"],
				     ["HB", "CB", "HB"], ["HB2", "CB", "HB2"], ["HB3", "CB", "HB3"],

				     ["H", "CB", "HB"], ["H-1", "CB", "HB"], ["H", "CB", "HB2"], ["H", "CB", "HB3"], ["H-1", "CB", "HB3"], ["H-1", "CB", "HB2"],
				     ["HA", "CB", "HB"], ["HA", "CB", "HB2"], ["HA", "CB", "HB3"],
				     ["HG1", "CB", "HB"], ["HG2", "CB", "HB"], ["HG3", "CB", "HB2"], ["HG3", "CB", "HB3"], ["HG2", "CB", "HB2"], ["HG2", "CB", "HB3"],
				     ["HG12", "CB", "HB"], ["HG13", "CB", "HB"], 
				     ["HD1", "CB", "HB"], ["HD2", "CB", "HB2"], ["HD2", "CB", "HB3"], ["HD3", "CB", "HB2"], ["HD3", "CB", "HB3"],
				     ["HD21", "CB", "HB2"], ["HD22", "CB", "HB2"], ["HD21", "CB", "HB3"], ["HD22", "CB", "HB3"], ["HD1", "CB", "HB2"], ["HD1", "CB", "HB3"],
				     ["HB3", "CB", "HB2"], ["HB2", "CB", "HB3"],

				     ["H", "CG", "HG"], ["H", "CG", "HG2"], ["H", "CG", "HG3"],
				     ["H", "CG2", "HG2"],
				     ["H-1", "CG2", "HG2"],
				     ["HA", "CG", "HG"], ["HA", "CG", "HG2"], ["HA", "CG", "HG3"],
				     ["HA", "CG2", "HG2"],
				     ["HB2", "CG", "HG"], ["HB3", "CG", "HG"],
				     ["HB2", "CG", "HG3"], ["HB3", "CG", "HG2"], ["HB3", "CG", "HG3"], ["HB2", "CG", "HG2"], 
				     ["HB", "CG2", "HG2"],
				     ["HG1", "CG2", "HG2"], ["HG2", "CG", "HG3"], ["HG3", "CG", "HG2"], ["HG3", "CG", "HG3"], ["HG2", "CG", "HG2"],
				     ["HG2", "CG1", "HG1"], ["HG2", "CG2", "HG2"], ["HG1", "CG1", "HG1"],
				     ["HG12", "CG2", "HG2"], ["HG13", "CG2", "HG2"],
				     ["HG2", "CG1", "HG12"], ["HG2", "CG1", "HG13"],
				     ["HG12", "CG1", "HG12"], ["HG13", "CG1", "HG13"], ["HG12", "CG1", "HG13"], ["HG13", "CG1", "HG12"],
				     ["HD1", "CG", "HG"], ["HD2", "CG", "HG"], ["HD2", "CG", "HG2"], ["HD3", "CG", "HG2"], ["HD2", "CG", "HG3"], ["HD3", "CG", "HG3"],
				     ["HD1", "CG2", "HG2"],
				     ["HD1", "CG1", "HG12"], ["HD1", "CG1", "HG13"],
				     ["HE", "CG", "HG2"], ["HE", "CG", "HG3"],
				     ["HE21", "CG", "HG2"], ["HE22", "CG", "HG2"], ["HE21", "CG", "HG3"], ["HE22", "CG", "HG3"],

				     ["H", "CD1", "HD1"], ["H-1", "CD1", "HD1"], 
				     ["HA", "CD1", "HD1"], ["HA", "CD2", "HD2"],
				     ["HB", "CD1", "HD1"], ["HB2", "CD1", "HD1"], ["HB2", "CD2", "HD2"], ["HB3", "CD1", "HD1"], ["HB3", "CD2", "HD2"],
				     ["HB3", "CD", "HD3"], ["HB2", "CD", "HD3"], ["HB3", "CD", "HD2"], ["HB2", "CD", "HD2"],
				     ["HG", "CD1", "HD1"], ["HG", "CD2", "HD2"],
				     ["HG3", "CD", "HD3"], ["HG2", "CD", "HD3"], ["HG3", "CD", "HD2"], ["HG2", "CD", "HD2"],
				     ["HG12", "CD1", "HD1"], ["HG13", "CD1", "HD1"], ["HG2", "CD1", "HD1"],
				     ["HD1", "CD1", "HD1"], ["HD2", "CD1", "HD1"], ["HD1", "CD2", "HD2"], ["HD2", "CD2", "HD2"],
				     ["HD3", "CD", "HD3"], ["HD2", "CD", "HD2"], ["HD2", "CD", "HD3"], ["HD3", "CD", "HD2"],
				     ["HE1", "CD1", "HD1"],
				     ["HE3", "CD", "HD3"], ["HE2", "CD", "HD3"], ["HE3", "CD", "HD2"], ["HE2", "CD", "HD2"],
				     ["HZ", "CD1", "HD1"], ["HZ", "CD2", "HD2"],

				     ["HB2", "CE3", "HE3"], ["HB3", "CE3", "HE3"],
				     ["HG2", "CE", "HE2"], ["HG3", "CE", "HE2"], ["HG2", "CE", "HE3"], ["HG3", "CE", "HE3"],
				     ["HD2", "CE", "HE2"], ["HD3", "CE", "HE2"], ["HD2", "CE", "HE3"], ["HD3", "CE", "HE3"],
				     ["HD2", "CE2", "HE2"], ["HD1", "CE1", "HE1"],
				     ["HE2", "CE", "HE2"], ["HE3", "CE", "HE2"], ["HE2", "CE", "HE3"], ["HE3", "CE", "HE3"],
				     ["HE3", "CE3", "HE3"], 
				     ["HE1", "CE1", "HE1"], ["HE2", "CE2", "HE2"],
				     ["HZ3", "CE3", "HE3"],
				     ["HZ", "CE1", "HE1"], ["HZ", "CE2", "HE2"],

				     ["HD1", "CZ", "HZ"], ["HD2", "CZ", "HZ"],
				     ["HE1", "CZ", "HZ"], ["HE2", "CZ", "HZ"],
				     ["HE1", "CZ2", "HZ2"], ["HE3", "CZ3", "HZ3"],
				     ["HZ", "CZ", "HZ"],
				     ["HZ2", "CZ2", "HZ2"], ["HZ3", "CZ3", "HZ3"],
				     ["HH2", "CZ2", "HZ2"], ["HH2", "CZ3", "HZ3"],

				     ["HZ3", "CH2", "HH2"], ["HZ2", "CH2", "HH2"],
				     ["HH2", "CH2", "HH2"]
				     ];


$peak_list_definitions{"N-NOESY"} = [ ["H", "N", "H"], ["H-1", "N", "H"],
				     ["HA", "N", "H"], ["HA2", "N", "H"], ["HA3", "N", "H"],
				     ["HA-1", "N", "H"], ["HA2-1", "N", "H"], ["HA3-1", "N", "H"],
				     ["HB", "N", "H"], ["HB2", "N", "H"], ["HB3", "N", "H"],
				     ["HB-1", "N", "H"], ["HB2-1", "N", "H"], ["HB3-1", "N", "H"],
				     ["HG", "N", "H"], ["HG2", "N", "H"], ["HG3", "N", "H"], ["HG1", "N", "H"], ["HG12", "N", "H"], ["HG13", "N", "H"],
				     ["HD1", "N", "H"], ["HD2", "N", "H"], ["HD3", "N", "H"],
				     ["HD21", "N", "H"], ["HD22", "N", "H"],
				     ["HE", "N", "H"], ["HE1", "N", "H"], ["HE2", "N", "H"], ["HE3", "N", "H"],
				     ["HE21", "N", "H"], ["HE22", "N", "H"],

				     ["H", "N-1", "H-1"],
				     ["HA", "N-1", "H-1"], ["HA2", "N-1", "H-1"], ["HA3", "N-1", "H-1"],
				     ["HB", "N-1", "H-1"], ["HB2", "N-1", "H-1"], ["HB3", "N-1", "H-1"],

				     ["H", "ND2", "HD21"], ["H", "ND2", "HD22"],
				     ["HA", "ND2", "HD22"], ["HA", "ND2", "HD21"],
				     ["HB1", "ND2", "HD21"], ["HB2", "ND2", "HD22"], ["HB1", "ND2", "HD22"], ["HB2", "ND2", "HD21"],
				     ["HD21", "ND2", "HD21"], ["HD22", "ND2", "HD22"], ["HD21", "ND2", "HD22"], ["HD22", "ND2", "HD21"],

				     ["H", "NE2", "HE21"], ["H", "NE2", "HE22"],
				     ["H", "NE1", "HE1"],
				     ["HA", "NE", "HE"],
				     ["HA", "NE2", "HE21"], ["HA", "NE2", "HE22"],
				     ["HG3", "NE2", "HE21"], ["HG2", "NE2", "HE22"], ["HG3", "NE2", "HE22"], ["HG2", "NE2", "HE21"],
				     ["HE", "NE", "HE"],
				     ["HE21", "NE2", "HE21"], ["HE22", "NE2", "HE22"], ["HE21", "NE2", "HE22"], ["HE22", "NE2", "HE21"],
				     ["HD2", "NE", "HE"], ["HD3", "NE", "HE"],
				     ];

$peak_list_definitions{"CN-NOESY"} = [ @{$peak_list_definitions{"N-NOESY"}}, @{$peak_list_definitions{"C-NOESY"}} ];


if ((@ARGV < 3) || ($ARGV[0] =~ /^-h/))
  {
  print STDERR "Error: not enough parameters given.\n\n" if (@ARGV < 3);
  print STDERR "createSparkyPeakList.pl\n";
  print STDERR "	Create specific Sparky peak list from assigned chemical shifts in bmrb file.\n";
  print STDERR "\n";
  print STDERR "  usage: createSparkyPeakList.pl input_bmrb_file output_peak_file peak_list_type\n";
  print STDERR "\n";
  print STDERR "		input_bmrb_file - bmrb file to read (\"-\" to read from standard input).\n";
  print STDERR "		output_peak_file - output file to write to (\"-\" to write to standard output).\n";
  print STDERR "		peak_list_type - type of peak list.  Known peak list types are listed below:\n";
  print STDERR join("\n",map {"\t\t\t" . $_; } (keys %peak_list_definitions)),"\n";
  print STDERR "\n";
  print STDERR "  Hint: Sparky reads them in using the rp command.\n";
  print STDERR "\n";
 
  exit(1);
  }

# read command line options
my $infile = shift @ARGV;
my $outfile = shift @ARGV;
my $peak_list_type = shift @ARGV;
my $peakdefinitions = 0;
foreach my $type (keys %peak_list_definitions)
  {
  if ($type eq $peak_list_type)
    {
    $peakdefinitions = $peak_list_definitions{$type};
    last;
    }
  }

if (! $peakdefinitions)
  {
  print STDERR "Error: unknown peak list type \"$peak_list_type\" given.\n";
  exit(1);
  }


# read bmrb file
my $bmrb_hlist = &read_bmrb_file($infile,{});


# open Sparky Assigned Peak list file
local *SPARKY_FILE;
if ($outfile eq "-")
  { *SPARKY_FILE = *STDOUT; }
else
  { open(SPARKY_FILE, ">$outfile") || die ("open " . $outfile . " failed."); }


# loop through all residues looking for peaks
foreach my $entity (values %{$$bmrb_hlist{"entity_list"}})
  {
  foreach my $residue ( @{$$entity{"name_array"}} )
    {	
    foreach my $peakdef (@$peakdefinitions)
      {
      my @shift_ahlist = ();
      
      # check for proper shift lists
      # this comment means that we make sure that every atomType (H,N,CA) exists for the residue so that it
      # is actually a peak.
      
      # iterate through each element in the peak definition (H,N,CA)
      for (my $count = 0; $count < @$peakdef; $count++)
	{
	# set your current atomType    
	my $atomType = $$peakdef[$count];
	
	# GSS 04/14/2001 add $ at end of regexp of atomType so that HA-12 will be a stupid and incorrect name,
	# but will be a current residue's atomType name
	
	# if you have a previous tag on the atomType and the previous residue is defined ...
	if (($atomType =~ /-1$/) && (defined $$entity{"rlist"}{$residue}{"prev"}))
	  {
	  my $prevresidue = $$entity{"rlist"}{$residue}{"prev"};
	  
	  # GSS 04/14/2001 deleted [A-Z]+ and replaced with \w+ because HG1-1 should be a valid atomType in CMAP naming scheme
	  # we get the name of the atomType without the previous tag and we should beable to read as pure BMRB ...
	  my ($true_atomType) = ($atomType =~ /^(\w+)\-1$/);
	  
	  # and see if it is defined as a standard atomType on the previous residue
	  if ( (exists $$entity{"rlist"}{$prevresidue}{"shifts"}{$true_atomType}{"list"}) && (@{$$entity{"rlist"}{$prevresidue}{"shifts"}{$true_atomType}{"list"}}))
	    { 
	    $shift_ahlist[$count] = { "atomType" => $true_atomType, "list" => $$entity{"rlist"}{$prevresidue}{"shifts"}{$true_atomType}{"list"}, "curr" => 0, "residue" => $prevresidue, "name" => $prevresidue };
	    if (exists $$entity{"rlist"}{$prevresidue}{"author_name"})
	      { $shift_ahlist[$count]{"name"} = $$entity{"rlist"}{$prevresidue}{"author_name"}; }
	    }
	  else
	    { last; }
	  }
	
	# otherwise, you are working with an atomType for the current residue.
	else
	  {
	  # you see if the atomType is defined for the current residue
	  if ( (exists $$entity{"rlist"}{$residue}{"shifts"}{$atomType}{"list"}) && (@{$$entity{"rlist"}{$residue}{"shifts"}{$atomType}{"list"}}))
	    { 
	    $shift_ahlist[$count] = { "atomType" => $atomType, "list" => $$entity{"rlist"}{$residue}{"shifts"}{$atomType}{"list"}, "curr" => 0, "residue" => $residue, "name" => $residue };
	    if (exists $$entity{"rlist"}{$residue}{"author_name"})
	      { $shift_ahlist[$count]{"name"} = $$entity{"rlist"}{$residue}{"author_name"}; }	      
	    }
	  else
	    { last; }
	  }
	}
      
      # start creating peaks if the proper shift lists are there.
      if (@shift_ahlist == @$peakdef)
	{
	while($shift_ahlist[$#shift_ahlist]{"curr"} < @{$shift_ahlist[$#shift_ahlist]{"list"}})
	  {
	  # write out peak
	  my $curr_residue = $shift_ahlist[0]{"residue"};
	  print SPARKY_FILE $shift_ahlist[0]{"name"},$shift_ahlist[0]{"atomType"};
	  for(my $count=1; $count < @shift_ahlist; $count++)
	    {
	    print SPARKY_FILE "-";
	    if ($curr_residue ne $shift_ahlist[$count]{"residue"})
	      { 
	      $curr_residue = $shift_ahlist[$count]{"residue"};
	      print SPARKY_FILE $shift_ahlist[$count]{"name"};
	      }
	    print SPARKY_FILE $shift_ahlist[$count]{"atomType"}; 
	    }
	  
	  print SPARKY_FILE " ";
	  for(my $count=0; $count < @shift_ahlist; $count++)
	    { print SPARKY_FILE "\t",$shift_ahlist[$count]{"list"}[$shift_ahlist[$count]{"curr"}]; }
	  
	  print SPARKY_FILE "\n";
	  
	  # increment curr counters in shift_ahlist.
	  # the curr counter tags allow us to have a list of resonances for each peak type and then 
	  # using the same logic as the recombine subroutine, we iterate until we have permuted through
	  # the entire list.
	  $shift_ahlist[0]{"curr"}++;
	  for (my $count = 0; $count < @$peakdef; $count++)
	    {
	    if (($shift_ahlist[$count]{"curr"} >= @{$shift_ahlist[$count]{"list"}}) && ($count+1 < @$peakdef))
	      { 
	      $shift_ahlist[$count]{"curr"} = 0;
	      $shift_ahlist[$count+1]{"curr"}++;
	      }
	    }
	  }
	}      
      }
    }
  }

