#!/usr/bin/perl
#  create_bmrb.pl
#	Converts a list of shift assignments in 3 column format into a BMRB file.
#
#  usage: create_bmrb.pl input_assignment_file output_bmrb_file sequence_start sequence [options]
#       
#	input_assignment_file - name of assignment file or "-" for standard input.
#		assignment file format:  ResidueName   AtomType    ShiftValue    [AmbiguityCode]
#			ResidueName - Residue followed by its position in the sequence  (ie. A32, G53).
#			AtomType - Type of Atom using BMRB conventions (ie. CA, CB, CG, HA1, HB3, HG22).
#			ShiftValue - The chemical shift value   (ie. 120.6 4.43, 3.84, 3.21).
#			AmbiguityCode - (optional) The ambiguity code (ie. 1 2 3 4 5 9).
#	output_bmrb_file - name of the output BMRB file or  "-" for standard output.
#	sequence_start - the starting residue number of the sequence.
#	sequence - the protein sequence using single letter codes.
#	options:
#		-diasterio - give diasteriomer resonancess an ambiguity code of 2 and aromatic ring resonances an ambiguity code of 3.
#		-unique_methyls - do not label the V and L methyls with an ambiguity code of 2.
#		-index index - assignment file uses the following indexing.
#		-clean - remove unknown atom types.
#		-alt - assignment file format is as follows:
#			ResidueNumber - residue position in the sequence (ie. 32, 53).
#			ResidueType - residue type (ie. A or Ala).
#			AtomType - Type of Atom using BMRB conventions (ie. CA, CB, CG, HA1, HB3, HG22).
#			ShiftValue - The chemical shift value   (ie. 120.6 4.43, 3.84, 3.21).
#			AmbiguityCode - (optional) The ambiguity code (ie. 1 2 3 4 5 9); a dot is intepreted as blank.
#		-bmrb2 - assignment file format is in bmrb 2.1 format; just the chemical shift assignment lines.
#
#
#
#   Written by Hunter Moseley 02/19/2001
#   Copyright:  Hunter Moseley, 2001. All rights reserved.
#
#   Modified by Hunter Moseley 3/19/2002
#
#
#  Updated to use new BMRBParsing/CMapParsing modules.
#  Copyright Hunter Moseley 2005. All rights reserved.
#

# These modules allow me to use modules rather than hardcoding everything into one script
use FindBin;
use lib $FindBin::Bin;

# Parser module parses in/out the cmap data and the peak file
use BMRBParsing qw(:ALL);
use strict;


# necessary conversion hash
my %aa_name_conversion = ( "ala" => "A", "arg" => "R", "asn" => "N", "asp" => "D",
			   "cys" => "C", "gln" => "Q", "glu" => "E", "gly" => "G",
			   "his" => "H", "ile" => "I", "leu" => "L", "lys" => "K",
			   "met" => "M", "phe" => "F", "pro" => "P", "ser" => "S",
			   "thr" => "T", "trp" => "W", "tyr" => "Y", "val" => "V",
			   "a"   => "A", "r"   => "R", "n"   => "N", "d"   => "D", 
			   "c"   => "C", "q"   => "Q", "e"   => "E", "g"   => "G",
			   "h"   => "H", "i"   => "I", "l"   => "L", "k"   => "K",
			   "m"   => "M", "f"   => "F", "p"   => "P", "s"   => "S",
			   "t"   => "T", "w"   => "W", "y"   => "Y", "v"   => "V"); 


# variables pertaining to the 3 column assignment file format.
my $ASSIGNMENT = 0;
my $RESIDUE_TYPE = -1;
my $RESIDUE_NUMBER = -1;
my $ATOMTYPE = 1;
my $SHIFT = 2;
my $AMBIGUITY = 3;

# list needed to set ambiguity code of diasteriomers.
my %diasteriomer_hlist = ( "R" => { "HB2" => 2, "HB3" => 2, "HG2" => 2, "HG3" => 2, "HD2" => 2, "HD3" => 2, "HH11" => 2, "HH12" => 2, "HH21" => 2, "HH22" => 2, "NH1" => 2, "NH2" => 2 },
			   "D" => { "HB2" => 2, "HB3" => 2 },
			   "N" => { "HB2" => 2, "HB3" => 2, "HD21" => 2, "HD22" => 2 },
			   "C" => { "HB2" => 2, "HB3" => 2 },
			   "E" => { "HB2" => 2, "HB3" => 2, "HG2" => 2, "HG3" => 2 },
			   "Q" => { "HB2" => 2, "HB3" => 2, "HG2" => 2, "HG3" => 2, "HE21" => 2, "HE22" => 2 },
			   "G" => { "HA2" => 2, "HA3" => 2 },
			   "H" => { "HB2" => 2, "HB3" => 2 },
			   "I" => { "HG12" => 2, "HG13" => 2 },
			   "L" => { "HB2" => 2, "HB3" => 2, "HD1" => 2, "HD2" => 2, "CD1" => 2, "CD2" => 2 },
			   "K" => { "HB2" => 2, "HB3" => 2, "HG2" => 2, "HG3" => 2, "HD2" => 2, "HD3" => 2, "HE2" => 2, "HE3" => 2 },
			   "M" => { "HB2" => 2, "HB3" => 2, "HG2" => 2, "HG3" => 2 },
			   "F" => { "HB2" => 2, "HB3" => 2, "HD1" => 3, "HD2" => 3, "CD1" => 3, "CD2" => 3, "HE1" => 3, "HE2" => 3, "CE1" => 3, "CE2" => 3 },
			   "P" => { "HB2" => 2, "HB3" => 2, "HG2" => 2, "HG3" => 2, "HD2" => 2, "HD3" => 2 },
			   "S" => { "HB2" => 2, "HB3" => 2 },
			   "W" => { "HB2" => 2, "HB3" => 2 },
			   "Y" => { "HB2" => 2, "HB3" => 2, "HD1" => 3, "HD2" => 3, "CD1" => 3, "CD2" => 3, "HE1" => 3, "HE2" => 3, "CE1" => 3, "CE2" => 3 },
			   "V" => { "HG1" => 2, "HG2" => 2, "CG1" => 2, "CG2" => 2 }
			   );

if ((@ARGV < 4) || ($ARGV[0] =~ /^-h/))
  {
  print "\nError: not enough parameters given.\n\n" if ($ARGV[0] !~ /^-h/);
  print "create_bmrb.pl\n";
  print "	Converts a list of shift assignments in 3 column format into a BMRB file.\n";
  print "\n";
  print "  usage: create_bmrb.pl input_assignment_file output_bmrb_file sequence_start sequence [options]\n";
  print "       \n";
  print "	input_assignment_file - name of assignment file or \"-\" for standard input.\n";
  print "		assignment file format:  ResidueName   AtomType    ShiftValue    [AmbiguityCode]\n";
  print "			ResidueName - Residue followed by its position in the sequence  (ie. A32, G53).\n";
  print "			AtomType - Type of Atom using BMRB conventions (ie. CA, CB, CG, HA1, HB3, HG22).\n";
  print "			ShiftValue - The chemical shift value   (ie. 120.6 4.43, 3.84, 3.21).\n";
  print "			AmbiguityCode - (optional) The ambiguity code (ie. 1 2 3 4 5 9).\n";
  print "	output_bmrb_file - name of the output BMRB file or  \"-\" for standard output.\n";
  print "	sequence_start - the starting residue number of the sequence.\n";
  print "	sequence - the protein sequence using single letter codes.\n";
  print "	options:\n";
  print "		-diasterio - give diasteriomer resonancess an ambiguity code of 2 and aromatic ring resonances an ambiguity code of 3.\n";
  print "		-unique_methyls - do not label the V and L methyls with an ambiguity code of 2.\n";
  print "		-index index - assignment file uses the following indexing.\n";
  print "		-clean - remove unknown atom types.\n";
  print "		-alt - assignment file format is as follows:\n";
  print "			ResidueNumber - residue position in the sequence (ie. 32, 53).\n";
  print "			ResidueType - residue type (ie. A or Ala).\n";
  print "			AtomType - Type of Atom using BMRB conventions (ie. CA, CB, CG, HA1, HB3, HG22).\n";
  print "			ShiftValue - The chemical shift value   (ie. 120.6 4.43, 3.84, 3.21).\n";
  print "			AmbiguityCode - (optional) The ambiguity code (ie. 1 2 3 4 5 9); a dot is intepreted as blank.\n";
  print "		-bmrb2 - assignment file format is in bmrb 2.1 format; just the chemical shift assignment lines.\n";
  print " \n";
  exit(1);
  }

# Read in command line parameters
my $input_assignment_file = shift @ARGV;
my $output_bmrb_file = shift @ARGV;
my $sequence_startpos = shift @ARGV;
my $sequence_string = shift @ARGV;
my $diasteriomer = 0;
my $use_alt_index = 0;
my $alt_index;
my $min_cols = 3;
my $clean = 0;

while (@ARGV && ($ARGV[0] =~ /^\-/))
  {
  my $switch = shift @ARGV;
  if ($switch =~ /^\-di/)
    { $diasteriomer = 1; }
  elsif ($switch =~ /^\-a/)
    { 
    $ASSIGNMENT = -1;
    $RESIDUE_NUMBER = 0;
    $RESIDUE_TYPE = 1;
    $ATOMTYPE = 2;
    $SHIFT = 3;
    $AMBIGUITY = 4;
    $min_cols = 4;
    }
  elsif ($switch =~ /^\-b/)
    { 
    $ASSIGNMENT = -1;
    $RESIDUE_NUMBER = 1;
    $RESIDUE_TYPE = 2;
    $ATOMTYPE = 3;
    $SHIFT = 5;
    $AMBIGUITY = 7;
    $min_cols = 8;
    }
  elsif ($switch =~ /^\-c/)
    { $clean = 1; }
  elsif ($switch =~ /^\-u/)
    {
    $diasteriomer_hlist{"V"} = {};
    $diasteriomer_hlist{"L"} = { "HB2" => 2, "HB3" => 2 };
    }
  elsif ($switch =~ /^\-i/)
    { 
    $use_alt_index = 1;
    $alt_index = shift @ARGV;
    }
  else
    { 
    print STDERR "INVALID OPTION: $switch\n\nExiting Program ...\n\n";  
    exit(1);
    }
  }

die "No sequence given.\n" if ($sequence_string eq "");

# attempt to open assignment file
local *ASSIGNMENT_FILE;
if ($input_assignment_file eq "-")
  { *ASSIGNMENT_FILE = *STDIN; }
else
  { open(ASSIGNMENT_FILE, $input_assignment_file) || die ("open " . $input_assignment_file . " failed."); }


my %author_mapping_hlist;
my $bmrb_hlist = {};
$$bmrb_hlist{"filename"} = $output_bmrb_file;

# create name_array and sequence fields
$$bmrb_hlist{"entity_list"}{1}{"entity_id"} = 1;
$$bmrb_hlist{"entity_list"}{1}{"sequence_startpos"} = $sequence_startpos;
$sequence_string =~ s/\s+//g;
my @sequence = split '', $sequence_string;
$$bmrb_hlist{"entity_list"}{1}{"sequence"} = $sequence_string;
my $sequence_pos = $sequence_startpos;

my $conversion_pos = $sequence_startpos;
if ($use_alt_index)
  { $conversion_pos = $alt_index; }

$$bmrb_hlist{"entity_list"}{1}{"name_array"} = [];
for(my $count = 0; $count < @sequence; $count++)
  {
  # There are no 0's in the sequence_position
  if ($sequence_pos == 0)
    { $sequence_pos++; }
  if ($conversion_pos == 0)
    { $conversion_pos++; }

  my $residue_name = $sequence[$count] . ($count + 1);
  $author_mapping_hlist{$sequence[$count] . $conversion_pos} = $residue_name;
  push @{$$bmrb_hlist{"entity_list"}{1}{"name_array"}}, $residue_name;
  $$bmrb_hlist{"entity_list"}{1}{"rlist"}{$residue_name}{"aa"} = $sequence[$count];
  $$bmrb_hlist{"entity_list"}{1}{"rlist"}{$residue_name}{"name"} = $residue_name;
  $$bmrb_hlist{"entity_list"}{1}{"rlist"}{$residue_name}{"index"} = $count+1;
  $$bmrb_hlist{"entity_list"}{1}{"rlist"}{$residue_name}{"author_index"} = $sequence_pos;
  $$bmrb_hlist{"entity_list"}{1}{"rlist"}{$residue_name}{"author_name"} = $sequence[$count] . $sequence_pos;

  $sequence_pos++;  
  $conversion_pos++;  
  }

while (my $line = <ASSIGNMENT_FILE>)
  {
  chomp $line;

  next if ($line =~ /^\#/);  # skip comments

  $line =~ s/^\s*//;
  next if ($line eq ""); # skip blank lines.
  next if ($line !~ /\d+/); # skip lines without numbers on them.

  my @line_elems = split /\s+/, $line;
  next if (@line_elems < $min_cols); # skip lines not having enough columns.

  my $aa;
  my $author_residue_number;
  if ($ASSIGNMENT != -1)
    {
    $aa = ($line_elems[$ASSIGNMENT] =~ /^([A-Za-z]+)\-?\d+$/)[0];
    $author_residue_number = ($line_elems[$ASSIGNMENT] =~ /^[A-Za-z]+(\-?\d+)$/)[0];
    }
  else
    {
    $aa = $line_elems[$RESIDUE_TYPE];
    $author_residue_number = $line_elems[$RESIDUE_NUMBER];
    }

  if (exists $aa_name_conversion{lc($aa)})
    { $aa = $aa_name_conversion{lc($aa)}; }
  my $author_residue_name = $aa . $author_residue_number;
  my $residue_name = $author_mapping_hlist{$author_residue_name};

  next if ($clean && ! &is_known_atom_name($aa,$line_elems[$ATOMTYPE]));

  $$bmrb_hlist{"entity_list"}{1}{"rlist"}{$residue_name}{"shifts"}{$line_elems[$ATOMTYPE]}{"atom_type"} = ($line_elems[$ATOMTYPE] =~ /^([HNC])/)[0];

  if (@line_elems > 3)
    { $$bmrb_hlist{"entity_list"}{1}{"rlist"}{$residue_name}{"shifts"}{$line_elems[$ATOMTYPE]}{"ambiguity_code"} = $line_elems[$AMBIGUITY]; }
  elsif ($diasteriomer && exists $diasteriomer_hlist{$aa} && exists $diasteriomer_hlist{$aa}{$line_elems[$ATOMTYPE]})
    { $$bmrb_hlist{"entity_list"}{1}{"rlist"}{$residue_name}{"shifts"}{$line_elems[$ATOMTYPE]}{"ambiguity_code"} = $diasteriomer_hlist{$aa}{$line_elems[$ATOMTYPE]}; }
  else
    { $$bmrb_hlist{"entity_list"}{1}{"rlist"}{$residue_name}{"shifts"}{$line_elems[$ATOMTYPE]}{"ambiguity_code"} = "1"; }

  push @{$$bmrb_hlist{"entity_list"}{1}{"rlist"}{$residue_name}{"shifts"}{$line_elems[$ATOMTYPE]}{"list"}}, $line_elems[$SHIFT];
  push @{$$bmrb_hlist{"entity_list"}{1}{"rlist"}{$residue_name}{"shifts"}{$line_elems[$ATOMTYPE]}{"list_id_list"}}, 1;
  }

close ASSIGNMENT_FILE;


#create next and prev fields
for(my $x=0; $x < @{$$bmrb_hlist{"entity_list"}{1}{"name_array"}}; $x++)
  {
  my $resname = $$bmrb_hlist{"entity_list"}{1}{"name_array"}[$x];
  
  if ($x)
    { $$bmrb_hlist{"entity_list"}{1}{"rlist"}{$resname}{"prev"} = $$bmrb_hlist{"entity_list"}{1}{"name_array"}[$x-1]; }
  if ($x < (@{$$bmrb_hlist{"entity_list"}{1}{"name_array"}} - 1))
    { $$bmrb_hlist{"entity_list"}{1}{"rlist"}{$resname}{"next"} = $$bmrb_hlist{"entity_list"}{1}{"name_array"}[$x+1]; }
  }


# write bmrb file
&write_bmrb_file($bmrb_hlist, $output_bmrb_file);
