#!/usr/bin/perl
#!/usr/bin/perl 
#
# create_split_doublet.pl
#	Create split doublet from central peak with splittings dimension.
# 
# usage: create_split_doublet.pl central_peaklist split_peaklist workbook_name split_dim
#
#	central_peaklist - peak list of central peaks with splittings in last dimension.
#	split_peaklist - peak list with split doublet.
#	workbook_name - string to use as workbook name.
#	split_dim - dimension to split.
#
# Hint: Useful to create peaklist to load into Sparky for assigning split doublets.
#
# Copyright Aneerban Bhattacharya, 2001. All rights reserved.
# Written by Aneerban Bhattacharya; Jul 8, 2001
# Modified by Aneerban Bhattacharya     Feb 16, 2002
#

use strict;

# Print usage information
if ((scalar(@ARGV) < 4) || ($ARGV[0] =~ /^-h/)) {
	print STDERR "\nError: not enough parameters given.\n" if ($ARGV[0] !~ /^-h/);
	print STDERR "\n";
        print STDERR " create_split_doublet.pl\n";
        print STDERR "	Create split doublet from central peak with splittings dimension.\n";
        print STDERR " \n";
        print STDERR " usage: create_split_doublet.pl central_peaklist split_peaklist workbook_name split_dim\n";
        print STDERR "\n";
        print STDERR "	central_peaklist - peak list of central peaks with splittings in last dimension.\n";
        print STDERR "	split_peaklist - peak list with split doublet.\n";
        print STDERR "	workbook_name - string to use as workbook name.\n";
        print STDERR "	split_dim - dimension to split.\n";
        print STDERR "\n";
        print STDERR " Hint: Useful to create peaklist to load into Sparky for assigning split doublets.\n";
	print("\n");
	exit(1);
}


my ($infile_s, $outfile_s, $workbook_name_s, $split_col_n) = @ARGV;
$split_col_n = int($split_col_n) - 1; # 1Dim is column 0 and so on

# get list of peak lines, and end coment lines
my ($in_peak_lines_alist, $end_comments_alist) = &read_peak_file($infile_s);

my $num_cols_n = scalar(@{$$in_peak_lines_alist[0]}); # number of columns in peak list
my %assigns_h; # to store the list of split peaks and difference assigned to particular residue
# structure: assigns_h -> resonance_s -> { peak dimensions { dim 1, dim 2, ... }, splitting-value, notes } 
# order of dimensions for the peaks must be in the SAME, for all peaks being put into the list

foreach my $peak_line_alist (@$in_peak_lines_alist) {
	$$peak_line_alist[$num_cols_n - 1] =~ /.*\.([A-Z]\d+)\S*$/; # extract the residue number and type
	my $residue_s = $1;
	
	# currently not being used
	my ($notes_s) = ($$peak_line_alist[$num_cols_n - 1] =~ /[^.]*[.](.*)/);
	
	my @peak_dims_a; # stores each dimension of this peak
	# assuming penultimate column reserved for intensity
	for (my $colnum_n = 1; $colnum_n < ($num_cols_n - 3); $colnum_n++) { # splitting value in last *Dim
		push(@peak_dims_a, $$peak_line_alist[$colnum_n]);
	}
	
	if (!exists ($assigns_h{$residue_s})) {
		$assigns_h{$residue_s} = [  [ @peak_dims_a ] ];
		push(@{$assigns_h{$residue_s}}, $notes_s, $$peak_line_alist[($num_cols_n - 3)]);
	}
	else {
		print("Error: > 1 central peak for $residue_s\n");
	}
}														

# simulate the upfield and downfield split peak for each central peak
# output the splittings for each residue
open(OF, ">$outfile_s") || die("Can't open $outfile_s : $!");
print(OF "#Index\t");
for (my $dim_n = 1; $dim_n < ($num_cols_n - 3); $dim_n++) { # 1 column being removed
		    print(OF "$dim_n", "Dim\t");
}   
print(OF "Intensity\tWorkbook\n");
close(OF);

my $residue_s;
my $index_n = 0;
foreach $residue_s (sort keys(%assigns_h)) {
	# first the upfield peak
	my $resonance_s = $residue_s . "up"; # identifier for upfield peak
	$assigns_h{$residue_s}[0][$split_col_n] += $assigns_h{$residue_s}[2];
	$index_n++;
	
	open(OF, ">>$outfile_s") || die("Can't open $outfile_s : $!");
    my $for_print_split_col_n = $split_col_n + 1; # for print output
	print(OF "$index_n\t");
	print(OF join("\t", @{$assigns_h{$residue_s}[0]}));
	print(OF "\t3000");
	print(OF "\t$workbook_name_s" . "_dim" . "$for_print_split_col_n\_" . $assigns_h{$residue_s}[1] . "\.$resonance_s\n");
#	close(OF);
	
	# now the downfield peak
	$resonance_s = $residue_s . "down"; # identifier for downfield peak
	$assigns_h{$residue_s}[0][$split_col_n] -= (2 *$assigns_h{$residue_s}[2]); # to compensate for previous addition of splitting-value
	$index_n++;				
	
#	open(OF, ">>$outfile_s") || die("Can't open $outfile_s : $!");
#	my $for_print_split_col_n = $split_col_n + 1; # for print output
	print(OF "$index_n\t");         
	print(OF join("\t", @{$assigns_h{$residue_s}[0]}));             
	print(OF "\t3000");
	print(OF "\t$workbook_name_s" . "_dim" . "$for_print_split_col_n\_" . $assigns_h{$residue_s}[1] . "\.$resonance_s\n");
	close(OF);          
}						

# print comments
open(OF, ">>$outfile_s") || die("Can't open $outfile_s : $!");

# modified from AutoPeak script written by Hunter Moseley
print(OF "*\n\n");
print(OF "\nFile History: $outfile_s \n\n");
print(OF $0, " ", join(' ',@ARGV),"\n");
print(OF "\n");

my $line_s;
my $history_found_b = 0;
foreach $line_s (@{$end_comments_alist}) {
	if ((!$history_found_b) && ($line_s =~ /^File History:/)) {
#       print(OF "File History: $outfile_s \n\n");
#       print(OF $0, " ", join(' ', @ARGV), "\n");
		$history_found_b = 1;
		next;
	}
	
	print(OF "$line_s");
}
print("\n");

#if (!$history_found_b) {
#   print(OF "\nFile History: $outfile_s \n\n");
#   print(OF $0, " ", join(' ',@ARGV),"\n");
#}

close(OF);


# Read in peaks from AutoPeak format files - modified from the subroutine written by Hunter Moseley
sub read_peak_file {
	my ($filename) = @_;
	
	local *PEAKFILE;
	if ($filename eq "-") {
		*PEAKFILE = *STDIN;
	}
	else {
		open(PEAKFILE, "<$filename") || die("Open " . $filename . " failed : $!");
	}
	
	my $peak_alist = []; # list of lines containing peak info (each line stored as an array of columns)
	my $line_s;
	while ($line_s = <PEAKFILE>) {
		next if ($line_s =~ /^\#/); # skip comment lines
		last if ($line_s =~ /^[*]/); # end loop if end_of_file marker is encountered.
		
		($line_s) = ($line_s =~/^\s*(.*)/); # skip leading white space.
		chomp($line_s);
		push(@$peak_alist, [ split(/\s+/, $line_s) ]);
	}
	
	my $comment_alist = []; # list of comment lines - History section
	while ($line_s = <PEAKFILE>) {
		push(@$comment_alist, $line_s);
	}
	close(PEAKFILE);
	
	return wantarray ? ($peak_alist, $comment_alist) : $peak_alist;
}
