#!/usr/bin/perl
#
#     dyanaUPL2cmap.pl
#               Converts the backbone NOE constraints from a DYANA UPL constraint file to the CMap format.
#              
#                usage: dyanaUPL2cmap.pl [options] input_dyana_file output_cmap_file
#                              input_dyana_file  - NOE distance constraints in DYANA UPL format
#                              output_cmap_file  - NOE information in CMap format
#              
#                              options:
#		                   -append                     - append CMap file.
#		                   -intra                      - include intraresidue NOEs.
#				   -shift num                  - shift index by num.
#                                  -convert strong medium weak - the NOEs with distance < strong are +++ (default 3.5)
#                                                                    < medium are ++ (default 4.5) 
#                                                                    < weak are + (default 8.5).  
#                                                                   NOEs with distance >= weak are disregarded   - .
#
# Copyright Gurmukh Sahota 2001.
# 05/05/01
# All Rights Reserved.
#

# These modules allow me to use modules rather than hardcoding everything into one script
use FindBin;
use lib $FindBin::Bin;
use strict;

use DyanaParsing qw(:ALL);
use CMapParsing qw(:ALL);

my %Dyana2CMap_AT = (HA =>  "A",
		      HA1 => "A",
		      HA2 => "A",
		      HA3 => "A",
		      QA =>  "A",
		      QA1 => "A",
		      QA2 => "A",
		      QA3 => "A",
		      HB =>  "B",
		      HB1 => "B",
		      HB2 => "B",
		      HB3 => "B",
		      QB =>  "B",
		      QB1 => "B",
		      QB2 => "B",
		      QB3 => "B",
		      HN =>  "N",
                      H => "N");


my %aa_name_3_to_1 = ( "ala" => "A", "arg" => "R", "asn" => "N", "asp" => "D",
		       "cys" => "C", "gln" => "Q", "glu" => "E", "gly" => "G",
		       "his" => "H", "ile" => "I", "leu" => "L", "lys" => "K",
		       "met" => "M", "phe" => "F", "pro" => "P", "ser" => "S",
		       "thr" => "T", "trp" => "W", "tyr" => "Y", "val" => "V",
		       "a"   => "A", "r"   => "R", "n"   => "N", "d"   => "D",
		       "c"   => "C", "q"   => "Q", "e"   => "E", "g"   => "G",
		       "h"   => "H", "i"   => "I", "l"   => "L", "k"   => "K",
		       "m"   => "M", "f"   => "F", "p"   => "P", "s"   => "S",
		       "t"   => "T", "w"   => "W", "y"   => "Y", "v"   => "V");
if ((scalar(@ARGV) < 1) || ($ARGV[0] eq "-h"))
  {
  if (!($ARGV[0] =~ /^-h/))
    {
    print STDERR "\nError:: not enough parameters given.\n";
    print STDERR "\n";
    }
  &print_help;
}
my $strong = "3.5";
my $medium = "4.5";
my $weak = "8.5";
my $append_cmap = 0;
my $include_intra = 0;
my $index_shift = 0;
# parse optional command line arguments
while (($ARGV[0] =~ /^\-/) && (@ARGV > 2))
  {
    my $switch = shift @ARGV;
    if ($switch =~ /^\-c/)
      	{
	 $strong = shift @ARGV; 
	 $medium = shift @ARGV; 
	 $weak = shift @ARGV; 
	}
    elsif ($switch =~ /^\-a/)
      	{ $append_cmap = 1; }
    elsif ($switch =~ /^\-i/)
      	{ $include_intra = 1; }
    elsif ($switch =~ /^\-s/)
      	{ $index_shift = shift @ARGV; }
    else
      {
      print STDERR "INVALID OPTION: $switch\n\nExiting Program ...\n\n";
      exit(1);
      }
}
if (!($strong < $medium) || !( $medium < $weak) ) 
{
   	print STDERR "Strong, Medium, and Weak are improperly ordered.  Exiting ...\n";
	print "\n"; 
	&print_help;
}
if (@ARGV < 2)
{
	print STDERR "Not Enough parameters given.  Exiting ...\n";
	print "\n";
	&print_help;
}

my $dyana_infile = shift @ARGV;
my $cmap_outfile = shift @ARGV;

if (! ((-e $dyana_infile) || ($dyana_infile eq "-"))  ) 
{
   	print STDERR "One or more of the files does not exist.  Exiting ...\n";
	print "\n"; 
	&print_help;
}



my $dyana_hlist = parseDyanaUplFile($dyana_infile);

my $cmap_hlist = {};
$cmap_hlist = parseCMapfile($cmap_outfile)   if ($append_cmap);

my @tokens = qw(dAN(i+1) dNN(i+1) dBN(i+1) dAN(i+2) dNN(i+2) dBN(i+2) dAN(i+3) dNN(i+3) dAB(i+3) dBN(i+3) dAN(i+4));

if ($include_intra)
  { push @tokens, qw(dAN(i+0) dBN(i+0) dAB(i+0)); }

if ($append_cmap)
{
    my %seen;
    my @uniq = grep {! $seen{$_}++} (@{$$cmap_hlist{"parsedtokens"}}, @tokens);
    @{$$cmap_hlist{"parsedtokens"}} = @uniq;
}
else
{
    @{$$cmap_hlist{"parsedtokens"}} = @tokens;
}

foreach my $residue (keys %{$$dyana_hlist{"residue"}})
{
    my $from_cmap_residuename = $aa_name_3_to_1{lc($$dyana_hlist{"residue"}{$residue})} . ($residue + $index_shift);
    $$cmap_hlist{"rlist"}{$from_cmap_residuename}{"index"} = $residue                              if (! exists $$cmap_hlist{"rlist"}{$from_cmap_residuename}{"index"});
    $$cmap_hlist{"rlist"}{$from_cmap_residuename}{"author_name"} = $from_cmap_residuename          if (! exists $$cmap_hlist{"rlist"}{$from_cmap_residuename}{"author_name"});
#    foreach my $token (@tokens)
#    {
#	$$cmap_hlist{"rlist"}{$from_cmap_residuename}{"shifts"}{$token}{"list"} = ["    -     "]   if (! exists $$cmap_hlist{"rlist"}{$from_cmap_residuename}{"shifts"}{$token}{"list"});
#    }
}

foreach my $fromRes (sort {$a <=> $b} keys %{$$dyana_hlist{"NOE"}})
  {
    my $from_cmap_residuename = $aa_name_3_to_1{lc($$dyana_hlist{"residue"}{$fromRes})} . ($fromRes + $index_shift);
    if (! grep {$_ eq $from_cmap_residuename; } (@{$$cmap_hlist{"name_array"}}))
      { push @ {$$cmap_hlist{"name_array"}}, $from_cmap_residuename; }
#    print "FromRES >> $fromRes\n";
    foreach my $fromAT ( keys %{$$dyana_hlist{"NOE"}{$fromRes}} )
      {
#	print "\tFromAT >> $fromAT", "\n";
	if (exists $Dyana2CMap_AT{$fromAT})
	  {
	    my $from_cmap_AT = $Dyana2CMap_AT{$fromAT};
	    foreach my $toRes (keys %{$$dyana_hlist{"NOE"}{$fromRes}{$fromAT}})
	      {

#		print "\t\tToRES >> $toRes\n";
		my $i_to_i_plus = int ($toRes - $fromRes);
		foreach my $toAT (keys %{$$dyana_hlist{"NOE"}{$fromRes}{$fromAT}{$toRes}})
		  {
#		    print "\t\t\ttoAT >> $toAT\n";
		    if (exists $Dyana2CMap_AT{$toAT})
		      {

			my $to_cmap_AT = $Dyana2CMap_AT{$toAT};

			my $cmap_column_name = "d" . $from_cmap_AT . $to_cmap_AT . "(i+" . $i_to_i_plus . ")";
#			print "\t\t\t\tTYPE >> >", $cmap_column_name, "< = ", $$dyana_hlist{"NOE"}{$fromRes}{$fromAT}{$toRes}{$toAT}{"distance"}, "\n";
#			print "\t\t\t\tTOKENS >> >", join ("< >", @tokens), "<\n";

			# Aneerban said that () caused problem, therefore this is fix.
			my $standard_cmap_column_name = $cmap_column_name;

			
			$cmap_column_name =~ s/\(/\\\(/;
			$cmap_column_name =~ s/\)/\\\)/;
			$cmap_column_name =~ s/\+/\\\+/;

			if ($i_to_i_plus == 0)
			  {
			  my $test_column_name = "d" . $to_cmap_AT . $from_cmap_AT . "(i+" . $i_to_i_plus . ")";
			  my $test_name = $test_column_name;
			  $test_column_name =~ s/\(/\\\(/;
			  $test_column_name =~ s/\)/\\\)/;
			  $test_column_name =~ s/\+/\\\+/;
			  if ((grep (/^$test_column_name/, @tokens))[0] eq $test_name)
			    {
			    $cmap_column_name = $test_column_name;
			    $standard_cmap_column_name = $test_name;
			    }
			  }

#			print "\t\t\t\tGREP >> ", grep (/^$cmap_column_name/, @tokens), "\t>", (grep (/^$cmap_column_name/, @tokens))[0],"<\n";
			if ((grep (/^$cmap_column_name/, @tokens))[0] eq $standard_cmap_column_name)
			  {
			    
			    if ($$dyana_hlist{"NOE"}{$fromRes}{$fromAT}{$toRes}{$toAT}{"distance"} < (&convert_CMap_to_distance($ {$$cmap_hlist{"rlist"}{$from_cmap_residuename}{"shifts"}{$standard_cmap_column_name}{"list"}}[0])))
			    {
	#$		      print "\t\t\t\t\tFound >> ",  $standard_cmap_column_name, "\n";
			      $ {$$cmap_hlist{"rlist"}{$from_cmap_residuename}{"shifts"}{$standard_cmap_column_name}{"list"}}[0] =  &convert_distance_to_CMap($$dyana_hlist{"NOE"}{$fromRes}{$fromAT}{$toRes}{$toAT}{"distance"});
			    }
			  }
		      }
		  }
	      }
	  }
      }
  }


#unshift @{$$cmap_hlist{"parsedtokens"}}, "AA";
outputCMapfile($cmap_hlist, $cmap_outfile);



sub convert_distance_to_CMap
  {
  my $distance = shift @_;
  
  if ($distance <= $strong)
    {
    return          "    +++   ";
    }
  elsif ($distance <= $medium)
    {
    return          "    ++    ";
    }
  elsif ($distance <= $weak)
    {
    return          "    +     ";
    }
  else
    {
    return          "    -     ";
    }
  }

sub convert_CMap_to_distance
  {
  my $strength = shift @_;
  
  if ($strength eq    "    +++   ")
    {
    return $strong;
    }
  elsif ($strength eq "    ++    ")
    {
    return $medium;
    }
  elsif ($strength eq "    +     ")
    {
    return $weak;
    }
  else
    {
    #really weak NOE
    return          1000000000;
    }
  }


sub print_help
{
  print STDERR "\n";
  print STDERR "  dyanaUPL2cmap.pl\n";
  print STDERR "        Converts the backbone NOEs from a DYANA UPL constraint file to the CMap format.\n";
  print STDERR "\n";
  print STDERR "  usage: dyanaUPL2cmap.pl [options] input_dyana_file output_cmap_file\n";
  print STDERR "                input_dyana_file  - NOE distance constraints in DYANA UPL format\n";
  print STDERR "                output_cmap_file  - NOE information in CMap format\n";
  print STDERR "\n";
  print STDERR "                options:\n";
  print STDERR "                    -append                     - append CMap file.\n";
  print STDERR "                    -intra                      - include intraresidue NOEs.\n";
  print STDERR "		    -shift num                  - shift index by num.\n";
  print STDERR "                    -convert strong medium weak - the NOEs with distance < strong are +++ (default 3.5)\n";
  print STDERR "                                                      < medium are ++ (default 4.5)\n"; 
  print STDERR "                                                      < weak are + (default 8.5).\n";  
  print STDERR "                                                     NOEs with distance >= weak are disregarded   - .\n";
  exit(1);
}


