#!/usr/bin/perl
#!/usr/bin/perl
#!/usr/local/bin/perl
# examine_spin_systems.pl 
# 	Groups peaks into spin systems and examines them for completeness and overlap.
# 
#	Copyright Hunter Moseley, 2002. All rights reserved.
#	Written by Hunter Moseley 7/19/2002
#	Modified by Hunter Moseley 2/26/2004
#
#
#   usage: examine_spin_systems.pl in_peak_file out_peak_file [options] Htol Ntol CACBtol HAtol Ctol
#
# 	peak_file 	- name of peak file or "-" for standard input/output.
#			  The dimensions of the peaks are Hdim Ndim Xdim.
# 	Htol Ntol  	- Amide H and N dimension tolerances for comparing peaks.
#	CACBtol		- CA/CB dimension tolerances for comparing peaks.
#	HAtol		- HA dimension tolerance for comparing peaks.
#	Ctol		- C dimension tolerance for comparing peaks.
#	options:
#		-deuteration fraction - set the amount of deuteration to the given fraction (default is 0).
#		-skip_matching_overlap - ignore overlap spin systems when determining nearest neighbors.
#		-skip_peak_overrides - ignore peak based assignment constraints.
#		-ignore peak_list - type of peak list to ignore in analysis.
#		-noroot - do not require HSQC or HNCO peak in evaluating ESS Scores.
#		-error_only - only list spin systems with errors.
#		-html filename - print a comprehensive report in html.
#		-sequence sequence - use given sequence to produce peak list based summary.
#		-phase workbookname - given peak list has intensity sign that distinguishing resonance type (i.e. CA+/CB- for HNCACB).
#		-gly_phase workbookname - given peak list has negative intensity sign indicate a glycine residue (i.e. CA+/GlyCA- for HNcoCA).
#
#
#   Hint:  First use concatenate.pl to create a peak list from all the other peak lists.
#	   Use a "-" for the workbookname and the "-old_index" option in the concatenation.
#	   List HSQC and HNCO peaklists first in the concatenation.
#	   Use standard workbooknames in the individual peak lists.
#	   Recognized workbooknames are: join(", ",keys %$expected_tests), hCCcoNH_ILV
#

# These modules allow me to use modules rather than hardcoding everything into one script
use FindBin;
use lib $FindBin::Bin;

use Statistics::Distributions;

# Load needed Modules for Bayesian statistics use BMRB::ChemicalShift;
use BMRB::BayesianCalculations2 qw(:ALL);
use BMRB::BayesianCalculations3 qw(:ALL);
use BMRB::BayesianCalculations4 qw(:ALL);

use strict;


my $peak_list_order = { "HSQC" => 1000, "HNCO" => 99, "HNcaCO" => 95, "HNcaCOintra" => 95, "HNCACACB" => 92, "HNcoCACACB" => 91, "HNCACB" => 89, "HNcoCACB" => 88, "HNCB" => 87, "HNcoCB" => 86, "HNCA" => 85, "HNcoCA" => 84, "HNHAHB" => 79, "HNcoHAHB" => 78, "HNHB" => 77, "HNcoHB" => 76, "HNHA" => 75, "HNcoHA" => 74 };

#
#  Peak List Descriptions for Peak List Summary
#
my $peak_list_type_tests = 
  { 
  "HSQC" => [ { "curr" => "ABCDEFGHIKLMNQRSTVWY", "increment" => 1 } ],
  "HNCO" => [ { "curr" => "ABCDEFGHIKLMNQRSTVWY", "increment" => 1 } ],
  "HNcaCO" => [ { "curr" => "ABCDEFGHIKLMNQRSTVWY", "increment" => 2 } ],
  "HNcaCOintra" => [ { "curr" => "ABCDEFHIKLMNQRSTVWY", "increment" => 1 } ],
  "HNcoCA" => [ { "curr" => "ABCDEFGHIKLMNQRSTVWY", "increment" => 1 } ],
  "HNCA" => [ { "curr" => "ABCDEFGHIKLMNQRSTVWY", "increment" => 2 } ],
  "HNcoCACB" => [ 
		    { "curr" => "ABCDEFGHIKLMNQRSTVWY", "increment" => 1 } , 
		    { "curr" => "ABCDEFGHIKLMNQRSTVWY", "prev" => "ABCDEFHIKLMNPQRSTVWY", "increment" => 1 } 
		  ],
  "HNcoCACACB" => [ 
		    { "curr" => "ABCDEFGHIKLMNQRSTVWY", "increment" => 1 } , 
		    { "curr" => "ABCDEFGHIKLMNQRSTVWY", "prev" => "ABCDEFHIKLMNPQRSTVWY", "increment" => 1 } 
		  ],
  "HNcoCB" => [ { "curr" => "ABCDEFGHIKLMNQRSTVWY", "prev" => "ABCDEFHIKLMNPQRSTVWY", "increment" => 1 } ],
  "HNCACB" => [ 
		  { "curr" => "ABCDEFGHIKLMNQRSTVWY", "increment" => 2 } , 
		  { "curr" => "ABCDEFHIKLMNQRSTVWY", "increment" => 1 } , 
		  { "curr" => "ABCDEFGHIKLMNQRSTVWY", "prev" => "ABCDEFHIKLMNPQRSTVWY", "increment" => 1 } 
		],
  "HNCACACB" => [ 
		  { "curr" => "ABCDEFGHIKLMNQRSTVWY", "increment" => 2 } , 
		  { "curr" => "ABCDEFHIKLMNQRSTVWY", "increment" => 1 } , 
		  { "curr" => "ABCDEFGHIKLMNQRSTVWY", "prev" => "ABCDEFHIKLMNPQRSTVWY", "increment" => 1 } 
		],
  "HNCB" => [ 
		  { "curr" => "ABCDEFHIKLMNQRSTVWY", "increment" => 1 } , 
		  { "curr" => "ABCDEFGHIKLMNQRSTVWY", "prev" => "ABCDEFHIKLMNPQRSTVWY", "increment" => 1 } 
		],
  "HNcoHA" => [ 
		  { "curr" => "ABCDEFGHIKLMNQRSTVWY", "increment" => 1 }, 
		  { "curr" => "ABCDEFGHIKLMNQRSTVWY", "prev" => "G", "increment" => 1 }
		],
  "HNHA" => [ 
		{ "curr" => "ABCDEFGHIKLMNQRSTVWY", "increment" => 2 }, 
		{ "curr" => "G", "increment" => 1 }, 
		{ "curr" => "ABCDEFGHIKLMNQRSTVWY", "prev" => "G", "increment" => 1 } 
	      ],
  "HNcoHAHB" => [ 
		  { "curr" => "ABCDEFGHIKLMNQRSTVWY", "increment" => 1 }, 
		  { "curr" => "ABCDEFGHIKLMNQRSTVWY", "prev" => "G", "increment" => 1 },
		  { "curr" => "ABCDEFGHIKLMNQRSTVWY", "prev" => "AITV", "increment" => 1 }, 
		  { "curr" => "ABCDEFGHIKLMNQRSTVWY", "prev" => "BCDEFHKLMNPQRSWY", "increment" => 2 } 
		],
  "HNcoHB" => [ 
		  { "curr" => "ABCDEFGHIKLMNQRSTVWY", "prev" => "AITV", "increment" => 1 }, 
		  { "curr" => "ABCDEFGHIKLMNQRSTVWY", "prev" => "BCDEFHKLMNPQRSWY", "increment" => 2 } 
		],
  "HNHAHB" => [ 
		  { "curr" => "ABCDEFGHIKLMNQRSTVWY", "increment" => 2 }, 
		  { "curr" => "G", "increment" => 1 }, 
		  { "curr" => "ABCDEFGHIKLMNQRSTVWY", "prev" => "G", "increment" => 1 }, 
		  { "curr" => "AITV", "increment" => 1 } , 
		  { "curr" => "BCDEFHKLMNQRSWY", "increment" => 2 } , 
		  { "curr" => "ABCDEFGHIKLMNQRSTVWY", "prev" => "AITV", "increment" => 1 }, 
		  { "curr" => "ABCDEFGHIKLMNQRSTVWY", "prev" => "BCDEFHKLMNPQRSWY", "increment" => 2 } 
		],
  "HNHB" => [ 
		  { "curr" => "AITV", "increment" => 1 } , 
		  { "curr" => "BCDEFHKLMNQRSWY", "increment" => 2 } , 
		  { "curr" => "ABCDEFGHIKLMNQRSTVWY", "prev" => "AITV", "increment" => 1 }, 
		  { "curr" => "ABCDEFGHIKLMNQRSTVWY", "prev" => "BCDEFHKLMNPQRSWY", "increment" => 2 } 
		],
  "hCCcoNH" => [ 
		   { "curr" => "ABCDEFGHIKLMNQRSTVWY", "increment" => 1 }, 
		   { "curr" => "ABCDEFGHIKLMNQRSTVWY", "prev" => "ABCDEFHIKLMNPQRSTVWY", "increment" => 1 }, 
		   { "curr" => "ABCDEFGHIKLMNQRSTVWY", "prev" => "DEHIKLMNPQRTV", "increment" => 1 }, 
		   { "curr" => "ABCDEFGHIKLMNQRSTVWY", "prev" => "EHIKLPQRV", "increment" => 1 }, 
		   { "curr" => "ABCDEFGHIKLMNQRSTVWY", "prev" => "HIKL", "increment" => 1 }, 
		 ],
  "hCCcoNH_ILV" => [ 
		       { "curr" => "ABCDEFGHIKLMNQRSTVWY", "prev" => "ILV", "increment" => 1 }, 
		       { "curr" => "ABCDEFGHIKLMNQRSTVWY", "prev" => "V", "increment" => 1 } 
		     ],
    };


# Print usage info
if ((@ARGV < 7) || ($ARGV[0] =~ /^-h/))
  {
  print STDERR "\nError: not enough parameters given.\n" if ($ARGV[0] !~ /^-h/);
  print STDERR "\n";
  print STDERR " examine_spin_systems.pl \n";
  print STDERR " 	Groups peaks into spin systems and examines them for completeness and overlap.\n";
  print STDERR " \n";
  print STDERR "   usage: examine_spin_systems.pl in_peak_file out_peak_file [options] Htol Ntol CACBtol HAtol Ctol \n";
  print STDERR "\n";
  print STDERR " 	peak_file 	- name of peak file or \"-\" for standard input/output.\n";
  print STDERR "			  The dimensions of the peaks are Hdim Ndim Xdim.\n";
  print STDERR " 	Htol Ntol  	- Amide H and N dimension tolerances for comparing peaks.\n";
  print STDERR "	CACBtol		- CA/CB dimension tolerances for comparing peaks.\n";
  print STDERR "	HAtol		- HA dimension tolerance for comparing peaks.\n";
  print STDERR "	Ctol		- C dimension tolerance for comparing peaks.\n";
  print STDERR "	options:\n";
  print STDERR "		-deuteration fraction - set the amount of deuteration to the given fraction (default is 0).\n";
  print STDERR "		-skip_matching_overlap - ignore overlap spin systems when determining nearest neighbors.\n";
  print STDERR "		-skip_peak_overrides - ignore peak based assignment constraints.\n";
  print STDERR "		-ignore peak_list - type of peak list to ignore in analysis.\n";
  print STDERR "		-noroot - do not require HSQC or HNCO peak in evaluating ESS Scores.\n";
  print STDERR "		-error_only - only list spin systems with errors.\n";
  print STDERR "		-html filename - print a comprehensive report in html.\n";
  print STDERR "		-sequence sequence - use given sequence to produce peak list based summary.\n";
  print STDERR "		-phase workbookname - given peak list has intensity sign that distinguishing resonance type (i.e. CA+/CB- for HNCACB).\n";
  print STDERR "		-gly_phase workbookname - given peak list has negative intensity sign indicate a glycine residue (i.e. CA+/GlyCA- for HNcoCA).\n";
  print STDERR "\n";
  print STDERR "\n";
  print STDERR "   Hint:  First use concatenate.pl to create a peak list from all the other peak lists.\n";
  print STDERR "	   Use a \"-\" for the workbookname and the \"-old_index\" option in the concatenation.\n";
  print STDERR "	   List HSQC and HNCO peaklists first in the concatenation.\n";
  print STDERR "	   Use standard workbooknames in the individual peak lists.\n";
  print STDERR "	   Recognized workbooknames are: ", join(", ", keys %$peak_list_type_tests),"\n";
  print STDERR "\n";
  exit(1);
  }

# Read in comand line parameters
my $command_line = $0 . " " . join(' ',@ARGV);

my $in_peak_file = shift @ARGV;
my $out_peak_file = shift @ARGV;

my $ignore_matching_overlap = 0;
my $ignore_peak_based_overrides = 0;
my @ignore_alist = ();
my $deuteration = 0;
my $require_root = 1;
my $error_only = 0;
my $prob_sum_cutoff = 0.9999;
my $use_sidechains = 1;
my $use_aromatics = 0;
my $html_filename = "";
my $sequence = [];
my $phase = {};
my $gly_phase = {};

while(@ARGV && $ARGV[0] =~ /^-/)
  {
  my $switch = shift @ARGV;
  if ($switch =~ /^-nor/)
    { $require_root = 0; }
  elsif ($switch =~ /^-d/)
    { $deuteration = shift @ARGV; }
  elsif ($switch =~ /^-e/)
    { $error_only = 1; }
  elsif ($switch =~ /^-skip_m/)
    { $ignore_matching_overlap = 1; }
  elsif ($switch =~ /^-skip_p/)
    { $ignore_peak_based_overrides = 1; }
  elsif ($switch =~ /^-i/)
    { push @ignore_alist, (shift @ARGV); }
  elsif ($switch =~ /^-h/)
    { $html_filename = shift @ARGV; }
  elsif ($switch =~ /^-ph/)
    {
    my $wb = shift @ARGV; 
    $$phase{$wb} = 1;
    }
  elsif ($switch =~ /^-gly/)
    {
    my $wb = shift @ARGV; 
    $$gly_phase{$wb} = 1;
    }
  elsif ($switch =~ /^-se/)
    {
    my $value = shift @ARGV;
    # Strip " from sequence, remove spaces and uppercase. Then split into an char array
    $value =~ s/\"//g; #"
    $value =~ s/\s//g;
    $value = uc($value);
    $sequence = [ split('',$value) ]; 
    }
  else
    {
    print STDERR "Error: unknown option ",$switch," given.\n";
    exit(1);
    }
  }

my $Htol = shift @ARGV;
my $Ntol = shift @ARGV;
my $CACBtol = shift @ARGV;
my $HAtol = shift @ARGV;
my $Ctol = shift @ARGV;

my @tolerance_alist = ($Htol, $Ntol);


my @amino_acid_alist = ("A", "B", "C", "D", "E", "F", "G", "H", "I", "K", "L", "M", "N", "P", "Q", "R", "S", "T", "V", "W", "Y");
my @amino_acid_alist2 = ("A", "C", "D", "E", "F", "G", "H", "I", "K", "L", "M", "N", "P", "Q", "R", "S", "T", "V", "W", "Y");

# Read Peak file
my ($peak_alist, $end_comments_alist) = &read_peak_file($in_peak_file);

# Initialize amino acid statistics for amino acid typing
my ($mean_count, $mean, $variance, $min_value, $max_value, $restricted_type_lists) = &initialize_amino_acid_stats($deuteration);

# remove duplicate peaks and peaks designated as deleted.
my $deleted_peaks_alist = [];
if (! $ignore_peak_based_overrides)
  {
  for(my $x=0; $x < $#$peak_alist; $x++)
    {
    if ($$peak_alist[$x] && (&pac($$peak_alist[$x]) < 0))
      {
      push @$deleted_peaks_alist, $$peak_alist[$x];
      $$peak_alist[$x] = 0;
      }
    elsif ($$peak_alist[$x])
      {
      for(my $y=$x+1; $y < @$peak_alist; $y++)
	{
	if ($$peak_alist[$y] && ! &pac($$peak_alist[$x]) && ! &pac($$peak_alist[$y]) && &is_duplicate($$peak_alist[$x],$$peak_alist[$y]))
	  {
	  my $peak = $$peak_alist[$y];
	  $$peak[$#$peak] .= " [Duplicate Peak to " . &peak_id($$peak_alist[$x]) . "]";
	  push @$deleted_peaks_alist, $peak;
	  $$peak_alist[$y] = 0;
	  }
	}
      }
    }

  @$peak_alist = (grep { $_; } (@$peak_alist));
  }

# create deleted peaks report
my $deleted_peaks_summary;
foreach my $peak (@$deleted_peaks_alist)
  {
  my @temp = @$peak;
  for(my $y=1; $y < @temp; $y++)
    { $temp[$y] =  ("\t" x (length($temp[$y-1]) < 9)) . $temp[$y]; }
  $deleted_peaks_summary .=  join("\t",@temp) . "\n";
  }

# return if a peak picking assignment constraint is present and what type it is
sub pac
  {
  my $peak = shift @_;

  if ($$peak[$#$peak] =~ /\;[Pp]\-/)
    { return -1; }
  elsif ($$peak[$#$peak] =~ /\;[Pp]/)
    { return 1; }

  return 0;
  }

sub is_duplicate
  {
  my $peak1 = shift @_;
  my $peak2 = shift @_;

  for(my $x=1; $x < $#$peak1; $x++)
    {
    if ($$peak1[$x] != $$peak2[$x])
      { return 0; }
    }

  return 1;
  }


# Count observed peaks
my $peak_type_observed_count = {};
foreach my $peak (@$peak_alist)
  {
  foreach my $type (keys %$peak_list_type_tests)
    {
    if (&peak_wb($peak) eq $type)
      { $$peak_type_observed_count{$type}++; }
    }
  }

if (! @$peak_alist)
  {
  foreach my $type (keys %$peak_list_type_tests)
    { $$peak_type_observed_count{$type} = 0; }
  }

#
# Create 3D peak lists from ND peak lists
# 
my %ND_conversions = ( "HNCACB" => ["HNCACACB",4,0,1], "HNCA" => ["HNCACACB",3,1,0], "HNcoCACB" => ["HNcoCACACB",4,0,1], "HNcoCA" => ["HNcoCACACB",3,1,0] );

my $ND_created = {};
foreach my $converted_type (keys %ND_conversions)
  {
  my $conversion_type = $ND_conversions{$converted_type}[0];
  if (! $$peak_type_observed_count{$converted_type} && $$peak_type_observed_count{$conversion_type})
    {
    my $new_peaks = [];
    my $dim = $ND_conversions{$converted_type}[1];
    foreach my $peak (@$peak_alist)
      {
      if (&peak_wb($peak) eq $conversion_type)
	{
	my $npeak = [];
	push @$npeak, $$peak[0], $$peak[1], $$peak[2], $$peak[$dim], $$peak[$#$peak-1], $$peak[$#$peak];
	$$npeak[$#{$npeak}] =~ s/^$conversion_type/$converted_type/;
	if ($$peak[$#{$peak}] !~ /\./)
	  { $$npeak[$#{$npeak}] .= "."; }
	$$npeak[$#{$npeak}] .= "[created_from_$conversion_type]";
	push @$new_peaks, $npeak;
	}
      }

    if ($ND_conversions{$converted_type}[2]) # delete duplicate peaks
      {
      my $new_list = [];
      foreach my $peak (@$new_peaks)
	{
	if (! grep { $$peak[1] == $$[1] && $$peak[2] == $$[2] && $$peak[3] == $$[3]; } (@$new_list))
	  { push @$new_list, $peak; }
	}
      $new_peaks = $new_list;
      }

    if ($ND_conversions{$converted_type}[3] && exists $$phase{$conversion_type})
      { $$phase{$converted_type} = 1; }

    $$ND_created{$converted_type} = 1;
    $$peak_type_observed_count{$converted_type} = scalar(@$new_peaks);
    push @$peak_alist, @$new_peaks;
    }
  }

#
# Create missing peak lists from phased peak lists.
#
my %phase_conversions = ( "HNCA" => ["HNCACB", 1], "HNCB" => ["HNCACB", -1, $$mean{"G"}{"CA"}, sqrt($$variance{"G"}{"CA"}) * 2], "HNcoCA" => ["HNcoCACB", 1], "HNcoCB" => ["HNcoCACB", -1, $$mean{"G"}{"CA"}, sqrt($$variance{"G"}{"CA"}) * 2], "HNHA" => ["HNHAHB", 1], "HNHB" => ["HNHAHB", -1], "HNcoHA" => ["HNcoHAHB", 1], "HNcoHB" => ["HNcoHAHB", -1] );

my $phase_created = {};
foreach my $converted_type (keys %phase_conversions)
  {
  my $conversion_type = $phase_conversions{$converted_type}[0];
  if (! $$peak_type_observed_count{$converted_type} && $$peak_type_observed_count{$conversion_type} && $$phase{$conversion_type})
    {
    my $new_peaks = [];
    my $sign = $phase_conversions{$converted_type}[1];
    foreach my $peak (@$peak_alist)
      {
      if ((&peak_wb($peak) eq $conversion_type) && 
	  ( ((! $$gly_phase{$conversion_type} && ($$peak[$#{$peak}-1] * $sign) > 0)) ||
	    ($$gly_phase{$conversion_type} && $phase_conversions{$converted_type}[3] != 0.0 && ((($$peak[$#{$peak}-1] * $sign) > 0) && ! (abs($$peak[$#{$peak}-2] - $phase_conversions{$converted_type}[3]) < $phase_conversions{$converted_type}[4]))) ||
	    ($$gly_phase{$conversion_type} && $phase_conversions{$converted_type}[3] == 0.0 && (($$peak[$#{$peak}-1] * $sign) > 0)) ))
	{
	my $npeak = [ @$peak ];
	$$npeak[$#{$npeak}] =~ s/^$conversion_type/$converted_type/;
	if ($$peak[$#{$peak}] !~ /\./)
	  { $$npeak[$#{$npeak}] .= "."; }
	$$npeak[$#{$npeak}] .= "[created_from_$conversion_type]";
	push @$new_peaks, $npeak;
	}
      }

    $$phase_created{$converted_type} = 1;
    $$peak_type_observed_count{$converted_type} = scalar(@$new_peaks);
    push @$peak_alist, @$new_peaks;
    }
  }


#
# Put peaks in their proper order
#
sub WB_CMP
  { return ($$peak_list_order{&peak_wb($b)} <=> $$peak_list_order{&peak_wb($a)}); }

@$peak_alist = sort WB_CMP (@$peak_alist);

#
# Create Peak List Summary
#
my $peak_list_summary = "";
if (@$sequence)
  {
  my $previous_residue = shift @$sequence; # skip first residue
  my $peak_type_expected_count = {};
  foreach my $aa (@$sequence)
    {
    foreach my $type (keys %$peak_type_observed_count)
      {
      foreach my $test (@{$$peak_list_type_tests{$type}})
	{
	if ((! exists $$test{"curr"} || ($$test{"curr"} =~ /$aa/)) && 
	    (! exists $$test{"prev"} || ($$test{"prev"} =~ /$previous_residue/)))
	  { $$peak_type_expected_count{$type} += $$test{"increment"}; }
	}
      }
    $previous_residue = $aa;
    }
  
  
  # Create Report
  $peak_list_summary .= "PeakList \t\t\tObserved \tExpected \tDifference\n---------------------------------------------------------------------------\n";
  
  foreach my $type (sort { $$peak_list_order{$b} <=> $$peak_list_order{$a}; } (keys %$peak_list_type_tests))
    {
    next if (! exists $$peak_type_observed_count{$type});
    
    my $type_title = $type . ("(phase created)"x$$phase_created{$type}) . ("(ND created)"x$$ND_created{$type}) ;
    $peak_list_summary .=  $type_title . (" "x(22 - length($type_title))) . " \t\t  " . $$peak_type_observed_count{$type} . " \t\t  " . $$peak_type_expected_count{$type} .  " \t\t  " .
    ($$peak_type_observed_count{$type} - $$peak_type_expected_count{$type}) . "\n";
    } 
  }

#
# Group peaks into globs using tolerances
#
my @peak_groups;
foreach my $peak (@$peak_alist)
  {
  if (@peak_groups)
    {
    # find peak_groups that the peak should be added to
    my @add_list = grep { &valid_grouping($peak,$peak_groups[$_]{"peaks"}, \@tolerance_alist); } (0..$#peak_groups);

    if (! @add_list) # create new peak_group because no suitable group was found.
      {
      push @peak_groups, { "peaks" => [ $peak ] };
      next;
      }
    elsif (@add_list > 1) # if there are more than one peak_group, then combine them.
      {
      my @hold_list;
      @add_list = sort { $b <=> $a; } @add_list; 
      foreach my $index (@add_list)
	{
	my $temp_group = splice(@peak_groups, $index, 1);
	push @hold_list, @{$$temp_group{"peaks"}}; 
	}

      push @peak_groups, { "peaks" => [ @hold_list ] };
      @add_list = ($#peak_groups);
      }

    push @{$peak_groups[$add_list[0]]{"peaks"}}, $peak;
    }
  else
    { push @peak_groups, { "peaks" => [ $peak ] }; }
  }


#
# Check for out of tolerance peaks (due to bad grouping constraints)
#
foreach my $peak_group (@peak_groups)
  {
  if ( @{$$peak_group{"peaks"}} > 1)
    {
    my $out_list = [ grep { my $test = $_; ! (grep { ($_ != $test) && &within_tolerances($_,$test,\@tolerance_alist); } (@{$$peak_group{"peaks"}})); } (@{$$peak_group{"peaks"}})];
    if (@$out_list)
      { $$peak_group{"out_tolerance_list"} = [ map { &peak_id($_); } (@$out_list)]; }
    }
  }


#
# Check for spin system completeness and overlap
#
my $expected_tests = { "HSQC" => 1, "HNCO" => 1, "HNcaCO" => 2, "HNcaCOintra" => 1, "HNcoCA" => 1, "HNCA" => 2, "HNcoCACB" => 2, "HNCACB" => 4, "HNcoCB" => 1, "HNCB" => 2, "HNcoHA" => 1, "HNHA" => 2, "HNHAHB" => 6, "HNcoHAHB" => 3, "HNHB" => 4, "HNcoHB" => 2 };

my $min_exception = { "HNcoCACB" => { "diff" => 1, "test" => "HNcoCA", "expected" => $$mean{"G"}{"CA"}, "range" => sqrt($$variance{"G"}{"CA"}) * 4 }, 
		      "HNCACB" => { "diff" => 1, "test" => "HNCA", "expected" => $$mean{"G"}{"CA"}, "range" => sqrt($$variance{"G"}{"CA"}) * 3 },
		      "HNcoCB" => { "diff" => 1, "test" => "HNcoCA", "expected" => $$mean{"G"}{"CA"}, "range" => sqrt($$variance{"G"}{"CA"}) * 4 }, 
		      "HNCB" => { "diff" => 1, "test" => "HNCA", "expected" => $$mean{"G"}{"CA"}, "range" => sqrt($$variance{"G"}{"CA"}) * 3 },
		      "HNHAHB" => { "default" => 2 },
		      "HNcoHAHB" => { "default" => 1 },
		      "HNHB" => { "default" => 2, "diff" => 2, "test" => "HNCA", "expected" => $$mean{"G"}{"CA"}, "range" => sqrt($$variance{"G"}{"CA"}) * 3 },
		      "HNcoHB" => { "default" => 1, "diff" => 2, "test" => "HNcoCA", "expected" => $$mean{"G"}{"CA"}, "range" => sqrt($$variance{"G"}{"CA"}) * 3 }
		      };
my $max_exception = { "HNcoHA" => { "diff" => 1, "test" => "HNcoCA", "expected" => $$mean{"G"}{"CA"}, "range" => sqrt($$variance{"G"}{"CA"}) * 4 }, 
		      "HNHA" => { "diff" => 1, "test" => "HNCA", "expected" => $$mean{"G"}{"CA"}, "range" => sqrt($$variance{"G"}{"CA"}) * 3 } 
		      };


my $observed_peak_types = {};
foreach my $peak_group (@peak_groups)
  {
  foreach my $test_type (grep { my $type = $_; ! (grep { $_ eq $type; } (@ignore_alist)); } (keys %$expected_tests))
    {
    my $expected = $$expected_tests{$test_type};
    my $number_observed = scalar(grep { (&peak_wb($_) eq $test_type); } (@{$$peak_group{"peaks"}}));
    my $exception = 0;
    my $default_exception = 0;
    if (exists $$min_exception{$test_type})
      {
      if (exists $$min_exception{$test_type}{"test"})
	{
	my $type2 = $$min_exception{$test_type}{"test"}; 
	$exception = $$min_exception{$test_type}{"diff"} * scalar(grep { abs($$_[3] - $$min_exception{$test_type}{"expected"}) < $$min_exception{$test_type}{"range"}; } (grep { (&peak_wb($_) eq $type2); } (@{$$peak_group{"peaks"}})));
	}

      if (exists $$min_exception{$test_type}{"default"})
	{ $default_exception = $$min_exception{$test_type}{"default"}; }
      
      if (exists $$min_exception{$test_type}{"default"} && ($$min_exception{$test_type}{"default"} > abs($exception)))
	{ $exception = $$min_exception{$test_type}{"default"}; }
      }
    elsif (exists $$max_exception{$test_type})
      {
      if (exists $$max_exception{$test_type}{"test"})
	{
	my $type2 = $$max_exception{$test_type}{"test"}; 
	$exception = -1 * $$max_exception{$test_type}{"diff"} * scalar(grep { abs($$_[3] - $$max_exception{$test_type}{"expected"}) < $$max_exception{$test_type}{"range"}; } (grep { (&peak_wb($_) eq $type2); } (@{$$peak_group{"peaks"}})));
	}

      if (exists $$max_exception{$test_type}{"default"})
	{ $default_exception = -1 * $$max_exception{$test_type}{"default"}; }
      
      if (exists $$max_exception{$test_type}{"default"} && ($$max_exception{$test_type}{"default"} > abs($exception)))
	{ $exception = -1 * $$max_exception{$test_type}{"default"}; }

      }

    if ($number_observed)
      { $$observed_peak_types{$test_type}++; }

    if ( ((($number_observed - $expected) > 0) && (($expected - $number_observed) >= $default_exception)) ||
	 ((($number_observed - $expected) < 0) && (($expected - $number_observed) <= $default_exception)) )
      { $exception = ($expected - $number_observed); }

    if ($number_observed - $expected)
      { $$peak_group{$test_type . "_test"} = ($number_observed - $expected + $exception); }
    else
      { $$peak_group{$test_type . "_test"} = ($number_observed - $expected); }

    if ($exception)
      { 
      $$peak_group{$test_type . "_exception"} = $exception; 
      $$peak_group{$test_type . "_exception_test"} = ($number_observed - $expected + $exception); 
      }
    }
  }

# Tabulation of results
my $statistics = {};
foreach my $peak_group (@peak_groups)
  {
  foreach my $test_type (keys %$expected_tests)
    {
    if ($$observed_peak_types{$test_type} && ($$peak_group{$test_type . "_test"} < 0))
      {
      $$peak_group{"missing_flags"}++;
      $$statistics{$test_type . "_missing_count"}++;
      }
    elsif ($$observed_peak_types{$test_type} && ($$peak_group{$test_type . "_test"} > 0))
      {
      $$peak_group{"overlap_flags"}++;
      $$statistics{$test_type . "_overlap_count"}++;
      }
    }
  }

#
# Detect Deleted Spin Systems
#
foreach my $peak_group (@peak_groups)
  {
  if (grep { ($$_[$#$_] =~ /\;[Dd]/); } (@{$$peak_group{"peaks"}}))
    { $$peak_group{"deleted_flag"} = 1; }
  }

#
# Calculate ESS Scores for each detected peaklist
#
my $ESS_scores = {};
foreach my $peak_group (@peak_groups)
  {
  foreach my $test_type (keys %$expected_tests)
    {
    if ($$observed_peak_types{$test_type})
      {
      if (! $require_root || scalar(&root_peaks(@{$$peak_group{"peaks"}})))
	{
	$$ESS_scores{$test_type . "_expected"} += $$expected_tests{$test_type};
	$$ESS_scores{$test_type . "_count"}++;
	if (! $$peak_group{"deleted_flag"})
	  { $$ESS_scores{$test_type . "_count2"}++; }

	if (exists $$peak_group{$test_type . "_test"})
	  { $$ESS_scores{$test_type . "_observed"} += exists $$peak_group{$test_type . "_test"} };

	if (exists $$peak_group{$test_type . "_test"} && (! $$peak_group{$test_type . "_test"} || (exists $$peak_group{$test_type . "_exception"} && ! $$peak_group{$test_type . "_exception_test"}))) 
	  {
	  $$ESS_scores{$test_type . "_correct"}++;
	  if (! $$peak_group{"deleted_flag"})
	    { $$ESS_scores{$test_type . "_correct2"}++; }
	  }
	elsif (exists $$peak_group{$test_type . "_test"} && ($$peak_group{$test_type . "_test"} < 0)) 
	  {
	  if (! exists $$peak_group{$test_type . "_exception"} || ($$peak_group{$test_type . "_exception_test"} <= $$peak_group{$test_type . "_test"}))
	    { $$ESS_scores{$test_type . "_missing"} -= $$peak_group{$test_type . "_test"}; }
	  elsif ($$peak_group{$test_type . "_exception_test"} < 0) 
	    { 
	    $$ESS_scores{$test_type . "_missing"} -= $$peak_group{$test_type . "_exception_test"}; 
	    $$ESS_scores{$test_type . "_expected"} -= $$peak_group{$test_type . "_exception"};
	    }
	  }
	elsif (exists $$peak_group{$test_type . "_test"} && ($$peak_group{$test_type . "_test"} > 0)) 
	  {
	  if (! exists $$peak_group{$test_type . "_exception"} || ($$peak_group{$test_type . "_exception_test"} >= $$peak_group{$test_type . "_test"}))
	    { $$ESS_scores{$test_type . "_extra"} += $$peak_group{$test_type . "_test"}; }
	  elsif ($$peak_group{$test_type . "_exception_test"} > 0) 
	    { 
	    $$ESS_scores{$test_type . "_extra"} += $$peak_group{$test_type . "_exception_test"}; 
	    $$ESS_scores{$test_type . "_expected"} += $$peak_group{$test_type . "_exception"};
	    }
	  }
	}
      }
    }
  }


my $spin_system_summary = "";
#
# Print ESS Scores for each detected peaklist
#
$spin_system_summary .= "                      #Peaks  \t #Peaks \t #Peaks  \t   #Peaks \t  Fract. SS\n";
$spin_system_summary .= "PeakList             Expected \t Missing\t  Extra  \t  Observed\t   Perfect\n";
$spin_system_summary .= "----------------------------------------------------------------------------------------------------\n";
foreach my $test_type (sort { $$peak_list_order{$b} <=> $$peak_list_order{$a}; } (keys %$expected_tests))
  {
  if ($$observed_peak_types{$test_type})
    {
    my $type_title = $test_type . ("(phase created)"x$$phase_created{$test_type}) . ("(ND created)"x$$ND_created{$test_type});
    $spin_system_summary .= sprintf "$type_title%s%4d\t %4d\t\t  %4d\t\t   %4d\t\t    %5.4f (%5.4f)\n"," "x(22-length($type_title)), $$ESS_scores{$test_type . "_expected"}, $$ESS_scores{$test_type . "_missing"},$$ESS_scores{$test_type . "_extra"},$$ESS_scores{$test_type . "_expected"}-$$ESS_scores{$test_type . "_missing"}+$$ESS_scores{$test_type . "_extra"},$$ESS_scores{$test_type . "_correct"} / $$ESS_scores{$test_type . "_count"},$$ESS_scores{$test_type . "_correct2"} / $$ESS_scores{$test_type . "_count2"};
    }
  }


# determine max overlap flags
my $max_overlap_flags = scalar(grep { $$observed_peak_types{$_}; } (keys %$observed_peak_types));


# 
#  Gly Tests
#
my %valid_gly_phase_types = { "HNCA" => 1, "HNcoCA" => 1, "HNHA" => 1, "HNcoHA" => 1 };

sub gly_phase_test
  {
  my $peak = shift @_;

  foreach my $phase_type (grep { $$gly_phase{$_}; } (@_))
    {
    if ((&peak_wb($peak) eq $phase_type) && $$peak[$#{$peak}-1] < 0)
      { return 1; }
    }

  return 0;
  }

foreach my $peak_group (@peak_groups)
  {
  my $inter_gly_phase = scalar(grep { &gly_phase_test($_,"HNcoCA","HNcoHA"); } (@{$$peak_group{"peaks"}}));
  my $intra_gly_phase = scalar(grep { &gly_phase_test($_,"HNCA","HNHA"); } (@{$$peak_group{"peaks"}}));
  
  if ($$peak_group{"HNcoCB_exception"} || $$peak_group{"HNcoCACB_exception"} || $$peak_group{"HNcoHA_exception"} || $inter_gly_phase)
    {
    $$peak_group{"Gly_inter_flags"} = ($$peak_group{"HNcoCB_exception"} && ! $$peak_group{"HNcoCB_exception_test"}) + 
      ($$peak_group{"HNcoCACB_exception"} && ! $$peak_group{"HNcoCACB_exception_test"}) + 
      ($$peak_group{"HNcoHA_exception"} && ! $$peak_group{"HNcoHA_exception_test"}) + $inter_gly_phase;
    }
  
  if ($$peak_group{"HNCB_exception"} || $$peak_group{"HNCACB_exception"} || $$peak_group{"HNHA_exception"} || $intra_gly_phase)
    {
    if ($$peak_group{"Gly_inter_flags"})
      {
      $$peak_group{"Gly_intra_flags"} = (($$peak_group{"HNCB_exception"} < -1) && ! $$peak_group{"HNCB_exception_test"}) + 
	(($$peak_group{"HNCACB_exception"} < -1) && ! $$peak_group{"HNCACB_exception_test"}) + 
	(($$peak_group{"HNHA_exception"} > 1) && ! $$peak_group{"HNHA_exception_test"}) + ($intra_gly_phase > $inter_gly_phase);
      }
    else
      {
      $$peak_group{"Gly_intra_flags"} = ($$peak_group{"HNCB_exception"} && ! $$peak_group{"HNCB_exception_test"}) + 
	($$peak_group{"HNCACB_exception"} && ! $$peak_group{"HNCACB_exception_test"}) + 
	($$peak_group{"HNHA_exception"} && ! $$peak_group{"HNHA_exception_test"}) + $intra_gly_phase;
      }

    }
  }
  
#
# Test for ROOT peaks
#
if (exists $$observed_peak_types{"HSQC"} || exists $$observed_peak_types{"HNCO"})
  {
  foreach my $peak_group (@peak_groups)
    {
    my $roots_observed = scalar(&root_peaks(@{$$peak_group{"peaks"}}));
    if (! $roots_observed || (@{$$peak_group{"peaks"}} == 1))
      {
      # create average HN for root search
      my @average_HN;
      foreach my $peak (@{$$peak_group{"peaks"}})
	{ 
	$average_HN[0] += $$peak[1];
	$average_HN[1] += $$peak[2];
	}
      $average_HN[0] /= scalar(@{$$peak_group{"peaks"}});
      $average_HN[1] /= scalar(@{$$peak_group{"peaks"}});

      my $closest = -1;
      my $distance2 = 100000000;
      my ($HN, $N);
      for(my $x=0; $x < @peak_groups; $x++)
	{
	next if ($peak_group == $peak_groups[$x]);

	foreach my $root_peak (&root_peaks(@{$peak_groups[$x]{"peaks"}}))
	  {
	  my $test_distance2 = ($$root_peak[1] - $average_HN[0]) ** 2 + (($$root_peak[2] - $average_HN[1]) * $tolerance_alist[0] / $tolerance_alist[1]) ** 2;
	  if ($test_distance2 < $distance2)
	    {
	    $closest = $x;
	    $distance2 = $test_distance2;
	    $HN = $$root_peak[1];
	    $N = $$root_peak[2];
	    }
	  }
	}

      $$peak_group{"closest_HN"}{"group"} = $closest;
      $$peak_group{"closest_HN"}{"HN_distance"} = sqrt($distance2);
      $$peak_group{"closest_HN"}{"HN"} = $HN;
      $$peak_group{"closest_HN"}{"N"} = $N;
      }
    }
  }

#
# Organize chemical shifts.
#
my %chem_shift_groups = ( "HNCOCA" => [ "CA-1" ], 
			  "HNCA" => [ "CA", "CA-1" ], 
			  "HNCOCACB" => [ "CA-1", "CB-1" ], 
			  "HNCACB" => [ "CA", "CB", "CA-1", "CB-1" ], 
			  "HNCOCB" => [ "CB-1" ], 
			  "HNCB" => [ "CB", "CB-1" ], 
			  "HNCOHA" => [ "HA-1" ], 
			  "HNHA" => [ "HA", "HA-1" ], 
			  "HNCOHB" => [ "HB-1" ], 
			  "HNHB" => [ "HB" ], 
			  "HNcoHAHB" => [ "HA", "HB" ], 
			  "HNHAHB" => [ "HA", "HB", "HA-1", "HB-1" ], 
			  "HNCO" => [ "CO-1" ], 
			  "HNCACO" => [ "CO", "CO-1" ],
			  "HNCACOINTRA" => [ "CO" ]
			  );

my @chem_shift_group_order = ("HNCOCA", "HNCA", "HNCOCB", "HNCB", "HNCOCACB", "HNCACB", "HNCOHA", "HNHA", "HNCOHB", "HNHB", "HNCOHAHB", "HNHAHB", "HNCO", "HNCACOINTRA", "HNCACO");

my %tol_hlist = ( "CA" => $CACBtol, "CA-1" => $CACBtol, "CB" => $CACBtol, "CB-1" => $CACBtol, 
		  "HA" => $HAtol, "HA-1" => $HAtol, 
		  "CO" => $Ctol, "CO-1" => $Ctol,
		  "H" => $Htol, "N" => $Ntol
		  );

foreach my $peak_group (@peak_groups)
  {
  # create chemical shift groups
  my %shift_groups_hlist = ();
  foreach my $peak (@{$$peak_group{"peaks"}})
    {
    my $peak_type = uc(&peak_wb($peak));
    if (exists $chem_shift_groups{$peak_type})
      { push @{$shift_groups_hlist{$peak_type}}, $$peak[$#{$peak}-2]; }
    }

  my @shift_groups_alist;
  foreach my $peak_type (grep { exists $shift_groups_hlist{$_}; } @chem_shift_group_order)
    { push @shift_groups_alist, [ [ @{$chem_shift_groups{$peak_type}} ], $shift_groups_hlist{$peak_type}, $peak_type ]; }

  # check all groups for changes due to matches.
  my $changed = 1;
  while($changed)
    {
    $changed = 0;
    my $best_match = {};
    for(my $x=0; $x < @shift_groups_alist; $x++)
      {
      for(my $y=0; $y < @shift_groups_alist; $y++)
	{
	next if ($x == $y);

	# are all the chem types in $x found in $y
	if (@{$shift_groups_alist[$x][0]} && (scalar(grep { my $test = $_; scalar (grep { $test eq $_; } (@{$shift_groups_alist[$y][0]})); } (@{$shift_groups_alist[$x][0]})) == @{$shift_groups_alist[$x][0]})) 
	  {
	  my $distance = 0;
	  my $tol = $tol_hlist{$shift_groups_alist[$y][0][0]};
	  my @used; 
	  foreach my $shift (@{$shift_groups_alist[$x][1]})
	    {
	    my $match = -1;
	    my $min_diff = 10000;
	    for(my $z=0; $z < @{$shift_groups_alist[$y][1]}; $z++)
	      {
	      my $diff = abs($shift - $shift_groups_alist[$y][1][$z]);
	      if (! $used[$z] && ($diff < $tol) && ($diff < $min_diff))
		{ 
		$match = $z;
		$min_diff = $diff;
		}
	      }
	    
	    last if ($match == -1);
	    $distance += $min_diff;
	    $used[$match] = 1;
	    }

	  # found match
	  if (scalar(grep { $_; } (@used)) == @{$shift_groups_alist[$x][1]})
	    {
	    if (! exists $$best_match{"distance"} || ($distance < $$best_match{"distance"}))
	      { $best_match = { "distance" => $distance, "sub" => $x, "super" => $y, "used" => [ @used ] }; }
	    }
	  }
	}
      }

    # restrict $y group
    if (exists $$best_match{"distance"})
      {
      $changed = 1;
      # strip matching shift types
      $shift_groups_alist[$$best_match{"super"}][0] = [ grep { my $test = $_; ! scalar (grep { $test eq $_; } (@{$shift_groups_alist[$$best_match{"sub"}][0]})); } (@{$shift_groups_alist[$$best_match{"super"}][0]}) ];
      # strip matching shifts
      for(my $x=0; $x < @{$$best_match{"used"}}; $x++)
	{
	if ($$best_match{"used"}[$x])
	  { splice(@{$shift_groups_alist[$$best_match{"super"}][1]},$x,1); }
	}
      }
    }

  # create aliphatic chemical shift lists
  @shift_groups_alist = sort { @{$$a[0]} <=> @{$$b[0]}; } (@shift_groups_alist);
  foreach my $shift_group (@shift_groups_alist)
    {
    foreach my $shift_type (@{$$shift_group[0]})
      {
      if (! exists $$peak_group{"shifts"}{$shift_type} && @{$$shift_group[1]})
	{ $$peak_group{"shifts"}{$shift_type} = [ @{$$shift_group[1]} ]; }
      }
    }

  # create root chemical shift lists
  my @peak_alist = (&root_peaks(@{$$peak_group{"peaks"}}));
  if (! @peak_alist)
    { @peak_alist = @{$$peak_group{"peaks"}}; }

  foreach my $root_peak (@peak_alist)
    {
    push @{$$peak_group{"shifts"}{"H"}}, $$root_peak[1]; 
    push @{$$peak_group{"shifts"}{"N"}}, $$root_peak[2]; 
    }
  }


#
# Create ChemicalShift objects and Test Average PRTL size
#
my @carbon_atom_type_alist = qw(CA CB CG CG1 CG2 CD CD1 CD2 CE CE1 CZ CZ2 CZ3 CH2);
my $junk=0;
foreach my $peak_group (@peak_groups)
  {
  foreach my $carbon_shift (@carbon_atom_type_alist)
    {
    if (exists $$peak_group{"shifts"}{$carbon_shift}  && @{$$peak_group{"shifts"}{$carbon_shift}})
      {
      $$peak_group{"chem_shifts"}{$carbon_shift} = [];
      foreach my $value (@{$$peak_group{"shifts"}{$carbon_shift}})
	{ push @{$$peak_group{"chem_shifts"}{$carbon_shift}}, BMRB::ChemicalShift->new( [ "ESS", $junk, "X", $carbon_shift, $value ] ); }
      }
    
    if (exists $$peak_group{"shifts"}{$carbon_shift . "-1"} && @{$$peak_group{"shifts"}{$carbon_shift . "-1"}})
      {
      $$peak_group{"chem_shifts-1"}{$carbon_shift} = [];
      foreach my $value (@{$$peak_group{"shifts"}{$carbon_shift . "-1"}})
	{ push @{$$peak_group{"chem_shifts-1"}{$carbon_shift}}, BMRB::ChemicalShift->new( [ "ESS", $junk, "X", $carbon_shift, $value ] ); }
      }
    }
  $junk++;
  }


my %PRTL_props;
$PRTL_props{"mean"} = $mean;
$PRTL_props{"mean_count"} = $mean_count;
$PRTL_props{"variance"} = $variance;
#$PRTL_props{"File_ResType_frequencies"} = &aa_frequencies($bmrb_hlist);
$PRTL_props{"restricted_type_lists"} = $restricted_type_lists;
$PRTL_props{"covariance_type_lists"} = $restricted_type_lists;
$PRTL_props{"single_bp_sub"} = \&singleBProbByMeanVarianceChiSquare; 
$PRTL_props{"prior_sub"} = \&simplePrior; 
$PRTL_props{"CX_prior_sub"} = \&simplePrior; 


foreach my $peak_group (@peak_groups)
  { 
 if (! $require_root || scalar(&root_peaks(@{$$peak_group{"peaks"}})))
    {
    $$ESS_scores{"aatyping_expected"}++;
    if (exists $$peak_group{"chem_shifts"})
      { $$ESS_scores{"aatyping_count_intra"} += &PRTL_size($$peak_group{"chem_shifts"}, \%PRTL_props, $prob_sum_cutoff); }
    else
      { $$ESS_scores{"aatyping_count_intra"} += 20; }

    if (exists $$peak_group{"chem_shifts-1"})
      { $$ESS_scores{"aatyping_count_sequential"} += &PRTL_size($$peak_group{"chem_shifts-1"}, \%PRTL_props, $prob_sum_cutoff); }
    else
      { $$ESS_scores{"aatyping_count_sequential"} += 20; }
    }
  }


if ($$ESS_scores{"aatyping_expected"} > 0)
  { $spin_system_summary .= sprintf "\n\nAverage PRTL Length: %5.4f\n",($$ESS_scores{"aatyping_count_intra"}+$$ESS_scores{"aatyping_count_sequential"})/$$ESS_scores{"aatyping_expected"}/2; }
else
  { $spin_system_summary .= "\n\nAverage PRTL Length: 0.0000\n";}




#
# Detect sidechains
#
foreach my $peak_group (@peak_groups)
  {
  # this commented out line prevented sidechains with no root peaks from being identified
  # next if (exists $$peak_group{"closest_HN"});
  my $sidechain_flags = ($$observed_peak_types{"HNCA"} && $$observed_peak_types{"HNcoCA"} && 
			 ($$peak_group{"HNCA_test"} < 0) && ($$peak_group{"HNcoCA_test"} == 0) && 
			 ((grep { (&peak_wb($_) eq "HNCA"); } (@{$$peak_group{"peaks"}})) > 0))
    + 
      ($$observed_peak_types{"HNCACB"} && $$observed_peak_types{"HNcoCACB"} && 
       ($$peak_group{"HNCACB_test"} < 0) && (($$peak_group{"HNcoCACB_test"} == 0) || ($$peak_group{"HNcoCACB_exception"} && ($$peak_group{"HNcoCACB_exception_test"} == 1))) && 
       ((grep { (&peak_wb($_) eq "HNCACB"); } (@{$$peak_group{"peaks"}})) > 0))
    + 
      ($$observed_peak_types{"HNCB"} && $$observed_peak_types{"HNcoCB"} && 
       ($$peak_group{"HNCB_test"} < 0) && (($$peak_group{"HNcoCB_test"} == 0) || ($$peak_group{"HNcoCB_exception"} && ($$peak_group{"HNcoCB_exception_test"} == 1))) && 
       ((grep { (&peak_wb($_) eq "HNCB"); } (@{$$peak_group{"peaks"}})) > 0))
    + 
      ($$observed_peak_types{"HNHA"} && $$observed_peak_types{"HNcoHA"} && 
       ($$peak_group{"HNHA_test"} < 0) && (($$peak_group{"HNcoHA_test"} == 0) || ($$peak_group{"HNcoHA_exception"} && ($$peak_group{"HNcoHA_exception_test"} == -1))) && 
			 ((grep { (&peak_wb($_) eq "HNHA"); } (@{$$peak_group{"peaks"}})) > 0))
    + 
      ($$observed_peak_types{"HNcaCO"} && $$observed_peak_types{"HNCO"} && 
       ($$peak_group{"HNcaCO_test"} < 0) && ($$peak_group{"HNCO_test"} == 0) && 
			 ((grep { (&peak_wb($_) eq "HNcaCO"); } (@{$$peak_group{"peaks"}})) > 0)) ;

  if ($sidechain_flags)
    { $$peak_group{"sidechain_flags"} = $sidechain_flags; }
  }

foreach my $peak_group (@peak_groups)
  {
  if (exists $$peak_group{"sidechain_flags"} && ($$peak_group{"sidechain_flags"} > 1))
    {
    # Determine sidechain amino acid type
    if (exists $$peak_group{"shifts"}{"CA-1"} && exists $$peak_group{"shifts"}{"CB-1"})
      {
      my $best_Q_prob = 0;
      my $best_N_prob = 0;
    
      foreach my $CA_shift (@{$$peak_group{"shifts"}{"CA-1"}})
	{
	foreach my $CB_shift (@{$$peak_group{"shifts"}{"CB-1"}})
	  {
	  my $test_Q_prob = Statistics::Distributions::chisqrprob(2, ((($CA_shift - $$mean{"Q"}{"CG"}) ** 2) / $$variance{"Q"}{"CG"}) + ((($CB_shift - $$mean{"Q"}{"CB"}) ** 2) / $$variance{"Q"}{"CB"}));
	  my $test_N_prob = Statistics::Distributions::chisqrprob(2, ((($CA_shift - $$mean{"N"}{"CB"}) ** 2) / $$variance{"N"}{"CB"})  + ((($CB_shift - $$mean{"N"}{"CA"}) ** 2) / $$variance{"N"}{"CA"}));
	  if ($test_Q_prob > $best_Q_prob)
	    { $best_Q_prob = $test_Q_prob; }
	  if ($test_N_prob > $best_N_prob)
	    { $best_N_prob = $test_N_prob; }
	  }
	}

      if (($best_N_prob > $best_Q_prob) && ($best_N_prob > 0.001))
	{ 
	$$peak_group{"sidechain_aa_prob"}  = $best_N_prob; 
	$$peak_group{"sidechain_aa_type"}  = "N"; 
	$$peak_group{"sidechain_flags"}++;
	}
      elsif (($best_Q_prob > $best_N_prob) && ($best_Q_prob > 0.001))
	{ 
	$$peak_group{"sidechain_aa_prob"}  = $best_Q_prob; 
	$$peak_group{"sidechain_aa_type"}  = "Q"; 
	$$peak_group{"sidechain_flags"}++;
	}
      }

    # Determine Parent spin systems
    if (exists $$peak_group{"sidechain_aa_type"})
      {
      my $intra_parent = -1;
      my $inter_parent = -1;
      if ($$peak_group{"sidechain_aa_type"} eq "N") # N Parent testing
	{
	for(my $x=0; $x < @peak_groups; $x++)
	  {
	  next if ($ignore_matching_overlap && ($peak_groups[$x]{"overlap_flags"} >= $max_overlap_flags/2));

	  if (exists $peak_groups[$x]{"shifts"}{"CA-1"} && exists $peak_groups[$x]{"shifts"}{"CB-1"})
	    {
	    # remember CA-1 matches to CB-1 and visa versa
	    my $CA_inter_match = scalar(grep {my $shift = $_; (grep { abs($shift - $_) < $CACBtol; } (@{$peak_groups[$x]{"shifts"}{"CA-1"}})); } (@{$$peak_group{"shifts"}{"CB-1"}}));
	    my $CB_inter_match = scalar(grep {my $shift = $_; (grep { abs($shift - $_) < $CACBtol; } (@{$peak_groups[$x]{"shifts"}{"CB-1"}})); } (@{$$peak_group{"shifts"}{"CA-1"}}));
	    my $CA_intra_match = scalar(grep {my $shift = $_; (grep { abs($shift - $_) < $CACBtol; } (@{$peak_groups[$x]{"shifts"}{"CA"}})); } (@{$$peak_group{"shifts"}{"CB-1"}}));
	    my $CB_intra_match = scalar(grep {my $shift = $_; (grep { abs($shift - $_) < $CACBtol; } (@{$peak_groups[$x]{"shifts"}{"CB"}})); } (@{$$peak_group{"shifts"}{"CA-1"}}));
	    my $CA_same = scalar(grep {my $shift = $_; (grep { $shift == $_; } (@{$peak_groups[$x]{"shifts"}{"CA"}})); } (@{$peak_groups[$x]{"shifts"}{"CA-1"}})) == @{$peak_groups[$x]{"shifts"}{"CA"}};
	    
	    if ($CA_inter_match && $CB_inter_match && $CA_intra_match && $CB_intra_match && $CA_same)
	      {
	      if (($inter_parent == -1) && ($intra_parent > -1)) 
		{ $inter_parent = $x; }
	      elsif (($inter_parent > -1) && ($intra_parent == -1)) 
		{ $intra_parent = $x; }
	      elsif (($inter_parent == -1) && ($intra_parent == -1)) 
		{ 
		$inter_parent = $x; 
		$intra_parent = $x; 
		}
	      else
		{ 
		$inter_parent = -2; 
		$intra_parent = -2; 
		}

	      }
	    elsif ($CA_inter_match && $CB_inter_match)
	      { 
	      if (($inter_parent == -1) || (($inter_parent == $intra_parent) && ($inter_parent != -2)))
		{ $inter_parent = $x; }
	      else 
		{ $inter_parent = -2; }
	      }
	    elsif ($CA_intra_match && $CB_intra_match)
	      { 
	      if (($intra_parent == -1) || (($inter_parent == $intra_parent) && ($inter_parent != -2)))
		{ $intra_parent = $x; }
	      else 
		{ $intra_parent = -2; }
	      }
	    }
	  }
	}
      else # Q Parent testing
	{
	for(my $x=0; $x < @peak_groups; $x++)
	  {
	  if (exists $peak_groups[$x]{"shifts"}{"CB-1"})
	    {
	    my $CB_inter_match = scalar(grep {my $shift = $_; (grep { abs($shift - $_) < $CACBtol; } (@{$peak_groups[$x]{"shifts"}{"CB-1"}})); } (@{$$peak_group{"shifts"}{"CB-1"}}));
	    my $CB_intra_match = scalar(grep {my $shift = $_; (grep { abs($shift - $_) < $CACBtol; } (@{$peak_groups[$x]{"shifts"}{"CB"}})); } (@{$$peak_group{"shifts"}{"CB-1"}}));
	    my $CB_same = scalar(grep {my $shift = $_; (grep { $shift == $_; } (@{$peak_groups[$x]{"shifts"}{"CB"}})); } (@{$peak_groups[$x]{"shifts"}{"CB-1"}})) == @{$peak_groups[$x]{"shifts"}{"CB"}};
	    
	    if ($CB_inter_match && $CB_intra_match && $CB_same)
	      {
	      if (($inter_parent == -1) && ($intra_parent > -1)) 
		{ $inter_parent = $x; }
	      elsif (($inter_parent > -1) && ($intra_parent == -1)) 
		{ $intra_parent = $x; }
	      elsif (($inter_parent == -1) && ($intra_parent == -1)) 
		{ 
		$inter_parent = $x; 
		$intra_parent = $x; 
		}
	      else
		{ 
		$inter_parent = -2; 
		$intra_parent = -2; 
		}

	      }
	    elsif ($CB_inter_match)
	      { 
	      if (($inter_parent == -1) || (($inter_parent == $intra_parent) && ($inter_parent != -2)))
		{ $inter_parent = $x; }
	      else 
		{ $inter_parent = -2; }
	      }
	    elsif ($CB_intra_match)
	      { 
	      if (($intra_parent == -1) || (($inter_parent == $intra_parent) && ($inter_parent != -2)))
		{ $intra_parent = $x; }
	      else 
		{ $intra_parent = -2; }
	      }
	    }
	  }
	}

      # # Set Inter Parent if found
      if (($inter_parent > -1) && ($inter_parent != $intra_parent))
	{ $$peak_group{"sidechain_inter_parent"} = $inter_parent; }
      
      # # Set Intra Parent if found
      if (($intra_parent > -1) && ($inter_parent != $intra_parent))
	{ $$peak_group{"sidechain_intra_parent"} = $intra_parent; }
      }


    # Determine Sidechain partner spin system
    my $best_partner = -1;
    my $partner_flags = 1;
    for(my $x=0; $x < @peak_groups; $x++)
      {
      next if ($peak_group == $peak_groups[$x]);
      next if (! exists $peak_groups[$x]{"sidechain_flags"} || ($peak_groups[$x]{"sidechain_flags"} < 2));
      # test for matching N shifts
      next if (! grep {my $shift = $_; scalar (grep { abs($shift - $_) <= $Ntol/2; } (@{$$peak_group{"shifts"}{"N"}}));} @{$peak_groups[$x]{"shifts"}{"N"}});
      # test for H shifts within right range.
      next if (! grep {my $shift = $_; scalar (grep {my $diff = abs($shift - $_); ($diff > 0.5) && ($diff < 1.7); } (@{$$peak_group{"shifts"}{"H"}}));} @{$peak_groups[$x]{"shifts"}{"H"}});

      # count each matching shift as a flag.
      my $flags = 1 + 
	scalar(grep { my $shift_type = $_; 
		      (grep {my $shift = $_; 
			     (grep { abs($shift - $_) < $tol_hlist{$shift_type}; } (@{$peak_groups[$x]{"shifts"}{$shift_type}})); 
			     } (@{$$peak_group{"shifts"}{$shift_type}})); 
		      } (grep {exists $$peak_group{"shifts"}{$_} && exists $peak_groups[$x]{"shifts"}{$_}; } ("CA-1", "CB-1", "CO-1", "HA-1")));

      if ($flags > abs($partner_flags))
	{
	$best_partner = $x;
	$partner_flags = $flags;
	}
      elsif ($flags == abs($partner_flags))
	{ $partner_flags = -$flags; }
      }

    if ($best_partner != -1)
      { 
      if ($partner_flags < 0)
	{ $$peak_group{"sidechain_partner"} = "Ambiguous"; }
      else
	{ $$peak_group{"sidechain_partner"} = $best_partner; }
      $$peak_group{"sidechain_partner_flags"} = abs($partner_flags);
      $$peak_group{"sidechain_flags"}++;
      }
    }
  }


#
# Detect nearest neighbors
#
my %prev_neighbors;
my %next_neighbors;
my %prev_matches;
my %next_matches;
for(my $y=0; $y < @peak_groups; $y++)
  {
  next if (exists $peak_groups[$y]{"sidechain_flags"} && $peak_groups[$y]{"sidechain_flags"} > 2); # skip sidechains
  next if ($ignore_matching_overlap && ($peak_groups[$y]{"overlap_flags"} >= $max_overlap_flags/2));

  my $peak_group = $peak_groups[$y];

  my $best_next = -1;
  my $best_next_count = 0;
  my $best_next_matches = 0;
  my $best_prev = -1;
  my $best_prev_count = 0;
  my $best_prev_matches = 0;
  for(my $x=0; $x < @peak_groups; $x++)
    {
    next if ($x == $y);
    next if (exists $peak_groups[$x]{"sidechain_flags"} && $peak_groups[$x]{"sidechain_flags"} > 2); # skip sidechains
    next if ($ignore_matching_overlap && ($peak_groups[$x]{"overlap_flags"} >= $max_overlap_flags/2));

    # count each matching shift
    my $prev_matches = 
      scalar(grep { my $shift_type = $_; 
		    (grep {my $shift = $_; 
			   (grep { abs($shift - $_) < $tol_hlist{$shift_type}; } (@{$peak_groups[$x]{"shifts"}{$shift_type}})); 
			   } (@{$$peak_group{"shifts"}{$shift_type . "-1"}})); 
		    } (grep {exists $$peak_group{"shifts"}{$_ . "-1"} && exists $peak_groups[$x]{"shifts"}{$_}; } ("CA", "CB", "CO", "HA")));

    my $next_matches = 
      scalar(grep { my $shift_type = $_; 
		    (grep {my $shift = $_; 
			   (grep { abs($shift - $_) < $tol_hlist{$shift_type}; } (@{$peak_groups[$x]{"shifts"}{$shift_type . "-1"}})); 
			   } (@{$$peak_group{"shifts"}{$shift_type}})); 
		    } (grep {exists $$peak_group{"shifts"}{$_} && exists $peak_groups[$x]{"shifts"}{$_ . "-1"}; } ("CA", "CB", "CO", "HA")));

    if ($prev_matches > $best_prev_matches)
      {
      $best_prev = $x;
      $best_prev_matches = $prev_matches;
      $best_prev_count = 1;
      }
    elsif ($prev_matches == $best_prev_matches)
      { $best_prev_count++; }
    
    if ($next_matches > $best_next_matches)
      {
      $best_next = $x;
      $best_next_matches = $next_matches;
      $best_next_count = 1;
      }
    elsif ($next_matches == $best_next_matches)
      { $best_next_count++; }
    }


  if (($best_prev != -1) && ($best_prev_count == 1) && (($best_prev_matches > 1) || $peak_groups[$y]{"Gly_inter_flags"}))
    { 
    $prev_neighbors{$y+1} = $best_prev+1; 
    $prev_matches{$y+1} = $best_prev_matches; 
    }

  if (($best_next != -1) && ($best_next_count == 1) && (($best_next_matches > 1) || $peak_groups[$best_next]{"Gly_inter_flags"}))
    { 
    $next_neighbors{$y+1} = $best_next+1; 
    $next_matches{$y+1} = $best_next_matches; 
    }
  }

# set unambiguous prev and next
for(my $x=0; $x < @peak_groups; $x++)
  {
  if ($next_neighbors{$prev_neighbors{$x+1}} == $x+1)
    {
    $peak_groups[$x]{"prev_spin_system"} = $prev_neighbors{$x+1} - 1;
    $peak_groups[$x]{"prev_spin_system_matches"} = $prev_matches{$x+1};
    $peak_groups[$peak_groups[$x]{"prev_spin_system"}]{"next_spin_system"} = $x;
    $peak_groups[$peak_groups[$x]{"prev_spin_system"}]{"next_spin_system_matches"} = $next_matches{$prev_neighbors{$x+1}};
    }
  }

#
# Print Linking Uniqueness Info
#
foreach my $peak_group (@peak_groups)
  {
  if (! $require_root || scalar(&root_peaks(@{$$peak_group{"peaks"}})))
    {
    $$ESS_scores{"linking_uniqueness_expected"}++;
    if (exists $$peak_group{"prev_spin_system"})
      { $$ESS_scores{"linking_uniqueness_count"}++; }
    }
  }

if ($$ESS_scores{"linking_uniqueness_expected"} > 0)
  { $spin_system_summary .= sprintf "\nLinking Uniqueness: %5.4f\n",$$ESS_scores{"linking_uniqueness_count"}/$$ESS_scores{"linking_uniqueness_expected"}; }
else
  { $spin_system_summary .= "\nLinking Uniqueness: 0.0000\n";}


#
#  Print Summary Report
#
if ($peak_list_summary ne "")
  { print "\n\nSequence Based Summary:\n\n",$peak_list_summary, "\n\n"; }
print "Spin System  Based Summary:\n\n",$spin_system_summary;

#
# Create Output peak list file with spin system report
#
# Open Output file
local *OUTPUT_FILE;
if ($out_peak_file eq "-")
  { *OUTPUT_FILE = *STDOUT; }
else
  { open(OUTPUT_FILE, ">$out_peak_file") || die ("open " . $out_peak_file . " failed."); }

# Print Summary
if ($peak_list_summary ne "")
  {
  my $temp = $peak_list_summary;
  $temp =~ s/\n/\n\#/g;
  print OUTPUT_FILE "#\n#\n#Sequence Based Summary:\n#\n","#",$temp,"\n#\n";
  }

my $temp = $spin_system_summary;
$temp =~ s/\n/\n\#/g;
print OUTPUT_FILE "#Spin System Based Summary:\n#\n","#",$temp,"\n#\n";


# Print Header
print OUTPUT_FILE "#Index	 1Dim	 2Dim	 3Dim	 Intensity	 Workbook\n";

if ($deleted_peaks_summary ne "")
  {
  my $temp = $deleted_peaks_summary;
  $temp =~ s/\n/\n\#/g;
  print OUTPUT_FILE "#\n#\n# Deleted Peaks:\n#$temp\n#\n";
  }

my $html_ss_javascript_switch;
for(my $x=0; $x < @peak_groups; $x++)
  {
  my $spin_system = "";
  if (exists $peak_groups[$x]{"overlap_flags"})
    {
    $spin_system .= "#\tOverlap Flags: " . $peak_groups[$x]{"overlap_flags"} . "\n#\t\t";
    foreach my $test_type (keys %$expected_tests)
      {
      if (exists $$observed_peak_types{$test_type} && ($peak_groups[$x]{$test_type . "_test"} > 0))
	{ $spin_system .=  "$test_type " . $peak_groups[$x]{$test_type . "_test"} . "  "; }
      }
    $spin_system .=  "\n";
    }
  
  if (exists $peak_groups[$x]{"missing_flags"})
    {
    $spin_system .=  "#\tIncomplete Flags: " .  $peak_groups[$x]{"missing_flags"} . "\n#\t\t";
    foreach my $test_type (keys %$expected_tests)
      {
      if (exists $$observed_peak_types{$test_type} && ($peak_groups[$x]{$test_type . "_test"} < 0))
	{ $spin_system .=  "$test_type " . $peak_groups[$x]{$test_type . "_test"} . "  "; }
      }
    $spin_system .=  "\n";
    }

  if (exists $peak_groups[$x]{"out_tolerance_list"} && @{$peak_groups[$x]{"out_tolerance_list"}})
    { $spin_system .=  "#\tOut of Tolerance: " .  join(" ", @{$peak_groups[$x]{"out_tolerance_list"}}) . "\n"; }
  
  if (exists $peak_groups[$x]{"prev_spin_system"})
    { $spin_system .=  "#\tPrev Spin System: " . $peak_groups[$x]{"prev_spin_system"} . "  Matches: " . $peak_groups[$x]{"prev_spin_system_matches"} . "\n"; }
  
  if (exists $peak_groups[$x]{"next_spin_system"})
    { $spin_system .=  "#\tNext Spin System: " . $peak_groups[$x]{"next_spin_system"} . "  Matches: " . $peak_groups[$x]{"next_spin_system_matches"} . "\n"; }
  
  if ($peak_groups[$x]{"Gly_inter_flags"})
    { $spin_system .=  "#\tGly Inter Flags: " . $peak_groups[$x]{"Gly_inter_flags"} . "\n"; }
  
  if ($peak_groups[$x]{"Gly_intra_flags"})
    { $spin_system .=  "#\tGly Intra Flags: " .  $peak_groups[$x]{"Gly_intra_flags"} . "\n"; }
  
  if (exists $peak_groups[$x]{"closest_HN"})
    { $spin_system .=  "#\tClosest Spin System: " . $peak_groups[$x]{"closest_HN"}{"group"} . "  HN: " . $peak_groups[$x]{"closest_HN"}{"HN"} . "  " . $peak_groups[$x]{"closest_HN"}{"N"} . "\n"; }
  
  if (exists $peak_groups[$x]{"sidechain_flags"} && $peak_groups[$x]{"sidechain_flags"} > 1)
    { $spin_system .=  "#\tSidechain Flags: " . $peak_groups[$x]{"sidechain_flags"} . "\n"; }
  
  if (exists $peak_groups[$x]{"sidechain_aa_type"})
    { $spin_system .=  "#\tSidechain AA Type: " . $peak_groups[$x]{"sidechain_aa_type"} . "  ChiProb: " . $peak_groups[$x]{"sidechain_aa_prob"} . "\n"; }
  
  if (exists $peak_groups[$x]{"sidechain_partner"})
    { $spin_system .=  "#\tSidechain Partner Flags: " . $peak_groups[$x]{"sidechain_partner_flags"} . "  Partner: " . $peak_groups[$x]{"sidechain_partner"} . "\n"; }
  
  if (exists $peak_groups[$x]{"sidechain_intra_parent"})
    { $spin_system .=  "#\tSidechain IntraResidue Backbone Spin System: " . $peak_groups[$x]{"sidechain_intra_parent"} . "\n"; }
  
  if (exists $peak_groups[$x]{"sidechain_inter_parent"})
    { $spin_system .=  "#\tSidechain InterResidue Backbone Spin System: ", $peak_groups[$x]{"sidechain_inter_parent"} . "\n"; }

  if (exists $peak_groups[$x]{"shifts"} && %{$peak_groups[$x]{"shifts"}})
    {
    $spin_system .= "#\tPossible Shift Assignments:\n";
    if (grep { exists $peak_groups[$x]{"shifts"}{$_}; } ("H", "N", "CO", "CA", "CB", "HA", "HB"))
      {
      $spin_system .= "#\t\t";
      foreach my $shift (grep { exists $peak_groups[$x]{"shifts"}{$_}; } ("H", "N", "CO", "CA", "CB", "HA", "HB"))
	{ $spin_system .= "$shift = [" . join(", ", @{$peak_groups[$x]{"shifts"}{$shift}}) . "]   "; }
      $spin_system .= "\n";
      }
    if (grep { exists $peak_groups[$x]{"shifts"}{$_}; } ("CO-1", "CA-1", "CB-1", "HA-1", "HB-1"))
      {
      $spin_system .= "#\t\t";
      foreach my $shift (grep { exists $peak_groups[$x]{"shifts"}{$_}; } ("CO-1", "CA-1", "CB-1", "HA-1", "HB-1"))
	{ $spin_system .= "$shift = [" . join(", ", @{$peak_groups[$x]{"shifts"}{$shift}}) . "]   "; }
      $spin_system .= "\n";
      }
    }
  
  $spin_system .=  "#\n#Peaks:\n";
  foreach my $peak (@{$peak_groups[$x]{"peaks"}})
    {
    my @temp = @$peak;
    for(my $y=1; $y < @temp; $y++)
      { $temp[$y] =  ("\t" x (length($temp[$y-1]) < 9)) . $temp[$y]; }
    $spin_system .=  join("\t",@temp) . "\n";
    }
  
  if (! $error_only || (exists $peak_groups[$x]{"overlap_flags"} || exists $peak_groups[$x]{"missing_flags"}))
    { print OUTPUT_FILE "#\n#\n#\n#\n# Spin System $x\n#\n", $spin_system; }

  $spin_system =~ s/\#//g;
  $spin_system =~ s/Spin System: (\d+)/Spin System: \<a href=\\\"javascript:display_ss($1)\\\"\>$1\<\/a\>/g;
  $spin_system =~ s/Partner: (\d+)/Partner: \<a href=\\\"javascript:display_ss($1)\\\"\>$1<\/a\>/g;
  $spin_system =~ s/\n/\<br\>/g;
  $html_ss_javascript_switch .= "case $x\:\nreturn \"<br><big><big><div style=\\\"margin-left: 80px;\\\"><h2>Spin System $x</h2><div style=\\\"margin-left: 40px;\\\"><pre>" . $spin_system . "</pre></div></div></big></big>\"; break;\n";
  }

# Print Comments
print OUTPUT_FILE "*\n";

my $line;
my $history_found = 0;
foreach $line (@$end_comments_alist)
  {
  if (! $history_found && ($line =~ /^File History:/))
    {
    print OUTPUT_FILE "File History: $out_peak_file \n\n";
    print OUTPUT_FILE "$command_line\n";
    $history_found = 1;
    next;
    }
  
  print OUTPUT_FILE $line;  
  }

if (! $history_found)
  {
  print OUTPUT_FILE "\nFile History: $out_peak_file \n\n";
  print OUTPUT_FILE "$command_line\n";
  }

close OUTPUT_FILE;

if ($html_filename ne "")
  {
  my $all_list = "<a href=\\\"javascript:display_previous(all_array)\\\">Previous</a> &nbsp; &nbsp; " . join("  &nbsp;",map { "<a href=\\\"javascript:display_ss2($_,all_array)\\\">$_</a>"; } (0 .. $#peak_groups)) . " &nbsp; &nbsp; <a href=\\\"javascript:display_next(all_array)\\\">Next</a> &nbsp; &nbsp; &nbsp; &nbsp; <a href=\\\"javascript:display_all_ss(all_array)\\\">All</a>";
  my $all_array = "var all_array = new Array(); " . join(" ",map { "all_array.push($_);"; } (0 .. $#peak_groups)) . " all_array.unshift(1);";
  my $error_list = "<a href=\\\"javascript:display_previous(error_array)\\\">Previous</a> &nbsp; &nbsp; " . join("  &nbsp;",map { "<a href=\\\"javascript:display_ss2($_,error_array)\\\">$_</a>"; } (grep { ! $peak_groups[$_]{"deleted_flag"} && ($peak_groups[$_]{"overlap_flags"} || $peak_groups[$_]{"missing_flags"}); } (0 .. $#peak_groups))) . " &nbsp; &nbsp; <a href=\\\"javascript:display_next(error_array)\\\">Next</a> &nbsp; &nbsp; &nbsp; &nbsp; <a href=\\\"javascript:display_all_ss(error_array)\\\">All</a>";
  my $error_array = "var error_array = new Array(); " . join(" ",map { "error_array.push($_);"; } (grep { ! $peak_groups[$_]{"deleted_flag"} && ($peak_groups[$_]{"overlap_flags"} || $peak_groups[$_]{"missing_flags"}); } (0 .. $#peak_groups))) . " error_array.unshift(1);";
  my $overlap_list = "<a href=\\\"javascript:display_previous(overlap_array)\\\">Previous</a> &nbsp; &nbsp; " . join("  &nbsp;",map { "<a href=\\\"javascript:display_ss2($_,overlap_array)\\\">$_</a>"; } (grep { ! $peak_groups[$_]{"deleted_flag"} && $peak_groups[$_]{"overlap_flags"}; } (0 .. $#peak_groups))) . " &nbsp; &nbsp; <a href=\\\"javascript:display_next(overlap_array)\\\">Next</a> &nbsp; &nbsp; &nbsp; &nbsp; <a href=\\\"javascript:display_all_ss(overlap_array)\\\">All</a>";
  my $overlap_array = "var overlap_array = new Array(); " . join(" ",map { "overlap_array.push($_);"; } (grep { ! $peak_groups[$_]{"deleted_flag"} && $peak_groups[$_]{"overlap_flags"}; } (0 .. $#peak_groups))) . " overlap_array.unshift(1);";
  my $incomplete_list = "<a href=\\\"javascript:display_previous(incomplete_array)\\\">Previous</a> &nbsp; &nbsp; " . join("  &nbsp;",map { "<a href=\\\"javascript:display_ss2($_,incomplete_array)\\\">$_</a>"; } (grep { ! $peak_groups[$_]{"deleted_flag"} && $peak_groups[$_]{"missing_flags"}; } (0 .. $#peak_groups))) . " &nbsp; &nbsp; <a href=\\\"javascript:display_next(incomplete_array)\\\">Next</a> &nbsp; &nbsp; &nbsp; &nbsp; <a href=\\\"javascript:display_all_ss(incomplete_array)\\\">All</a>";
  my $incomplete_array = "var incomplete_array = new Array(); " . join(" ",map { "incomplete_array.push($_);"; } (grep { ! $peak_groups[$_]{"deleted_flag"} && $peak_groups[$_]{"missing_flags"}; } (0 .. $#peak_groups))) . " incomplete_array.unshift(1);";
  my $gly_list = "<a href=\\\"javascript:display_previous(gly_array)\\\">Previous</a> &nbsp; &nbsp; " . join("  &nbsp;",map { "<a href=\\\"javascript:display_ss2($_,gly_array)\\\">$_</a>"; } (grep { ! $peak_groups[$_]{"deleted_flag"} && ($peak_groups[$_]{"Gly_inter_flags"} || $peak_groups[$_]{"Gly_intra_flags"}); } (0 .. $#peak_groups))) . " &nbsp; &nbsp; <a href=\\\"javascript:display_next(gly_array)\\\">Next</a> &nbsp; &nbsp; &nbsp; &nbsp; <a href=\\\"javascript:display_all_ss(gly_array)\\\">All</a>";
  my $gly_array = "var gly_array = new Array(); " . join(" ",map { "gly_array.push($_);"; } (grep { ! $peak_groups[$_]{"deleted_flag"} && ($peak_groups[$_]{"Gly_inter_flags"} || $peak_groups[$_]{"Gly_intra_flags"}); } (0 .. $#peak_groups))) . " gly_array.unshift(1);";
  my $inter_gly_list = "<a href=\\\"javascript:display_previous(inter_gly_array)\\\">Previous</a> &nbsp; &nbsp; " . join("  &nbsp;",map { "<a href=\\\"javascript:display_ss2($_,inter_gly_array)\\\">$_</a>"; } (grep { ! $peak_groups[$_]{"deleted_flag"} && $peak_groups[$_]{"Gly_inter_flags"}; } (0 .. $#peak_groups))) . " &nbsp; &nbsp; <a href=\\\"javascript:display_next(inter_gly_array)\\\">Next</a> &nbsp; &nbsp; &nbsp; &nbsp; <a href=\\\"javascript:display_all_ss(inter_gly_array)\\\">All</a>";
  my $inter_gly_array = "var inter_gly_array = new Array(); " . join(" ",map { "inter_gly_array.push($_);"; } (grep { ! $peak_groups[$_]{"deleted_flag"} && $peak_groups[$_]{"Gly_inter_flags"}; } (0 .. $#peak_groups))) . " inter_gly_array.unshift(1);";
  my $intra_gly_list = "<a href=\\\"javascript:display_previous(intra_gly_array)\\\">Previous</a> &nbsp; &nbsp; " . join("  &nbsp;",map { "<a href=\\\"javascript:display_ss2($_,intra_gly_array)\\\">$_</a>"; } (grep { ! $peak_groups[$_]{"deleted_flag"} && $peak_groups[$_]{"Gly_intra_flags"}; } (0 .. $#peak_groups))) . " &nbsp; &nbsp; <a href=\\\"javascript:display_next(intra_gly_array)\\\">Next</a> &nbsp; &nbsp; &nbsp; &nbsp; <a href=\\\"javascript:display_all_ss(intra_gly_array)\\\">All</a>";
  my $intra_gly_array = "var intra_gly_array = new Array(); " . join(" ",map { "intra_gly_array.push($_);"; } (grep { ! $peak_groups[$_]{"deleted_flag"} && $peak_groups[$_]{"Gly_intra_flags"}; } (0 .. $#peak_groups))) . " intra_gly_array.unshift(1);";
  my $sidechain_list = "<a href=\\\"javascript:display_previous(sidechain_array)\\\">Previous</a> &nbsp; &nbsp; " . join("  &nbsp;",map { "<a href=\\\"javascript:display_ss2($_,sidechain_array)\\\">$_</a>"; } (grep { ! $peak_groups[$_]{"deleted_flag"} && $peak_groups[$_]{"sidechain_flags"} > 1; } (0 .. $#peak_groups))) . " &nbsp; &nbsp; <a href=\\\"javascript:display_next(sidechain_array)\\\">Next</a> &nbsp; &nbsp; &nbsp; &nbsp; <a href=\\\"javascript:display_all_ss(sidechain_array)\\\">All</a>";
  my $sidechain_array = "var sidechain_array = new Array(); " . join(" ",map { "sidechain_array.push($_);"; } (grep { ! $peak_groups[$_]{"deleted_flag"} && $peak_groups[$_]{"sidechain_flags"} > 1; } (0 .. $#peak_groups))) . " sidechain_array.unshift(1);";
  my $deleted_list = "<a href=\\\"javascript:display_previous(deleted_array)\\\">Previous</a> &nbsp; &nbsp; " . join("  &nbsp;",map { "<a href=\\\"javascript:display_ss2($_,deleted_array)\\\">$_</a>"; } (grep { $peak_groups[$_]{"deleted_flag"}; } (0 .. $#peak_groups))) . " &nbsp; &nbsp; <a href=\\\"javascript:display_next(deleted_array)\\\">Next</a> &nbsp; &nbsp; &nbsp; &nbsp; <a href=\\\"javascript:display_all_ss(deleted_array)\\\">All</a>";
  my $deleted_array = "var deleted_array = new Array(); " . join(" ",map { "deleted_array.push($_);"; } (grep { $peak_groups[$_]{"deleted_flag"}; } (0 .. $#peak_groups))) . " deleted_array.unshift(1);";
  my $out_tolerance_list = "<a href=\\\"javascript:display_previous(out_tolerance_array)\\\">Previous</a> &nbsp; &nbsp; " . join("  &nbsp;",map { "<a href=\\\"javascript:display_ss2($_,out_tolerance_array)\\\">$_</a>"; } (grep { exists $peak_groups[$_]{"out_tolerance_list"} && @{$peak_groups[$_]{"out_tolerance_list"}}; } (0 .. $#peak_groups))) . " &nbsp; &nbsp; <a href=\\\"javascript:display_next(out_tolerance_array)\\\">Next</a> &nbsp; &nbsp; &nbsp; &nbsp; <a href=\\\"javascript:display_all_ss(out_tolerance_array)\\\">All</a>";
  my $out_tolerance_array = "var out_tolerance_array = new Array(); " . join(" ",map { "out_tolerance_array.push($_);"; } (grep { $peak_groups[$_]{"out_tolerance_list"} && @{$peak_groups[$_]{"out_tolerance_list"}}; } (0 .. $#peak_groups))) . " out_tolerance_array.unshift(1);";

  $peak_list_summary =~ s/\n/\<br\>/g;
  $spin_system_summary =~ s/\n/\<br\>/g;
  $deleted_peaks_summary =~ s/\n/\<br\>/g;


  my $html_output = <<"ENDHTML1";
<!DOCTYPE html
PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
<head>
<title>Peak Lists Quality Assessment Report</title>
<script type="text/javascript">
function display_summary()
  { document.getElementById("area_1").innerHTML = "<div style=\\\"margin-left: 40px;\\\"><big><h2 style=\\\"text-align: center;\\\">Peak List Based Summary</h2></big><div style=\\\"margin-left: 40px;\\\"><big><big><pre>$peak_list_summary<br></pre></big></big></div><big><h2 style=\\\"text-align: center;\\\">Spin System Based Summary</h2></big><div style=\\\"margin-left: 40px;\\\"><big><big><pre>$spin_system_summary<br></pre></big></big></div></div>"; }
function display_deleted_peaks()
  { document.getElementById("area_1").innerHTML = "<div style=\\\"margin-left: 40px;\\\"><big><h2 style=\\\"text-align: center;\\\">Deleted Peaks</h2></big><div style=\\\"margin-left: 40px;\\\"><big><big><pre>$deleted_peaks_summary<br></pre></big></big></div>"; }
function display_all_list()
  { document.getElementById("area_1").innerHTML = "<div style=\\\"margin-left: 40px;\\\"><big><h2 style=\\\"text-align: center;\\\">All Spin Systems</h2></big><div style=\\\"margin-left: 40px;\\\"><big><big>$all_list<br></big></big></div></div>"; }
function display_error_list()
  { document.getElementById("area_1").innerHTML = "<div style=\\\"margin-left: 40px;\\\"><big><h2 style=\\\"text-align: center;\\\">Error Spin Systems</h2></big><div style=\\\"margin-left: 40px;\\\"><big><big>$error_list<br></big></big></div></div>"; }
function display_overlap_list()
  { document.getElementById("area_1").innerHTML = "<div style=\\\"margin-left: 40px;\\\"><big><h2 style=\\\"text-align: center;\\\">Overlapped Spin Systems</h2></big><div style=\\\"margin-left: 40px;\\\"><big><big>$overlap_list<br></big></big></div></div>"; }
function display_incomplete_list()
  { document.getElementById("area_1").innerHTML = "<div style=\\\"margin-left: 40px;\\\"><big><h2 style=\\\"text-align: center;\\\">Incomplete Spin Systems</h2></big><div style=\\\"margin-left: 40px;\\\"><big><big>$incomplete_list<br></big></big></div></div>"; }
function display_gly_list()
  { document.getElementById("area_1").innerHTML = "<div style=\\\"margin-left: 40px;\\\"><big><h2 style=\\\"text-align: center;\\\">Gly Spin Systems</h2></big><div style=\\\"margin-left: 40px;\\\"><big><big>$gly_list<br></big></big></div></div>"; }
function display_inter_gly_list()
  { document.getElementById("area_1").innerHTML = "<div style=\\\"margin-left: 40px;\\\"><big><h2 style=\\\"text-align: center;\\\">Inter Gly Spin Systems</h2></big><div style=\\\"margin-left: 40px;\\\"><big><big>$inter_gly_list<br></big></big></div></div>"; }
function display_intra_gly_list()
  { document.getElementById("area_1").innerHTML = "<div style=\\\"margin-left: 40px;\\\"><big><h2 style=\\\"text-align: center;\\\">Intra Gly Spin Systems</h2></big><div style=\\\"margin-left: 40px;\\\"><big><big>$intra_gly_list<br></big></big></div></div>"; }
function display_sidechain_list()
  {   document.getElementById("area_1").innerHTML = "<div style=\\\"margin-left: 40px;\\\"><big><h2 style=\\\"text-align: center;\\\">Sidechain NH Spin Systems</h2></big><div style=\\\"margin-left: 40px;\\\"><big><big>$sidechain_list<br></big></big></div></div>"; }  
function display_deleted_list()
  {   document.getElementById("area_1").innerHTML = "<div style=\\\"margin-left: 40px;\\\"><big><h2 style=\\\"text-align: center;\\\">Deleted Spin Systems</h2></big><div style=\\\"margin-left: 40px;\\\"><big><big>$deleted_list<br></big></big></div></div>"; }  
function display_out_tolerance_list()
  {   document.getElementById("area_1").innerHTML = "<div style=\\\"margin-left: 40px;\\\"><big><h2 style=\\\"text-align: center;\\\">Spin Systems with Out of Tolerance Peaks</h2></big><div style=\\\"margin-left: 40px;\\\"><big><big>$out_tolerance_list<br></big></big></div></div>"; }  
function get_ss_string(ss_num)
  {
  switch(ss_num)
    {
    $html_ss_javascript_switch;

    default :
      return "";
    }

  }

function display_ss(ss_num)
  {
  var display_string = new String();
  display_string = get_ss_string(ss_num);
  if (display_string != "")
    document.getElementById("area_2").innerHTML = display_string;
  }

function display_all_ss(array_var)
  {
  if (array_var.length > 1)
    {
    var display_string = new String();
    for(i=1; i < array_var.length; i++)
      {
      display_string += get_ss_string(array_var[i]);
      }

    if (display_string != "")
      document.getElementById("area_2").innerHTML = display_string;
    }
  }


  $all_array;
  $error_array;
  $overlap_array;
  $incomplete_array;
  $gly_array;
  $inter_gly_array;
  $intra_gly_array;
  $sidechain_array;
  $deleted_array;
  $out_tolerance_array;
function display_ss2(ss_num,array_var)
  {
  for(i=1; i < array_var.length; i++)
    {
    if (ss_num == array_var[i])
      { array_var[0] = i; }
    }

  display_ss(ss_num);
  }

  function display_next(array_var)
    {
    if (array_var.length > 1)
      {
      array_var[0]++;
      if (array_var[0] >= array_var.length)
	{ array_var[0] = 1; }

      display_ss(array_var[array_var[0]]);
      }
    }
  function display_previous(array_var)
    {
    if (array_var.length > 1)
      {
      array_var[0]--;
      if (array_var[0] < 1)
	{ array_var[0] = array_var.length-1; }

      display_ss(array_var[array_var[0]]);
      }
    }
</script>

</head>


<body>
<h1 style="text-align: center;"><b><font><b><font face="Arial,Helvetica,Geneva,Swiss,SunSans-Regular">Peak Lists Quality Assessment Report<br></font></b></font></b></h1>
<font size="1">

<form>
<input type="button" id="Summary" value="Summary" onclick="display_summary()" />
<input type="button" id="All List" value="All List" onclick="display_all_list()" />
<input type="button" id="Error List" value="Error List" onclick="display_error_list()" />
<input type="button" id="Overlap List" value="Overlap List" onclick="display_overlap_list()" />
<input type="button" id="Incomplete List" value="Incomplete List" onclick="display_incomplete_list()" />
<input type="button" id="Gly List" value="Gly List" onclick="display_gly_list()" />
<input type="button" id="Inter Gly List" value="Inter Gly List" onclick="display_inter_gly_list()" />
<input type="button" id="Intra Gly List" value="Intra Gly List" onclick="display_intra_gly_list()" />
<input type="button" id="Sidechain NH List" value="Sidechain NH List" onclick="display_sidechain_list()" />
<input type="button" id="Deleted List" value="Deleted List" onclick="display_deleted_list()" />
<input type="button" id="Out of Tolerance List" value="Out of Tolerance List" onclick="display_out_tolerance_list()" />
<input type="button" id="Deleted Peaks" value="Deleted Peaks" onclick="display_deleted_peaks()" />
</form>

<hr style="width: 90%; height: 4px;">
<a id="area_1">  </a>

<hr style="width: 90%; height: 4px;">
<a id="area_2">  </a>

<script type="text/javascript">
display_all_list();
display_ss(0);
</script>

</body>
</html>
ENDHTML1


  local *HTML_FILE;
  open(HTML_FILE, ">$html_filename") || die ("open " . $html_filename . " failed."); 
  print HTML_FILE $html_output;
  close HTML_FILE;
  }

#
#  Subroutines
#

# read_peak_file
#   Reads an AutoAssign peak list file and returns an array of peaks.
# 
# Parameters
#	$filename - name of peak list file.
#
sub read_peak_file 
  {
  my $filename = shift @_;
    
  local *PEAKFILE;
  if ($filename eq "-")
    { *PEAKFILE = *STDIN; }
  else
    { open(PEAKFILE, $filename) || die ("open " . $filename . " failed."); }

  my $peak_alist = [];
  my $line;
  while ($line = <PEAKFILE>) 
    {
    next if ($line =~ /^\#/); # skip comment lines
    last if ($line =~ /^[*]/); # end loop if end_of_file marker is encountered.
    
    ($line) = ($line =~/^\s*(.*)/); # skip leading white space.
    chomp $line;
    push @$peak_alist, [ split(/\s+/, $line) ];
    }

  my $comment_alist = [];
  while ($line = <PEAKFILE>)
    { push @$comment_alist, $line; }

  close PEAKFILE;
  return wantarray ? ($peak_alist, $comment_alist) : $peak_alist;
  }

# valid_grouping
#   Tests whether a peak can be joined to a peakgroup
#
# Parameters:
#	$peak1 - ref to peak 1.
#	$peakgroup - ref to array of peaks.
#	$tolerance_alist - ref to array of tolerances.
sub valid_grouping
  {
  my $peak = shift @_;
  my $peakgroup = shift @_;
  my $tolerance_alist = shift @_;

  my $peak_group_id = &group_id($peak);
  my $peak_group_id2 = &group_id($$peakgroup[0]);

  return ( (((($peak_group_id eq "") && ($peak_group_id2 eq "")) || $ignore_peak_based_overrides) && (grep { &within_tolerances($_, $peak, $tolerance_alist); } (@$peakgroup))) ||
	  (! $ignore_peak_based_overrides && ($peak_group_id ne "") && ($peak_group_id2 ne "") && ($peak_group_id eq $peak_group_id2)));     
  }

# within_tolerances
#   Tests whether two peaks are within tolerances
#
# Parameters:
#	$peak1 - ref to peak 1.
#	$peak2 - ref to peak 2.
#	$tolerance_alist - ref to array of tolerances.
#
sub within_tolerances
  {
  my $peak = shift @_;
  my $peak2 = shift @_;
  my $tolerance_alist = shift @_;

  my $x;
  for($x=0; $x < @$tolerance_alist; $x++)
    {
    if (($$tolerance_alist[$x] != 0.0) && (abs($$peak[$x+1] - $$peak2[$x+1]) >= $$tolerance_alist[$x]))
      { return 0; }
    }

  return 1;
  }



#
# Peak-based subroutines
#


# peak_wb
#   Returns the peak's workbook name
#
# Parameters:
#	$peak - ref to peak.
#
sub peak_wb
  {
  my $peak = shift @_;

  my ($wb) = ($$peak[$#$peak] =~ /^(\w+)/); 
  return $wb;
  }

# Test for being a root peak
sub is_root_peak
  {
  my $wb = &peak_wb(@_);
  return ($wb eq "HSQC") || ($wb eq "HNCO");
  }

#
sub root_peaks
  { return grep { &is_root_peak($_); } (@_); }

# peak_id
#   Returns the peak id.
#
# Parameters:
#	$peak - ref to peak.
#
sub peak_id
  {
  my $peak = shift @_;

  return &peak_wb($peak) . "." . $$peak[0];
  }

# group_id
#   Returns any positive group id
#
# Parameters:
#	$peak - ref to peak.
#
sub group_id
  {
  my $peak = shift @_;

  if ($$peak[$#$peak] =~ /\;[Gg](\d+)/)
    { return $1; }

  return "";
  }


# PRTL_size
#   Creates and returns a PRTL by calling the appropriate Bayesian Typing method
#
# Parameters:
#	$res_shift_hlist - hash of shift_name to ChemicalShift
#	$PRTL_props - hash of properties used by Bayesian Typing methods
#	$cutoff - probability cutoff to use.
#
sub PRTL_size
  {
  my $res_shift_hlist = shift @_;
  my $PRTL_props = shift @_;
  my $cutoff = shift @_;
  
  my @prob_list = ();
  my $total_prob = 0.0;
  
  # creates an array of arrays where each row is the aminoacid type and the its probability
  foreach my $aa_type (@amino_acid_alist2)
    { 
    my $used_res_shift_hlist;
    
    my ($prob_value, $junk) = &allBayesianProbCombinationsCXrecomb2($aa_type, $res_shift_hlist, $PRTL_props, 0, 0);
    push @prob_list, [ $aa_type, $prob_value ];
    $total_prob += $prob_list[$#prob_list][1]; 
    }
  
  
  # normalize each probablilty by the total
  if ($total_prob > 0)
    {
    foreach my $prob (@prob_list)
      { $$prob[1] = $$prob[1] / $total_prob; } 
    }
  
  # return the probabilites in descending order
  @prob_list = (sort { $$b[1] <=> $$a[1] } @prob_list);
  
  my $sum = 0;
  
  return scalar(grep {my $test = ($sum <= $cutoff); $sum += $$_[1]; $test; } @prob_list);
  }



sub initialize_amino_acid_stats
  {
  my $deuteration = shift @_;

  my $count = {};
  my $mean = {};
  my $variance = {};
  my $min = {};
  my $max = {};

  $$min{A}{H} = 3.97;
  $$max{A}{H} = 11.58;
  $$mean{A}{H} = 8.20;
  $$count{A}{H} = 1;
  $$variance{A}{H} = 0.60 * 0.60;
  
  $$min{A}{HA} = 1.92;
  $$max{A}{HA} = 6.16;
  $$mean{A}{HA} = 4.26;
  $$count{A}{HA} = 1;
  $$variance{A}{HA} = 0.42 * 0.42;
  
  $$min{A}{HB} = -0.62;
  $$max{A}{HB} = 2.63;
  $$mean{A}{HB} = 1.38;
  $$count{A}{HB} = 1;
  $$variance{A}{HB} = 0.25 * 0.25;
  
  $$min{A}{C} = 168.80;
  $$max{A}{C} = 183.70;
  $$mean{A}{C} = 177.80;
  $$count{A}{C} = 1;
  $$variance{A}{C} = 2.18 * 2.18;
  
  $$min{A}{CA} = 43.00;
  $$max{A}{CA} = 63.50;
  $$mean{A}{CA} = 53.16;
  $$count{A}{CA} = 1;
  $$variance{A}{CA} = 2.06 * 2.06;
  
  $$min{A}{CB} = 0.00;
  $$max{A}{CB} = 26.30;
  $$mean{A}{CB} = 18.90;
  $$count{A}{CB} = 1;
  $$variance{A}{CB} = 1.85 * 1.85;
  
  $$min{A}{N} = 99.44;
  $$max{A}{N} = 137.20;
  $$mean{A}{N} = 123.23;
  $$count{A}{N} = 1;
  $$variance{A}{N} = 3.70 * 3.70;
  
  
  $$min{R}{H} = 5.02;
  $$max{R}{H} = 11.38;
  $$mean{R}{H} = 8.24;
  $$count{R}{H} = 1;
  $$variance{R}{H} = 0.60 * 0.60;
  
  $$min{R}{HA} = 2.12;
  $$max{R}{HA} = 5.94;
  $$mean{R}{HA} = 4.27;
  $$count{R}{HA} = 1;
  $$variance{R}{HA} = 0.44 * 0.44;
  
  $$min{R}{HB2} = -0.38;
  $$max{R}{HB2} = 3.29;
  $$mean{R}{HB2} = 1.79;
  $$count{R}{HB2} = 1;
  $$variance{R}{HB2} = 0.28 * 0.28;
  
  $$min{R}{HB3} = 0.00;
  $$max{R}{HB3} = 3.29;
  $$mean{R}{HB3} = 1.78;
  $$count{R}{HB3} = 1;
  $$variance{R}{HB3} = 0.28 * 0.28;
  
  $$min{R}{HG2} = -0.64;
  $$max{R}{HG2} = 3.12;
  $$mean{R}{HG2} = 1.58;
  $$count{R}{HG2} = 1;
  $$variance{R}{HG2} = 0.27 * 0.27;
  
  $$min{R}{HG3} = -0.23;
  $$max{R}{HG3} = 3.12;
  $$mean{R}{HG3} = 1.58;
  $$count{R}{HG3} = 1;
  $$variance{R}{HG3} = 0.27 * 0.27;
  
  $$min{R}{HD2} = 1.43;
  $$max{R}{HD2} = 4.28;
  $$mean{R}{HD2} = 3.13;
  $$count{R}{HD2} = 1;
  $$variance{R}{HD2} = 0.23 * 0.23;
  
  $$min{R}{HD3} = 1.22;
  $$max{R}{HD3} = 4.28;
  $$mean{R}{HD3} = 3.13;
  $$count{R}{HD3} = 1;
  $$variance{R}{HD3} = 0.22 * 0.22;
  
  $$min{R}{HE} = 3.25;
  $$max{R}{HE} = 11.82;
  $$mean{R}{HE} = 7.32;
  $$count{R}{HE} = 1;
  $$variance{R}{HE} = 0.52 * 0.52;
  
  $$min{R}{HH11} = 5.82;
  $$max{R}{HH11} = 8.76;
  $$mean{R}{HH11} = 6.74;
  $$count{R}{HH11} = 1;
  $$variance{R}{HH11} = 0.31 * 0.31;
  
  $$min{R}{HH12} = 5.82;
  $$max{R}{HH12} = 8.76;
  $$mean{R}{HH12} = 6.75;
  $$count{R}{HH12} = 1;
  $$variance{R}{HH12} = 0.32 * 0.32;
  
  $$min{R}{HH21} = 4.59;
  $$max{R}{HH21} = 7.62;
  $$mean{R}{HH21} = 6.72;
  $$count{R}{HH21} = 1;
  $$variance{R}{HH21} = 0.32 * 0.32;
  
  $$min{R}{HH22} = 4.59;
  $$max{R}{HH22} = 7.62;
  $$mean{R}{HH22} = 6.72;
  $$count{R}{HH22} = 1;
  $$variance{R}{HH22} = 0.33 * 0.33;
  
  $$min{R}{C} = 168.67;
  $$max{R}{C} = 181.50;
  $$mean{R}{C} = 176.49;
  $$count{R}{C} = 1;
  $$variance{R}{C} = 2.16 * 2.16;
  
  $$min{R}{CA} = 46.10;
  $$max{R}{CA} = 65.13;
  $$mean{R}{CA} = 56.95;
  $$count{R}{CA} = 1;
  $$variance{R}{CA} = 2.41 * 2.41;
  
  $$min{R}{CB} = 24.50;
  $$max{R}{CB} = 38.27;
  $$mean{R}{CB} = 30.66;
  $$count{R}{CB} = 1;
  $$variance{R}{CB} = 1.77 * 1.77;
  
  $$min{R}{CG} = 18.22;
  $$max{R}{CG} = 49.39;
  $$mean{R}{CG} = 27.31;
  $$count{R}{CG} = 1;
  $$variance{R}{CG} = 1.65 * 1.65;
  
  $$min{R}{CD} = 29.71;
  $$max{R}{CD} = 45.60;
  $$mean{R}{CD} = 43.10;
  $$count{R}{CD} = 1;
  $$variance{R}{CD} = 1.10 * 1.10;
  
  $$min{R}{CZ} = 157.52;
  $$max{R}{CZ} = 160.60;
  $$mean{R}{CZ} = 159.01;
  $$count{R}{CZ} = 1;
  $$variance{R}{CZ} = 0.87 * 0.87;
  
  $$min{R}{N} = 103.60;
  $$max{R}{N} = 137.60;
  $$mean{R}{N} = 120.61;
  $$count{R}{N} = 1;
  $$variance{R}{N} = 3.90 * 3.90;
  
  $$min{R}{NE} = 7.28;
  $$max{R}{NE} = 128.90;
  $$mean{R}{NE} = 89.80;
  $$count{R}{NE} = 1;
  $$variance{R}{NE} = 12.83 * 12.83;
  
  $$min{R}{NH1} = 70.10;
  $$max{R}{NH1} = 112.50;
  $$mean{R}{NH1} = 74.78;
  $$count{R}{NH1} = 1;
  $$variance{R}{NH1} = 11.38 * 11.38;
  
  $$min{R}{NH2} = 70.10;
  $$max{R}{NH2} = 111.90;
  $$mean{R}{NH2} = 75.84;
  $$count{R}{NH2} = 1;
  $$variance{R}{NH2} = 11.44 * 11.44;
  
  
  $$min{D}{H} = 6.12;
  $$max{D}{H} = 11.54;
  $$mean{D}{H} = 8.33;
  $$count{D}{H} = 1;
  $$variance{D}{H} = 0.57 * 0.57;
  
  $$min{D}{HA} = 2.59;
  $$max{D}{HA} = 6.33;
  $$mean{D}{HA} = 4.61;
  $$count{D}{HA} = 1;
  $$variance{D}{HA} = 0.31 * 0.31;
  
  $$min{D}{HB2} = 1.25;
  $$max{D}{HB2} = 6.60;
  $$mean{D}{HB2} = 2.74;
  $$count{D}{HB2} = 1;
  $$variance{D}{HB2} = 0.29 * 0.29;
  
  $$min{D}{HB3} = 0.65;
  $$max{D}{HB3} = 4.58;
  $$mean{D}{HB3} = 2.70;
  $$count{D}{HB3} = 1;
  $$variance{D}{HB3} = 0.29 * 0.29;
  
  $$min{D}{C} = 166.80;
  $$max{D}{C} = 181.65;
  $$mean{D}{C} = 176.41;
  $$count{D}{C} = 1;
  $$variance{D}{C} = 1.77 * 1.77;
  
  $$min{D}{CA} = 39.80;
  $$max{D}{CA} = 60.37;
  $$mean{D}{CA} = 54.52;
  $$count{D}{CA} = 1;
  $$variance{D}{CA} = 2.10 * 2.10;
  
  $$min{D}{CB} = 29.51;
  $$max{D}{CB} = 48.30;
  $$mean{D}{CB} = 40.70;
  $$count{D}{CB} = 1;
  $$variance{D}{CB} = 1.67 * 1.67;
  
  $$min{D}{CG} = 173.49;
  $$max{D}{CG} = 181.31;
  $$mean{D}{CG} = 178.41;
  $$count{D}{CG} = 1;
  $$variance{D}{CG} = 1.84 * 1.84;
  
  $$min{D}{N} = 105.50;
  $$max{D}{N} = 136.80;
  $$mean{D}{N} = 120.73;
  $$count{D}{N} = 1;
  $$variance{D}{N} = 4.20 * 4.20;
  
  
  $$min{N}{H} = 3.61;
  $$max{N}{H} = 12.40;
  $$mean{N}{H} = 8.37;
  $$count{N}{H} = 1;
  $$variance{N}{H} = 0.65 * 0.65;
  
  $$min{N}{HA} = 2.92;
  $$max{N}{HA} = 6.32;
  $$mean{N}{HA} = 4.70;
  $$count{N}{HA} = 1;
  $$variance{N}{HA} = 0.39 * 0.39;
  
  $$min{N}{HB2} = 0.10;
  $$max{N}{HB2} = 4.47;
  $$mean{N}{HB2} = 2.80;
  $$count{N}{HB2} = 1;
  $$variance{N}{HB2} = 0.34 * 0.34;
  
  $$min{N}{HB3} = 0.26;
  $$max{N}{HB3} = 4.47;
  $$mean{N}{HB3} = 2.78;
  $$count{N}{HB3} = 1;
  $$variance{N}{HB3} = 0.34 * 0.34;
  
  $$min{N}{HD21} = 3.33;
  $$max{N}{HD21} = 9.30;
  $$mean{N}{HD21} = 7.27;
  $$count{N}{HD21} = 1;
  $$variance{N}{HD21} = 0.52 * 0.52;
  
  $$min{N}{HD22} = 3.14;
  $$max{N}{HD22} = 10.33;
  $$mean{N}{HD22} = 7.20;
  $$count{N}{HD22} = 1;
  $$variance{N}{HD22} = 0.52 * 0.52;
  
  $$min{N}{C} = 168.60;
  $$max{N}{C} = 180.20;
  $$mean{N}{C} = 175.16;
  $$count{N}{C} = 1;
  $$variance{N}{C} = 1.76 * 1.76;
  
  $$min{N}{CA} = 43.30;
  $$max{N}{CA} = 62.30;
  $$mean{N}{CA} = 53.43;
  $$count{N}{CA} = 1;
  $$variance{N}{CA} = 1.94 * 1.94;
  
  $$min{N}{CB} = 29.20;
  $$max{N}{CB} = 45.81;
  $$mean{N}{CB} = 38.66;
  $$count{N}{CB} = 1;
  $$variance{N}{CB} = 1.78 * 1.78;
  
  $$min{N}{CG} = 173.50;
  $$max{N}{CG} = 179.43;
  $$mean{N}{CG} = 176.61;
  $$count{N}{CG} = 1;
  $$variance{N}{CG} = 1.33 * 1.33;
  
  $$min{N}{N} = 105.00;
  $$max{N}{N} = 132.70;
  $$mean{N}{N} = 119.13;
  $$count{N}{N} = 1;
  $$variance{N}{N} = 4.42 * 4.42;
  
  $$min{N}{ND2} = 104.00;
  $$max{N}{ND2} = 124.77;
  $$mean{N}{ND2} = 112.83;
  $$count{N}{ND2} = 1;
  $$variance{N}{ND2} = 2.60 * 2.60;
  
  
  $$min{C}{H} = 5.05;
  $$max{C}{H} = 10.74;
  $$mean{C}{H} = 8.42;
  $$count{C}{H} = 1;
  $$variance{C}{H} = 0.66 * 0.66;
  
  $$min{C}{HA} = 1.70;
  $$max{C}{HA} = 6.19;
  $$mean{C}{HA} = 4.73;
  $$count{C}{HA} = 1;
  $$variance{C}{HA} = 0.57 * 0.57;
  
  $$min{C}{HB2} = 0.29;
  $$max{C}{HB2} = 4.65;
  $$mean{C}{HB2} = 2.95;
  $$count{C}{HB2} = 1;
  $$variance{C}{HB2} = 0.43 * 0.43;
  
  $$min{C}{HB3} = -0.28;
  $$max{C}{HB3} = 4.65;
  $$mean{C}{HB3} = 2.98;
  $$count{C}{HB3} = 1;
  $$variance{C}{HB3} = 0.45 * 0.45;
  
  $$min{C}{HG} = 1.38;
  $$max{C}{HG} = 2.22;
  $$mean{C}{HG} = 1.66;
  $$count{C}{HG} = 1;
  $$variance{C}{HG} = 0.48 * 0.48;
  
  $$min{C}{C} = 168.50;
  $$max{C}{C} = 180.71;
  $$mean{C}{C} = 174.78;
  $$count{C}{C} = 1;
  $$variance{C}{C} = 1.99 * 1.99;
  
  $$min{C}{CA} = 49.74;
  $$max{C}{CA} = 66.25;
  $$mean{C}{CA} = 57.43;
  $$count{C}{CA} = 1;
  $$variance{C}{CA} = 3.41 * 3.41;
  
  $$min{C}{CB} = 22.30;
  $$max{C}{CB} = 57.00;
  $$mean{C}{CB} = 34.15;
  $$count{C}{CB} = 1;
  $$variance{C}{CB} = 6.77 * 6.77;
  
  $$min{C}{N} = 106.60;
  $$max{C}{N} = 135.89;
  $$mean{C}{N} = 119.95;
  $$count{C}{N} = 1;
  $$variance{C}{N} = 4.79 * 4.79;
  
  
  $$min{E}{H} = 4.98;
  $$max{E}{H} = 11.72;
  $$mean{E}{H} = 8.34;
  $$count{E}{H} = 1;
  $$variance{E}{H} = 0.61 * 0.61;
  
  $$min{E}{HA} = 1.77;
  $$max{E}{HA} = 6.29;
  $$mean{E}{HA} = 4.26;
  $$count{E}{HA} = 1;
  $$variance{E}{HA} = 0.43 * 0.43;
  
  $$min{E}{HB2} = 0.12;
  $$max{E}{HB2} = 3.21;
  $$mean{E}{HB2} = 2.04;
  $$count{E}{HB2} = 1;
  $$variance{E}{HB2} = 0.22 * 0.22;
  
  $$min{E}{HB3} = 0.06;
  $$max{E}{HB3} = 3.13;
  $$mean{E}{HB3} = 2.04;
  $$count{E}{HB3} = 1;
  $$variance{E}{HB3} = 0.23 * 0.23;
  
  $$min{E}{HG2} = 0.85;
  $$max{E}{HG2} = 3.68;
  $$mean{E}{HG2} = 2.31;
  $$count{E}{HG2} = 1;
  $$variance{E}{HG2} = 0.21 * 0.21;
  
  $$min{E}{HG3} = 1.06;
  $$max{E}{HG3} = 3.68;
  $$mean{E}{HG3} = 2.31;
  $$count{E}{HG3} = 1;
  $$variance{E}{HG3} = 0.21 * 0.21;
  
  $$min{E}{C} = 167.90;
  $$max{E}{C} = 181.70;
  $$mean{E}{C} = 176.93;
  $$count{E}{C} = 1;
  $$variance{E}{C} = 2.11 * 2.11;
  
  $$min{E}{CA} = 46.80;
  $$max{E}{CA} = 66.60;
  $$mean{E}{CA} = 57.42;
  $$count{E}{CA} = 1;
  $$variance{E}{CA} = 2.19 * 2.19;
  
  $$min{E}{CB} = 21.70;
  $$max{E}{CB} = 44.18;
  $$mean{E}{CB} = 30.07;
  $$count{E}{CB} = 1;
  $$variance{E}{CB} = 1.84 * 1.84;
  
  $$min{E}{CG} = 25.10;
  $$max{E}{CG} = 51.92;
  $$mean{E}{CG} = 36.01;
  $$count{E}{CG} = 1;
  $$variance{E}{CG} = 1.51 * 1.51;
  
  $$min{E}{CD} = 177.76;
  $$max{E}{CD} = 184.74;
  $$mean{E}{CD} = 181.94;
  $$count{E}{CD} = 1;
  $$variance{E}{CD} = 1.85 * 1.85;
  
  $$min{E}{N} = 109.80;
  $$max{E}{N} = 137.30;
  $$mean{E}{N} = 120.68;
  $$count{E}{N} = 1;
  $$variance{E}{N} = 3.68 * 3.68;
  
  
  $$min{Q}{H} = 5.93;
  $$max{Q}{H} = 11.94;
  $$mean{Q}{H} = 8.22;
  $$count{Q}{H} = 1;
  $$variance{Q}{H} = 0.62 * 0.62;
  
  $$min{Q}{HA} = 2.39;
  $$max{Q}{HA} = 6.18;
  $$mean{Q}{HA} = 4.28;
  $$count{Q}{HA} = 1;
  $$variance{Q}{HA} = 0.44 * 0.44;
  
  $$min{Q}{HB2} = -0.22;
  $$max{Q}{HB2} = 4.00;
  $$mean{Q}{HB2} = 2.05;
  $$count{Q}{HB2} = 1;
  $$variance{Q}{HB2} = 0.28 * 0.28;
  
  $$min{Q}{HB3} = -0.22;
  $$max{Q}{HB3} = 4.04;
  $$mean{Q}{HB3} = 2.04;
  $$count{Q}{HB3} = 1;
  $$variance{Q}{HB3} = 0.29 * 0.29;
  
  $$min{Q}{HG2} = 0.06;
  $$max{Q}{HG2} = 3.66;
  $$mean{Q}{HG2} = 2.32;
  $$count{Q}{HG2} = 1;
  $$variance{Q}{HG2} = 0.29 * 0.29;
  
  $$min{Q}{HG3} = 0.26;
  $$max{Q}{HG3} = 3.66;
  $$mean{Q}{HG3} = 2.32;
  $$count{Q}{HG3} = 1;
  $$variance{Q}{HG3} = 0.29 * 0.29;
  
  $$min{Q}{HE21} = 5.21;
  $$max{Q}{HE21} = 9.14;
  $$mean{Q}{HE21} = 7.17;
  $$count{Q}{HE21} = 1;
  $$variance{Q}{HE21} = 0.46 * 0.46;
  
  $$min{Q}{HE22} = 4.30;
  $$max{Q}{HE22} = 9.02;
  $$mean{Q}{HE22} = 7.07;
  $$count{Q}{HE22} = 1;
  $$variance{Q}{HE22} = 0.49 * 0.49;
  
  $$min{Q}{C} = 170.37;
  $$max{Q}{C} = 182.22;
  $$mean{Q}{C} = 176.39;
  $$count{Q}{C} = 1;
  $$variance{Q}{C} = 1.96 * 1.96;
  
  $$min{Q}{CA} = 47.70;
  $$max{Q}{CA} = 61.42;
  $$mean{Q}{CA} = 56.62;
  $$count{Q}{CA} = 1;
  $$variance{Q}{CA} = 2.19 * 2.19;
  
  $$min{Q}{CB} = 21.70;
  $$max{Q}{CB} = 41.54;
  $$mean{Q}{CB} = 29.10;
  $$count{Q}{CB} = 1;
  $$variance{Q}{CB} = 1.97 * 1.97;
  
  $$min{Q}{CG} = 26.10;
  $$max{Q}{CG} = 39.44;
  $$mean{Q}{CG} = 33.72;
  $$count{Q}{CG} = 1;
  $$variance{Q}{CG} = 1.10 * 1.10;
  
  $$min{Q}{CD} = 172.80;
  $$max{Q}{CD} = 181.25;
  $$mean{Q}{CD} = 179.68;
  $$count{Q}{CD} = 1;
  $$variance{Q}{CD} = 1.17 * 1.17;
  
  $$min{Q}{N} = 105.10;
  $$max{Q}{N} = 136.00;
  $$mean{Q}{N} = 119.92;
  $$count{Q}{N} = 1;
  $$variance{Q}{N} = 4.01 * 4.01;
  
  $$min{Q}{NE2} = 92.49;
  $$max{Q}{NE2} = 122.30;
  $$mean{Q}{NE2} = 111.81;
  $$count{Q}{NE2} = 1;
  $$variance{Q}{NE2} = 2.20 * 2.20;
  
  
  $$min{G}{H} = 3.15;
  $$max{G}{H} = 12.22;
  $$mean{G}{H} = 8.34;
  $$count{G}{H} = 1;
  $$variance{G}{H} = 0.74 * 0.74;
  
  $$min{G}{HA2} = 0.34;
  $$max{G}{HA2} = 6.17;
  $$mean{G}{HA2} = 3.95;
  $$count{G}{HA2} = 1;
  $$variance{G}{HA2} = 0.40 * 0.40;
  
  $$min{G}{HA3} = 1.27;
  $$max{G}{HA3} = 6.01;
  $$mean{G}{HA3} = 3.93;
  $$count{G}{HA3} = 1;
  $$variance{G}{HA3} = 0.40 * 0.40;
  
  $$min{G}{C} = 165.80;
  $$max{G}{C} = 182.40;
  $$mean{G}{C} = 174.04;
  $$count{G}{C} = 1;
  $$variance{G}{C} = 1.94 * 1.94;
  
  $$min{G}{CA} = 35.50;
  $$max{G}{CA} = 56.69;
  $$mean{G}{CA} = 45.33;
  $$count{G}{CA} = 1;
  $$variance{G}{CA} = 1.42 * 1.42;
  
  $$min{G}{N} = 88.30;
  $$max{G}{N} = 135.66;
  $$mean{G}{N} = 109.91;
  $$count{G}{N} = 1;
  $$variance{G}{N} = 4.29 * 4.29;
  
  
  $$min{H}{H} = 4.60;
  $$max{H}{H} = 10.80;
  $$mean{H}{H} = 8.25;
  $$count{H}{H} = 1;
  $$variance{H}{H} = 0.74 * 0.74;
  
  $$min{H}{HA} = 1.95;
  $$max{H}{HA} = 8.90;
  $$mean{H}{HA} = 4.62;
  $$count{H}{HA} = 1;
  $$variance{H}{HA} = 0.51 * 0.51;
  
  $$min{H}{HB2} = 0.37;
  $$max{H}{HB2} = 8.70;
  $$mean{H}{HB2} = 3.12;
  $$count{H}{HB2} = 1;
  $$variance{H}{HB2} = 0.43 * 0.43;
  
  $$min{H}{HB3} = 0.70;
  $$max{H}{HB3} = 8.70;
  $$mean{H}{HB3} = 3.11;
  $$count{H}{HB3} = 1;
  $$variance{H}{HB3} = 0.45 * 0.45;
  
  $$min{H}{HD1} = 6.29;
  $$max{H}{HD1} = 17.20;
  $$mean{H}{HD1} = 10.14;
  $$count{H}{HD1} = 1;
  $$variance{H}{HD1} = 3.50 * 3.50;
  
  $$min{H}{HD2} = 3.46;
  $$max{H}{HD2} = 8.82;
  $$mean{H}{HD2} = 7.08;
  $$count{H}{HD2} = 1;
  $$variance{H}{HD2} = 0.50 * 0.50;
  
  $$min{H}{HE1} = 3.21;
  $$max{H}{HE1} = 9.60;
  $$mean{H}{HE1} = 8.08;
  $$count{H}{HE1} = 1;
  $$variance{H}{HE1} = 0.57 * 0.57;
  
  $$min{H}{HE2} = 6.87;
  $$max{H}{HE2} = 16.53;
  $$mean{H}{HE2} = 10.43;
  $$count{H}{HE2} = 1;
  $$variance{H}{HE2} = 3.11 * 3.11;
  
  $$min{H}{C} = 167.60;
  $$max{H}{C} = 179.90;
  $$mean{H}{C} = 175.19;
  $$count{H}{C} = 1;
  $$variance{H}{C} = 2.14 * 2.14;
  
  $$min{H}{CA} = 45.80;
  $$max{H}{CA} = 65.40;
  $$mean{H}{CA} = 56.37;
  $$count{H}{CA} = 1;
  $$variance{H}{CA} = 2.40 * 2.40;
  
  $$min{H}{CB} = 23.70;
  $$max{H}{CB} = 40.86;
  $$mean{H}{CB} = 29.95;
  $$count{H}{CB} = 1;
  $$variance{H}{CB} = 2.15 * 2.15;
  
  $$min{H}{CG} = 122.67;
  $$max{H}{CG} = 136.80;
  $$mean{H}{CG} = 130.59;
  $$count{H}{CG} = 1;
  $$variance{H}{CG} = 3.41 * 3.41;
  
  $$min{H}{CD2} = 112.80;
  $$max{H}{CD2} = 138.85;
  $$mean{H}{CD2} = 119.84;
  $$count{H}{CD2} = 1;
  $$variance{H}{CD2} = 3.30 * 3.30;
  
  $$min{H}{CE1} = 115.52;
  $$max{H}{CE1} = 140.50;
  $$mean{H}{CE1} = 136.05;
  $$count{H}{CE1} = 1;
  $$variance{H}{CE1} = 3.58 * 3.58;
  
  $$min{H}{N} = 105.00;
  $$max{H}{N} = 133.70;
  $$mean{H}{N} = 119.44;
  $$count{H}{N} = 1;
  $$variance{H}{N} = 4.52 * 4.52;
  
  $$min{H}{ND1} = 106.60;
  $$max{H}{ND1} = 248.00;
  $$mean{H}{ND1} = 190.74;
  $$count{H}{ND1} = 1;
  $$variance{H}{ND1} = 22.84 * 22.84;
  
  $$min{H}{NE2} = 161.70;
  $$max{H}{NE2} = 231.50;
  $$mean{H}{NE2} = 179.81;
  $$count{H}{NE2} = 1;
  $$variance{H}{NE2} = 13.86 * 13.86;
  
  
  $$min{I}{H} = 3.75;
  $$max{I}{H} = 10.91;
  $$mean{I}{H} = 8.26;
  $$count{I}{H} = 1;
  $$variance{I}{H} = 0.71 * 0.71;
  
  $$min{I}{HA} = 1.03;
  $$max{I}{HA} = 6.30;
  $$mean{I}{HA} = 4.20;
  $$count{I}{HA} = 1;
  $$variance{I}{HA} = 0.56 * 0.56;
  
  $$min{I}{HB} = -0.57;
  $$max{I}{HB} = 2.79;
  $$mean{I}{HB} = 1.80;
  $$count{I}{HB} = 1;
  $$variance{I}{HB} = 0.31 * 0.31;
  
  $$min{I}{HG12} = -2.02;
  $$max{I}{HG12} = 2.57;
  $$mean{I}{HG12} = 1.30;
  $$count{I}{HG12} = 1;
  $$variance{I}{HG12} = 0.40 * 0.40;
  
  $$min{I}{HG13} = -2.04;
  $$max{I}{HG13} = 2.88;
  $$mean{I}{HG13} = 1.24;
  $$count{I}{HG13} = 1;
  $$variance{I}{HG13} = 0.40 * 0.40;
  
  $$min{I}{HG2} = -2.07;
  $$max{I}{HG2} = 1.77;
  $$mean{I}{HG2} = 0.80;
  $$count{I}{HG2} = 1;
  $$variance{I}{HG2} = 0.29 * 0.29;
  
  $$min{I}{HD1} = -1.06;
  $$max{I}{HD1} = 1.94;
  $$mean{I}{HD1} = 0.70;
  $$count{I}{HD1} = 1;
  $$variance{I}{HD1} = 0.30 * 0.30;
  
  $$min{I}{C} = 167.50;
  $$max{I}{C} = 182.70;
  $$mean{I}{C} = 175.82;
  $$count{I}{C} = 1;
  $$variance{I}{C} = 1.95 * 1.95;
  
  $$min{I}{CA} = 51.50;
  $$max{I}{CA} = 69.14;
  $$mean{I}{CA} = 61.59;
  $$count{I}{CA} = 1;
  $$variance{I}{CA} = 2.78 * 2.78;
  
  $$min{I}{CB} = 20.94;
  $$max{I}{CB} = 44.58;
  $$mean{I}{CB} = 38.58;
  $$count{I}{CB} = 1;
  $$variance{I}{CB} = 2.09 * 2.09;
  
  $$min{I}{CG1} = 9.80;
  $$max{I}{CG1} = 58.63;
  $$mean{I}{CG1} = 27.65;
  $$count{I}{CG1} = 1;
  $$variance{I}{CG1} = 2.32 * 2.32;
  
  $$min{I}{CG2} = 7.30;
  $$max{I}{CG2} = 24.40;
  $$mean{I}{CG2} = 17.36;
  $$count{I}{CG2} = 1;
  $$variance{I}{CG2} = 1.60 * 1.60;
  
  $$min{I}{CD1} = 3.20;
  $$max{I}{CD1} = 27.00;
  $$mean{I}{CD1} = 13.41;
  $$count{I}{CD1} = 1;
  $$variance{I}{CD1} = 1.91 * 1.91;
  
  $$min{I}{N} = 99.00;
  $$max{I}{N} = 135.00;
  $$mean{I}{N} = 121.60;
  $$count{I}{N} = 1;
  $$variance{I}{N} = 4.72 * 4.72;
  
  
  $$min{L}{H} = 5.38;
  $$max{L}{H} = 11.75;
  $$mean{L}{H} = 8.22;
  $$count{L}{H} = 1;
  $$variance{L}{H} = 0.65 * 0.65;
  
  $$min{L}{HA} = 2.31;
  $$max{L}{HA} = 6.15;
  $$mean{L}{HA} = 4.32;
  $$count{L}{HA} = 1;
  $$variance{L}{HA} = 0.47 * 0.47;
  
  $$min{L}{HB2} = -1.31;
  $$max{L}{HB2} = 3.02;
  $$mean{L}{HB2} = 1.63;
  $$count{L}{HB2} = 1;
  $$variance{L}{HB2} = 0.34 * 0.34;
  
  $$min{L}{HB3} = -1.21;
  $$max{L}{HB3} = 3.11;
  $$mean{L}{HB3} = 1.57;
  $$count{L}{HB3} = 1;
  $$variance{L}{HB3} = 0.36 * 0.36;
  
  $$min{L}{HG} = -1.06;
  $$max{L}{HG} = 3.90;
  $$mean{L}{HG} = 1.54;
  $$count{L}{HG} = 1;
  $$variance{L}{HG} = 0.33 * 0.33;
  
  $$min{L}{HD1} = -1.23;
  $$max{L}{HD1} = 2.36;
  $$mean{L}{HD1} = 0.77;
  $$count{L}{HD1} = 1;
  $$variance{L}{HD1} = 0.27 * 0.27;
  
  $$min{L}{HD2} = -1.45;
  $$max{L}{HD2} = 2.78;
  $$mean{L}{HD2} = 0.76;
  $$count{L}{HD2} = 1;
  $$variance{L}{HD2} = 0.28 * 0.28;
  
  $$min{L}{C} = 168.90;
  $$max{L}{C} = 189.78;
  $$mean{L}{C} = 176.97;
  $$count{L}{C} = 1;
  $$variance{L}{C} = 2.09 * 2.09;
  
  $$min{L}{CA} = 44.60;
  $$max{L}{CA} = 65.10;
  $$mean{L}{CA} = 55.65;
  $$count{L}{CA} = 1;
  $$variance{L}{CA} = 2.21 * 2.21;
  
  $$min{L}{CB} = 32.60;
  $$max{L}{CB} = 53.70;
  $$mean{L}{CB} = 42.37;
  $$count{L}{CB} = 1;
  $$variance{L}{CB} = 1.86 * 1.86;
  
  $$min{L}{CG} = 20.53;
  $$max{L}{CG} = 37.80;
  $$mean{L}{CG} = 26.77;
  $$count{L}{CG} = 1;
  $$variance{L}{CG} = 1.25 * 1.25;
  
  $$min{L}{CD1} = 11.70;
  $$max{L}{CD1} = 29.60;
  $$mean{L}{CD1} = 24.73;
  $$count{L}{CD1} = 1;
  $$variance{L}{CD1} = 1.71 * 1.71;
  
  $$min{L}{CD2} = 12.50;
  $$max{L}{CD2} = 28.60;
  $$mean{L}{CD2} = 24.10;
  $$count{L}{CD2} = 1;
  $$variance{L}{CD2} = 1.78 * 1.78;
  
  $$min{L}{N} = 109.76;
  $$max{L}{N} = 136.00;
  $$mean{L}{N} = 121.98;
  $$count{L}{N} = 1;
  $$variance{L}{N} = 4.07 * 4.07;
  
  
  $$min{K}{H} = 5.36;
  $$max{K}{H} = 11.53;
  $$mean{K}{H} = 8.22;
  $$count{K}{H} = 1;
  $$variance{K}{H} = 0.64 * 0.64;
  
  $$min{K}{HA} = 0.68;
  $$max{K}{HA} = 7.96;
  $$mean{K}{HA} = 4.28;
  $$count{K}{HA} = 1;
  $$variance{K}{HA} = 0.45 * 0.45;
  
  $$min{K}{HB2} = -0.15;
  $$max{K}{HB2} = 3.94;
  $$mean{K}{HB2} = 1.79;
  $$count{K}{HB2} = 1;
  $$variance{K}{HB2} = 0.25 * 0.25;
  
  $$min{K}{HB3} = -0.37;
  $$max{K}{HB3} = 3.94;
  $$mean{K}{HB3} = 1.78;
  $$count{K}{HB3} = 1;
  $$variance{K}{HB3} = 0.26 * 0.26;
  
  $$min{K}{HG2} = -0.69;
  $$max{K}{HG2} = 3.01;
  $$mean{K}{HG2} = 1.38;
  $$count{K}{HG2} = 1;
  $$variance{K}{HG2} = 0.27 * 0.27;
  
  $$min{K}{HG3} = -0.77;
  $$max{K}{HG3} = 2.99;
  $$mean{K}{HG3} = 1.37;
  $$count{K}{HG3} = 1;
  $$variance{K}{HG3} = 0.29 * 0.29;
  
  $$min{K}{HD2} = 0.32;
  $$max{K}{HD2} = 3.61;
  $$mean{K}{HD2} = 1.61;
  $$count{K}{HD2} = 1;
  $$variance{K}{HD2} = 0.25 * 0.25;
  
  $$min{K}{HD3} = 0.28;
  $$max{K}{HD3} = 3.61;
  $$mean{K}{HD3} = 1.61;
  $$count{K}{HD3} = 1;
  $$variance{K}{HD3} = 0.25 * 0.25;
  
  $$min{K}{HE2} = 1.25;
  $$max{K}{HE2} = 4.32;
  $$mean{K}{HE2} = 2.93;
  $$count{K}{HE2} = 1;
  $$variance{K}{HE2} = 0.20 * 0.20;
  
  $$min{K}{HE3} = 1.49;
  $$max{K}{HE3} = 4.32;
  $$mean{K}{HE3} = 2.93;
  $$count{K}{HE3} = 1;
  $$variance{K}{HE3} = 0.20 * 0.20;
  
  $$min{K}{HZ} = 2.85;
  $$max{K}{HZ} = 9.90;
  $$mean{K}{HZ} = 7.52;
  $$count{K}{HZ} = 1;
  $$variance{K}{HZ} = 0.36 * 0.36;
  
  $$min{K}{C} = 170.15;
  $$max{K}{C} = 181.50;
  $$mean{K}{C} = 176.46;
  $$count{K}{C} = 1;
  $$variance{K}{C} = 2.05 * 2.05;
  
  $$min{K}{CA} = 46.10;
  $$max{K}{CA} = 62.71;
  $$mean{K}{CA} = 56.84;
  $$count{K}{CA} = 1;
  $$variance{K}{CA} = 2.25 * 2.25;
  
  $$min{K}{CB} = 22.90;
  $$max{K}{CB} = 46.60;
  $$mean{K}{CB} = 32.83;
  $$count{K}{CB} = 1;
  $$variance{K}{CB} = 1.88 * 1.88;
  
  $$min{K}{CG} = 17.30;
  $$max{K}{CG} = 31.28;
  $$mean{K}{CG} = 24.91;
  $$count{K}{CG} = 1;
  $$variance{K}{CG} = 1.31 * 1.31;
  
  $$min{K}{CD} = 21.30;
  $$max{K}{CD} = 42.60;
  $$mean{K}{CD} = 28.78;
  $$count{K}{CD} = 1;
  $$variance{K}{CD} = 1.39 * 1.39;
  
  $$min{K}{CE} = 34.20;
  $$max{K}{CE} = 56.00;
  $$mean{K}{CE} = 41.78;
  $$count{K}{CE} = 1;
  $$variance{K}{CE} = 0.98 * 0.98;
  
  $$min{K}{N} = 105.45;
  $$max{K}{N} = 136.10;
  $$mean{K}{N} = 121.16;
  $$count{K}{N} = 1;
  $$variance{K}{N} = 4.11 * 4.11;
  
  $$min{K}{NZ} = 33.00;
  $$max{K}{NZ} = 131.04;
  $$mean{K}{NZ} = 71.86;
  $$count{K}{NZ} = 1;
  $$variance{K}{NZ} = 52.41 * 52.41;
  
  
  $$min{M}{H} = 5.80;
  $$max{M}{H} = 10.36;
  $$mean{M}{H} = 8.26;
  $$count{M}{H} = 1;
  $$variance{M}{H} = 0.60 * 0.60;
  
  $$min{M}{HA} = 2.80;
  $$max{M}{HA} = 6.22;
  $$mean{M}{HA} = 4.39;
  $$count{M}{HA} = 1;
  $$variance{M}{HA} = 0.46 * 0.46;
  
  $$min{M}{HB2} = -0.99;
  $$max{M}{HB2} = 4.07;
  $$mean{M}{HB2} = 2.03;
  $$count{M}{HB2} = 1;
  $$variance{M}{HB2} = 0.38 * 0.38;
  
  $$min{M}{HB3} = -0.97;
  $$max{M}{HB3} = 3.15;
  $$mean{M}{HB3} = 2.01;
  $$count{M}{HB3} = 1;
  $$variance{M}{HB3} = 0.36 * 0.36;
  
  $$min{M}{HG2} = -0.36;
  $$max{M}{HG2} = 3.80;
  $$mean{M}{HG2} = 2.44;
  $$count{M}{HG2} = 1;
  $$variance{M}{HG2} = 0.36 * 0.36;
  
  $$min{M}{HG3} = 0.18;
  $$max{M}{HG3} = 3.80;
  $$mean{M}{HG3} = 2.41;
  $$count{M}{HG3} = 1;
  $$variance{M}{HG3} = 0.40 * 0.40;
  
  $$min{M}{HE} = -0.19;
  $$max{M}{HE} = 3.30;
  $$mean{M}{HE} = 1.86;
  $$count{M}{HE} = 1;
  $$variance{M}{HE} = 0.47 * 0.47;
  
  $$min{M}{C} = 169.13;
  $$max{M}{C} = 181.20;
  $$mean{M}{C} = 176.30;
  $$count{M}{C} = 1;
  $$variance{M}{C} = 2.13 * 2.13;
  
  $$min{M}{CA} = 47.70;
  $$max{M}{CA} = 62.24;
  $$mean{M}{CA} = 56.16;
  $$count{M}{CA} = 1;
  $$variance{M}{CA} = 2.34 * 2.34;
  
  $$min{M}{CB} = 24.86;
  $$max{M}{CB} = 40.75;
  $$mean{M}{CB} = 32.90;
  $$count{M}{CB} = 1;
  $$variance{M}{CB} = 2.23 * 2.23;
  
  $$min{M}{CG} = 20.83;
  $$max{M}{CG} = 37.31;
  $$mean{M}{CG} = 32.07;
  $$count{M}{CG} = 1;
  $$variance{M}{CG} = 1.45 * 1.45;
  
  $$min{M}{CE} = 9.50;
  $$max{M}{CE} = 48.46;
  $$mean{M}{CE} = 17.21;
  $$count{M}{CE} = 1;
  $$variance{M}{CE} = 3.42 * 3.42;
  
  $$min{M}{N} = 107.82;
  $$max{M}{N} = 131.60;
  $$mean{M}{N} = 120.10;
  $$count{M}{N} = 1;
  $$variance{M}{N} = 3.88 * 3.88;
  
  
  $$min{F}{H} = 5.72;
  $$max{F}{H} = 11.46;
  $$mean{F}{H} = 8.42;
  $$count{F}{H} = 1;
  $$variance{F}{H} = 0.74 * 0.74;
  
  $$min{F}{HA} = 2.72;
  $$max{F}{HA} = 6.56;
  $$mean{F}{HA} = 4.62;
  $$count{F}{HA} = 1;
  $$variance{F}{HA} = 0.58 * 0.58;
  
  $$min{F}{HB2} = 0.43;
  $$max{F}{HB2} = 4.32;
  $$mean{F}{HB2} = 2.99;
  $$count{F}{HB2} = 1;
  $$variance{F}{HB2} = 0.37 * 0.37;
  
  $$min{F}{HB3} = 0.59;
  $$max{F}{HB3} = 3.99;
  $$mean{F}{HB3} = 2.97;
  $$count{F}{HB3} = 1;
  $$variance{F}{HB3} = 0.38 * 0.38;
  
  $$min{F}{HD1} = 2.02;
  $$max{F}{HD1} = 7.94;
  $$mean{F}{HD1} = 6.91;
  $$count{F}{HD1} = 1;
  $$variance{F}{HD1} = 0.82 * 0.82;
  
  $$min{F}{HD2} = 2.02;
  $$max{F}{HD2} = 8.08;
  $$mean{F}{HD2} = 6.89;
  $$count{F}{HD2} = 1;
  $$variance{F}{HD2} = 0.86 * 0.86;
  
  $$min{F}{HE1} = 2.95;
  $$max{F}{HE1} = 8.80;
  $$mean{F}{HE1} = 6.95;
  $$count{F}{HE1} = 1;
  $$variance{F}{HE1} = 0.82 * 0.82;
  
  $$min{F}{HE2} = 2.95;
  $$max{F}{HE2} = 8.80;
  $$mean{F}{HE2} = 6.94;
  $$count{F}{HE2} = 1;
  $$variance{F}{HE2} = 0.86 * 0.86;
  
  $$min{F}{HZ} = 3.04;
  $$max{F}{HZ} = 9.50;
  $$mean{F}{HZ} = 6.92;
  $$count{F}{HZ} = 1;
  $$variance{F}{HZ} = 0.80 * 0.80;
  
  $$min{F}{C} = 169.00;
  $$max{F}{C} = 181.00;
  $$mean{F}{C} = 175.49;
  $$count{F}{C} = 1;
  $$variance{F}{C} = 2.13 * 2.13;
  
  $$min{F}{CA} = 48.30;
  $$max{F}{CA} = 67.10;
  $$mean{F}{CA} = 58.25;
  $$count{F}{CA} = 1;
  $$variance{F}{CA} = 2.73 * 2.73;
  
  $$min{F}{CB} = 32.00;
  $$max{F}{CB} = 50.40;
  $$mean{F}{CB} = 39.95;
  $$count{F}{CB} = 1;
  $$variance{F}{CB} = 2.04 * 2.04;
  
  $$min{F}{CG} = 136.20;
  $$max{F}{CG} = 140.40;
  $$mean{F}{CG} = 138.35;
  $$count{F}{CG} = 1;
  $$variance{F}{CG} = 1.12 * 1.12;
  
  $$min{F}{CD1} = 118.50;
  $$max{F}{CD1} = 134.00;
  $$mean{F}{CD1} = 131.38;
  $$count{F}{CD1} = 1;
  $$variance{F}{CD1} = 1.38 * 1.38;
  
  $$min{F}{CD2} = 119.10;
  $$max{F}{CD2} = 137.20;
  $$mean{F}{CD2} = 131.33;
  $$count{F}{CD2} = 1;
  $$variance{F}{CD2} = 1.48 * 1.48;
  
  $$min{F}{CE1} = 117.50;
  $$max{F}{CE1} = 133.30;
  $$mean{F}{CE1} = 130.48;
  $$count{F}{CE1} = 1;
  $$variance{F}{CE1} = 1.29 * 1.29;
  
  $$min{F}{CE2} = 126.40;
  $$max{F}{CE2} = 133.30;
  $$mean{F}{CE2} = 130.60;
  $$count{F}{CE2} = 1;
  $$variance{F}{CE2} = 0.97 * 0.97;
  
  $$min{F}{CZ} = 118.70;
  $$max{F}{CZ} = 138.60;
  $$mean{F}{CZ} = 129.11;
  $$count{F}{CZ} = 1;
  $$variance{F}{CZ} = 1.68 * 1.68;
  
  $$min{F}{N} = 104.78;
  $$max{F}{N} = 134.60;
  $$mean{F}{N} = 120.69;
  $$count{F}{N} = 1;
  $$variance{F}{N} = 4.18 * 4.18;
  
  
  $$min{P}{HA} = 1.63;
  $$max{P}{HA} = 6.05;
  $$mean{P}{HA} = 4.41;
  $$count{P}{HA} = 1;
  $$variance{P}{HA} = 0.36 * 0.36;
  
  $$min{P}{HB2} = -0.78;
  $$max{P}{HB2} = 3.98;
  $$mean{P}{HB2} = 2.05;
  $$count{P}{HB2} = 1;
  $$variance{P}{HB2} = 0.39 * 0.39;
  
  $$min{P}{HB3} = -0.15;
  $$max{P}{HB3} = 3.98;
  $$mean{P}{HB3} = 2.05;
  $$count{P}{HB3} = 1;
  $$variance{P}{HB3} = 0.39 * 0.39;
  
  $$min{P}{HG2} = -0.93;
  $$max{P}{HG2} = 4.92;
  $$mean{P}{HG2} = 1.93;
  $$count{P}{HG2} = 1;
  $$variance{P}{HG2} = 0.36 * 0.36;
  
  $$min{P}{HG3} = -0.90;
  $$max{P}{HG3} = 4.92;
  $$mean{P}{HG3} = 1.92;
  $$count{P}{HG3} = 1;
  $$variance{P}{HG3} = 0.37 * 0.37;
  
  $$min{P}{HD2} = 1.01;
  $$max{P}{HD2} = 4.62;
  $$mean{P}{HD2} = 3.64;
  $$count{P}{HD2} = 1;
  $$variance{P}{HD2} = 0.36 * 0.36;
  
  $$min{P}{HD3} = 0.99;
  $$max{P}{HD3} = 5.16;
  $$mean{P}{HD3} = 3.63;
  $$count{P}{HD3} = 1;
  $$variance{P}{HD3} = 0.40 * 0.40;
  
  $$min{P}{C} = 170.95;
  $$max{P}{C} = 182.30;
  $$mean{P}{C} = 176.70;
  $$count{P}{C} = 1;
  $$variance{P}{C} = 1.69 * 1.69;
  
  $$min{P}{CA} = 50.12;
  $$max{P}{CA} = 68.10;
  $$mean{P}{CA} = 63.30;
  $$count{P}{CA} = 1;
  $$variance{P}{CA} = 1.64 * 1.64;
  
  $$min{P}{CB} = 26.32;
  $$max{P}{CB} = 56.76;
  $$mean{P}{CB} = 31.81;
  $$count{P}{CB} = 1;
  $$variance{P}{CB} = 1.54 * 1.54;
  
  $$min{P}{CG} = 19.31;
  $$max{P}{CG} = 33.39;
  $$mean{P}{CG} = 27.14;
  $$count{P}{CG} = 1;
  $$variance{P}{CG} = 1.18 * 1.18;
  
  $$min{P}{CD} = 40.44;
  $$max{P}{CD} = 56.00;
  $$mean{P}{CD} = 50.28;
  $$count{P}{CD} = 1;
  $$variance{P}{CD} = 1.04 * 1.04;
  
  $$min{P}{N} = 106.00;
  $$max{P}{N} = 144.29;
  $$mean{P}{N} = 127.40;
  $$count{P}{N} = 1;
  $$variance{P}{N} = 10.05 * 10.05;
  
  
  $$min{S}{H} = 2.96;
  $$max{S}{H} = 11.68;
  $$mean{S}{H} = 8.29;
  $$count{S}{H} = 1;
  $$variance{S}{H} = 0.62 * 0.62;
  
  $$min{S}{HA} = 1.43;
  $$max{S}{HA} = 6.37;
  $$mean{S}{HA} = 4.51;
  $$count{S}{HA} = 1;
  $$variance{S}{HA} = 0.43 * 0.43;
  
  $$min{S}{HB2} = 1.48;
  $$max{S}{HB2} = 4.99;
  $$mean{S}{HB2} = 3.88;
  $$count{S}{HB2} = 1;
  $$variance{S}{HB2} = 0.29 * 0.29;
  
  $$min{S}{HB3} = 1.51;
  $$max{S}{HB3} = 4.96;
  $$mean{S}{HB3} = 3.87;
  $$count{S}{HB3} = 1;
  $$variance{S}{HB3} = 0.31 * 0.31;
  
  $$min{S}{HG} = 1.10;
  $$max{S}{HG} = 8.97;
  $$mean{S}{HG} = 5.33;
  $$count{S}{HG} = 1;
  $$variance{S}{HG} = 1.16 * 1.16;
  
  $$min{S}{C} = 166.90;
  $$max{S}{C} = 181.40;
  $$mean{S}{C} = 174.53;
  $$count{S}{C} = 1;
  $$variance{S}{C} = 1.73 * 1.73;
  
  $$min{S}{CA} = 48.90;
  $$max{S}{CA} = 66.91;
  $$mean{S}{CA} = 58.57;
  $$count{S}{CA} = 1;
  $$variance{S}{CA} = 2.12 * 2.12;
  
  $$min{S}{CB} = 55.50;
  $$max{S}{CB} = 70.80;
  $$mean{S}{CB} = 63.80;
  $$count{S}{CB} = 1;
  $$variance{S}{CB} = 1.57 * 1.57;
  
  $$min{S}{N} = 102.60;
  $$max{S}{N} = 133.10;
  $$mean{S}{N} = 116.30;
  $$count{S}{N} = 1;
  $$variance{S}{N} = 3.95 * 3.95;
  
  
  $$min{T}{H} = 5.93;
  $$max{T}{H} = 11.01;
  $$mean{T}{H} = 8.27;
  $$count{T}{H} = 1;
  $$variance{T}{H} = 0.62 * 0.62;
  
  $$min{T}{HA} = 0.87;
  $$max{T}{HA} = 6.35;
  $$mean{T}{HA} = 4.48;
  $$count{T}{HA} = 1;
  $$variance{T}{HA} = 0.50 * 0.50;
  
  $$min{T}{HB} = 0.92;
  $$max{T}{HB} = 5.90;
  $$mean{T}{HB} = 4.17;
  $$count{T}{HB} = 1;
  $$variance{T}{HB} = 0.37 * 0.37;
  
  $$min{T}{HG1} = 0.32;
  $$max{T}{HG1} = 8.21;
  $$mean{T}{HG1} = 4.40;
  $$count{T}{HG1} = 1;
  $$variance{T}{HG1} = 1.92 * 1.92;
  
  $$min{T}{HG2} = -0.83;
  $$max{T}{HG2} = 4.35;
  $$mean{T}{HG2} = 1.16;
  $$count{T}{HG2} = 1;
  $$variance{T}{HG2} = 0.28 * 0.28;
  
  $$min{T}{C} = 165.50;
  $$max{T}{C} = 181.39;
  $$mean{T}{C} = 174.60;
  $$count{T}{C} = 1;
  $$variance{T}{C} = 1.80 * 1.80;
  
  $$min{T}{CA} = 52.00;
  $$max{T}{CA} = 69.18;
  $$mean{T}{CA} = 62.15;
  $$count{T}{CA} = 1;
  $$variance{T}{CA} = 2.72 * 2.72;
  
  $$min{T}{CB} = 58.60;
  $$max{T}{CB} = 78.30;
  $$mean{T}{CB} = 69.64;
  $$count{T}{CB} = 1;
  $$variance{T}{CB} = 1.70 * 1.70;
  
  $$min{T}{CG2} = 13.20;
  $$max{T}{CG2} = 38.90;
  $$mean{T}{CG2} = 21.44;
  $$count{T}{CG2} = 1;
  $$variance{T}{CG2} = 1.39 * 1.39;
  
  $$min{T}{N} = 97.70;
  $$max{T}{N} = 132.80;
  $$mean{T}{N} = 115.70;
  $$count{T}{N} = 1;
  $$variance{T}{N} = 5.05 * 5.05;
  
  
  $$min{W}{H} = 5.73;
  $$max{W}{H} = 10.76;
  $$mean{W}{H} = 8.35;
  $$count{W}{H} = 1;
  $$variance{W}{H} = 0.84 * 0.84;
  
  $$min{W}{HA} = 2.28;
  $$max{W}{HA} = 6.75;
  $$mean{W}{HA} = 4.74;
  $$count{W}{HA} = 1;
  $$variance{W}{HA} = 0.55 * 0.55;
  
  $$min{W}{HB2} = 0.42;
  $$max{W}{HB2} = 4.54;
  $$mean{W}{HB2} = 3.22;
  $$count{W}{HB2} = 1;
  $$variance{W}{HB2} = 0.36 * 0.36;
  
  $$min{W}{HB3} = 1.02;
  $$max{W}{HB3} = 4.49;
  $$mean{W}{HB3} = 3.18;
  $$count{W}{HB3} = 1;
  $$variance{W}{HB3} = 0.36 * 0.36;
  
  $$min{W}{HD1} = 5.81;
  $$max{W}{HD1} = 8.93;
  $$mean{W}{HD1} = 7.16;
  $$count{W}{HD1} = 1;
  $$variance{W}{HD1} = 0.34 * 0.34;
  
  $$min{W}{HE1} = 6.79;
  $$max{W}{HE1} = 11.90;
  $$mean{W}{HE1} = 10.13;
  $$count{W}{HE1} = 1;
  $$variance{W}{HE1} = 0.53 * 0.53;
  
  $$min{W}{HE3} = 1.85;
  $$max{W}{HE3} = 8.98;
  $$mean{W}{HE3} = 7.17;
  $$count{W}{HE3} = 1;
  $$variance{W}{HE3} = 1.03 * 1.03;
  
  $$min{W}{HZ2} = 2.63;
  $$max{W}{HZ2} = 8.60;
  $$mean{W}{HZ2} = 7.14;
  $$count{W}{HZ2} = 1;
  $$variance{W}{HZ2} = 0.89 * 0.89;
  
  $$min{W}{HZ3} = 0.76;
  $$max{W}{HZ3} = 8.20;
  $$mean{W}{HZ3} = 6.68;
  $$count{W}{HZ3} = 1;
  $$variance{W}{HZ3} = 1.02 * 1.02;
  
  $$min{W}{HH2} = 2.84;
  $$max{W}{HH2} = 10.17;
  $$mean{W}{HH2} = 6.82;
  $$count{W}{HH2} = 1;
  $$variance{W}{HH2} = 0.94 * 0.94;
  
  $$min{W}{C} = 170.70;
  $$max{W}{C} = 180.82;
  $$mean{W}{C} = 176.10;
  $$count{W}{C} = 1;
  $$variance{W}{C} = 1.89 * 1.89;
  
  $$min{W}{CA} = 47.70;
  $$max{W}{CA} = 65.12;
  $$mean{W}{CA} = 57.71;
  $$count{W}{CA} = 1;
  $$variance{W}{CA} = 2.65 * 2.65;
  
  $$min{W}{CB} = 24.10;
  $$max{W}{CB} = 43.02;
  $$mean{W}{CB} = 30.16;
  $$count{W}{CB} = 1;
  $$variance{W}{CB} = 2.19 * 2.19;
  
  $$min{W}{CG} = 107.50;
  $$max{W}{CG} = 114.60;
  $$mean{W}{CG} = 110.37;
  $$count{W}{CG} = 1;
  $$variance{W}{CG} = 1.55 * 1.55;
  
  $$min{W}{CD1} = 119.50;
  $$max{W}{CD1} = 129.80;
  $$mean{W}{CD1} = 126.16;
  $$count{W}{CD1} = 1;
  $$variance{W}{CD1} = 1.76 * 1.76;
  
  $$min{W}{CD2} = 120.20;
  $$max{W}{CD2} = 130.10;
  $$mean{W}{CD2} = 127.40;
  $$count{W}{CD2} = 1;
  $$variance{W}{CD2} = 1.33 * 1.33;
  
  $$min{W}{CE2} = 118.37;
  $$max{W}{CE2} = 139.20;
  $$mean{W}{CE2} = 137.38;
  $$count{W}{CE2} = 1;
  $$variance{W}{CE2} = 2.86 * 2.86;
  
  $$min{W}{CE3} = 116.50;
  $$max{W}{CE3} = 127.30;
  $$mean{W}{CE3} = 120.38;
  $$count{W}{CE3} = 1;
  $$variance{W}{CE3} = 1.50 * 1.50;
  
  $$min{W}{CZ2} = 111.18;
  $$max{W}{CZ2} = 118.90;
  $$mean{W}{CZ2} = 114.19;
  $$count{W}{CZ2} = 1;
  $$variance{W}{CZ2} = 1.09 * 1.09;
  
  $$min{W}{CZ3} = 116.46;
  $$max{W}{CZ3} = 129.70;
  $$mean{W}{CZ3} = 121.57;
  $$count{W}{CZ3} = 1;
  $$variance{W}{CZ3} = 1.76 * 1.76;
  
  $$min{W}{CH2} = 119.60;
  $$max{W}{CH2} = 128.73;
  $$mean{W}{CH2} = 123.75;
  $$count{W}{CH2} = 1;
  $$variance{W}{CH2} = 1.34 * 1.34;
  
  $$min{W}{N} = 101.75;
  $$max{W}{N} = 134.59;
  $$mean{W}{N} = 121.87;
  $$count{W}{N} = 1;
  $$variance{W}{N} = 4.58 * 4.58;
  
  $$min{W}{NE1} = 106.75;
  $$max{W}{NE1} = 136.30;
  $$mean{W}{NE1} = 129.48;
  $$count{W}{NE1} = 1;
  $$variance{W}{NE1} = 2.56 * 2.56;
  
  
  $$min{Y}{H} = 5.64;
  $$max{Y}{H} = 11.92;
  $$mean{Y}{H} = 8.37;
  $$count{Y}{H} = 1;
  $$variance{Y}{H} = 0.75 * 0.75;
  
  $$min{Y}{HA} = 1.20;
  $$max{Y}{HA} = 6.65;
  $$mean{Y}{HA} = 4.63;
  $$count{Y}{HA} = 1;
  $$variance{Y}{HA} = 0.56 * 0.56;
  
  $$min{Y}{HB2} = 0.31;
  $$max{Y}{HB2} = 4.70;
  $$mean{Y}{HB2} = 2.91;
  $$count{Y}{HB2} = 1;
  $$variance{Y}{HB2} = 0.37 * 0.37;
  
  $$min{Y}{HB3} = 0.34;
  $$max{Y}{HB3} = 4.70;
  $$mean{Y}{HB3} = 2.91;
  $$count{Y}{HB3} = 1;
  $$variance{Y}{HB3} = 0.37 * 0.37;
  
  $$min{Y}{HD1} = 2.77;
  $$max{Y}{HD1} = 8.00;
  $$mean{Y}{HD1} = 6.87;
  $$count{Y}{HD1} = 1;
  $$variance{Y}{HD1} = 0.65 * 0.65;
  
  $$min{Y}{HD2} = 2.77;
  $$max{Y}{HD2} = 8.50;
  $$mean{Y}{HD2} = 6.86;
  $$count{Y}{HD2} = 1;
  $$variance{Y}{HD2} = 0.68 * 0.68;
  
  $$min{Y}{HE1} = 2.61;
  $$max{Y}{HE1} = 7.86;
  $$mean{Y}{HE1} = 6.64;
  $$count{Y}{HE1} = 1;
  $$variance{Y}{HE1} = 0.58 * 0.58;
  
  $$min{Y}{HE2} = 2.61;
  $$max{Y}{HE2} = 8.50;
  $$mean{Y}{HE2} = 6.63;
  $$count{Y}{HE2} = 1;
  $$variance{Y}{HE2} = 0.61 * 0.61;
  
  $$min{Y}{HH} = 5.96;
  $$max{Y}{HH} = 13.03;
  $$mean{Y}{HH} = 9.25;
  $$count{Y}{HH} = 1;
  $$variance{Y}{HH} = 1.34 * 1.34;
  
  $$min{Y}{C} = 169.20;
  $$max{Y}{C} = 180.34;
  $$mean{Y}{C} = 175.39;
  $$count{Y}{C} = 1;
  $$variance{Y}{C} = 1.89 * 1.89;
  
  $$min{Y}{CA} = 45.20;
  $$max{Y}{CA} = 65.80;
  $$mean{Y}{CA} = 58.02;
  $$count{Y}{CA} = 1;
  $$variance{Y}{CA} = 2.64 * 2.64;
  
  $$min{Y}{CB} = 29.20;
  $$max{Y}{CB} = 46.06;
  $$mean{Y}{CB} = 39.16;
  $$count{Y}{CB} = 1;
  $$variance{Y}{CB} = 2.23 * 2.23;
  
  $$min{Y}{CG} = 125.70;
  $$max{Y}{CG} = 131.10;
  $$mean{Y}{CG} = 129.29;
  $$count{Y}{CG} = 1;
  $$variance{Y}{CG} = 1.21 * 1.21;
  
  $$min{Y}{CD1} = 122.10;
  $$max{Y}{CD1} = 136.70;
  $$mean{Y}{CD1} = 132.54;
  $$count{Y}{CD1} = 1;
  $$variance{Y}{CD1} = 1.35 * 1.35;
  
  $$min{Y}{CD2} = 118.15;
  $$max{Y}{CD2} = 136.70;
  $$mean{Y}{CD2} = 132.26;
  $$count{Y}{CD2} = 1;
  $$variance{Y}{CD2} = 1.99 * 1.99;
  
  $$min{Y}{CE1} = 110.70;
  $$max{Y}{CE1} = 133.31;
  $$mean{Y}{CE1} = 117.81;
  $$count{Y}{CE1} = 1;
  $$variance{Y}{CE1} = 2.09 * 2.09;
  
  $$min{Y}{CE2} = 115.00;
  $$max{Y}{CE2} = 133.31;
  $$mean{Y}{CE2} = 117.91;
  $$count{Y}{CE2} = 1;
  $$variance{Y}{CE2} = 2.60 * 2.60;
  
  $$min{Y}{CZ} = 153.54;
  $$max{Y}{CZ} = 162.70;
  $$mean{Y}{CZ} = 156.37;
  $$count{Y}{CZ} = 1;
  $$variance{Y}{CZ} = 2.50 * 2.50;
  
  $$min{Y}{N} = 104.60;
  $$max{Y}{N} = 134.53;
  $$mean{Y}{N} = 120.79;
  $$count{Y}{N} = 1;
  $$variance{Y}{N} = 4.69 * 4.69;
  
  
  $$min{V}{H} = 3.98;
  $$max{V}{H} = 11.03;
  $$mean{V}{H} = 8.29;
  $$count{V}{H} = 1;
  $$variance{V}{H} = 0.71 * 0.71;
  
  $$min{V}{HA} = 1.73;
  $$max{V}{HA} = 6.13;
  $$mean{V}{HA} = 4.16;
  $$count{V}{HA} = 1;
  $$variance{V}{HA} = 0.57 * 0.57;
  
  $$min{V}{HB} = -0.56;
  $$max{V}{HB} = 3.42;
  $$mean{V}{HB} = 1.99;
  $$count{V}{HB} = 1;
  $$variance{V}{HB} = 0.33 * 0.33;
  
  $$min{V}{HG1} = -1.09;
  $$max{V}{HG1} = 2.31;
  $$mean{V}{HG1} = 0.84;
  $$count{V}{HG1} = 1;
  $$variance{V}{HG1} = 0.27 * 0.27;
  
  $$min{V}{HG2} = -2.32;
  $$max{V}{HG2} = 3.32;
  $$mean{V}{HG2} = 0.83;
  $$count{V}{HG2} = 1;
  $$variance{V}{HG2} = 0.29 * 0.29;
  
  $$min{V}{C} = 168.50;
  $$max{V}{C} = 181.44;
  $$mean{V}{C} = 175.69;
  $$count{V}{C} = 1;
  $$variance{V}{C} = 2.03 * 2.03;
  
  $$min{V}{CA} = 51.10;
  $$max{V}{CA} = 69.74;
  $$mean{V}{CA} = 62.48;
  $$count{V}{CA} = 1;
  $$variance{V}{CA} = 3.04 * 3.04;
  
  $$min{V}{CB} = 22.04;
  $$max{V}{CB} = 40.75;
  $$mean{V}{CB} = 32.66;
  $$count{V}{CB} = 1;
  $$variance{V}{CB} = 1.91 * 1.91;
  
  $$min{V}{CG1} = 11.60;
  $$max{V}{CG1} = 26.90;
  $$mean{V}{CG1} = 21.38;
  $$count{V}{CG1} = 1;
  $$variance{V}{CG1} = 1.53 * 1.53;
  
  $$min{V}{CG2} = 11.30;
  $$max{V}{CG2} = 28.74;
  $$mean{V}{CG2} = 21.31;
  $$count{V}{CG2} = 1;
  $$variance{V}{CG2} = 1.76 * 1.76;
  
  $$min{V}{N} = 96.29;
  $$max{V}{N} = 136.80;
  $$mean{V}{N} = 121.02;
  $$count{V}{N} = 1;
  $$variance{V}{N} = 4.96 * 4.96;

  # calculate deuteration isotope corrections if deuteration is present
  if ($deuteration > 0)
    {
    my $deuteration_corrections = {};


    # corrections for CA's calculated from Sandy Farmer's macro values.
    # corrections for sidechain carbons calculated from small molecule studies.
    # corrections for nitrogens come from a rough estimate of 0.16 for 2-bond effects and 0.06 for 3-bond effects
    # for A
    $$deuteration_corrections{A}{CA} =0.752;
    $$deuteration_corrections{A}{CB} =0.932;
    $$deuteration_corrections{A}{N} =0.34;

    # for C
    $$deuteration_corrections{C}{CA} =0.632;
    $$deuteration_corrections{C}{CB} =0.837;
    $$deuteration_corrections{C}{N} =0.28;

    # for D
    $$deuteration_corrections{D}{CA} =0.632;
    $$deuteration_corrections{D}{CB} =0.69;
    $$deuteration_corrections{D}{N} =0.28;

    # for E
    $$deuteration_corrections{E}{CA} =0.716;
    $$deuteration_corrections{E}{CB} =0.937;
    $$deuteration_corrections{E}{CG} =0.83;
    $$deuteration_corrections{E}{N} =0.28;

    # for F
    $$deuteration_corrections{F}{CA} =0.632;
    $$deuteration_corrections{F}{CB} =0.919;
    $$deuteration_corrections{F}{CG} =0.274;
    $$deuteration_corrections{F}{CD1} =0.429;
    $$deuteration_corrections{F}{CD2} =0.429;
    $$deuteration_corrections{F}{CE1} =0.500;
    $$deuteration_corrections{F}{CE2} =0.500;
    $$deuteration_corrections{F}{CZ} =0.496;
    $$deuteration_corrections{F}{N} =0.28;

    # for G
    $$deuteration_corrections{G}{CA} =0.784;
    $$deuteration_corrections{G}{N} =0.32;

    # for H
    $$deuteration_corrections{H}{CA} =0.632;
    $$deuteration_corrections{H}{CB} =0.832;
    $$deuteration_corrections{H}{CG} =0.365;
    $$deuteration_corrections{H}{CD2} =0.545;
    $$deuteration_corrections{H}{CE1} =0.475;
    $$deuteration_corrections{H}{N} =0.28;
    $$deuteration_corrections{H}{ND1} =0.34;
    $$deuteration_corrections{H}{NE2} =0.32;

    # for I
    $$deuteration_corrections{I}{CA} =0.722;
    $$deuteration_corrections{I}{CB} =1.092;
    $$deuteration_corrections{I}{CG1} =1.130;
    $$deuteration_corrections{I}{CG2} =1.020;
    $$deuteration_corrections{I}{CD1} =1.090;
    $$deuteration_corrections{I}{N} =0.22;

    # for K
    $$deuteration_corrections{K}{CA} =0.716;
    $$deuteration_corrections{K}{CB} =1.067;
    $$deuteration_corrections{K}{CG} =1.225;
    $$deuteration_corrections{K}{CD} =1.190;
    $$deuteration_corrections{K}{CE} =0.990;
    $$deuteration_corrections{K}{N} =0.28;
    $$deuteration_corrections{K}{NZ} =0.44;

    # for L
    $$deuteration_corrections{L}{CA} =0.674;
    $$deuteration_corrections{L}{CB} =1.067;
    $$deuteration_corrections{L}{CG} =1.215;
    $$deuteration_corrections{L}{CD1} =1.090;
    $$deuteration_corrections{L}{CD2} =1.090;
    $$deuteration_corrections{L}{N} =0.28;

    # for M
    $$deuteration_corrections{M}{CA} =0.716;
    $$deuteration_corrections{M}{CB} =0.917;
    $$deuteration_corrections{M}{CG} =0.995;
    $$deuteration_corrections{M}{CE} =0.855;
    $$deuteration_corrections{M}{N} =0.28;

    # for N
    $$deuteration_corrections{N}{CA} =0.632;
    $$deuteration_corrections{N}{CB} =0.757;
    $$deuteration_corrections{N}{N} =0.28;

    # for P
    $$deuteration_corrections{P}{CA} =0.716;
    $$deuteration_corrections{P}{CB} =1.027;
    $$deuteration_corrections{P}{CG} =1.158;
    $$deuteration_corrections{P}{CD} =0.952;
    $$deuteration_corrections{P}{N} =0.72;

    # for Q
    $$deuteration_corrections{Q}{CA} =0.716;
    $$deuteration_corrections{Q}{CB} =0.951;
    $$deuteration_corrections{Q}{CG} =0.915;
    $$deuteration_corrections{Q}{N} =0.28;

    # for R
    $$deuteration_corrections{R}{CA} =0.716;
    $$deuteration_corrections{R}{CB} =1.067;
    $$deuteration_corrections{R}{CG} =1.155;
    $$deuteration_corrections{R}{CD} =0.990;
    $$deuteration_corrections{R}{CZ} =0.070;
    $$deuteration_corrections{R}{N} =0.28;
    $$deuteration_corrections{R}{NE} =0.44;

    # for S
    $$deuteration_corrections{S}{CA} =0.632;
    $$deuteration_corrections{S}{CB} =0.837;
    $$deuteration_corrections{S}{N} =0.28;

    # for T
    $$deuteration_corrections{T}{CA} =0.638;
    $$deuteration_corrections{T}{CB} =0.867;
    $$deuteration_corrections{T}{CG} =0.950;
    $$deuteration_corrections{T}{N} =0.22;

    # for V
    $$deuteration_corrections{V}{CA} =0.764;
    $$deuteration_corrections{V}{CB} =1.057;
    $$deuteration_corrections{V}{CG1} =0.950;
    $$deuteration_corrections{V}{CG2} =0.950;
    $$deuteration_corrections{V}{N} =0.22;

    # for W
    $$deuteration_corrections{W}{CA} =0.632;
    $$deuteration_corrections{W}{CB} =0.832;
    $$deuteration_corrections{W}{CG} =0.365;
    $$deuteration_corrections{W}{CD1} =0.350;
    $$deuteration_corrections{W}{CD2} =0.213;
    $$deuteration_corrections{W}{CE2} =0.167;
    $$deuteration_corrections{W}{CE3} =0.429;
    $$deuteration_corrections{W}{CZ2} =0.429;
    $$deuteration_corrections{W}{CZ3} =0.511;
    $$deuteration_corrections{W}{CH2} =0.511;
    $$deuteration_corrections{W}{N} =0.28;
    $$deuteration_corrections{W}{NE1} =0.22;

    # for Y
    $$deuteration_corrections{Y}{CA} =0.632;
    $$deuteration_corrections{Y}{CB} =0.919;
    $$deuteration_corrections{Y}{CG} =0.299;
    $$deuteration_corrections{Y}{CD1} =0.418;
    $$deuteration_corrections{Y}{CD2} =0.418;
    $$deuteration_corrections{Y}{CE1} =0.390;
    $$deuteration_corrections{Y}{CE2} =0.390;
    $$deuteration_corrections{Y}{CZ} =0.220;
    $$deuteration_corrections{Y}{N} =0.28;


    # make the corrections
    foreach my $aa (keys %{$deuteration_corrections})
      {
      foreach my $atom (keys %{$$deuteration_corrections{$aa}})
	{
	$$min{$aa}{$atom} -= ($$deuteration_corrections{$aa}{$atom} * $deuteration);
	$$max{$aa}{$atom} -= ($$deuteration_corrections{$aa}{$atom} * $deuteration);
	$$mean{$aa}{$atom} -= ($$deuteration_corrections{$aa}{$atom} * $deuteration);
	}
      }
    }


  my $restricted_type_lists = {};
  if (! $use_sidechains)
    {
    $$restricted_type_lists{P} = ['CA', 'CB'];
    $$restricted_type_lists{Q} = ['CA', 'CB'];
    $$restricted_type_lists{A} = ['CA', 'CB'];
    $$restricted_type_lists{R} = ['CA', 'CB'];
    $$restricted_type_lists{S} = ['CA', 'CB'];
    $$restricted_type_lists{C} = ['CA', 'CB'];
    $$restricted_type_lists{T} = ['CA', 'CB'];
    $$restricted_type_lists{D} = ['CA', 'CB'];
    $$restricted_type_lists{E} = ['CA', 'CB'];
    $$restricted_type_lists{V} = ['CA', 'CB'];
    $$restricted_type_lists{F} = ['CA', 'CB'];
    $$restricted_type_lists{W} = ['CA', 'CB'];
    $$restricted_type_lists{G} = ['CA'];
    $$restricted_type_lists{H} = ['CA', 'CB'];
    $$restricted_type_lists{Y} = ['CA', 'CB'];
    $$restricted_type_lists{I} = ['CA', 'CB'];
    $$restricted_type_lists{K} = ['CA', 'CB'];
    $$restricted_type_lists{L} = ['CA', 'CB'];
    $$restricted_type_lists{M} = ['CA', 'CB'];
    $$restricted_type_lists{N} = ['CA', 'CB'];
    }
  else
    {
    $$restricted_type_lists{P} = ['CA', 'CB', 'CD', 'CG'];
    $$restricted_type_lists{Q} = ['CA', 'CB', 'CD', 'CG'];
    $$restricted_type_lists{A} = ['CA', 'CB'];
    $$restricted_type_lists{R} = ['CA', 'CB', 'CD', 'CG'];
    $$restricted_type_lists{S} = ['CA', 'CB'];
    $$restricted_type_lists{C} = ['CA', 'CB'];
    $$restricted_type_lists{T} = ['CA', 'CB', 'CG2'];
    $$restricted_type_lists{D} = ['CA', 'CB', 'CG'];
    $$restricted_type_lists{E} = ['CA', 'CB', 'CD', 'CG'];
    $$restricted_type_lists{V} = ['CA', 'CB', 'CG1', 'CG2'];
    $$restricted_type_lists{F} = ['CA', 'CB', 'CD1', 'CD2', 'CE1', 'CE2', 'CG', 'CZ'];
    $$restricted_type_lists{W} = ['CA', 'CB', 'CD1', 'CD2', 'CE2', 'CE3', 'CG', 'CH2', 'CZ2', 'CZ3'];
    $$restricted_type_lists{G} = ['CA'];
    $$restricted_type_lists{H} = ['CA', 'CB', 'CD2', 'CE1','CG'];
    $$restricted_type_lists{Y} = ['CA', 'CB', 'CD1', 'CD2', 'CE1', 'CE2', 'CG', 'CZ'];
    $$restricted_type_lists{I} = ['CA', 'CB', 'CD1', 'CG1', 'CG2'];
    $$restricted_type_lists{K} = ['CA', 'CB', 'CD', 'CE', 'CG'];
    $$restricted_type_lists{L} = ['CA', 'CB', 'CD1', 'CD2', 'CG'];
    $$restricted_type_lists{M} = ['CA', 'CB', 'CE', 'CG'];
    $$restricted_type_lists{N} = ['CA', 'CB', 'CG'];
    }


  if (! $use_aromatics)
    {
    foreach my $aa (qw(F Y W))
      { @{$$restricted_type_lists{$aa}} = grep { $_ eq "CA" || $_ eq "CB"; } @{$$restricted_type_lists{$aa}}; }
    }


  return $count, $mean, $variance, $min, $max, $restricted_type_lists;  
  }
