#!/usr/bin/perl
#!/usr/bin/perl
# extract_by_filter.pl 
# 	Extract a group of peaks using a set of filter peaks.
# 
#	Copyright Hunter Moseley, 1999. All rights reserved.
#	Written by Hunter Moseley 4/2/1999
#
#   usage: extract_by_filter.pl in_peak_file filter_peak_file inclusion_toggle \ 
# 			      new_workbook_name out_peak_file tol1 tol2 ...
# 
# 	peak_file	 - name of peak file or "-" for standard input/output.
# 	filter_peak_file - name of peak file.
# 	inclusion_toggle: 
# 		inc(lusive) - include peaks that match filter. 
# 		exc(lusive) - exclude peak that match filter.
# 	workbook_name	 - string to use as workbook name or "-" to use old name.
# 	tol1 tol2 ...	 - dimension tolerances for comparison or "0" not to use specific 
# 			   dimension.
#
use strict;

# Print usage info
if ((@ARGV < 6) || ($ARGV[0] =~ /^-h/))
  {
  print "\nError: not enough parameters given.\n" if ($ARGV[0] !~ /^-h/);
  print "\n";
  print "extract_by_filter.pl \n";
  print "	Extract a group of peaks using a set of filter peaks.\n";
  print "\n";
  print "  usage: extract_by_filter.pl in_peak_file filter_peak_file inclusion_toggle \\\n";
  print "			      new_workbook_name out_peak_file tol1 tol2 ...\n";
  print "\n";
  print "	peak_file	 - name of peak file or \"-\" for standard input/output.\n";
  print "	filter_peak_file - name of peak file.\n";
  print "	inclusion_toggle: \n";
  print "		inc(lusive) - include peaks that match filter. \n";
  print "		exc(lusive) - exclude peak that match filter.\n";
  print "	workbook_name	 - string to use as workbook name or \"-\" to use old name.\n";
  print "	tol1 tol2 ...	 - dimension tolerances for comparison or \"0\" not to use specific \n";
  print "			   dimension.\n";
  exit(1);
  }

# Read in comand line parameters
my $in_peak_file = $ARGV[0];
my $filter_peak_file = $ARGV[1];
my $inclusion;
if ($ARGV[2] =~ /^inc/)
  { $inclusion = 1; }
else
  { $inclusion = 0; }
my $workbook_name = $ARGV[3];
my $out_peak_file = $ARGV[4];

my $x;
my @tolerance_alist;
for($x = 5; $x < @ARGV; $x++)
  { push @tolerance_alist, $ARGV[$x]; }


# Read Peak file
my ($peak_alist, $end_comments_alist) = &read_peak_file($in_peak_file);
my $num_cols = @{$$peak_alist[0]};

# Read Filter file
my $filter_alist = &read_peak_file($filter_peak_file);

# Open Output file
local *OUTPUT_FILE;
if ($out_peak_file eq "-")
  { *OUTPUT_FILE = *STDOUT; }
else
  { open(OUTPUT_FILE, ">$out_peak_file") || die ("open " . $out_peak_file . " failed."); }

# Print Header
print OUTPUT_FILE "#Index	";
my $x;
for ($x=1; $x < $num_cols-2; $x++)
  { print OUTPUT_FILE $x, "Dim	"; }
print OUTPUT_FILE "	Intensity	Workbook\n";

# Print filtered peak list. 
my $peak;
my $x = 1;
foreach $peak (@$peak_alist) 
  {
  if (&within_tolerances($peak, $filter_alist, \@tolerance_alist) == $inclusion)
    {
    $$peak[0] = $x++; 
    $$peak[$num_cols-2] = "\t" . $$peak[$num_cols-2];
    if ($workbook_name eq "-")
      {
      $$peak[$num_cols-1] = ("\t" x (length($$peak[$num_cols-2]) < 9)) . $$peak[$num_cols-1];
      }
    else
      {
      my ($notes) = ($$peak[$num_cols-1] =~ /[^.]*[.](.*)/);
      $$peak[$num_cols-1] = ("\t" x (length($$peak[$num_cols-2]) < 9)) . $workbook_name . ("."x($notes ne "")) . $notes; 
      }
    print OUTPUT_FILE join("\t",@$peak), "\n";
    }
  }

# Print Comments
print OUTPUT_FILE "*\n";

my $line;
my $history_found = 0;
foreach $line (@$end_comments_alist)
  {
  if (! $history_found && ($line =~ /^File History:/))
    {
    print OUTPUT_FILE "File History: $out_peak_file \n\n";
    print OUTPUT_FILE $0, " ", join(' ',@ARGV),"\n";
    $history_found = 1;
    next;
    }
  
  print OUTPUT_FILE $line;  
  }

if (! $history_found)
  {
  print OUTPUT_FILE "\nFile History: $out_peak_file \n\n";
  print OUTPUT_FILE $0, " ", join(' ',@ARGV),"\n";
  }

close OUTPUT_FILE;


#
#  Subroutines
#
sub read_peak_file 
  {
  my $filename = shift @_;
    
  local *PEAKFILE;
  if ($filename eq "-")
    { *PEAKFILE = *STDIN; }
  else
    { open(PEAKFILE, $filename) || die ("open " . $filename . " failed."); }

  my $peak_alist = [];
  my $line;
  while ($line = <PEAKFILE>) 
    {
    next if ($line =~ /^\#/); # skip comment lines
    last if ($line =~ /^[*]/); # end loop if end_of_file marker is encountered.
    
    ($line) = ($line =~/^\s*(.*)/); # skip leading white space.
    chomp $line;
    push @$peak_alist, [ split(/\s+/, $line) ];
    }

  my $comment_alist = [];
  while ($line = <PEAKFILE>)
    { push @$comment_alist, $line; }

  close PEAKFILE;
  return wantarray ? ($peak_alist, $comment_alist) : $peak_alist;
  }

sub within_tolerances
  {
  my $peak = shift @_;
  my $filter_alist = shift @_;
  my $tolerance_alist = shift @_;

  my $filter_peak;
  my $x;
  foreach $filter_peak (@$filter_alist)
    {
    for($x=0; $x < @$tolerance_alist; $x++)
      {
      if (($$tolerance_alist[$x] != 0.0) && (abs($$peak[$x+1] - $$filter_peak[$x+1]) >= $$tolerance_alist[$x]))
	{ last; }
      }

    if ($x == @$tolerance_alist)
      { return 1; }
    }

    return 0;
  }
