#!/usr/bin/perl
#!/usr/bin/perl

# extract_dup_assignments.pl
#
# Function: Looks in the 'Notes' column for peaks with duplicate assignments
#
# Copyright Aneerban Bhattacharya, 2001. All rights reserved.
# Written by Aneerban Bhattacharya	Jun 2, 2001
# Modified by Aneerban Bhattacharya	Mar 21, 2002
#
# Subroutine read_peak_file, procedure for writing out file history and interactive help format
# taken from AutoPeak scripts written by Hunter Moseley.
#
# Usage: extract_dup_assignments.pl in_peak_file workbook_name out_peak_file inclusion_toggle
#
# peak_file: name of peak file or "-" for standard input
# workbook_name: string to use as name of workbook
# inclusion_toggle:
#	inc(lusive) - include only peaks with duplicates
#	exc(lusive) - exclude peaks with duplicates
#

use strict;

if ((scalar(@ARGV) < 4) || ($ARGV[0] =~ /^-h/)) {
	print(STDERR "\nError: not enough parameters given.\n") if ($ARGV[0] !~ /^-h/);
	print(STDERR "\n");
	print(STDERR "extract_dup_assignments.pl\n");
	print(STDERR "\tExtracts groups of peaks that have duplicate assignments in 'Notes' column.\n");
	print(STDERR "\n");
	print(STDERR "\tUsage: find_duplicate_assignments.pl in_peak_file workbook_name out_peak_file inclusion_toggle\n");
	print(STDERR "\n");
	print(STDERR "\t\tpeak_file\t\t- name of peak file or \"-\" for standard input\n");
	print(STDERR "\t\tworkbook_name\t\t- name of workbook\n");
	print(STDERR "\t\tinclusion_toggle:\n");
	print(STDERR "\t\t\tinc(lusive)\t- include only peaks with duplicates\n");
	print(STDERR "\t\t\texc(lusive)\t- exclude peaks with duplicates\n");
	print(STDERR "\n");
	exit(1);
}

my $in_peak_file_s = $ARGV[0];
my $workbook_name_s = $ARGV[1];
my $out_peak_file_s = $ARGV[2];

my $inclusion_b = 1; # = 1 if to include duplicate peaks in output
if ($ARGV[3] =~ /^exc/) { # to write out only non-duplicate peaks
	$inclusion_b = 0;
}


my $line_s;
my %assignments_h; # stores all the assigned peak names (residue name and number) and 

my ($all_peak_lines_alist, $end_comments_alist) = &read_peak_file($in_peak_file_s);
my $num_cols_n = scalar(@{$$all_peak_lines_alist[0]}); # number of columns in peak list

open(OUTFILE, ">$out_peak_file_s") || die("Can't open $out_peak_file_s : $!");
print(OUTFILE "#Index\t");
for (my $dim_n = 1; $dim_n < $num_cols_n - 2; $dim_n++) {
    print(OUTFILE "$dim_n", "Dim\t");
}
print(OUTFILE "Intensity\tWorkbook\n");
close(OUTFILE);

foreach my $peak_alist (@{$all_peak_lines_alist}) {

	$$peak_alist[$num_cols_n - 1] =~ /.*\.([A-Z]\d+\S*)$/; # extract the residue number and type
	my $assg_s = $1;


	my @peak_dims_a; # stores each dimension of this peak
    # assuming penultimate column reserved for intensity
    for (my $colnum_n = 1; $colnum_n < ($num_cols_n - 2); $colnum_n++) {
        push(@peak_dims_a, $$peak_alist[$colnum_n]);
    }

	push(@{$assignments_h{$assg_s}}, [ @peak_dims_a ]);
	
}

my $index_n = 0;
foreach my $residue_s (sort keys(%assignments_h)) {
	if ($inclusion_b == 1) { # include only assignments with duplicate peaks in output
		if (scalar(@{$assignments_h{$residue_s}}) > 1) { # duplicate assignment for this residue
			foreach my $peak_dims_alist (@{$assignments_h{$residue_s}}) {
				$index_n++;
		    	open(OF, ">>$out_peak_file_s") || die("Can't open $out_peak_file_s : $!");
	    		print(OF "$index_n\t");
			    print(OF join("\t", @{$peak_dims_alist}));
    			print(OF "\t3000");
	    		print(OF "\t$workbook_name_s" . "_dupl_assgn" . "\.$residue_s\n");
			    close(OF);
			}
		}
	}
	else { # print out non-duplicated assignments
		if (scalar(@{$assignments_h{$residue_s}}) == 1) { # no duplicate assignment for this residue
			$index_n++;
		   	open(OF, ">>$out_peak_file_s") || die("Can't open $out_peak_file_s : $!");
	    	print(OF "$index_n\t");
		    print(OF join("\t", @{$assignments_h{$residue_s}[0]}));
    		print(OF "\t3000");
	    	print(OF "\t$workbook_name_s" . "_unique_assgn" . "\.$residue_s\n");
		    close(OF);
		}
	}
}

open(OF, ">>$out_peak_file_s") || die("Can't open $out_peak_file_s : $!");

# modified from AutoPeak script written by Hunter Moseley
print(OF "*\n\n");
print(OF "\nFile History: $out_peak_file_s \n\n");
print(OF $0, " ", join(' ',@ARGV),"\n");
print(OF "\n");

my $history_found_b = 0;
foreach $line_s (@{$end_comments_alist}) {
    if ((!$history_found_b) && ($line_s =~ /^File History:/)) {
#       print(OF "File History: $outfile_s \n\n");
#       print(OF $0, " ", join(' ', @ARGV), "\n");
        $history_found_b = 1;
        next;
    }

    print(OF "$line_s");
}
print("\n");

#if (!$history_found_b) {
#   print(OF "\nFile History: $outfile_s \n\n");
#   print(OF $0, " ", join(' ',@ARGV),"\n");
#}

close(OF);



# Read in peaks from AutoPeak format files - modified from the subroutine written by Hunter Moseley
sub read_peak_file {
    my ($filename) = @_;

    local *PEAKFILE;
    if ($filename eq "-") {
        *PEAKFILE = *STDIN;
    }
    else {
        open(PEAKFILE, "<$filename") || die("Open " . $filename . " failed : $!");
    }

    my $peak_alist = []; # list of lines containing peak info (each line stored as an array of columns)
    my $line_s;
    while ($line_s = <PEAKFILE>) {
        next if ($line_s =~ /^\#/); # skip comment lines
        last if ($line_s =~ /^[*]/); # end loop if end_of_file marker is encountered.

        ($line_s) = ($line_s =~/^\s*(.*)/); # skip leading white space.
        chomp($line_s);
        push(@$peak_alist, [ split(/\s+/, $line_s) ]);
    }

    my $comment_alist = []; # list of comment lines - History section
    while ($line_s = <PEAKFILE>) {
        push(@$comment_alist, $line_s);
    }

    close(PEAKFILE);

    return wantarray ? ($peak_alist, $comment_alist) : $peak_alist;
}
