#!/usr/bin/perl
#!/usr/bin/perl
#  manage_cmap.pl
#	Manage the sections and columns in a cmap file.
#
#  usage: manage_cmap.pl input_cmap_file [options]
#
#		input_cmap_file - cmap file to read ("-" to read from standard input).
#		no options: lists sections of the input_cmap_file.
#		options:
#			-add cmap_file "column_list1 column_list2 ..." - add quoted list of sections from another cmap file.
#				column_list - list of comma-separated columns (use "all" for all remaining columns).
#			-delete column_list - delete the list of comma-separated columns.
#			-force - force the changes.
#			-nocollapse - make all sections full (no collapse).
#			-collapse - collapse all sections.
#			-resort - resort row order.
#			-trim range_list - remove rows in the given comma-separated index ranges (i.e. 4-7,9-12).
#			-output cmap_file - cmap_file to save changes to ("-" to write to standard output).
#			-extract "column_list1 column_list2 ..." - extract quoted list of sections in the given order.
#				column_list - list of comma-separated columns (use "all" for all remaining columns).
#
#
#  Written by Hunter Moseley 12/3/2001
#  Copyright Hunter Moseley, 2001. All rights reserved.
#

# These modules allow me to use modules rather than hardcoding everything into one script
use FindBin;
use lib $FindBin::Bin;

# Parser module parses in/out the cmap data and the peak file
use CMapParsing qw(:ALL);

# Use the strict module: This is off because of silly bareword problems with the initialize_matirices
 use strict;

if ((@ARGV < 1) || ($ARGV[0] =~ /^-h/))
  {
  print STDERR "Error: not enough parameters given.\n\n" if (@ARGV < 1);
  print STDERR "  manage_cmap.pl\n";
  print STDERR "	Manage the sections and columns in a cmap file.\n";
  print STDERR "\n";
  print STDERR "  usage: manage_cmap.pl input_cmap_file [options]\n";
  print STDERR "\n";
  print STDERR "		input_cmap_file - cmap file to read (\"-\" to read from standard input).\n";
  print STDERR "		no options: lists sections of the input_cmap_file.\n";
  print STDERR "		options:\n";
  print STDERR "			-add cmap_file \"column_list1 column_list2 ...\" - add quoted list of sections from another cmap file.\n";
  print STDERR "				column_list - list of comma-separated columns (use \"all\" for all remaining columns).\n";
  print STDERR "			-delete column_list - delete the list of comma-separated columns.\n";
  print STDERR "			-force - force the changes.\n";
  print STDERR "			-nocollapse - make all sections full (no collapse).\n";
  print STDERR "			-collapse - collapse all sections.\n";
  print STDERR "			-resort - resort row order.\n";
  print STDERR "			-trim range_list - remove rows in the given comma-separated index ranges (i.e. 4-7,9-12).\n";
  print STDERR "			-output cmap_file - cmap_file to save changes to (\"-\" to write to standard output).\n";
  print STDERR "			-extract \"column_list1 column_list2 ...\" - extract quoted list of sections in the given order.\n";
  print STDERR "				column_list - list of comma-separated columns (use \"all\" for all remaining columns).\n";
  print STDERR "\n";
  exit(1);
  }

# read command line options
my $infile = shift @ARGV;
my $add_filename = "";
my $add_alist;
my $delete_alist;
my $output_filename = "";
my $extract_alist;
my $collapse_type = "";
my $force = 0;
my $resort = 0;
my $sections_only = 0;
my $trim_ranges;

for(my $x=0; $x < @ARGV; $x++) 
  {
  if ($ARGV[$x] =~ /-a/)
    { 
    $x++;
    $add_filename = $ARGV[$x];
    $x++;
    $add_alist = [];
    foreach my $section (split(/\s+/,$ARGV[$x]))
      { 
      if ($section eq "all")
	{ push @$add_alist, $section; }
      else
	{ push @$add_alist, [ split(/[,]/,$section) ]; }
      }
    }
  elsif ($ARGV[$x] =~ /^-d/)
    { $x++; $delete_alist = [ sort { $b <=> $a; } (split(/[,]/,$ARGV[$x])) ]; }
  elsif ($ARGV[$x] =~ /^-o/)
    { $x++; $output_filename = $ARGV[$x]; }
  elsif ($ARGV[$x] =~ /^-n/)
    { $collapse_type = "none"; }
  elsif ($ARGV[$x] =~ /^-c/)
    { $collapse_type = "all"; }
  elsif ($ARGV[$x] =~ /^-f/)
    { $force = 1; }
  elsif ($ARGV[$x] =~ /^-t/)
    { 
    $x++;
    $trim_ranges = [];
    foreach my $range (split(/[,]/,$ARGV[$x]))
      { push @$trim_ranges, [ $range =~ /([-]?\d+)[-]([-]?\d+)/ ]; }
    }
  elsif ($ARGV[$x] =~ /^-r/)
    { $resort = 1; }
  elsif ($ARGV[$x] =~ /^-e/)
    { 
    $x++; 
    $extract_alist = [];
    foreach my $section (split(/\s+/,$ARGV[$x]))
      {
      if ($section eq "all")
	{ push @$extract_alist, $section; }
      else
	{ push @$extract_alist, [ split(/[,]/,$section) ]; }
      }
    }
  else
    {
    print STDERR "Error: unknown option ",$ARGV[$x]," given.\n";
    exit(1);
    }
  }

if (( (ref $delete_alist eq "ARRAY") || (ref $add_alist eq "ARRAY") || (ref $add_alist eq "ARRAY") || (ref $extract_alist eq "ARRAY") || (ref $trim_ranges eq "ARRAY") || ($collapse_type ne "") || $resort) && ($output_filename eq ""))
  { print STDERR "\nWarning: no output file given with modifying options.\n\n"; }



# read cmap file
my $cmap_hlist = &parseCMapfile($infile);

#
# delete columns if indicated.
#
if (ref $delete_alist eq "ARRAY")
  {
  foreach my $column (@$delete_alist)
    {
    if (($column <= @{$$cmap_hlist{"parsedtokens"}}) && ($column > 0))
      { splice(@{$$cmap_hlist{"parsedtokens"}},($column-1),1); }
    }
  }

#
# add sections if indicated.
#
if (ref $add_alist eq "ARRAY")
  {
  my $cmap_hlist2 = &parseCMapfile($add_filename); 

  # testing for superset/subset between name arrays
  my $name_conversion1 = {};
  foreach my $name (@{$$cmap_hlist{"name_array"}})
    { $$name_conversion1{$$cmap_hlist{"rlist"}{$name}{"author_name"}} = $name; }

  my $name_conversion2 = {};
  foreach my $name (@{$$cmap_hlist2{"name_array"}})
    { $$name_conversion2{$$cmap_hlist2{"rlist"}{$name}{"author_name"}} = $name; }

  my $test1_super = ! scalar(grep { ! exists $$name_conversion1{$_}; } (keys %$name_conversion2));
  my $test2_super = ! scalar(grep { ! exists $$name_conversion2{$_}; } (keys %$name_conversion1));
  
  if (! $test1_super &&  ! $test2_super && ! $force)
    {
    print STDERR "\nError: incompatible list of residues between CMap files.  Must use -force option to add columns.\n";
    exit(1);
    }
  elsif(! $test1_super && $test2_super)
    { $$cmap_hlist{"name_array"} = $$cmap_hlist2{"name_array"}; }

  # add new residues
  foreach my $res_name (@{$$cmap_hlist2{"name_array"}})
    {    
    my $author_name = $$cmap_hlist2{"rlist"}{$res_name}{"author_name"};
    if (! exists $$name_conversion1{$author_name})
      {
      my $index = @{$$cmap_hlist{"name_array"}} + 1;
      my $name = $$cmap_hlist2{"rlist"}{$res_name}{"aa"} . $index; 
      $$cmap_hlist{"rlist"}{$name}{"aa"} = $$cmap_hlist2{"rlist"}{$res_name}{"aa"}; 
      $$cmap_hlist{"rlist"}{$name}{"name"} = $$cmap_hlist2{"rlist"}{$res_name}{"aa"} . $index; 
      $$cmap_hlist{"rlist"}{$name}{"index"} = $index; 
      $$cmap_hlist{"rlist"}{$name}{"author_index"} = $$cmap_hlist2{"rlist"}{$res_name}{"author_index"}; 
      $$cmap_hlist{"rlist"}{$name}{"author_name"} = $$cmap_hlist2{"rlist"}{$res_name}{"author_name"}; 
      push @{$$cmap_hlist{"name_array"}}, $name;
      $$name_conversion1{$author_name} = $name;
      }
    }

  # add each new section
  my $used_columns = {};
  foreach my $section (@$add_alist)
    {
    my $new_section = [];

    if ($section eq "all")
      { $section = [ grep {! $$used_columns{$_}; } (1 .. (@{$$cmap_hlist2{"parsedtokens"}})) ]; }

    # add each new column of each new section
    foreach my $column (grep {! $$used_columns{$_}; } (@$section))
      {
      next if (($column < 1) || ($column > @{$$cmap_hlist2{"parsedtokens"}}));
 
      $$used_columns{$column} = 1;

      my $token = $$cmap_hlist2{"parsedtokens"}[$column-1];
      push @{$$cmap_hlist{"parsedtokens"}}, $token;
      push @$new_section, $token;

      foreach my $res_name (@{$$cmap_hlist2{"name_array"}})
	{
	my $new_name = $$name_conversion1{$$cmap_hlist2{"rlist"}{$res_name}{"author_name"}};
	if (ref($$cmap_hlist2{"rlist"}{$res_name}{"shifts"}{$token}{"list"}) eq "ARRAY")
	  {
	  if (! exists $$cmap_hlist{"rlist"}{$new_name}{"shifts"}{$token}{"list"})
	    { $$cmap_hlist{"rlist"}{$new_name}{"shifts"}{$token}{"list"} = []; }

	  for(my $x=0; $x < @{$$cmap_hlist2{"rlist"}{$res_name}{"shifts"}{$token}{"list"}}; $x++)
	    { $$cmap_hlist{"rlist"}{$new_name}{"shifts"}{$token}{"list"}[$x] = $$cmap_hlist2{"rlist"}{$res_name}{"shifts"}{$token}{"list"}[$x]; }
	  }
	}
      }
    
    # only add a section if it contains usable columns
    if (@$new_section)
      { push @{$$cmap_hlist{"sections"}}, $new_section; }
    }
  }


#
# extract sections
#
if (ref($extract_alist) eq "ARRAY")
  {
  my $section_alist = [];
  my $used_columns = {};  
  foreach my $section (@$extract_alist)
    {
    my $new_section = [];
    if ($section eq "all")
      { $section = [ grep {! $$used_columns{$_}; } (1..(@{$$cmap_hlist{"parsedtokens"}})) ]; }

    # add each new column of each new section
    foreach my $column (grep {! $$used_columns{$_}; } (@$section))
      {
      next if (($column < 1) || ($column > @{$$cmap_hlist{"parsedtokens"}})); 
      $$used_columns{$column} = 1;

      push @$new_section, $$cmap_hlist{"parsedtokens"}[$column-1];
      }

    if (@$new_section)
      { push @$section_alist, $new_section; }
    }

  if (@$section_alist)
    {
    $$cmap_hlist{"sections"} = $section_alist;
    $sections_only = 1;
    }
  }

# 
# trim row names if indicated
#
if (ref $trim_ranges eq "ARRAY")
  {
  my $index_conversion = {};
  foreach my $index (1 .. @{$$cmap_hlist{"name_array"}})
    { $$index_conversion{$$cmap_hlist{"rlist"}{$$cmap_hlist{"name_array"}[$index-1]}{"author_index"}} = $index; }
  
  my @delete_alist;
  foreach my $range (@$trim_ranges)
    {
    if ($$range[0] <= $$range[1])
      { push @delete_alist, map { $$index_conversion{$_}; } (grep { exists $$index_conversion{$_}; } ($$range[0] .. $$range[1])); }
    }

  @delete_alist = (sort { $b <=> $a; } (@delete_alist));

  foreach my $index (@delete_alist)
    { splice(@{$$cmap_hlist{"name_array"}},$index-1,1); }
  }


# 
# resort row names if indicated
#
if ($resort)
  {
  my $name_conversion1 = {};
  foreach my $name (@{$$cmap_hlist{"name_array"}})
    { $$name_conversion1{$$cmap_hlist{"rlist"}{$name}{"author_index"}} = $name; }
  
  $$cmap_hlist{"name_array"} = [ map { $$name_conversion1{$_}; } (sort { $a <=> $b; } (keys %$name_conversion1)) ]; 
  }

#
# print a list of sections and columns
#
if ($output_filename ne "")
  { print "\nList of CMap sections and columns in $output_filename\n----------------------------------\n\n"; }
else
  { print "\nList of CMap sections and columns in $infile\n----------------------------------\n\n"; }

my $used_tokens = {};

my $all_tokens = {};
foreach my $token (@{$$cmap_hlist{"parsedtokens"}}) 
  { $$all_tokens{$token} = 1; }

my $column_count = 1;
for(my $x=0; $x < @{$$cmap_hlist{"sections"}}; $x++)
  {
  my $section = $$cmap_hlist{"sections"}[$x];

  my $usable_tokens = [ grep {! $$used_tokens{$_} && $$all_tokens{$_}; } (@$section) ];
  if (@$usable_tokens)
    {
    print "Section ",$x+1,":\n";
    foreach my $token (@$usable_tokens)
      { 
      print "\t",$column_count, " - ", $token, "\n"; 
      $$used_tokens{$token} = 1;
      $column_count++;
      }

    print "\n";
    }
  }


#
# write new CMap file if indicated
#
if ($output_filename ne "")
  {
  my $write_options = { "collapse" => $collapse_type };
  
  if ($sections_only)
    { $$write_options{"sections_only"} = 1; }
  
  &outputCMapfile($cmap_hlist, $output_filename, $write_options); 
  }

