#!/usr/bin/perl
#!/usr/bin/perl
# match_recombine.pl 
# 	Generate new peaks from the parts of other peaks that match along certain 
# 	dimensions.
# 
#	Copyright Hunter Moseley, 1999. All rights reserved.
#	Written by Hunter Moseley 4/2/1999
#
#   usage: match_recombine.pl in_peak_file1 in_peak_file2 new_workbook_name out_peak_file \
# 			    intensity_source triple1 triple2 ...
# 
# 	peak_file 	 - name of peak file or "-" for standard input/output.
# 	in_peak_file2 	 - name of peak file.
# 	workbook_name	 - string to use as workbook name.
# 	intensity_source - [1,2] = which peak_file to use for intensity.
# 	triple1 ... 	 - (tolerance toggle1_include toggle2_include) for each dimension.
# 		tolerance      - tolerance for comparison or "0" not to use dimension.
# 		toggle_include - "1" to include dimension from file or "0" not to include.
#
use strict;

# Print usage info
if ((@ARGV < 8) || ((@ARGV-5) % 3) || ($ARGV[0] =~ /^-h/))
  {
  print "\nError: not enough parameters given.\n" if ($ARGV[0] !~ /^-h/);
  print "\n";
  print "match_recombine.pl \n";
  print "	Generate new peaks from the parts of other peaks that match along certain \n";
  print "	dimensions.\n";
  print "\n";
  print "  usage: match_recombine.pl in_peak_file1 in_peak_file2 new_workbook_name out_peak_file \\\n";
  print "			    intensity_source triple1 triple2 ...\n";
  print "\n";
  print "	peak_file 	 - name of peak file or \"-\" for standard input/output.\n";
  print "	in_peak_file2 	 - name of peak file.\n";
  print "	workbook_name	 - string to use as workbook name.\n";
  print "	intensity_source - [1,2] = which peak_file to use for intensity.\n";
  print "	triple1 ... 	 - (tolerance toggle1_include toggle2_include) for each dimension.\n";
  print "		tolerance      - tolerance for comparison or \"0\" not to use dimension.\n";
  print "		toggle_include - \"1\" to include dimension from file or \"0\" not to include.\n";
  exit(1);
  }

# Read in comand line parameters
my $in_peak_file1 = $ARGV[0];
my $in_peak_file2 = $ARGV[1];
my $workbook_name = $ARGV[2];
my $out_peak_file = $ARGV[3];
my $intensity_source = $ARGV[4];

my $x;
my @tolerance_alist;
my @include_alist1;
my @include_alist2;
for($x = 5; $x < @ARGV; $x=$x+3)
  { 
  push @tolerance_alist, $ARGV[$x]; 
  push @include_alist1, $ARGV[$x+1];
  push @include_alist2, $ARGV[$x+2];
  }

# Read Peak files
my ($peak_alist1, $end_comments_alist1) = &read_peak_file($in_peak_file1);
my @orig_cols;
$orig_cols[0] = @{$$peak_alist1[0]};

my ($peak_alist2, $end_comments_alist2) = &read_peak_file($in_peak_file2);
$orig_cols[1] = @{$$peak_alist2[0]};

# Calculate number of dimensions in new peak file
my $x;
my $num_dims;
for($x=0; $x < @include_alist1; $x++)
  {
  if ($include_alist1[$x])
    { $num_dims++; }
  }

for($x=0; $x < @include_alist2; $x++)
  {
  if ($include_alist2[$x])
    { $num_dims++; }
  }

# Create equivalence pairs for recombination
my @equiv_pair_alist;
my $peak1;
my $peak2;
foreach $peak1 (@$peak_alist1)
  {
  foreach $peak2 (@$peak_alist2)
    {
    if (&within_tolerances($peak1, $peak2, \@tolerance_alist))
      { push @equiv_pair_alist, [$peak1, $peak2]; }
    }
  }


# Open Output file
local *OUTPUT_FILE;
if ($out_peak_file eq "-")
  { *OUTPUT_FILE = *STDOUT; }
else
  { open(OUTPUT_FILE, ">$out_peak_file") || die ("open " . $out_peak_file . " failed."); }

# Print Header
print OUTPUT_FILE "#Index	";
my $x;
for ($x=1; $x <= $num_dims; $x++)
  { print OUTPUT_FILE $x, "Dim	"; }
print OUTPUT_FILE "	Intensity	Workbook\n";

# Print recombined peaks
my $peak_pair;
my $x = 1;
foreach $peak_pair (@equiv_pair_alist)
  {
  my @new_peak = ();
  push @new_peak, $x++;
  my $dim;
  for($dim=0; $dim < @include_alist1; $dim++)
    {
    if ($include_alist1[$dim])
      { push @new_peak, $$peak_pair[0][$dim+1]; }
    if ($include_alist2[$dim])
      { push @new_peak, $$peak_pair[1][$dim+1]; }
    }
  push @new_peak, "\t" . $$peak_pair[$intensity_source-1][$orig_cols[$intensity_source-1]-2];
  my ($notes) = ($$peak_pair[$intensity_source-1][$orig_cols[$intensity_source-1]-1] =~ /[^.]*[.](.*)/);
  push @new_peak, ("\t" x (length($new_peak[$#new_peak]) < 9)) . $workbook_name . ("."x($notes ne "")) . $notes; 
  print OUTPUT_FILE join("\t",@new_peak), "\n";
  }


# Print Comments
print OUTPUT_FILE "*\n";
print OUTPUT_FILE "File History: $out_peak_file \n\n";
print OUTPUT_FILE $0, " ", join(' ',@ARGV), "\n\n";

print OUTPUT_FILE "\n\nComments: $in_peak_file1\n";
print OUTPUT_FILE "-" x 80, "\n";
print OUTPUT_FILE @$end_comments_alist1;

print OUTPUT_FILE "\n\nComments: $in_peak_file2\n";
print OUTPUT_FILE "-" x 80, "\n";
print OUTPUT_FILE @$end_comments_alist2;

close OUTPUT_FILE;


#
#  Subroutines
#
sub read_peak_file 
  {
  my $filename = shift @_;
    
  local *PEAKFILE;
  if ($filename eq "-")
    { *PEAKFILE = *STDIN; }
  else
    { open(PEAKFILE, $filename) || die ("open " . $filename . " failed."); }

  my $peak_alist = [];
  my $line;
  while ($line = <PEAKFILE>) 
    {
    next if ($line =~ /^\#/); # skip comment lines
    last if ($line =~ /^[*]/); # end loop if end_of_file marker is encountered.
    
    ($line) = ($line =~/^\s*(.*)/); # skip leading white space.
    chomp $line;
    push @$peak_alist, [ split(/\s+/, $line) ];
    }

  my $comment_alist = [];
  while ($line = <PEAKFILE>)
    { push @$comment_alist, $line; }

  close PEAKFILE;
  return wantarray ? ($peak_alist, $comment_alist) : $peak_alist;
  }

sub within_tolerances
  {
  my $peak1 = shift @_;
  my $peak2 = shift @_;
  my $tolerance_alist = shift @_;

  my $x;
  for($x=0; $x < @$tolerance_alist; $x++)
    {
    if (($$tolerance_alist[$x] != 0.0) && (abs($$peak1[$x+1] - $$peak2[$x+1]) >= $$tolerance_alist[$x]))
      { last; }
    }

  if ($x == @$tolerance_alist)
    { return 1; }

  return 0;
  }
