#!/usr/bin/perl
# n15NOESY2SimNOESY.pl
#     Converts real nitrogen chemical shifts to the simultaneous NOESY pseudo shifts.
# 
# usage: n15NOESY2SimNOESY.pl input_file output_file [options]
#
#       input_file - sparky input file
#       output_file sparky output file
#       options:
#           -column index - column containing the nitrogen chemical shifts (default 3).
#           -carbon_center value - carbon center (default 41.748).
#           -nitrogen_center value - nitrogen center (default 118.109).
#           -conversion_factor - conversion factor (default 2.59).
#           -intensity index - column containing the intensity (default 5).
#
#   Conversion Equation:  pseudo_carbon = (nitrogen_value - nitrogen_center) * conversion_factor + carbon_center
#

# Written by Hunter Moseley 7/16/2007
# Copyright Hunter Moseley 2007. All rights reserved.
#

use strict;

if (! @ARGV)
  {
  print STDERR "n15NOESY2SimNOESY.pl\n";
  print STDERR "     Converts real nitrogen chemical shifts to the simultaneous NOESY pseudo shifts.\n";
  print STDERR "\n";
  print STDERR " usage: n15NOESY2SimNOESY.pl input_file output_file [options]\n";
  print STDERR "\n";
  print STDERR "       input_file - sparky input file\n";
  print STDERR "       output_file sparky output file\n";
  print STDERR "       options:\n";
  print STDERR "           -column index - column containing the nitrogen chemical shifts (default 3).\n";
  print STDERR "           -carbon_center value - carbon center (default 41.748).\n";
  print STDERR "           -nitrogen_center value - nitrogen center (default 118.109).\n";
  print STDERR "           -conversion_factor - conversion factor (default 2.59).\n";
  print STDERR "           -intensity index - column containing the intensity (default 5).\n";
  print STDERR "\n";
  print STDERR "   Conversion Equation:  pseudo_carbon = (nitrogen_value - nitrogen_center) * conversion_factor + carbon_center\n"; 
  print STDERR "\n";
  exit(1);
  }

my $input_file = shift @ARGV;
my $output_file = shift @ARGV;

my $nitrogen_column = 3;
my $carbon_center = 41.748 ;   # Put here spectrum center for C
my $nitrogen_center = 118.109 ;  # Put here spectrum center for N
my $conversion_factor = 2.59;
my $intensity_column = 5;

while (@ARGV)
  {
  my $switch = shift @ARGV;
  if ($switch =~ /^\-col/)
    { $nitrogen_column = shift @ARGV;}
  elsif ($switch =~ /^\-car/)
    { $carbon_center = shift @ARGV; }
  elsif ($switch =~ /^\-n/)
    { $nitrogen_center = shift @ARGV; }
  elsif ($switch =~ /^\-con/)
    { $conversion_factor = shift @ARGV; }
  elsif ($switch =~ /^\-i/)
    { $intensity_column = shift @ARGV; }
  else
    { print STDERR "Error:  Invalid parameter $switch given.\n"; exit(1); }
  }

open(INPUT_FILE, "<$input_file") || die ("open " . $input_file . " failed.");
open(OUTPUT_FILE, ">$output_file") || die ("open " . $output_file . " failed.");


while (my $line = <INPUT_FILE>) 
  {
  next if ($_ =~ /^\#/ );       # Discard comment lines
  next if ($_ =~ / Assig/ );   # Discard possible title lines
  chomp $line;
  ($line) = ($line =~/^\s*(.*)/); # skip leading white space.
  my @columns = split(/\s+/,$line);
  next if (@columns < 4);
  

  # Convert the nitrogen shift to a pseudo carbon shift
  $columns[$nitrogen_column-1] = ($columns[$nitrogen_column-1] - $nitrogen_center) * $conversion_factor + $carbon_center;

  # flip the intensity
  if ($intensity_column > 0)
    { $columns[$intensity_column-1] *= -1; }

  print OUTPUT_FILE join("\t ",@columns),"\n";
  }

close(INPUT_FILE);
close(OUTPUT_FILE);
