#!/usr/bin/perl
#!/usr/bin/perl
# pks2bmrb.pl
# 	Converts AutoAssign peak list into BMRB spectral peak list save frame.
# 
#	Copyright Hunter Moseley, 2002. All rights reserved.
#	Written by Hunter Moseley 3/15/2002
#
#   usage: pks2bmrb.pl in_bmrb_file in_peak_file out_bmrb_file [options] [-dim descrip1 descrip2 ...]
# 
# 	peak_file 	 - name of peak file or "-" for standard input/output.
# 	bmrb_file 	 - name of BMRB file or "-" for standard input/output.
#	options:
#		-label experiment_label - experiment label (default is workbook name of first peak).
#		-dim descrip1 ... - list of dimension descriptions.
#			nuclei_type - type of nuclei in dimension (i.e. 1H, 13C, etc.).
#			spectral_region - spectral region for dimension (i.e. CA, H-C, CA/CB, etc.).
#			shift_error - shift error.
#
#
#  Updated to use new BMRBParsing/CMapParsing modules.
#  Copyright Hunter Moseley 2005. All rights reserved.
#
#  Hunter Moseley: Updated program to not overwrite an existing file.
#  Partial rewrite of the program on 08/19/2005
#

#
# These modules allow me to use modules rather than hardcoding everything into one script
use FindBin;
use lib $FindBin::Bin;

# BMRB Parser
use BMRBParsing qw(:ALL);

use strict;

#
# Print usage info
if ((@ARGV < 3) || ($ARGV[0] =~ /^-h/))
  {
  print STDERR "\nError: not enough parameters given.\n" if ($ARGV[0] !~ /^-h/);
  print STDERR "\n";
  print STDERR " pks2bmrb.pl\n";
  print STDERR " 	Converts AutoAssign peak list into BMRB spectral peak list save frame.\n";
  print STDERR " \n";
  print STDERR "   usage: pks2bmrb.pl in_bmrb_file in_peak_file out_bmrb_file [options] [-dim descrip1 descrip2 ...]\n";
  print STDERR " \n";
  print STDERR " 	peak_file 	 - name of peak file or \"-\" for standard input/output.\n";
  print STDERR " 	bmrb_file 	 - name of BMRB file or \"-\" for standard input/output.\n";
  print STDERR "	options:\n";
  print STDERR "		-label experiment_label - experiment label (default is workbook name of first peak).\n";
  print STDERR "		-dim descrip1 ... - list of dimension descriptions.\n";
  print STDERR "			nuclei_type - type of nuclei in dimension (i.e. 1H, 13C, etc.).\n";
  print STDERR "			spectral_region - spectral region for dimension (i.e. CA, H-C, CA/CB, etc.).\n";
  print STDERR "			shift_error - shift error.\n";
  print STDERR "\n";
  exit(1);
  }

# Read command line arguements.
my $in_bmrb_file = shift @ARGV;
my $in_peak_file = shift @ARGV;
my $out_bmrb_file = shift @ARGV;
my $experiment_label = "";
my @dim_descriptions = ();

while (($ARGV[0] =~ /^\-/) && @ARGV)
  {
  my $switch = shift @ARGV;
  if ($switch =~ /^\-l/)
    { $experiment_label = shift @ARGV; }
  elsif ($switch =~ /^\-d/)
    { 
    while(@ARGV > 1)
      { push @dim_descriptions, [ splice(@ARGV,0,4) ]; }

    foreach my $dim (@dim_descriptions)
      {
      my $nuclei = shift @$dim;
      my ($isotope, $type) = ($nuclei =~ /^(\d*)(\w)/);
      unshift @$dim, $isotope, $type;
      }
    }
  else
    { 
    print STDERR "INVALID OPTION: $switch\n\nExiting Program ...\n\n";  
    exit(1);
    }
  }


#
# Read Peak file
#
my ($peak_alist, $end_comments_alist) = &read_peak_file($in_peak_file);
my $num_cols = @{$$peak_alist[0]};

# read bmrb file
my $bmrb_hlist = &read_bmrb_file($in_bmrb_file);


# set experiment label if not set.
if ($experiment_label eq "")
  { 
  $experiment_label = $$peak_alist[0][$num_cols-1]; 
  $experiment_label =~ s/^(.*?)\..*/$1/
  }

# create dimension descriptions if they are not given.
if (@dim_descriptions == 0)
  {
  for(my $x = 1; $x < $num_cols-2; $x++)
    { push @dim_descriptions, [ ".", "." , "."]; }

  my @test_ranges = ([6,10, [ 1, "H", "HN", "0.02"] ],
		     [105,135, [ 15, "N", "HN", "0.25"] ],
		     [1,5.5, [ 1, "H", "H-C", "0.04" ] ],
		     [20,60, [ 13, "C", "C", "0.35" ] ],
		     [160,190, [ 13, "C", "C'", "0.35" ] ]);

  my @average_per_dim;
  foreach my $peak (@$peak_alist)
    {
    for(my $x=1; $x < $num_cols-2; $x++)
      { $average_per_dim[$x-1] += $$peak[$x]; }  
    }
  
  for(my $x=0; $x < @average_per_dim; $x++)
    { $average_per_dim[$x] /= (@$peak_alist); }  
  
  for(my $x=0; $x < @average_per_dim; $x++)
    {
    foreach my $tester (@test_ranges)
      {
      if (($average_per_dim[$x] >= $$tester[0]) && ($average_per_dim[$x] <= $$tester[1]))
	{
	$dim_descriptions[$x] = $$tester[2];
	last;
	}
      }
    }
  }

# pop off excess dimension descriptions
while(@dim_descriptions > ($num_cols-3))
  { pop @dim_descriptions; }


# determine last spectral peal list id
my $spl_id = 0;
foreach my $spl (values %{$$bmrb_hlist{"peak_lists"}})
  {
  if ($$spl{"peak_list_id"} > $spl_id)
    { $spl_id = $$spl{"peak_list_id"}; }
  }

$spl_id++;

my $spl = {};
# add dimension descriptions to save frame
for(my $x=0; $x < @dim_descriptions; $x++)
  {
  $$spl{"dimension_descriptions"}[$x]{"dim_id"} = $x+1;
  $$spl{"dimension_descriptions"}[$x]{"isotope"} = $dim_descriptions[$x][0];
  $$spl{"dimension_descriptions"}[$x]{"atom_type"} = $dim_descriptions[$x][1];
  $$spl{"dimension_descriptions"}[$x]{"spectral_region"} = $dim_descriptions[$x][2];
  }

# add miscelaneous info to save frame
$$spl{"peak_list_id"} = $spl_id;
$$spl{"details"} = [ @$end_comments_alist ];
$$spl{"num_dimensions"} = scalar(@dim_descriptions);
$$spl{"experiment_label"} = "\$" . $experiment_label . "_" . $spl_id;
$$spl{"index_array"} = [];

# add peak info to save_frame
for(my $index = 0; $index < @$peak_alist; $index++)
  {
  my $peak = {};
  
  $$peak{"index"} = $index+1;
  push @{$$spl{"index_array"}}, $index+1;
  $$peak{"intensity"} = $$peak_alist[$index][$num_cols-2];
  my $details = $$peak_alist[0][$num_cols-1]; 
  if ($details =~ /\.[^.]+/)
    {
    $details =~ s/^.*?\.(.*)$/$1/;
    if ($details ne "")
      { $$peak{"details"} = $details; }
    }
  
  for(my $dim = 0; $dim < @dim_descriptions; $dim++)
    {
    $$peak{"dimensions"}{$dim+1}{"dim_id"} = $dim+1;
    $$peak{"dimensions"}{$dim+1}{"shift"} = $$peak_alist[$index][$dim+1];
    $$peak{"dimensions"}{$dim+1}{"shift_error"} = $dim_descriptions[$dim][3];
    }
  
  $$spl{"plist"}{$$peak{"index"}} = $peak;
  push @{$$spl{"index_array"}}, $$peak{"index"};
  }

$$bmrb_hlist{"peak_lists"}{$$spl{"peak_list_id"}} = $spl;

#
# write bmrb file out
#
&write_bmrb_file($bmrb_hlist, $out_bmrb_file);


#
#  Subroutines
#
sub read_peak_file 
  {
  my $filename = shift @_;
    
  local *PEAKFILE;
  if ($filename eq "-")
    { *PEAKFILE = *STDIN; }
  else
    { open(PEAKFILE, $filename) || die ("open " . $filename . " failed."); }

  my $peak_alist = [];
  my $line;
  while ($line = <PEAKFILE>) 
    {
    next if ($line =~ /^\#/); # skip comment lines
    last if ($line =~ /^[*]/); # end loop if end_of_file marker is encountered.
    
    ($line) = ($line =~/^\s*(.*)\s*$/); # skip leading and ending white space.
    chomp $line;
    push @$peak_alist, [ split(/\s+/, $line) ];
    }

  my $comment_alist = [];
  while ($line = <PEAKFILE>)
    {
    chomp $line;
    push @$comment_alist, $line; 
    }

  close PEAKFILE;
  return wantarray ? ($peak_alist, $comment_alist) : $peak_alist;
  }
