#!/usr/bin/perl
#!/usr/bin/perl 
#
# rdc2cns.pl
#	Converts RDCs from peak list format to cns format.
#
# usage: rdc2cns.pl in_peak_file cns_out_file dim1_atom_type dim2_atom_type [option]
#
#	in_peak_file		- name of peak file or "-" for standard input/output.
#	cns_out_file		- name of CNS-formatted output file.
#	dim1_atom_type		- Atom type for the 1Dim column.
#	dim2_atom_type		- Atom type for the 2Dim column.
#	options:
#		-margin n	- n is the error margin to use in CNS calculations.
#
# Assumption: Residue name and numbers are stored in the Notes column.
#             RDCs in Hz are stored in the last *Dim column.
#             CNS convention is used for naming atom types for the each dimension.
#             RDCs are only for intra-residue atoms.
#
#
# Copyright Aneerban Bhattacharya, 2001. All rights reserved.
# Originally by Deyou Zheng
# Modified by Aneerban Bhattacharya, Apr 4, 2001
# Modified by Aneerban Bhattacharya	Mar 21, 2002
#
# Procedure for reading peak file, writing out file history, and interactive help format
# taken from AutoPeak scripts written by Hunter Moseley.
#
# 

use strict;

# Print usage information
if ((scalar(@ARGV) < 4) || ($ARGV[0] =~ /^-h/)) {
    print(STDERR "\nError: not enough parameters given.\n") if ($ARGV[0] !~ /^-h/);
    print(STDERR "\n");
	print(STDERR " rdc2cns.pl\n");
	print(STDERR "\tConverts RDCs from peak list format to cns format.\n");
	print(STDERR "\n");
	print(STDERR " usage: rdc2cns.pl in_peak_file cns_out_file dim1_atom_type dim2_atom_type [option]\n");
	print(STDERR "\n");
	print(STDERR "\tin_peak_file\t\t- name of peak file or \"-\" for standard input/output.\n");
	print(STDERR "\tcns_out_file\t\t- name of CNS-formatted output file.\n");
	print(STDERR "\tdim1_atom_type\t\t- Atom type for the 1Dim column.\n");
	print(STDERR "\tdim2_atom_type\t\t- Atom type for the 2Dim column.\n");
	print(STDERR "\toptions:\n");
	print(STDERR "\t\t-margin n\t- n is the error margin to use in CNS calculations.\n");
	print(STDERR "\n");
	print(STDERR " Assumption: Residue name and numbers are stored in the Notes column.\n");
	print(STDERR "             RDCs in Hz are stored in the last *Dim column.\n");
	print(STDERR "             CNS convention is used for naming atom types for the each dimension.\n");
	print(STDERR "             RDCs are only for intra-residue atoms.\n");
	print(STDERR "\n");
	exit(1);
}


my ($infile_s, $outfile_s, $dim1_atom_type_s, $dim2_atom_type_s) = @ARGV;

my $line_s;
my $error_margin_n = 0.500; # margin to use in CNS calculations

if ((scalar(@ARGV) > 5) && ($ARGV[4] =~ /^-mar/) && $ARGV[5] =~ /^\-?\d*\.\d+$/) {
	$error_margin_n = $ARGV[5];
}

my ($in_peak_lines_alist, $end_comments_alist) = &read_peak_file($infile_s);
my $num_cols_n = scalar(@{$$in_peak_lines_alist[0]}); # number of columns in peak list

open (OF, ">$outfile_s") || die ("Can't open $outfile_s :  $!\n");

foreach my $peak_alist (@{$in_peak_lines_alist}) {
	$$peak_alist[$num_cols_n - 1] =~ /.*\.[A-Z](\d+)\S*$/; # extract the residue number
	
	my $resnum_n = $1;
	my $RDC_val_n = $$peak_alist[$num_cols_n - 3]; # assumed that the last *Dim column has RDC in Hz
	
	print(OF "assign ( resid 500  and name OO  )\n       ( resid 500  and name Z   )\n       ( resid 500  and name X   )\n       ( resid 500  and name Y   )\n");
	print(OF sprintf("       ( resid %3d  and name $dim2_atom_type_s   )\n", $resnum_n));
	print(OF sprintf("       ( resid %3d  and name $dim1_atom_type_s  )  %.4f %.4f \n\n", $resnum_n, $RDC_val_n, $error_margin_n));
}

close(OF);



# Read in peaks from AutoPeak format files - modified from the subroutine written by Hunter Moseley
sub read_peak_file {
    my ($filename) = @_;

    local *PEAKFILE;
    if ($filename eq "-") {
        *PEAKFILE = *STDIN;
    }
    else {
        open(PEAKFILE, "<$filename") || die("Open " . $filename . " failed : $!");
    }

    my $peak_alist = []; # list of lines containing peak info (each line stored as an array of columns)
    my $line_s; 
    while ($line_s = <PEAKFILE>) {
        next if ($line_s =~ /^\#/); # skip comment lines
        last if ($line_s =~ /^[*]/); # end loop if end_of_file marker is encountered.

        ($line_s) = ($line_s =~/^\s*(.*)/); # skip leading white space.
        chomp($line_s);
        push(@$peak_alist, [ split(/\s+/, $line_s) ]);
    }

    my $comment_alist = []; # list of comment lines - History section
    while ($line_s = <PEAKFILE>) {
        push(@$comment_alist, $line_s);
    }

    close(PEAKFILE);

    return wantarray ? ($peak_alist, $comment_alist) : $peak_alist;
}

