#!/usr/bin/perl
#!/usr/bin/perl 
#
# rdc2pales.pl
#	Converts RDCs from peak list format to PALES input table.
#
# usage: rdc2pales.pl in_peak_file pales_inp_tab_file dim1_atom_type dim2_atom_type [option]
#
#	in_peak_file		- name of peak file or "-" for standard input.
#	pales_inp_tab_file	- name of PALES input-format table file.
#	dim1_atom_type		- Atom type for the 1Dim column.
#	dim2_atom_type		- Atom type for the 2Dim column.
#	option:
#		-seq sequence_file: sequence of ENTIRE, in 1-letter format.
#
# Assumption: Residue name and numbers are stored in the Notes column.
#             RDCs in Hz are stored in the last *Dim column.
#             PALES convention is used for naming atom types for the each dimension.
#             RDCs are only for intra-residue atoms.
#
# 
# Copyright Aneerban Bhattacharya, 2001. All rights reserved.
# Modified Aneerban Bhattacharya Apr 9, 2002
#
# Procedure for reading peak file, and interactive help format
# taken from AutoPeak scripts written by Hunter Moseley.
#
#

use strict;

# Print usage information
if ((scalar(@ARGV) < 4) || ($ARGV[0] =~ /^-h/)) {
    print(STDERR "\nError: not enough parameters given.\n") if ($ARGV[0] !~ /^-h/);
    print(STDERR "\n");
	print(STDERR " rdc2pales.pl\n");
	print(STDERR "\tConverts RDCs from peak list format to PALES input table.\n");
	print(STDERR "\n");
	print(STDERR " usage: rdc2pales.pl in_peak_file pales_inp_tab_file dim1_atom_type dim2_atom_type [option]\n");
	print(STDERR "\n");
	print(STDERR "\tin_peak_file\t\t- name of peak file or \"-\" for standard input.\n");
	print(STDERR "\tpales_inp_tab_file\t- name of PALES input-format table file.\n");
	print(STDERR "\tdim1_atom_type\t\t- Atom type for the 1Dim column.\n");
	print(STDERR "\tdim2_atom_type\t\t- Atom type for the 2Dim column.\n");
	print(STDERR "\toption:\n");
	print(STDERR "\t\t-seq sequence_file: sequence of ENTIRE, in 1-letter format.\n");
	print(STDERR "\n");
	print(STDERR " Assumption: Residue name and numbers are stored in the Notes column.\n");
	print(STDERR "             RDCs in Hz are stored in the last *Dim column.\n");
	print(STDERR "             PALES convention is used for naming atom types for the each dimension.\n");
	print(STDERR "             RDCs are only for intra-residue atoms.\n");
	print(STDERR "\n");
	exit(1);
}

my ($peakfile_s, $tabfile_s, $dim1_atom_type_s, $dim2_atom_type_s) = @ARGV;

my $seq_file_s = ""; # stores 1-letter AA sequence of ENTIRE protein

if ((scalar(@ARGV) > 5) && ($ARGV[4] =~ /^-seq/)) {
	$seq_file_s = $ARGV[5];
}

my %RDCs_in_sequence_h; # stores RDCs along sequence; AA stored using 1-letter code
my %AAcode_h = (A => 'ALA', C => 'CYS', D => 'ASP', E => 'GLU', F => 'PHE', G => 'GLY', H => 'HIS', I => 'ILE', 
K => 'LYS', L => 'LEU', M => 'MET', N => 'ASN', P => 'PRO', Q => 'GLN', R => 'ARG', S => 'SER', T => 'THR', V =>
 'VAL', W => 'TRP', Y => 'TYR', B => 'ASX', Z => 'GLX');

#my %AAcode_h = ('ALA' => 'A', 'CYS' => 'C', 'ASP' => 'D', 'GLU' => 'E', 'PHE' => 'F', 'GLY' => 'G', 'HIS' => 'H', 'ILE' => 'I', 'LYS' => 'K', 'LEU' => 'L', 'MET' => 'M', 'ASN' => 'N', 'PRO' => 'P', 'GLN' => 'Q', 'ARG' => 'R', 'SER' => 'S',  'THR'=> 'T', 'VAL' => 'V', 'TRP' => 'W', 'TYR' => 'Y', 'ASX' => 'B', 'GLX' => 'Z');

my $in_peak_lines_alist = &read_peak_file($peakfile_s);
my $num_cols_n = scalar(@{$$in_peak_lines_alist[0]}); # number of columns in peak list


my $line_s;

foreach my $peak_alist (@{$in_peak_lines_alist}) {
    $$peak_alist[$num_cols_n - 1] =~ /.*\.([A-Z])(\d+)\S*$/; # extract the residue number
	$RDCs_in_sequence_h{$2} = [ $1, $$peak_alist[$num_cols_n - 3] ]; # the residue name and RDC value for each residue
	# assumed that the last *Dim column has RDC in Hz
}

open(OF, ">$tabfile_s") || die("Can't open $tabfile_s : $!");

if ($seq_file_s ne "") { # 1-letter protein sequence provided in 1-letter code, possibly FASTA format
	my @sequence_a;
	open(SF, "<$seq_file_s") || die("Can't open $seq_file_s : $!");
	while ($line_s = <SF>) {
		chomp($line_s);
		if ($line_s !~ /^>/) {
			push(@sequence_a, split(//, $line_s));
		}
	}
	# print the sequence to the table file
	my $index_n = 0;
	foreach my $residue_n (@sequence_a) {
		if (($index_n % 40) == 0) { # for the 1st, 41st etc residue
	        print(OF "\nDATA SEQUENCE");
    	}
	    if (($index_n % 10) == 0) { # for 1st, 11th etc residue
    	    print(OF " ");
	    }
	
		print(OF "$residue_n");
	    $index_n++; 
	}
}

# print the VARS and the FORMAT lines
print(OF "\n\n");
print(OF "VARS   RESID_I RESNAME_I ATOMNAME_I RESID_J RESNAME_J ATOMNAME_J D DD W\n");
print(OF "FORMAT %5d     %6s       %6s        %5d     %6s     %6s  %9.3f %9.3f %.2f\n");

# Print the atom pairs and other values for dipolar couplings
# NOTE: values for Residue_ID are being store as strings within this program - thus the slight difference in the sprintf format below, and what is shown as the table format in the output file

foreach my $residue_n (sort {$a <=> $b} keys (%RDCs_in_sequence_h)) {
	print(OF sprintf("%5s%6s%6s%5s%6s%6s%9.3f%9.3f%6.2f\n", $residue_n, $AAcode_h{$RDCs_in_sequence_h{$residue_n}[0]}, $dim1_atom_type_s, $residue_n, $AAcode_h{$RDCs_in_sequence_h{$residue_n}[0]}, $dim2_atom_type_s, $RDCs_in_sequence_h{$residue_n}[1], 1.000, 1.00));
#	print(OF sprintf("%5s%6s%6s%5s%6s%6s%9.3f%9.3f%6.2f\n", $atom_pairs_a[$index_n]{'residue1'}, $atom_pairs_a[$index_n]{'resname1'}, $atom_pairs_a[$index_n]{'atom1'}, $atom_pairs_a[$index_n]{'residue2'}, $atom_pairs_a[$index_n]{'resname2'}, $atom_pairs_a[$index_n]{'atom2'}, $atom_pairs_a[$index_n]{'RDC'}, 1.000, 1.00));
}

close(OF);


# Read in peaks from AutoPeak format files - modified from the subroutine written by Hunter Moseley
sub read_peak_file {
    my ($filename) = @_;

    local *PEAKFILE;
    if ($filename eq "-") {
        *PEAKFILE = *STDIN;
    }
    else {
        open(PEAKFILE, "<$filename") || die("Open " . $filename . " failed : $!");
    }

    my $peak_alist = []; # list of lines containing peak info (each line stored as an array of columns)
    my $line_s;
    while ($line_s = <PEAKFILE>) {
        next if ($line_s =~ /^\#/); # skip comment lines
        last if ($line_s =~ /^[*]/); # end loop if end_of_file marker is encountered.

        ($line_s) = ($line_s =~/^\s*(.*)/); # skip leading white space.
        chomp($line_s);
        push(@$peak_alist, [ split(/\s+/, $line_s) ]);
    }

    my $comment_alist = []; # list of comment lines - History section
    while ($line_s = <PEAKFILE>) {
        push(@$comment_alist, $line_s);
    }

    close(PEAKFILE);

    return wantarray ? ($peak_alist, $comment_alist) : $peak_alist;
}

