#!/usr/bin/perl
#  simNOESYbmrb2bmrb.pl
#	Convert pseudo carbon chemical shift values for a simultaneous NOESY to real nitrogen shifts
#
#  usage: simNOESYbmrb2bmrb.pl input_filename output_filename [options]
#
#	filename - bmrb filename or "-" for standard input/output.
#	options:
#           -carbon_center value - carbon center (default 41.748).
#           -nitrogen_center value - nitrogen center (default 118.109).
#           -conversion_factor - conversion factor (default 2.59).
#
#  Copyright Hunter Moseley, 2007. All rights reserved.
#  Written by Hunter Moseley 7/26/2007
#
#
#
use strict;

# allows perl to fine local modules.
use FindBin;
use lib $FindBin::Bin;

# contains subroutines for reading and writing bmrb files.
use BMRBParsing qw(:ALL);


# print help info.
if ((@ARGV < 2) || ($ARGV[0] =~ /^-h/))
  {
  print STDERR "Error: not enough parameters given.\n\n" if (@ARGV < 2);
  print STDERR "simNOESYbmrb2bmrb.pl\n";
  print STDERR "	Convert pseudo carbon chemical shift values for a simultaneous NOESY to real nitrogen shifts\n";
  print STDERR "\n";
  print STDERR "  usage: simNOESYbmrb2bmrb.pl input_filename output_filename [options]\n";
  print STDERR "\n";
  print STDERR "	filename - bmrb filename or \"-\" for standard input/output.\n";
  print STDERR "	options:\n";
  print STDERR "           -carbon_center value - carbon center (default 41.748).\n";
  print STDERR "           -nitrogen_center value - nitrogen center (default 118.109).\n";
  print STDERR "           -conversion_factor - conversion factor (default 2.59).\n";

  exit(1);
  }

# read command line arguments.
my $input_filename = shift @ARGV;
my $output_filename = shift @ARGV;

my $carbon_center = 41.748 ;   # Put here spectrum center for C
my $nitrogen_center = 118.109 ;  # Put here spectrum center for N
my $conversion_factor = 2.59;

while (@ARGV)
  {
  my $switch = shift @ARGV;
  if ($switch =~ /^\-car/)
    { $carbon_center = shift @ARGV; }
  elsif ($switch =~ /^\-n/)
    { $nitrogen_center = shift @ARGV; }
  elsif ($switch =~ /^\-con/)
    { $conversion_factor = shift @ARGV; }
  else
    { print STDERR "Error:  Invalid parameter $switch given.\n"; exit(1); }
  }


# read the bmrb file
my $input_bmrb_hlist = &read_bmrb_file($input_filename, { } );

# cycle through each residue
foreach my $entity (values %{$$input_bmrb_hlist{"entity_list"}})
  {
  foreach my $residue_name (keys %{$$entity{"rlist"}})
    {
    # cycle through each shift
    foreach my $shift_name (keys %{$$entity{"rlist"}{$residue_name}{"shifts"}})
      {
      if ($$entity{"rlist"}{$residue_name}{"shifts"}{$shift_name}{"atom_type"} eq "N")
	{
	foreach my $value (@{$$entity{"rlist"}{$residue_name}{"shifts"}{$shift_name}{"list"}})
	  { $value = ($value - $carbon_center) / $conversion_factor + $nitrogen_center; } # update the shift
	}
      }
    }
  }

# write updated bmrb file
&write_bmrb_file($input_bmrb_hlist, $output_filename);
