#!/usr/bin/perl
#!/usr/bin/perl
# sparkyPks2bmrb.pl 
# 	Converts a sparky peaklist into a BMRB assigned peak list save frame
# 
#	Copyright Hunter Moseley, 2005. All rights reserved.
#	Written by Hunter Moseley 05/9/2005
#
#   usage: sparkyPks2bmrb.pl in_bmrb_file out_bmrb_file [options] sparky_peak_files
#
# 	sparky_peak_file - name of sparky peak file.
#	bmrb_file	 - name of the bmrb file.
#	options:
#		-no_intensity    - sparky peak lists do not have intensities.
#		-no_error        - do not put a shift_error into the bmrb file. 
#
#
#  Updated to use new BMRBParsing/CMapParsing modules.
#  Copyright Hunter Moseley 2005. All rights reserved.
#
#  Hunter Moseley: Partial rewrite of the program on 08/22/2005

use strict;

# Print usage info
if ((@ARGV < 3) || ($ARGV[0] =~ /^-h/))
  {
  print STDERR "\nError: not enough parameters given.\n" if ($ARGV[0] !~ /^-h/);
  print STDERR "\n";
  print STDERR " sparkyPks2bmrb.pl \n";
  print STDERR " 	Converts a sparky peaklist into a BMRB assigned peak list save frame\n";
  print STDERR " \n";
  print STDERR "   usage: sparkyPks2bmrb.pl in_bmrb_file out_bmrb_file [options] sparky_peak_files\n";
  print STDERR "\n";
  print STDERR " 	sparky_peak_file - name of sparky peak file.\n";
  print STDERR "	bmrb_file	 - name of the bmrb file.\n";
  print STDERR "	options:\n";
  print STDERR "		-no_intensity    - sparky peak lists do not have intensities.\n";
  print STDERR "		-no_error        - do not put a shift_error into the bmrb file.\n";
  exit(1);
  }


use FindBin;                    
use lib $FindBin::Bin;    

use PeakParsing qw(:ALL);
use BMRBParsing qw(:ALL);
use strict;


my %ranges = ("HN"  => [6, 10, "1H", "HN", "H", 1, 0.02 ],
	      "N15" => [105, 135, "15N", "HN", "N", 15, 0.25 ],
	      "HX"  => [-1, 6, "1H", "H", "H", 1, 0.04 ],
	      "CX"  => [15, 80, "13C", "C", "C", 13, 0.35 ],
	      "CO"  => [160, 190, "13C", "C", "C", 13, 0.35 ]);


my $bmrb_infile = shift @ARGV;
my $bmrb_outfile = shift @ARGV;
my $missing_intensity = 0;
my $use_shift_error = 1;

while (($ARGV[0] =~ /^\-/) && @ARGV)
  {
  my $switch = shift @ARGV;
  if ($switch =~ /^\-no_i/)
    { $missing_intensity = 1; }
  elsif ($switch =~ /^\-no_e/)
    { $use_shift_error = 0; }
  else
    { 
    print STDERR "INVALID OPTION: $switch\n\nExiting Program ...\n\n";  
    exit(1);
    }
  }

my @sparky_infiles = @ARGV;

my $bmrb_hlist = &read_bmrb_file($bmrb_infile);


foreach my $sparky_file (@sparky_infiles)
  {
  my $sparky_peakfile_hlist = &parseSparkyPeakfile($sparky_file, $missing_intensity);
  
  my $spl = &generate_spl($sparky_peakfile_hlist,$bmrb_hlist);
  $$bmrb_hlist{"peak_lists"}{$$spl{"peak_list_id"}} = $spl;
  }


&write_bmrb_file($bmrb_hlist, $bmrb_outfile);

exit;

sub generate_spl
  {
  my $sparky_hlist = shift @_;
  my $bmrb_hlist = shift @_;

  my $spl_id = 0;
  foreach my $test_id (keys %{$$bmrb_hlist{"peak_lists"}})
    {
    if ($test_id > $spl_id)
      { $spl_id = $test_id; }
    }
  $spl_id++;

  my $experiment_label = $$sparky_hlist{"filename"};
  if ($experiment_label =~ /^(\w+)[.]/)
    { $experiment_label = $1; }

  my $dim_guess = &guess_peak_dimensions($sparky_hlist);
  my @dims = @{$$sparky_hlist{"dimlist"}};


  my $spl = {};
  # add dimension descriptions to save frame
  for(my $x=0; $x < @dims; $x++)
    {
    $$spl{"dimension_descriptions"}[$x]{"dim_id"} = $x+1;
    $$spl{"dimension_descriptions"}[$x]{"isotope"} = $$dim_guess{$dims[$x]}{"isotope"};
    $$spl{"dimension_descriptions"}[$x]{"atom_type"} = $$dim_guess{$dims[$x]}{"nuclei"};
    $$spl{"dimension_descriptions"}[$x]{"spectral_region"} = $$dim_guess{$dims[$x]}{"region"};
    }

  # add miscelaneous info to save frame
  $$spl{"peak_list_id"} = $spl_id;
  $$spl{"num_dimensions"} = scalar(@dims);
  $$spl{"experiment_label"} = "\$" . $experiment_label . "_" . $spl_id;
  $$spl{"details"} = [ "Derived from Sparky Peak List file " . $$sparky_hlist{"filename"} ];
  $$spl{"index_array"} = [];


  foreach my $peak_index (sort { $a <=> $b } (keys %{$$sparky_hlist{"plist"}}))
    {
    push @{$$spl{"index_array"}}, $peak_index;
    $$spl{"plist"}{$peak_index}{"index"} = $peak_index;
    if (exists $$sparky_hlist{"plist"}{$peak_index}{"intensity"} && $$sparky_hlist{"plist"}{$peak_index}{"intensity"} ne "")
      { $$spl{"plist"}{$peak_index}{"intensity"} = $$sparky_hlist{"plist"}{$peak_index}{"intensity"}; }
    if (exists $$sparky_hlist{"plist"}{$peak_index}{"sparky_note"} && $$sparky_hlist{"plist"}{$peak_index}{"sparky_note"} ne "")
      { $$spl{"plist"}{$peak_index}{"details"} = $$sparky_hlist{"plist"}{$peak_index}{"sparky_note"}; }

    my $assignments = &find_bmrb_assignments($$sparky_hlist{"plist"}{$peak_index}{"sparky_name"}, $bmrb_hlist); 
    for(my $x=0; $x < @dims; $x++)
      {
      $$spl{"plist"}{$peak_index}{"dimensions"}{$x+1}{"dim_id"} = $x+1; 
      $$spl{"plist"}{$peak_index}{"dimensions"}{$x+1}{"shift"} = $$sparky_hlist{"plist"}{$peak_index}{"shifts"}{$dims[$x]}; 
      if ($use_shift_error)
	{ $$spl{"plist"}{$peak_index}{"dimensions"}{$x+1}{"shift_error"} = $$dim_guess{$dims[$x]}{"shift_error"}; }

      if ($assignments)
	{
	$$spl{"plist"}{$peak_index}{"dimensions"}{$x+1}{"assignments"}[0]{"aa"} = $$assignments[$x]{"res_type"};
	$$spl{"plist"}{$peak_index}{"dimensions"}{$x+1}{"assignments"}[0]{"index"} = $$assignments[$x]{"res_num"};
	$$spl{"plist"}{$peak_index}{"dimensions"}{$x+1}{"assignments"}[0]{"atom_name"} = $$assignments[$x]{"atom_name"};
	$$spl{"plist"}{$peak_index}{"dimensions"}{$x+1}{"assignments"}[0]{"set_id"} = 1;
	$$spl{"plist"}{$peak_index}{"dimensions"}{$x+1}{"assignments"}[0]{"shift_id"} = $$assignments[$x]{"shift_id"};
	if (exists $$assignments[$x]{"list_id"})
	  { $$spl{"plist"}{$peak_index}{"dimensions"}{$x+1}{"assignments"}[0]{"list_id"} = $$assignments[$x]{"list_id"}; }
	}
      }
    }
  

  return $spl;
  }

sub guess_peak_dimensions
  {
  my $sparky_hlist = shift @_;

  my $dim_guess = {};
  my @dims = @{$$sparky_hlist{"dimlist"}};

  my @assigned_peaks = (grep { $$_{"sparky_name"} !~ /[?]-[?]/; } (values %{$$sparky_hlist{"plist"}}));
  if (@assigned_peaks)
    {
    my $test_dims = &translate_sparky_name($assigned_peaks[0]{"sparky_name"});
    for(my $x=0; $x < @dims; $x++)
      {
      my $atom = $$test_dims[$x]{"atom_type"};
      if ($atom eq "H")
	{ $$dim_guess{$dims[$x]} = { "type" => "1H", "region" => "HN", "nuclei" => "H", "isotope" => 1, "shift_error" => 0.02 }; }
      elsif ($atom eq "N")
	{ $$dim_guess{$dims[$x]} = { "type" => "15N", "region" => "HN", "nuclei" => "N", "isotope" => 15, "shift_error" => 0.25 }; }
      elsif ($atom =~ /^H/)
	{ $$dim_guess{$dims[$x]} = { "type" => "1H", "region" => "H", "nuclei" => "H", "isotope" => 1, "shift_error" => 0.04 }; }
      elsif ($atom =~ /^C/)
	{ $$dim_guess{$dims[$x]} = { "type" => "13C", "region" => "C", "nuclei" => "C", "isotope" => 13, "shift_error" => 0.35 }; }
      }
    }
  else
    {  
    my %average_per_dim;
    foreach my $peak_index (keys %{$$sparky_hlist{"plist"}})
      {
      foreach my $dim (@dims)
	{ $average_per_dim{$dim}+= $$sparky_hlist{"plist"}{$peak_index}{"shifts"}{$dim}; }      
      }
    
    foreach my $dim (@dims)
      { $average_per_dim{$dim}/= scalar(keys %{$$sparky_hlist{"plist"}}); }    
  
    foreach my $dim (@dims)
      {
      foreach my $atomType (keys %ranges)
	{
	if (($average_per_dim{$dim} > $ranges{$atomType}[0]) && ($average_per_dim{$dim} < $ranges{$atomType}[1]))
	  {
	  $$dim_guess{$dim} = { "type" => $ranges{$atomType}[2], "region" => $ranges{$atomType}[3], "nuclei" => $ranges{$atomType}[4], "isotope" => $ranges{$atomType}[5], "shift_error" => $ranges{$atomType}[6] };
	  last;
	  }
	}
      
      $$dim_guess{$dim} = { "type" => "X", "region" => "X", "nuclei" => ".", "isotope" => ".", "shift_error" => "." }  if (! (exists $$dim_guess{$dim}) );
      }
    }

  return $dim_guess;
  }


sub find_bmrb_assignments
  {
  my $sparky_name = shift @_;
  my $bmrb_hlist = shift @_;

  my $translation = &translate_sparky_name($sparky_name);
  return 0 if (! $translation);

  foreach my $dim (@$translation)
    {
    my $entity = &find_entity($bmrb_hlist, $$dim{"res_name"});
    return 0 if (! $entity);
    
    LOOP1: foreach my $shift_name (keys %{$$entity{"rlist"}{$$dim{"res_name"}}{"shifts"}})
      {
      if ($$dim{"atom_type"} eq $shift_name)
	{ 
	$$dim{"shift_id"} = $$entity{"rlist"}{$$dim{"res_name"}}{"shifts"}{$shift_name}{"idlist"}[0]; 
	if (@{$$entity{"rlist"}{$$dim{"res_name"}}{"shifts"}{$shift_name}{"list_id_list"}})
	  { $$dim{"list_id"} = $$entity{"rlist"}{$$dim{"res_name"}}{"shifts"}{$shift_name}{"list_id_list"}[0]; }
	$$dim{"atom_name"} = $shift_name;
	last LOOP1;
	}
      }
    
    if ( ! exists $$dim{"shift_id"})
      { return 0; }
    }
  
  return $translation;
  }

