#!/usr/bin/perl
#!/usr/bin/perl
# tag_peaks.pl 
# 	Extract a group of peaks using a set of filter peaks and tag them.
# 
#	Copyright Hunter Moseley, 1999. All rights reserved.
#	Written by Hunter Moseley 4/2/1999
#
#   usage: tag_peaks.pl in_peak_file filter_peak_file \ 
# 			      new_workbook_name out_peak_file tol1 tol2 ...
# 
# 	peak_file	 - name of peak file or "-" for standard input/output.
# 	filter_peak_file - name of peak file.
# 	workbook_name	 - string to use as workbook name.
# 	tol1 tol2 ...	 - dimension tolerances for comparison or "0" not to use specific 
# 			   dimension.
#
use strict;

# Print usage info
if ((@ARGV < 5) || ($ARGV[0] =~ /^-h/))
  {
  print "\nError: not enough parameters given.\n" if ($ARGV[0] !~ /^-h/);
  print "\n";
  print "tag_peaks.pl \n";
  print "	Extract a group of peaks using a set of filter peaks and tag them.\n";
  print "\n";
  print "  usage: tag_peaks.pl in_peak_file filter_peak_file \\\n";
  print "			      new_workbook_name out_peak_file tol1 tol2 ...\n";
  print "\n";
  print "	peak_file	 - name of peak file or \"-\" for standard input/output.\n";
  print "	filter_peak_file - name of peak file.\n";
  print "	workbook_name	 - string to use as workbook name.\n";
  print "	tol1 tol2 ...	 - dimension tolerances for comparison or \"0\" not to use specific \n";
  print "			   dimension.\n";
  exit(1);
  }

# Read in comand line parameters
my $in_peak_file = $ARGV[0];
my $filter_peak_file = $ARGV[1];
my $workbook_name = $ARGV[2];
my $out_peak_file = $ARGV[3];

my $x;
my @tolerance_alist;
for($x = 4; $x < @ARGV; $x++)
  { push @tolerance_alist, $ARGV[$x]; }


# Read Peak file
my ($peak_alist, $end_comments_alist) = &read_peak_file($in_peak_file);
my $num_cols = @{$$peak_alist[0]};

# Read Filter file
my $filter_alist = &read_peak_file($filter_peak_file);

# Open Output file
local *OUTPUT_FILE;
if ($out_peak_file eq "-")
  { *OUTPUT_FILE = *STDOUT; }
else
  { open(OUTPUT_FILE, ">$out_peak_file") || die ("open " . $out_peak_file . " failed."); }

# Print Header
print OUTPUT_FILE "#Index	";
my $x;
for ($x=1; $x < $num_cols-2; $x++)
  { print OUTPUT_FILE $x, "Dim	"; }
print OUTPUT_FILE "	Intensity	Workbook\n";

# Print filtered peak list. 
my $peak;
my $x = 1;
foreach $peak (@$peak_alist) 
  {
  my $filter_peak;
  if ($filter_peak = &find_best_filter_peak($peak, $filter_alist, \@tolerance_alist))
    {
    my $num_filter_cols = @$filter_peak;
    $$peak[0] = $x++; 
    $$peak[$num_cols-2] = "\t" . $$peak[$num_cols-2];
    my ($notes) = ($$filter_peak[$num_filter_cols-1] =~ /[^.]*[.](.*)/);
    $$peak[$num_cols-1] = ("\t" x (length($$peak[$num_cols-2]) < 9)) . $workbook_name . ("."x($notes ne "")) . $notes; 
    print OUTPUT_FILE join("\t",@$peak), "\n";
    }
  }

# Print Comments
print OUTPUT_FILE "*\n";

my $line;
my $history_found = 0;
foreach $line (@$end_comments_alist)
  {
  if (! $history_found && ($line =~ /^File History:/))
    {
    print OUTPUT_FILE "File History: $out_peak_file \n\n";
    print OUTPUT_FILE $0, " ", join(' ',@ARGV),"\n";
    $history_found = 1;
    next;
    }
  
  print OUTPUT_FILE $line;  
  }

if (! $history_found)
  {
  print OUTPUT_FILE "\nFile History: $out_peak_file \n\n";
  print OUTPUT_FILE $0, " ", join(' ',@ARGV),"\n";
  }

close OUTPUT_FILE;


#
#  Subroutines
#
sub read_peak_file 
  {
  my $filename = shift @_;
    
  local *PEAKFILE;
  if ($filename eq "-")
    { *PEAKFILE = *STDIN; }
  else
    { open(PEAKFILE, $filename) || die ("open " . $filename . " failed."); }

  my $peak_alist = [];
  my $line;
  while ($line = <PEAKFILE>) 
    {
    next if ($line =~ /^\#/); # skip comment lines
    last if ($line =~ /^[*]/); # end loop if end_of_file marker is encountered.
    
    ($line) = ($line =~/^\s*(.*)/); # skip leading white space.
    chomp $line;
    push @$peak_alist, [ split(/\s+/, $line) ];
    }

  my $comment_alist = [];
  while ($line = <PEAKFILE>)
    { push @$comment_alist, $line; }

  close PEAKFILE;
  return wantarray ? ($peak_alist, $comment_alist) : $peak_alist;
  }

sub find_best_filter_peak
  {
  my $peak = shift @_;
  my $filter_alist = shift @_;
  my $tolerance_alist = shift @_;

  my $filter_peak;
  my $x;
  my $best_filter_peak = 0;
  my $best_square_distance = 1000000;
  foreach $filter_peak (@$filter_alist)
    {
    my $square_distance = 0;
    for($x=0; $x < @$tolerance_alist; $x++)
      {
      if ($$tolerance_alist[$x] != 0.0)
	{
	if (abs($$peak[$x+1] - $$filter_peak[$x+1]) >= $$tolerance_alist[$x])
	  { $square_distance = 100000000; last; }
	else
	  {
	  my $difference = abs($$peak[$x+1] - $$filter_peak[$x+1]) / $$tolerance_alist[$x];
	  $square_distance += $difference * $difference; 
	  }
	}      
      }

    if (($x == @$tolerance_alist) && ($best_square_distance > $square_distance))
      {
      $best_filter_peak = $filter_peak; 
      $best_square_distance = $square_distance;
      }
    }

  return $best_filter_peak;
  }
