#!/usr/bin/perl
#!/usr/bin/perl
# typing_degeneracy.pl
#	Calculates the typing degeneracy for a given sequence.
#
#  usage:  typing_degeneracy.pl [options]
#
#		options:
#			-cmap cmap_file - input cmap file to analyze (overrides -sequence).
#			-bmrb bmrb_file - input bmrb file to analyze (overrides -sequence).
#			-entity id_num  - which entity to analyze in the input bmrb file.
#			-sequence sequence - sequence to analyze.
#			-output cmap_file - output file to write bar graphs in.
#			-length min_length - minimum length of degenerate sequences to print (default 4).
#			-similarity min - minimum segment similarity to consider degenerate (default 0.5).
#			-CG - use a residue similarity matrix which reflects CG's being used in the amino acid typing.
#			-linkage - create linkage strength row only.
#			-strength min_strength - minimum linkage strength for defining assigned segments (default 2).
#			-typing - create typing degeneracy bar graph only. 
#			-aromatic - use aromatics in typing for evaluation of mapping uniqueness.
#			-deuteration fraction - set the amount of deuteration to the given fraction (default is 0).
#			-unique ratio - minimum probability ratio for a uniquely mapped segment (default 1000).
#			-noesy strength - strength of a noesy link (default 1). 
#			-dangerous color - graphic color to represent dangerous (highly suspicious) segments in type degeneracy BarGraph Row (default A).
#			-suspicious color - graphic color to represent suspicious segments in type degeneracy BarGraph Row (default B).
#			-tolerance atom1 tol1 atom2 tol2 ... - tolerances for matching intra and sequential shifts.
#				atom - name of atom.
#				tol - maximum tolerance to accept when comparing chemical shift values.
#
#   Written by Hunter Moseley, 2/21/2002
#   Copyright Hunter Moseley, 2/21/2002. All rights reserved.   
#
#
#  Updated to use new BMRBParsing/CMapParsing modules.
#  Copyright Hunter Moseley 2005. All rights reserved.
#
#  Hunter Moseley: added ability to directly read a bmrb file to analyze.
#

# These modules allow me to use modules rather than hardcoding everything into one script
use FindBin;
use lib $FindBin::Bin;

# Parser module parses in/out the cmap data and the peak file
use CMapParsing qw(:ALL);
use BMRB::BayesianCalculations4 qw(:ALL);

use strict;


# Set up the standard variables
my @amino_acid_alist = ("A", "C", "D", "E", "F", "G", "H", "I", "K", "L", "M", "N", "P", "Q", "R", "S", "T", "V", "W", "Y");


# Usage info if not enough arguments or the user asks for help

if ((@ARGV < 1) || ($ARGV[0] =~ /^-h/)) 
  {
  if (!($ARGV[0] =~ /^-h/))
    {
    print STDERR "\nError:: not enough parameters given.\n";
    print STDERR "\n";
    }

  print STDERR "typing_degeneracy.pl\n";
  print STDERR "	Calculates the typing degeneracy for a given sequence.\n";
  print STDERR "\n";
  print STDERR "  usage:  typing_degeneracy.pl [options]\n";
  print STDERR "\n";
  print STDERR "		options:\n";
  print STDERR "			-cmap cmap_file - input cmap file to analyze (overrides -sequence).\n";
  print STDERR "			-bmrb bmrb_file - input bmrb file to analyze (overrides -sequence).\n";
  print STDERR "			-entity id_num  - which entity to analyze in the input bmrb file.\n";
  print STDERR "			-sequence sequence - sequence to analyze.\n";
  print STDERR "			-output cmap_file - output file to write bar graphs in.\n";
  print STDERR "			-length min_length - minimum length of degenerate sequences to print (default 4).\n";
  print STDERR "			-similarity min - minimum segment similarity to consider degenerate (default 0.5).\n";
  print STDERR "			-CG - use a residue similarity matrix which reflects CG's being used in the amino acid typing.\n";
  print STDERR "			-linkage - create linkage strength row only.\n";
  print STDERR "			-strength min_strength - minimum linkage strength for defining assigned segments (default 2).\n";
  print STDERR "			-typing - create typing degeneracy bar graph only. \n";
  print STDERR "			-aromatic - use aromatics in typing for evaluation of mapping uniqueness.\n";
  print STDERR "			-deuteration fraction - set the amount of deuteration to the given fraction (default is 0).\n";
  print STDERR "			-unique ratio - minimum probability ratio for a uniquely mapped segment (default 1000).\n";
  print STDERR "			-noesy strength - strength of a noesy link (default 1). \n";
  print STDERR "			-dangerous color - graphic color to represent dangerous (highly suspicious) segments in type degeneracy BarGraph Row (default A).\n";
  print STDERR "			-suspicious color - graphic color to represent suspicious segments in type degeneracy BarGraph Row (default B).\n";
  print STDERR "			-tolerance atom1 tol1 atom2 tol2 ... - tolerances for matching intra and sequential shifts.\n";
  print STDERR "				atom - name of atom.\n";
  print STDERR "				tol - maximum tolerance to accept when comparing chemical shift values.\n";
  print STDERR "\n";
  exit(1);
  }


# process input parameters
my $input_cmap_file = "";
my $input_bmrb_file = "";
my $entity_id = -1;
my $output_cmap_file = ""; 
my $min_segment_similarity = 0.5;
my $min_print_length = 4;
my $min_linkage_strength = 2;
my $noesy_link_strength = 1;
my $dangerous_color = "A";
my $suspicious_color = "B";
my $use_CG = 0;
my $input_sequence = "";
my $create_linkage_strength_row = 1;
my $create_typing_degeneracy_row = 1;
my $unique_ratio = 1000;
my $deuteration_fraction = 0;
my $use_aromatics = 0;
my $use_sidechains = 1;
my $tol_hlist = {};

while (@ARGV)
  {
  my $switch = shift @ARGV;
  if ($switch =~ /^-o/)
    { $output_cmap_file = shift @ARGV; }
  elsif ($switch =~ /^-le/)
    { $min_print_length = shift @ARGV; }
  elsif ($switch =~ /^-c/)
    { $input_cmap_file = shift @ARGV; }
  elsif ($switch =~ /^-b/)
    { $input_bmrb_file = shift @ARGV; }
  elsif ($switch =~ /^-e/)
    { $entity_id = shift @ARGV; }
  elsif ($switch =~ /^-si/)
    { $min_segment_similarity = shift @ARGV; }
  elsif ($switch =~ /^-li/)
    { 
    $create_linkage_strength_row = 1; 
    $create_typing_degeneracy_row = 0;
    }
  elsif ($switch =~ /^-ty/)
    { 
    $create_linkage_strength_row = 0; 
    $create_typing_degeneracy_row = 1;
    }
  elsif ($switch =~ /^-to/)
    { 
    while (@ARGV > 1)
      {
      my $atom = shift @ARGV;
      my $tol = shift @ARGV;
      $$tol_hlist{$atom} = $tol;
      }
      
    last;
    }
  elsif ($switch =~ /^-se/)
    { $input_sequence = shift @ARGV; }
  elsif ($switch =~ /^-st/)
    { $min_linkage_strength = shift @ARGV; }
  elsif ($switch =~ /^-n/)
    { $noesy_link_strength = shift @ARGV; }
  elsif ($switch =~ /^-da/)
    { $dangerous_color = shift @ARGV; }
  elsif ($switch =~ /^-su/)
    { $suspicious_color = shift @ARGV; }
  elsif ($switch =~ /^-CG/)
    { $use_CG = 1; }
  elsif ($switch =~ /^-u/)
    { $unique_ratio = shift @ARGV; }
  elsif ($switch =~ /^-a/)
    { $use_aromatics = 1; }
  elsif ($switch =~ /^-de/)
    { $deuteration_fraction = shift @ARGV; }
  else
    {
    print STDERR "BAD SWITCH: $switch\n"; exit(1);
    }    
  }

# initialize amino acid and typing statistics
my ($mean_count, $mean, $variance, $restricted_type_lists, $similarity_matrix) = &initialize_typing_stats($use_CG, $use_aromatics, $use_sidechains, $deuteration_fraction);




# read cmap file and create sequence_array
#
# $sequence_array[$x]
#	$ss[0] - cmap residue name.
#	$ss[1] - amino acid type.
#	$ss[2] - typing redundancy.
#	$ss[3] - author residue name.
#	$ss[4] - dangerous/suspicious color for segment (if set).
#	$ss[5] - linkage strength.
#	$ss[6] - PRTL.
#	$ss[7] - dangerous color for linkage (if set).
#	$ss[8] - linkage mixmatch.	
#
my $cmap_hlist = {};
my $sequence_array = [];
if ($input_cmap_file ne "")
  { $cmap_hlist = &parseCMapfile($input_cmap_file); }
elsif ($input_bmrb_file ne "")
  { $cmap_hlist = &readBMRBasCMap($input_bmrb_file, $entity_id); }


if ($input_cmap_file ne "" || $input_bmrb_file ne "")
  { 
  # set PRTL hash values for amino acid typing
  my %PRTL_props;
  $PRTL_props{"mean"} = $mean;
  $PRTL_props{"mean_count"} = $mean_count;
  $PRTL_props{"variance"} = $variance;
  $PRTL_props{"File_ResType_frequencies"} = &aa_frequencies($cmap_hlist);
  $PRTL_props{"restricted_type_lists"} = $restricted_type_lists;
  $PRTL_props{"covariance_type_lists"} = $restricted_type_lists;
  $PRTL_props{"single_bp_sub"} = \&singleBProbByMeanVarianceChiSquare; 
  $PRTL_props{"prior_sub"} = \&deltaPrior; 
  $PRTL_props{"CX_prior_sub"} = \&simplePrior; 

  # create sequence array structure
  foreach my $res_name (@{$$cmap_hlist{"name_array"}})
    {
    my $aa = uc($$cmap_hlist{"rlist"}{$res_name}{"aa"});
    $aa =~ s/B/C/;
    my $residue = $$cmap_hlist{"rlist"}{$res_name};
    my $next_residue = 0;
    if (exists $$residue{"next"} && $$residue{"next"} ne "")
      { $next_residue = $$cmap_hlist{"rlist"}{$$residue{"next"}}; }

    # create ChemicalShift objects
    my $chemical_shifts = {};
    my @all_shifts = (grep { ($_ =~ /^C/) && ($_ !~ /^CO/); } (keys %{$$residue{"shifts"}}));
    my @intra_shifts = (grep { ($_ !~ /-1/) && @{$$residue{"shifts"}{$_}{"list"}}; } (@all_shifts));
    my @inter_shifts = ();
    if ($next_residue) # use next residue if it exists for inter shifts.
      { @inter_shifts = (grep { ($_ =~ /^C.*-1/) && ($_ !~ /^CO/) && @{$$next_residue{"shifts"}{$_}{"list"}}; } (keys %{$$next_residue{"shifts"}})); }
    my @usable_shifts = (@intra_shifts, (grep {my ($test) = ($_ =~ /(.*)-1/); ! (grep { $_ eq $test; } (@intra_shifts)); } (@inter_shifts)));
    my $CX_count = 1;
    foreach my $shift_name (@usable_shifts)
      {
      my $converted_shift_name = $shift_name;
      my $test_residue = $residue;
      if ($shift_name =~ /-1/)
	{
	($converted_shift_name) = ($shift_name =~ /(.*)(-1)?/);
	$test_residue = $next_residue;
	}

      foreach my $shift_value (@{$$test_residue{"shifts"}{$shift_name}{"list"}})
	{
	if (($converted_shift_name ne "CA") && ($converted_shift_name ne "CB"))
	  { $converted_shift_name = "CX" . $CX_count++; }
	$$chemical_shifts{$converted_shift_name} = BMRB::ChemicalShift->new( [ $$cmap_hlist{"filename"}, $$residue{"index"}, $aa, $converted_shift_name, $shift_value ] );
	}
      }

    # create PRTL
    my $PRTL;
    if (scalar(%$chemical_shifts))
      {
      my $prtl_hash = &createPRTL($chemical_shifts, \%PRTL_props);
      my @sorted = (sort {$$prtl_hash{$b} <=> $$prtl_hash{$a}; } (@amino_acid_alist));
      $PRTL = [ shift @sorted, $prtl_hash ];
      }
    else
      {
      my $prtl_hash = {};
      foreach my $aa (@amino_acid_alist)
	{ $$prtl_hash{$aa} = 1; }
      $PRTL = ["A", $prtl_hash ]; 
      }

    my $true_name = $res_name;
    if (exists $$cmap_hlist{"rlist"}{$res_name}{"author_name"} && $$cmap_hlist{"rlist"}{$res_name}{"author_name"} ne "")
      { $true_name = $$cmap_hlist{"rlist"}{$res_name}{"author_name"}; }

    push @$sequence_array, [ $res_name, $aa, 0,  $true_name, "" , "", $PRTL, "" ];
    }
  }
elsif ($input_sequence ne "")
  {
  my @name_array = split("",$input_sequence);
  # create sequence array structure
  for(my $x=1; $x <= @name_array; $x++)
    {
    my $aa = uc($name_array[$x-1]);
    $aa =~ s/B/C/;
    push @$sequence_array, [ $aa . $x, $aa, 0 , $aa . $x];
    }
  }
else 
  {
  print STDERR "Error:  No sequence or input cmap file given.\n";
  exit;
  }


# analyze the sequence with the similarity matrix and return the degeneracy results in an array of arrays
my $degeneracy_lists = &analyze_sequence($sequence_array);

# Find and print Suspicious segment mapping.
if ($input_cmap_file ne "" || $input_bmrb_file ne "")
  { 
  my $suspicious_segments = &analyze_cmap_segments($sequence_array,$cmap_hlist, $degeneracy_lists, $tol_hlist); 

  print "\nSuspicious Spin System Segments:\n\n  Segment - Suspicion\n  -------------------\n\n";
  foreach my $segment (@$suspicious_segments)
    {
    print "  ", substr($$sequence_array[$$segment[0]][3],1),"_";
    for(my $z=0; $z < $$segment[1]; $z++)
      { print $$sequence_array[$$segment[0]+$z][1]; }
    print " - ", " (Highly Suspicious) "x($$sequence_array[$$segment[0]][4] ne $suspicious_color),$$segment[2],"\n";
    }

  print "\n\n";

  if (%$tol_hlist)
    {
    print "\nSuspicious Linkages:\n\n  Linkage - Mismatch Strength\n  ---------------------------\n\n";
    for(my $x=0; $x < @$sequence_array; $x++)
      {
      if ($$sequence_array[$x][8])
	{ print "  ", $$sequence_array[$x][3],"-",$$sequence_array[$x+1][3], " mismatch = ", $$sequence_array[$x][8],"\n"; }
      }

    print "\n\n";
    }
  }


# print degeneracy lists
if ($$degeneracy_lists[$#$degeneracy_lists][0] > $min_print_length)
  {
  print "Similar Sequence Segments:\n\n";
  for(my $x=$#$degeneracy_lists; $x >= 0; $x--)
    {
    if ($$degeneracy_lists[$x][0] >= $min_print_length)
      {
      print "Length ",$$degeneracy_lists[$x][0], ":\n";
      for(my $y = 0; $y < @{$$degeneracy_lists[$x][1]}; $y ++)
	{
	print "  ",substr($$sequence_array[$$degeneracy_lists[$x][1][$y][0]][3],1),"_";
	for(my $z=0; $z < $$degeneracy_lists[$x][0]; $z++)
	  { print $$sequence_array[$$degeneracy_lists[$x][1][$y][0]+$z][1]; }
	print " =>";
	for(my $w=0; $w < @{$$degeneracy_lists[$x][1][$y][1]}; $w++)
	  {
	  print " ",substr($$sequence_array[$$degeneracy_lists[$x][1][$y][1][$w][0]][3],1),"_";
	  for(my $z=0; $z < $$degeneracy_lists[$x][0]; $z++)
	    { print $$sequence_array[$$degeneracy_lists[$x][1][$y][1][$w][0]+$z][1]; }
	  printf "=%5.4f", $$degeneracy_lists[$x][1][$y][1][$w][1];
	  }

	print "\n";
	}
      print "\n\n";
      }
    }
  }



# Create Typing Degeneracy and Linkage Strength bar graphs.
if ($output_cmap_file ne "")
  {
  my $output_cmap_hlist = &create_degeneracy_bar_graph($sequence_array, $create_linkage_strength_row, $create_typing_degeneracy_row, $cmap_hlist);
  &outputCMapfile($output_cmap_hlist,$output_cmap_file);
  }




#
# Subroutines
#

# analyze_cmap_segments
#   Finds linked segments, marks dangerous segments, and returns list of dangerous segments.
#
# Parameters:
#	$sequence_array - reference to sequence array of arrays.
#	$cmap_hlist - ref to cmap hash structure.
#	$degeneracy_lists - list of degerate sequence segments
#	$tol_hlist - tolerances to use in finding resonance connectivity.
#
sub analyze_cmap_segments
  {
  my ($sequence_array,$cmap_hlist, $degeneracy_lists, $tol_hlist) = @_;

  my $suspicious_segments = [];

  # analyze resonance rows
  my @intra_rows = (grep { $_ =~ /^[CcHhNnMm]/ && $_ !~ /-1$/; } (@{$$cmap_hlist{"parsedtokens"}}));
  my @seq_rows = (grep { $_ =~ /^[CcHhNnMm]/ && $_ =~ /-1$/; } (@{$$cmap_hlist{"parsedtokens"}}));
  my @usable_intra = (grep { my $a = $_; (grep { $_ eq ($a . "-1"); } (@seq_rows)); } (grep { $_ =~ /^[CcHhNnMm]/ && $_ !~ /-1$/; } (@{$$cmap_hlist{"parsedtokens"}})));

  for (my $x=0; $x < @{$$cmap_hlist{"name_array"}}; $x++)
    {
    my $res_name = $$cmap_hlist{"name_array"}[$x];
    my $next_res_name = $$cmap_hlist{"rlist"}{$res_name}{"next"};
    next if ($next_res_name eq "");
    $$sequence_array[$x][5] += (grep { my $row_name = $_; 
				       (exists $$cmap_hlist{"rlist"}{$res_name}{"shifts"}{$row_name} && 
					(grep {$_ ne "-"; } (@{$$cmap_hlist{"rlist"}{$res_name}{"shifts"}{$row_name}{"list"}})) && 
					exists $$cmap_hlist{"rlist"}{$next_res_name}{"shifts"}{$row_name . "-1"} && 
					(grep {$_ ne "-"; } (@{$$cmap_hlist{"rlist"}{$next_res_name}{"shifts"}{$row_name . "-1"}{"list"}})) &&
					(! exists $$tol_hlist{$row_name} || 
					 (grep { my $value = $_; ($value ne "-") && 
						   (grep {($_ ne "-") && (abs($value - $_) < $$tol_hlist{$row_name}); } 
						    (@{$$cmap_hlist{"rlist"}{$next_res_name}{"shifts"}{$row_name . "-1"}{"list"}})); } 
					  (@{$$cmap_hlist{"rlist"}{$res_name}{"shifts"}{$row_name}{"list"}})) )); 
				       } (@usable_intra));
    
    $$sequence_array[$x][8] = (grep { my $row_name = $_; 
				      (exists $$cmap_hlist{"rlist"}{$res_name}{"shifts"}{$row_name} && 
				       (grep {$_ ne "-"; } (@{$$cmap_hlist{"rlist"}{$res_name}{"shifts"}{$row_name}{"list"}})) && 
				       exists $$cmap_hlist{"rlist"}{$next_res_name}{"shifts"}{$row_name . "-1"} && 
				       (grep {$_ ne "-"; } (@{$$cmap_hlist{"rlist"}{$next_res_name}{"shifts"}{$row_name . "-1"}{"list"}})) &&
				       exists $$tol_hlist{$row_name} && 
				       ! (grep { my $value = $_; ($value ne "-") && 
						   (grep {($_ ne "-") && (abs($value - $_) < $$tol_hlist{$row_name}); } 
						    (@{$$cmap_hlist{"rlist"}{$next_res_name}{"shifts"}{$row_name . "-1"}{"list"}})); } 
					  (@{$$cmap_hlist{"rlist"}{$res_name}{"shifts"}{$row_name}{"list"}})) ); 
				      } (@usable_intra));
    if ($$sequence_array[$x][8])
      { $$sequence_array[$x][7] = $dangerous_color; }
    }
  
  # analyze noesy_rows
  my @noesy_rows = (grep { $_ =~ /^[Dd]/; } (@{$$cmap_hlist{"parsedtokens"}}));
  my @noesy_offset = map { ($_ =~ /i\+(\d+)/);} (@noesy_rows);
  for (my $x=0; $x < @{$$cmap_hlist{"name_array"}}; $x++)
    {
    my $res_name = $$cmap_hlist{"name_array"}[$x];
    for(my $y=0; $y < @noesy_rows; $y++)
      {
      my $row_name = $noesy_rows[$y];
      if (exists $$cmap_hlist{"rlist"}{$res_name}{"shifts"}{$row_name} && (grep {$_ ne "-"; } (@{$$cmap_hlist{"rlist"}{$res_name}{"shifts"}{$row_name}{"list"}})))
	{
	for(my $z=0; $z < $noesy_offset[$y]; $z++)
	  { $$sequence_array[$x+$z][5] += $noesy_link_strength; }
	}
      }
    }

  # find segments
  my $segment_start = -1;
  for (my $x=0; $x <= @$sequence_array; $x++)
    {
    if (($x == @$sequence_array) || $$sequence_array[$x][5] < $min_linkage_strength)
      {
      # test segment
      if ($segment_start > -1)
	{
	my $test_sequential = 0;
	my $res_name = $$cmap_hlist{"name_array"}[$segment_start];

	# test for sequential resonances in beginning of segment
	if (grep {my $row_name = $_; (exists $$cmap_hlist{"rlist"}{$res_name}{"shifts"}{$row_name} && (grep {$_ ne "-"; } (@{$$cmap_hlist{"rlist"}{$res_name}{"shifts"}{$row_name}{"list"}}))); } (@seq_rows))
	  { $segment_start--; }
	
	my $segment_length = $x - $segment_start + 1;
	my $segment_mapping_prob = &calc_segment_mapping_prob($sequence_array, $segment_start, $segment_start, $segment_length);

	# test if segment is dangerous
	my $suspicion = "";
	my $other_sites = [];
	my $degen_sites = [];
	if ($segment_mapping_prob == 0)
	  { $suspicion = "relative segment mapping probability is zero"; }
	else
	  {
	  $other_sites = &find_nonunique_sites($sequence_array, $segment_start, $segment_length,$segment_mapping_prob, $unique_ratio);
	  $degen_sites = &find_degen_sites($sequence_array, $segment_start, $segment_length, $degeneracy_lists);

	  if (@$other_sites)
	    {
	    $suspicion = "segment mapping is not unique - Likelyhood = " . sprintf("%5.4e\n",$segment_mapping_prob);
	    foreach my $site (@$other_sites)
	      { 
	      $suspicion .= "\t\tLikelyhood(" . substr($$sequence_array[$$site[0]][3],1) . "_";
	      for(my $z=0; $z < $segment_length; $z++)
		{ $suspicion .= $$sequence_array[$$site[0]+$z][1]; }
	      $suspicion .= ") = " . sprintf("%5.4e",$$site[1]) . "  \tRatio = " . sprintf("%5.4f\n",$segment_mapping_prob / $$site[1]);
	      }
	    }
	  elsif(@$degen_sites)
	    { $suspicion = "shorter than typing degeneracy"; }
	  }

	if ($suspicion ne "")
	  {
	  push @$suspicious_segments, [ $segment_start, $segment_length, $suspicion, [ @$other_sites, @$degen_sites ] ]; 
	  for(my $z=0; $z < $segment_length; $z++)
	    { $$sequence_array[$segment_start+$z][4] = $dangerous_color; }
	  }

	$segment_start = -1;
	}
      # mark dangerous if intra rows are present
      elsif (($x < @$sequence_array) && 
	     (grep {my $row_name = $_; (exists $$cmap_hlist{"rlist"}{$$cmap_hlist{"name_array"}[$x]}{"shifts"}{$row_name} && 
					(grep {$_ ne "-"; } (@{$$cmap_hlist{"rlist"}{$$cmap_hlist{"name_array"}[$x]}{"shifts"}{$row_name}{"list"}}))); } (@intra_rows)))
	{
	push @$suspicious_segments, [ $x, 1, "isolated spin system", [] ];
	$$sequence_array[$x][4] = $dangerous_color; 
	}
      }
    elsif ($segment_start == -1)
      { $segment_start = $x; }
    }


  # check suspicious segments for low suspicion.
  my $change = 1;
  while ($change)
    {
    $change = 0;
    foreach my $suspicious (@$suspicious_segments)
      {
      next if (($$sequence_array[$$suspicious[0]][4] eq $suspicious_color) || ! @{$$suspicious[3]});

      if (! (grep { &check_unassigned_or_dangerous($sequence_array, $cmap_hlist, $$_[0], $$suspicious[1], \@intra_rows, \@seq_rows); } (@{$$suspicious[3]})) )
	{
	$change = 1;
	for(my $x=0; $x < $$suspicious[1]; $x++)
	  { $$sequence_array[$$suspicious[0]+$x][4] = $suspicious_color; }
	}
      }
    }

  return $suspicious_segments;
  }


# check_unassigned_or_dangerous
#   Returns false if any part of given segment is assigned and not labeled dangerous.
#
# Parameters:
#	$sequence_array - reference to sequence array of arrays.
#	$cmap_hlist - ref to cmap hash structure.
#	$offset - offset of target segment in the sequence array.
#	$length - length of segment.
#	$intra_rows - list of intra resonance rows to check.
#	$seq_rows - list of sequential resonance rows to check.
#
sub check_unassigned_or_dangerous
  {
  my $sequence_array = shift @_;
  my $cmap_hlist = shift @_;
  my $offset = shift @_;
  my $length = shift @_;
  my $intra_rows = shift @_;
  my $seq_rows = shift @_;

  for(my $x=$offset; $x < ($length+$offset); $x++)
    {
    my $res_name = $$cmap_hlist{"name_array"}[$x];
    my $next_res_name = $$cmap_hlist{"rlist"}{$res_name}{"next"};

    if (grep { my $row_name = $_; (($$sequence_array[$x][4] ne $dangerous_color) &&
				  (exists $$cmap_hlist{"rlist"}{$$cmap_hlist{"name_array"}[$x]}{"shifts"}{$row_name} && 
				  (grep {$_ ne "-"; } (@{$$cmap_hlist{"rlist"}{$$cmap_hlist{"name_array"}[$x]}{"shifts"}{$row_name}{"list"}})))); 
	       } (@$intra_rows))
      { return 0; }

    if (($next_res_name ne "") && grep { my $row_name = $_; (($$sequence_array[$x][4] ne $dangerous_color) &&
				  (exists $$cmap_hlist{"rlist"}{$$cmap_hlist{"name_array"}[$x+1]}{"shifts"}{$row_name . "-1"} && 
				  (grep {$_ ne "-"; } (@{$$cmap_hlist{"rlist"}{$$cmap_hlist{"name_array"}[$x+1]}{"shifts"}{$row_name . "-1"}{"list"}})))); 
	       } (@$seq_rows))
      { return 0; }

    }

  return 1;
  }


# find_degen_sites
#   Returns list of degenerate sequence sites for the given sequence segment.
#
# Parameters:
#	$sequence_array - reference to sequence array of arrays.
#	$segment_start - offset of target segment in the sequence array.
#	$segment_length - length of segment.
#	$degeneracy_lists - lists of degenerate sequence sites broken down by length of degeneracy.
#
sub find_degen_sites
  {
  my $sequence_array = shift @_;
  my $segment_start = shift @_;
  my $segment_length = shift @_;
  my $degeneracy_lists = shift @_;


  foreach my $degen_sublist (@$degeneracy_lists)
    {
    if ($$degen_sublist[0] == $segment_length)
      {
      foreach my $degen_sublist2 (@{$$degen_sublist[1]})
	{
	if ($segment_start == $$degen_sublist2[0])
	  { return $$degen_sublist2[1]; }
	}
      }
    }

  return [];
  }


# find_nonunique_sites
#   Returns list of sites that break the uniqueness criterium for the segment mapping.
#
# Parameters:
#	$sequence_array - reference to sequence array of arrays.
#	$segment_pos - offset of target segment in the sequence array.
#	$segment_length - length of segment.
#	$segment_mapping_prob - relative segment mapping prob.
#	$uniqueness - uniqueness ratio.
#
sub find_nonunique_sites
  {
  my $sequence_array = shift @_;
  my $segment_pos = shift @_;
  my $segment_length = shift @_;
  my $segment_mapping_prob = shift @_;
  my $uniqueness = shift @_;

  my $other_sites = [];

  for(my $x=0; $x <= (@$sequence_array - $segment_length); $x++)
    {
    if ($x != $segment_pos)
      {
      my $test_prob = &calc_segment_mapping_prob($sequence_array,$segment_pos,$x, $segment_length);
      if (($test_prob / $segment_mapping_prob) > (1 / $uniqueness))
	{ push @$other_sites, [$x, $test_prob]; }
      }
    }

  return $other_sites; 
  }



# create_degeneracy_bar_graph
#   Creates and returns cmap hash structure containing optionally two bar graphs: Typing Degeneracy and Linkage Strength.
#
# Parameters:
#	$sequence_array - reference to sequence array of arrays.
#	$create_row2 - toggle to create Linkage Strength.
#	$create_row1 - toggle to create Typing Degeneracy.
#	$begin_cmap - cmap hash structure to begin with (optional).
#
sub create_degeneracy_bar_graph
  {
  my $sequence_array = shift @_;
  my $create_row2 = shift @_;
  my $create_row1 = shift @_;

  my $cmap_hlist = {};
  if (@_)
    { $cmap_hlist = shift @_; }

  my $row_name1 = "BDegeneracy\$Length?P";
  my $row_name2 = "BLinkage\$Strength?P";
  if (! exists $$cmap_hlist{"parsedtokens"})
    { $$cmap_hlist{"parsedtokens"} = [ ]; }


  if (! exists $$cmap_hlist{"name_array"})
    { $$cmap_hlist{"name_array"} = [ (map { $$_[0]; } (@$sequence_array)) ]; }

  if (! exists $$cmap_hlist{"settings"})
    { $$cmap_hlist{"settings"} = []; }

  if ($create_row1)
    { 
    push @{$$cmap_hlist{"parsedtokens"}}, $row_name1; 
    
    my @junk = sort {$b <=> $a; } (map {$$_[2]; } (@$sequence_array));
    my $max = shift @junk;

    push @{$$cmap_hlist{"settings"}}, "Settings: Range $row_name1 0 $max\n"
    }

  if ($create_row2)
    { 
    push @{$$cmap_hlist{"parsedtokens"}}, $row_name2; 
    
    my @junk = sort {$b <=> $a; } (map {$$_[5]; } (@$sequence_array));
    my $max = shift @junk;

    push @{$$cmap_hlist{"settings"}}, "Settings: Range $row_name2 0 $max\n"
    }

  my $prev_name = "";
  for(my $x=0; $x < @$sequence_array; $x++)
    {
    my $aa_array = $$sequence_array[$x];
    if ($prev_name ne "" && ! exists $$cmap_hlist{"rlist"}{$$aa_array[0]}{"prev"})
      { 
      $$cmap_hlist{"rlist"}{$$aa_array[0]}{"prev"} = $prev_name;
      $$cmap_hlist{"rlist"}{$prev_name}{"next"} = $$aa_array[0];
      }

    if (! exists $$cmap_hlist{"rlist"}{$$aa_array[0]}{"name"})
      { $$cmap_hlist{"rlist"}{$$aa_array[0]}{"name"} = $$aa_array[0]; }

    if (! exists $$cmap_hlist{"rlist"}{$$aa_array[0]}{"author_name"})
      { $$cmap_hlist{"rlist"}{$$aa_array[0]}{"author_name"} = $$aa_array[3]; }

    if (! exists $$cmap_hlist{"rlist"}{$$aa_array[0]}{"index"})
      { $$cmap_hlist{"rlist"}{$$aa_array[0]}{"index"} = $x; }

    if ($create_row1)
      { $$cmap_hlist{"rlist"}{$$aa_array[0]}{"shifts"}{$row_name1}{"list"} = [ $$aa_array[4] . $$aa_array[2] ]; }
    if ($create_row2)
      { $$cmap_hlist{"rlist"}{$$aa_array[0]}{"shifts"}{$row_name2}{"list"} = [ $$aa_array[7] . $$aa_array[5] ]; }
    }

  return $cmap_hlist;
  }

# analyze_sequence
#   Find typing degeneracies and array of array of degeneracies.
#
# Parameters:
#	$sequence_array - reference to sequence array of arrays.
#
sub analyze_sequence
  { 
  my ($sequence_array) = @_;
  
  my $results = [];
  my $degeneracy_list = [ 0 ];
  my $search_peptide_length = 2;
  while (@$degeneracy_list && ($search_peptide_length <= @$sequence_array)) 
    {
    $degeneracy_list = &find_degeneracies($sequence_array, $search_peptide_length);

    if (@$degeneracy_list)
      { push @$results, [ $search_peptide_length, $degeneracy_list ]; }

    $search_peptide_length++;
    }
  
  return $results;   
  } 


# find_degeneracies
#   Finds degeneracies of a specified length
#
# Parameters
#	$sequence_array - reference to sequence array of arrays.
#	$search_peptide_length - length of peptide degeneracy to look for.
#
sub find_degeneracies
  { 
  my ($sequence_array, $search_peptide_length) = @_; 
  
  my $pair_lists = [];
  
  for (my $offset = 0; $offset <= (@$sequence_array - $search_peptide_length); $offset++)
    {
    my $match_list = &find_string($sequence_array, $offset, $search_peptide_length);
    if (@$match_list)
      { 
      push @$pair_lists, [ $offset, $match_list ]; 
      for(my $x = 0; $x < $search_peptide_length; $x++)
	{ $$sequence_array[$offset+$x][2] = $search_peptide_length; }
      }
    }
  
  return $pair_lists;
  } 


# find_string
#   Returns all matches to a peptide in the sequence.
#
# Parameters:
#	$sequence_a - reference to sequence array of arrays.
#	$search_start - offset of target peptide sequence in the sequence array.
#	$search_length - length of peptide degeneracy to look for.
#
sub find_string
  { 
  my ($sequence_a, $search_start, $search_length) = @_;

  my $match_list = [];
  for(my $test_start = 0; $test_start <= @$sequence_a - $search_length; $test_start++)
    {
    if ($search_start != $test_start)
      {
      my $score =  &calc_match_score($sequence_a,$search_start,$test_start,$search_length);
      if ($score > $min_segment_similarity)
	{ 
	push @$match_list, [ $test_start, $score ]; 
	for(my $x = 0; $x < $search_length; $x++)
	  { $$sequence_array[$test_start+$x][2] = $search_length; }
	}
      }
    }

  return $match_list;  
  } 


# calc_match_score
#   Returns how close two peptide sequence match each other using a particular similarity matrix.
#
# Parameters:
#	$sequence_a - reference to sequence array of arrays.
#	$search_pos - offset of target peptide sequence in the sequence array.
#	$test_pos - offset of test peptide sequence in the sequence array.
#	$search_length - length of peptide degeneracy to look for.
#
sub calc_match_score
  {
  my ($sequence_a, $search_pos, $test_pos, $search_length) = @_;

  # test to see if the calculation would go beyond the length of the sequence
  if (($test_pos+$search_length) > @$sequence_a)
    { return 0; }

  my $score = 1;
  for(my $count =0; $count < $search_length; $count++, $search_pos++, $test_pos++)
    {
    $score *= ($$similarity_matrix[(ord($$sequence_a[$test_pos][1]) - 65)][((ord ($$sequence_a[$search_pos][1]))-65)]);    
    }

  return $score;
  }


# createPRTL
#   Creates and returns a PRTL hash.
#
# Parameters:
#	$res_shift_hlist - hash of shift_name to ChemicalShift
#	$PRTL_props - hash of properties used by Bayesian Typing methods
#
sub createPRTL
  {
  my $res_shift_hlist = shift @_;
  my $PRTL_props = shift @_;

  my $prtl_hash = {};
  my $total_prob = 0.0;

  # creates an array of arrays where each row is the aminoacid type and the its probability
  foreach my $aa_type (@amino_acid_alist)
    { 
    my ($prob_value, $junk) = &allBayesianProbCombinationsCXrecomb($aa_type, $res_shift_hlist, $PRTL_props, 0);
    $$prtl_hash{$aa_type} = $prob_value;
    $total_prob += $prob_value;
    }

  # normalize each probablilty by the total
  if ($total_prob != 0)
    {
    foreach my $aa_type (@amino_acid_alist)
      { $$prtl_hash{$aa_type} /= $total_prob; } 
    }
  else
    {
    foreach my $aa_type (@amino_acid_alist)
      { $$prtl_hash{$aa_type} = 1; } 
    }
    
  return $prtl_hash;
  }


# calc_segment_mapping_prob
#   Calculates and returns the relative segment mapping probability.
#
# Parameters:
#	$sequence_array - reference to sequence array of arrays.
#	$segment_pos - offset of target segment in the sequence array.
#	$test_pos - offset of test peptide sequence in the sequence array.
#	$segment_length - length of segment.
#
#
sub calc_segment_mapping_prob
  {
  my $sequence_array = shift @_;
  my $segment_pos = shift @_;
  my $test_pos = shift @_;
  my $segment_length = shift @_;

  my $prob = 1;
  
  for(my $z=0; $z < $segment_length; $z++)
    {
    $prob *= $$sequence_array[$segment_pos+$z][6][1]{$$sequence_array[$test_pos+$z][1]} / $$sequence_array[$segment_pos+$z][6][1]{$$sequence_array[$segment_pos+$z][6][0]};
    }

  return $prob;
  }

# aa_frequencies
#   returns the frequency of the 20 amino acids in the given cmap file as a hash.
#	$$protein_hlist{$cmap_filename}{$residue_type}
#
# Parameters:
#	$cmap_hlist - reference to cmap hash structure.
#
sub aa_frequencies
  {
  my $cmap_hlist = shift @_;
  
  my $protein_hlist={};
  foreach my $aa (@{$$cmap_hlist{"sequence"}})
    {
    my $new_aa = $aa;
    $new_aa =~ s/B/C/;
    $$protein_hlist{$$cmap_hlist{"filename"}}{$new_aa}++; 
    }
  
  return $protein_hlist;
  }


# initialize_matching_tables
#   Returns similarity matrix to use in comparing peptide sequences.
#
# Parameters:
#	$use_CG - toggle for using similarity matrix that represents CG typing information. 
#
sub initialize_typing_stats
  {
  my $use_CG = shift @_;
  my $use_aromatics = shift @_;
  my $use_sidechains = shift @_;
  my $deuteration = shift @_;

  my $similarity;
  if (! $use_CG)
    {
    $similarity = [
		      [ 1.0, 0.000000, 0.520307, 0.000000, 0.007995, 0.000000, 0.218420, 0.103614, 0.000000, 0.000000, 0.000959, 0.000000, 0.034858, 0.000000, 0.000000, 0.000000, 0.024624, 0.012152, 0.000000, 0.000000, 0.000000, 0.000000, 0.014071, 0.000000, 0.000000, 0.000000 ],
		      [ 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000 ],
		      [ 0.008503, 0.000000, 1.0, 0.404762, 0.642857, 0.459184, 0.028912, 0.671769, 0.437075, 0.000000, 0.741497, 0.389456, 0.787415, 0.416667, 0.000000, 0.345238, 0.608844, 0.654762, 0.000000, 0.000000, 0.000000, 0.613946, 0.500000, 0.000000, 0.517007, 0.000000 ],
		      [ 0.000000, 0.000000, 0.595805, 1.0, 0.022590, 0.974587, 0.047196, 0.128278, 0.977007, 0.000000, 0.321097, 0.989109, 0.820089, 0.988302, 0.000000, 0.000403, 0.011698, 0.052844, 0.000000, 0.000000, 0.000000, 0.296894, 0.044373, 0.000000, 0.944736, 0.000000 ],
		      [ 0.003947, 0.000000, 0.574671, 0.040132, 1.0, 0.085526, 0.002303, 0.874342, 0.054605, 0.000000, 0.987171, 0.015461, 0.919737, 0.151974, 0.000000, 0.433882, 0.971711, 0.970395, 0.000000, 0.000000, 0.000000, 0.728618, 0.630263, 0.000000, 0.163487, 0.000000 ],
		      [ 0.000000, 0.000000, 0.601578, 0.972387, 0.063116, 1.0, 0.003287, 0.205786, 0.974359, 0.000000, 0.360947, 0.988166, 0.769231, 0.849441, 0.000000, 0.015122, 0.024984, 0.088100, 0.000000, 0.000000, 0.000000, 0.523340, 0.066404, 0.000000, 0.954635, 0.000000 ],
		      [ 0.756250, 0.000000, 0.576786, 0.327976, 0.029464, 0.165476, 1.0, 0.252381, 0.003571, 0.000000, 0.073214, 0.228869, 0.237798, 0.478571, 0.000000, 0.000000, 0.107738, 0.192560, 0.008333, 0.000893, 0.000000, 0.001190, 0.094940, 0.000000, 0.123214, 0.000000 ],
		      [ 0.026030, 0.000000, 0.623644, 0.048807, 0.989154, 0.134490, 0.013015, 1.0, 0.114967, 0.000000, 0.960954, 0.022777, 0.894794, 0.197397, 0.000000, 0.254881, 0.972885, 0.970716, 0.000000, 0.000000, 0.000000, 0.496746, 0.732104, 0.000000, 0.224512, 0.000000 ],
		      [ 0.000000, 0.000000, 0.568451, 0.674088, 0.063477, 0.963051, 0.000000, 0.228328, 1.0, 0.000000, 0.316911, 0.821885, 0.664140, 0.563714, 0.000000, 0.138797, 0.029844, 0.098532, 0.000000, 0.000000, 0.000000, 0.674088, 0.068214, 0.000000, 0.936523, 0.000000 ],
		      [ 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000 ],
		      [ 0.000639, 0.000000, 0.565217, 0.193095, 0.981458, 0.667839, 0.005754, 0.864770, 0.536765, 0.000000, 1.0, 0.113171, 0.928389, 0.558504, 0.000000, 0.407928, 0.943414, 0.955243, 0.000000, 0.000000, 0.000000, 0.934143, 0.666240, 0.000000, 0.796355, 0.000000 ],
		      [ 0.000000, 0.000000, 0.575542, 0.985456, 0.005343, 0.975957, 0.024636, 0.032354, 0.978925, 0.000000, 0.082814, 1.0, 0.533986, 0.976551, 0.000000, 0.000000, 0.002968, 0.011576, 0.000297, 0.000000, 0.000000, 0.086376, 0.009795, 0.000000, 0.941822, 0.000000 ],
		      [ 0.006742, 0.000000, 0.615730, 0.246067, 0.950562, 0.562921, 0.012360, 0.857303, 0.442697, 0.000000, 0.980899, 0.179775, 1.000000, 0.617978, 0.000000, 0.287640, 0.915730, 0.946067, 0.000000, 0.000000, 0.000000, 0.875281, 0.629213, 0.000000, 0.720225, 0.000000 ],
		      [ 0.000000, 0.000000, 0.598758, 0.982506, 0.276524, 0.974041, 0.126975, 0.604966, 0.934537, 0.000000, 0.817720, 0.953160, 0.883183, 1.0, 0.000000, 0.000000, 0.134312, 0.404628, 0.000000, 0.000000, 0.000000, 0.778781, 0.342551, 0.000000, 0.945824, 0.000000 ],
		      [ 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000 ],
		      [ 0.000000, 0.000000, 0.627639, 0.002559, 0.736404, 0.528471, 0.000000, 0.887396, 0.893154, 0.000000, 0.975688, 0.010877, 0.878439, 0.004479, 0.000000, 1.0, 0.326296, 0.916187, 0.000000, 0.000000, 0.000000, 0.993602, 0.701216, 0.000000, 0.844530, 0.000000 ],
		      [ 0.017337, 0.000000, 0.623529, 0.015480, 0.996904, 0.034675, 0.007430, 0.902167, 0.018576, 0.000000, 0.969659, 0.004954, 0.925697, 0.108359, 0.000000, 0.253251, 1.0, 0.980186, 0.000000, 0.000000, 0.000000, 0.393189, 0.668731, 0.000000, 0.076780, 0.000000 ],
		      [ 0.003130, 0.000000, 0.617632, 0.062076, 0.991654, 0.123109, 0.010433, 0.894627, 0.086594, 0.000000, 0.985394, 0.016693, 0.928013, 0.208138, 0.000000, 0.366719, 0.971831, 1.0, 0.000000, 0.000000, 0.000000, 0.758998, 0.679186, 0.000000, 0.257173, 0.000000 ],
		      [ 0.000000, 0.000000, 0.015905, 0.000000, 0.000000, 0.000000, 0.001223, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 1.0, 0.305873, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000 ],
		      [ 0.000000, 0.000000, 0.000436, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.232649, 1.0, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000 ],
		      [ 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000 ],
		      [ 0.000000, 0.000000, 0.591971, 0.134672, 0.684307, 0.656204, 0.000000, 0.820073, 0.925182, 0.000000, 0.915693, 0.195985, 0.760584, 0.207299, 0.000000, 0.900730, 0.413139, 0.847080, 0.000000, 0.000000, 0.000000, 1.0, 0.650365, 0.000000, 0.848175, 0.000000 ],
		      [ 0.006494, 0.000000, 0.645022, 0.038961, 0.995671, 0.110390, 0.006494, 0.919913, 0.086580, 0.000000, 0.978355, 0.021645, 0.935065, 0.166667, 0.000000, 0.424242, 0.982684, 0.976190, 0.000000, 0.000000, 0.000000, 0.742424, 1.000000, 0.000000, 0.227273, 0.000000 ],
		      [ 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000 ],
		      [ 0.000000, 0.000000, 0.593574, 0.967871, 0.110843, 0.967068, 0.007229, 0.363052, 0.971084, 0.000000, 0.510040, 0.976707, 0.813655, 0.877912, 0.000000, 0.042570, 0.056225, 0.151807, 0.000000, 0.000000, 0.000000, 0.630522, 0.130924, 0.000000, 1.0, 0.000000 ],
		      [ 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000 ]
		      ];
    }
  else
    {
    $similarity = [
		      [ 1.0, 0.000000, 0.520307, 0.000000, 0.007995, 0.000000, 0.218420, 0.103614, 0.000000, 0.000000, 0.000959, 0.000000, 0.034858, 0.000000, 0.000000, 0.000000, 0.024624, 0.012152, 0.000000, 0.000000, 0.000000, 0.000000, 0.014071, 0.000000, 0.000000, 0.000000 ],
		      [ 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000 ],
		      [ 0.008503, 0.000000, 1.0, 0.404762, 0.642857, 0.459184, 0.028912, 0.671769, 0.437075, 0.000000, 0.741497, 0.389456, 0.787415, 0.416667, 0.000000, 0.345238, 0.608844, 0.654762, 0.000000, 0.000000, 0.000000, 0.613946, 0.500000, 0.000000, 0.517007, 0.000000 ],
		      [ 0.000000, 0.000000, 0.604534, 1.0, 0.023510, 0.973971, 0.047439, 0.130563, 0.976490, 0.000000, 0.321159, 0.989085, 0.822838, 0.988245, 0.000000, 0.000420, 0.012175, 0.054576, 0.000000, 0.000000, 0.000000, 0.299328, 0.045760, 0.000000, 0.945004, 0.000000 ],
		      [ 0.008040, 0.000000, 0.596985, 0.000000, 1.0, 0.000000, 0.005528, 0.000000, 0.014573, 0.000000, 0.001508, 0.000503, 0.768844, 0.000000, 0.000000, 0.000503, 0.950754, 0.010553, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000 ],
		      [ 0.000000, 0.000000, 0.600662, 0.972185, 0.063576, 1.0, 0.003311, 0.206623, 0.974172, 0.000000, 0.362252, 0.988079, 0.768874, 0.850331, 0.000000, 0.015232, 0.025166, 0.088742, 0.000000, 0.000000, 0.000000, 0.525166, 0.066887, 0.000000, 0.954305, 0.000000 ],
		      [ 0.756250, 0.000000, 0.576786, 0.327976, 0.029464, 0.165476, 1.0, 0.252381, 0.003571, 0.000000, 0.073214, 0.228869, 0.237798, 0.478571, 0.000000, 0.000000, 0.107738, 0.192560, 0.008333, 0.000893, 0.000000, 0.001190, 0.094940, 0.000000, 0.123214, 0.000000 ],
		      [ 0.023991, 0.000000, 0.624864, 0.049073, 0.989095, 0.135224, 0.013086, 1.0, 0.115594, 0.000000, 0.961832, 0.022901, 0.896401, 0.198473, 0.000000, 0.256270, 0.972737, 0.970556, 0.000000, 0.000000, 0.000000, 0.498364, 0.730643, 0.000000, 0.225736, 0.000000 ],
		      [ 0.000000, 0.000000, 0.577306, 0.000000, 0.000000, 0.000000, 0.000664, 0.000000, 1.000000, 0.000000, 0.010617, 0.022561, 0.003318, 0.000000, 0.000000, 0.002654, 0.000000, 0.001991, 0.000000, 0.000000, 0.000000, 0.232250, 0.000000, 0.000000, 0.000000, 0.000000 ],
		      [ 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000 ],
		      [ 0.001626, 0.000000, 0.596748, 0.000000, 0.000000, 0.000000, 0.011924, 0.000000, 0.495393, 0.000000, 1.0, 0.139295, 0.041734, 0.000000, 0.000000, 0.453117, 0.000542, 0.957182, 0.000000, 0.000000, 0.000000, 0.363144, 0.000000, 0.000000, 0.000000, 0.000000 ],
		      [ 0.000000, 0.000000, 0.604107, 0.000000, 0.000000, 0.000000, 0.035817, 0.000000, 0.960363, 0.000000, 0.101242, 1.0, 0.011461, 0.000000, 0.000000, 0.000000, 0.000478, 0.007641, 0.000000, 0.000000, 0.000000, 0.017192, 0.000000, 0.000000, 0.000000, 0.000000 ],
		      [ 0.011152, 0.000000, 0.633829, 0.000000, 0.453532, 0.000000, 0.014870, 0.000000, 0.338290, 0.000000, 0.048327, 0.009294, 1.0, 0.000000, 0.000000, 0.011152, 0.691450, 0.650558, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000 ],
		      [ 0.000000, 0.000000, 0.598758, 0.982506, 0.276524, 0.974041, 0.126975, 0.604966, 0.934537, 0.000000, 0.817720, 0.953160, 0.883183, 1.0, 0.000000, 0.000000, 0.134312, 0.404628, 0.000000, 0.000000, 0.000000, 0.778781, 0.342551, 0.000000, 0.945824, 0.000000 ],
		      [ 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000 ],
		      [ 0.000000, 0.000000, 0.660104, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.812435, 0.000000, 0.958549, 0.014508, 0.030052, 0.000000, 0.000000, 1.0, 0.001036, 0.795855, 0.000000, 0.000000, 0.000000, 0.410363, 0.000000, 0.000000, 0.000000, 0.000000 ],
		      [ 0.025070, 0.000000, 0.672238, 0.000000, 0.984215, 0.000000, 0.009285, 0.000000, 0.006500, 0.000000, 0.001857, 0.000000, 0.921077, 0.000000, 0.000000, 0.001857, 1.0, 0.042711, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000 ],
		      [ 0.006950, 0.000000, 0.634231, 0.000000, 0.008688, 0.000000, 0.015639, 0.000000, 0.065161, 0.000000, 0.950478, 0.013032, 0.612511, 0.000000, 0.000000, 0.379670, 0.016507, 1.0, 0.000000, 0.000000, 0.000000, 0.006950, 0.000000, 0.000000, 0.000000, 0.000000 ],
		      [ 0.000000, 0.000000, 0.015905, 0.000000, 0.000000, 0.000000, 0.001223, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 1.0, 0.305873, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000 ],
		      [ 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.250823, 1.0, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000 ],
		      [ 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000 ],
		      [ 0.000000, 0.000000, 0.626556, 0.000000, 0.000519, 0.000000, 0.000000, 0.000000, 0.944502, 0.000000, 0.164938, 0.031639, 0.000000, 0.000000, 0.000000, 0.104772, 0.000000, 0.004668, 0.000000, 0.000000, 0.000000, 1.0, 0.000000, 0.000000, 0.000000, 0.000000 ],
		      [ 0.006550, 0.000000, 0.644105, 0.039301, 0.995633, 0.109170, 0.006550, 0.919214, 0.085153, 0.000000, 0.978166, 0.021834, 0.938865, 0.163755, 0.000000, 0.427948, 0.982533, 0.975983, 0.000000, 0.000000, 0.000000, 0.744541, 1.000000, 0.000000, 0.227074, 0.000000 ],
		      [ 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000 ],
		      [ 0.000000, 0.000000, 0.591169, 0.968929, 0.109567, 0.972200, 0.006541, 0.358953, 0.971382, 0.000000, 0.506132, 0.977923, 0.818479, 0.876533, 0.000000, 0.043336, 0.053966, 0.150450, 0.000000, 0.000000, 0.000000, 0.628782, 0.128373, 0.000000, 1.0, 0.000000 ],
		      [ 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000 ]
		      ];
    }


  my $count = {};
  my $mean = {};
  my $variance = {};

  
  $$mean{A}{CA} = 53.16;
  $$count{A}{CA} = 1;
  $$variance{A}{CA} = 2.06 * 2.06;
  
  $$mean{A}{CB} = 18.90;
  $$count{A}{CB} = 1;
  $$variance{A}{CB} = 1.85 * 1.85;
  
  
  $$mean{R}{CA} = 56.95;
  $$count{R}{CA} = 1;
  $$variance{R}{CA} = 2.41 * 2.41;
  
  $$mean{R}{CB} = 30.66;
  $$count{R}{CB} = 1;
  $$variance{R}{CB} = 1.77 * 1.77;
  
  $$mean{R}{CG} = 27.31;
  $$count{R}{CG} = 1;
  $$variance{R}{CG} = 1.65 * 1.65;
  
  $$mean{R}{CD} = 43.10;
  $$count{R}{CD} = 1;
  $$variance{R}{CD} = 1.10 * 1.10;
  
  $$mean{R}{CZ} = 159.01;
  $$count{R}{CZ} = 1;
  $$variance{R}{CZ} = 0.87 * 0.87;
  
  
  $$mean{D}{CA} = 54.52;
  $$count{D}{CA} = 1;
  $$variance{D}{CA} = 2.10 * 2.10;
  
  $$mean{D}{CB} = 40.70;
  $$count{D}{CB} = 1;
  $$variance{D}{CB} = 1.67 * 1.67;
  
  $$mean{D}{CG} = 178.41;
  $$count{D}{CG} = 1;
  $$variance{D}{CG} = 1.84 * 1.84;
  
  
  $$mean{N}{CA} = 53.43;
  $$count{N}{CA} = 1;
  $$variance{N}{CA} = 1.94 * 1.94;
  
  $$mean{N}{CB} = 38.66;
  $$count{N}{CB} = 1;
  $$variance{N}{CB} = 1.78 * 1.78;
  
  $$mean{N}{CG} = 176.61;
  $$count{N}{CG} = 1;
  $$variance{N}{CG} = 1.33 * 1.33;
  
  
  $$mean{C}{CA} = 57.43;
  $$count{C}{CA} = 1;
  $$variance{C}{CA} = 3.41 * 3.41;
  
  $$mean{C}{CB} = 34.15;
  $$count{C}{CB} = 1;
  $$variance{C}{CB} = 6.77 * 6.77;
  
  
  $$mean{E}{CA} = 57.42;
  $$count{E}{CA} = 1;
  $$variance{E}{CA} = 2.19 * 2.19;
  
  $$mean{E}{CB} = 30.07;
  $$count{E}{CB} = 1;
  $$variance{E}{CB} = 1.84 * 1.84;
  
  $$mean{E}{CG} = 36.01;
  $$count{E}{CG} = 1;
  $$variance{E}{CG} = 1.51 * 1.51;
  
  $$mean{E}{CD} = 181.94;
  $$count{E}{CD} = 1;
  $$variance{E}{CD} = 1.85 * 1.85;
  
  
  $$mean{Q}{CA} = 56.62;
  $$count{Q}{CA} = 1;
  $$variance{Q}{CA} = 2.19 * 2.19;
  
  $$mean{Q}{CB} = 29.10;
  $$count{Q}{CB} = 1;
  $$variance{Q}{CB} = 1.97 * 1.97;
  
  $$mean{Q}{CG} = 33.72;
  $$count{Q}{CG} = 1;
  $$variance{Q}{CG} = 1.10 * 1.10;
  
  $$mean{Q}{CD} = 179.68;
  $$count{Q}{CD} = 1;
  $$variance{Q}{CD} = 1.17 * 1.17;
  
  
  $$mean{G}{CA} = 45.33;
  $$count{G}{CA} = 1;
  $$variance{G}{CA} = 1.42 * 1.42;
  
  
  $$mean{H}{CA} = 56.37;
  $$count{H}{CA} = 1;
  $$variance{H}{CA} = 2.40 * 2.40;
  
  $$mean{H}{CB} = 29.95;
  $$count{H}{CB} = 1;
  $$variance{H}{CB} = 2.15 * 2.15;
  
  $$mean{H}{CG} = 130.59;
  $$count{H}{CG} = 1;
  $$variance{H}{CG} = 3.41 * 3.41;
  
  $$mean{H}{CD2} = 119.84;
  $$count{H}{CD2} = 1;
  $$variance{H}{CD2} = 3.30 * 3.30;
  
  $$mean{H}{CE1} = 136.05;
  $$count{H}{CE1} = 1;
  $$variance{H}{CE1} = 3.58 * 3.58;
  
  
  $$mean{I}{CA} = 61.59;
  $$count{I}{CA} = 1;
  $$variance{I}{CA} = 2.78 * 2.78;
  
  $$mean{I}{CB} = 38.58;
  $$count{I}{CB} = 1;
  $$variance{I}{CB} = 2.09 * 2.09;
  
  $$mean{I}{CG1} = 27.65;
  $$count{I}{CG1} = 1;
  $$variance{I}{CG1} = 2.32 * 2.32;
  
  $$mean{I}{CG2} = 17.36;
  $$count{I}{CG2} = 1;
  $$variance{I}{CG2} = 1.60 * 1.60;
  
  $$mean{I}{CD1} = 13.41;
  $$count{I}{CD1} = 1;
  $$variance{I}{CD1} = 1.91 * 1.91;
  
  
  $$mean{L}{CA} = 55.65;
  $$count{L}{CA} = 1;
  $$variance{L}{CA} = 2.21 * 2.21;
  
  $$mean{L}{CB} = 42.37;
  $$count{L}{CB} = 1;
  $$variance{L}{CB} = 1.86 * 1.86;
  
  $$mean{L}{CG} = 26.77;
  $$count{L}{CG} = 1;
  $$variance{L}{CG} = 1.25 * 1.25;
  
  $$mean{L}{CD1} = 24.73;
  $$count{L}{CD1} = 1;
  $$variance{L}{CD1} = 1.71 * 1.71;
  
  $$mean{L}{CD2} = 24.10;
  $$count{L}{CD2} = 1;
  $$variance{L}{CD2} = 1.78 * 1.78;
  
  
  $$mean{K}{CA} = 56.84;
  $$count{K}{CA} = 1;
  $$variance{K}{CA} = 2.25 * 2.25;
  
  $$mean{K}{CB} = 32.83;
  $$count{K}{CB} = 1;
  $$variance{K}{CB} = 1.88 * 1.88;
  
  $$mean{K}{CG} = 24.91;
  $$count{K}{CG} = 1;
  $$variance{K}{CG} = 1.31 * 1.31;
  
  $$mean{K}{CD} = 28.78;
  $$count{K}{CD} = 1;
  $$variance{K}{CD} = 1.39 * 1.39;
  
  $$mean{K}{CE} = 41.78;
  $$count{K}{CE} = 1;
  $$variance{K}{CE} = 0.98 * 0.98;
  
  
  $$mean{M}{CA} = 56.16;
  $$count{M}{CA} = 1;
  $$variance{M}{CA} = 2.34 * 2.34;
  
  $$mean{M}{CB} = 32.90;
  $$count{M}{CB} = 1;
  $$variance{M}{CB} = 2.23 * 2.23;
  
  $$mean{M}{CG} = 32.07;
  $$count{M}{CG} = 1;
  $$variance{M}{CG} = 1.45 * 1.45;
  
  $$mean{M}{CE} = 17.21;
  $$count{M}{CE} = 1;
  $$variance{M}{CE} = 3.42 * 3.42;
  
  
  $$mean{F}{CA} = 58.25;
  $$count{F}{CA} = 1;
  $$variance{F}{CA} = 2.73 * 2.73;
  
  $$mean{F}{CB} = 39.95;
  $$count{F}{CB} = 1;
  $$variance{F}{CB} = 2.04 * 2.04;
  
  $$mean{F}{CG} = 138.35;
  $$count{F}{CG} = 1;
  $$variance{F}{CG} = 1.12 * 1.12;
  
  $$mean{F}{CD1} = 131.38;
  $$count{F}{CD1} = 1;
  $$variance{F}{CD1} = 1.38 * 1.38;
  
  $$mean{F}{CD2} = 131.33;
  $$count{F}{CD2} = 1;
  $$variance{F}{CD2} = 1.48 * 1.48;
  
  $$mean{F}{CE1} = 130.48;
  $$count{F}{CE1} = 1;
  $$variance{F}{CE1} = 1.29 * 1.29;
  
  $$mean{F}{CE2} = 130.60;
  $$count{F}{CE2} = 1;
  $$variance{F}{CE2} = 0.97 * 0.97;
  
  $$mean{F}{CZ} = 129.11;
  $$count{F}{CZ} = 1;
  $$variance{F}{CZ} = 1.68 * 1.68;
  
  
  $$mean{P}{CA} = 63.30;
  $$count{P}{CA} = 1;
  $$variance{P}{CA} = 1.64 * 1.64;
  
  $$mean{P}{CB} = 31.81;
  $$count{P}{CB} = 1;
  $$variance{P}{CB} = 1.54 * 1.54;
  
  $$mean{P}{CG} = 27.14;
  $$count{P}{CG} = 1;
  $$variance{P}{CG} = 1.18 * 1.18;
  
  $$mean{P}{CD} = 50.28;
  $$count{P}{CD} = 1;
  $$variance{P}{CD} = 1.04 * 1.04;
  
  
  $$mean{S}{CA} = 58.57;
  $$count{S}{CA} = 1;
  $$variance{S}{CA} = 2.12 * 2.12;
  
  $$mean{S}{CB} = 63.80;
  $$count{S}{CB} = 1;
  $$variance{S}{CB} = 1.57 * 1.57;
  
  
  $$mean{T}{CA} = 62.15;
  $$count{T}{CA} = 1;
  $$variance{T}{CA} = 2.72 * 2.72;
  
  $$mean{T}{CB} = 69.64;
  $$count{T}{CB} = 1;
  $$variance{T}{CB} = 1.70 * 1.70;
  
  $$mean{T}{CG2} = 21.44;
  $$count{T}{CG2} = 1;
  $$variance{T}{CG2} = 1.39 * 1.39;
  
  
  $$mean{W}{CA} = 57.71;
  $$count{W}{CA} = 1;
  $$variance{W}{CA} = 2.65 * 2.65;
  
  $$mean{W}{CB} = 30.16;
  $$count{W}{CB} = 1;
  $$variance{W}{CB} = 2.19 * 2.19;
  
  $$mean{W}{CG} = 110.37;
  $$count{W}{CG} = 1;
  $$variance{W}{CG} = 1.55 * 1.55;
  
  $$mean{W}{CD1} = 126.16;
  $$count{W}{CD1} = 1;
  $$variance{W}{CD1} = 1.76 * 1.76;
  
  $$mean{W}{CD2} = 127.40;
  $$count{W}{CD2} = 1;
  $$variance{W}{CD2} = 1.33 * 1.33;
  
  $$mean{W}{CE2} = 137.38;
  $$count{W}{CE2} = 1;
  $$variance{W}{CE2} = 2.86 * 2.86;
  
  $$mean{W}{CE3} = 120.38;
  $$count{W}{CE3} = 1;
  $$variance{W}{CE3} = 1.50 * 1.50;
  
  $$mean{W}{CZ2} = 114.19;
  $$count{W}{CZ2} = 1;
  $$variance{W}{CZ2} = 1.09 * 1.09;
  
  $$mean{W}{CZ3} = 121.57;
  $$count{W}{CZ3} = 1;
  $$variance{W}{CZ3} = 1.76 * 1.76;
  
  $$mean{W}{CH2} = 123.75;
  $$count{W}{CH2} = 1;
  $$variance{W}{CH2} = 1.34 * 1.34;
  
  
  $$mean{Y}{CA} = 58.02;
  $$count{Y}{CA} = 1;
  $$variance{Y}{CA} = 2.64 * 2.64;
  
  $$mean{Y}{CB} = 39.16;
  $$count{Y}{CB} = 1;
  $$variance{Y}{CB} = 2.23 * 2.23;
  
  $$mean{Y}{CG} = 129.29;
  $$count{Y}{CG} = 1;
  $$variance{Y}{CG} = 1.21 * 1.21;
  
  $$mean{Y}{CD1} = 132.54;
  $$count{Y}{CD1} = 1;
  $$variance{Y}{CD1} = 1.35 * 1.35;
  
  $$mean{Y}{CD2} = 132.26;
  $$count{Y}{CD2} = 1;
  $$variance{Y}{CD2} = 1.99 * 1.99;
  
  $$mean{Y}{CE1} = 117.81;
  $$count{Y}{CE1} = 1;
  $$variance{Y}{CE1} = 2.09 * 2.09;
  
  $$mean{Y}{CE2} = 117.91;
  $$count{Y}{CE2} = 1;
  $$variance{Y}{CE2} = 2.60 * 2.60;
  
  $$mean{Y}{CZ} = 156.37;
  $$count{Y}{CZ} = 1;
  $$variance{Y}{CZ} = 2.50 * 2.50;
  
  
  $$mean{V}{CA} = 62.48;
  $$count{V}{CA} = 1;
  $$variance{V}{CA} = 3.04 * 3.04;
  
  $$mean{V}{CB} = 32.66;
  $$count{V}{CB} = 1;
  $$variance{V}{CB} = 1.91 * 1.91;
  
  $$mean{V}{CG1} = 21.38;
  $$count{V}{CG1} = 1;
  $$variance{V}{CG1} = 1.53 * 1.53;
  
  $$mean{V}{CG2} = 21.31;
  $$count{V}{CG2} = 1;
  $$variance{V}{CG2} = 1.76 * 1.76;
  
  
  my $restricted_type_lists = {};
  if (! $use_sidechains)
    {
    $$restricted_type_lists{P} = ['CA', 'CB'];
    $$restricted_type_lists{Q} = ['CA', 'CB'];
    $$restricted_type_lists{A} = ['CA', 'CB'];
    $$restricted_type_lists{R} = ['CA', 'CB'];
    $$restricted_type_lists{S} = ['CA', 'CB'];
    $$restricted_type_lists{C} = ['CA', 'CB'];
    $$restricted_type_lists{T} = ['CA', 'CB'];
    $$restricted_type_lists{D} = ['CA', 'CB'];
    $$restricted_type_lists{E} = ['CA', 'CB'];
    $$restricted_type_lists{V} = ['CA', 'CB'];
    $$restricted_type_lists{F} = ['CA', 'CB'];
    $$restricted_type_lists{W} = ['CA', 'CB'];
    $$restricted_type_lists{G} = ['CA'];
    $$restricted_type_lists{H} = ['CA', 'CB'];
    $$restricted_type_lists{Y} = ['CA', 'CB'];
    $$restricted_type_lists{I} = ['CA', 'CB'];
    $$restricted_type_lists{K} = ['CA', 'CB'];
    $$restricted_type_lists{L} = ['CA', 'CB'];
    $$restricted_type_lists{M} = ['CA', 'CB'];
    $$restricted_type_lists{N} = ['CA', 'CB'];
    }
  else
    {
    $$restricted_type_lists{P} = ['CA', 'CB', 'CD', 'CG'];
    $$restricted_type_lists{Q} = ['CA', 'CB', 'CD', 'CG'];
    $$restricted_type_lists{A} = ['CA', 'CB'];
    $$restricted_type_lists{R} = ['CA', 'CB', 'CD', 'CG'];
    $$restricted_type_lists{S} = ['CA', 'CB'];
    $$restricted_type_lists{C} = ['CA', 'CB'];
    $$restricted_type_lists{T} = ['CA', 'CB', 'CG2'];
    $$restricted_type_lists{D} = ['CA', 'CB', 'CG'];
    $$restricted_type_lists{E} = ['CA', 'CB', 'CD', 'CG'];
    $$restricted_type_lists{V} = ['CA', 'CB', 'CG1', 'CG2'];
    $$restricted_type_lists{F} = ['CA', 'CB', 'CD1', 'CD2', 'CE1', 'CE2', 'CG', 'CZ'];
    $$restricted_type_lists{W} = ['CA', 'CB', 'CD1', 'CD2', 'CE2', 'CE3', 'CG', 'CH2', 'CZ2', 'CZ3'];
    $$restricted_type_lists{G} = ['CA'];
    $$restricted_type_lists{H} = ['CA', 'CB', 'CD2', 'CE1','CG'];
    $$restricted_type_lists{Y} = ['CA', 'CB', 'CD1', 'CD2', 'CE1', 'CE2', 'CG', 'CZ'];
    $$restricted_type_lists{I} = ['CA', 'CB', 'CD1', 'CG1', 'CG2'];
    $$restricted_type_lists{K} = ['CA', 'CB', 'CD', 'CE', 'CG'];
    $$restricted_type_lists{L} = ['CA', 'CB', 'CD1', 'CD2', 'CG'];
    $$restricted_type_lists{M} = ['CA', 'CB', 'CE', 'CG'];
    $$restricted_type_lists{N} = ['CA', 'CB', 'CG'];
    }


  if (! $use_aromatics)
    {
    foreach my $aa (qw(F Y W))
      { @{$$restricted_type_lists{$aa}} = grep { $_ eq "CA" || $_ eq "CB"; } @{$$restricted_type_lists{$aa}}; }
    }


  # calculate deuteration isotope corrections if deuteration is present
  if ($deuteration > 0)
    {
    my $deuteration_corrections = {};


    # corrections for CA's calculated from Sandy Farmer's macro values.
    # corrections for sidechain carbons calculated from small molecule studies.
    # corrections for nitrogens come from a rough estimate of 0.16 for 2-bond effects and 0.06 for 3-bond effects
    # for A
    $$deuteration_corrections{A}{CA} =0.752;
    $$deuteration_corrections{A}{CB} =0.932;

    # for C
    $$deuteration_corrections{C}{CA} =0.632;
    $$deuteration_corrections{C}{CB} =0.837;

    # for D
    $$deuteration_corrections{D}{CA} =0.632;
    $$deuteration_corrections{D}{CB} =0.69;

    # for E
    $$deuteration_corrections{E}{CA} =0.716;
    $$deuteration_corrections{E}{CB} =0.937;
    $$deuteration_corrections{E}{CG} =0.83;

    # for F
    $$deuteration_corrections{F}{CA} =0.632;
    $$deuteration_corrections{F}{CB} =0.919;
    $$deuteration_corrections{F}{CG} =0.274;
    $$deuteration_corrections{F}{CD1} =0.429;
    $$deuteration_corrections{F}{CD2} =0.429;
    $$deuteration_corrections{F}{CE1} =0.500;
    $$deuteration_corrections{F}{CE2} =0.500;
    $$deuteration_corrections{F}{CZ} =0.496;

    # for G
    $$deuteration_corrections{G}{CA} =0.784;

    # for H
    $$deuteration_corrections{H}{CA} =0.632;
    $$deuteration_corrections{H}{CB} =0.832;
    $$deuteration_corrections{H}{CG} =0.365;
    $$deuteration_corrections{H}{CD2} =0.545;
    $$deuteration_corrections{H}{CE1} =0.475;

    # for I
    $$deuteration_corrections{I}{CA} =0.722;
    $$deuteration_corrections{I}{CB} =1.092;
    $$deuteration_corrections{I}{CG1} =1.130;
    $$deuteration_corrections{I}{CG2} =1.020;
    $$deuteration_corrections{I}{CD1} =1.090;

    # for K
    $$deuteration_corrections{K}{CA} =0.716;
    $$deuteration_corrections{K}{CB} =1.067;
    $$deuteration_corrections{K}{CG} =1.225;
    $$deuteration_corrections{K}{CD} =1.190;
    $$deuteration_corrections{K}{CE} =0.990;

    # for L
    $$deuteration_corrections{L}{CA} =0.674;
    $$deuteration_corrections{L}{CB} =1.067;
    $$deuteration_corrections{L}{CG} =1.215;
    $$deuteration_corrections{L}{CD1} =1.090;
    $$deuteration_corrections{L}{CD2} =1.090;

    # for M
    $$deuteration_corrections{M}{CA} =0.716;
    $$deuteration_corrections{M}{CB} =0.917;
    $$deuteration_corrections{M}{CG} =0.995;
    $$deuteration_corrections{M}{CE} =0.855;

    # for N
    $$deuteration_corrections{N}{CA} =0.632;
    $$deuteration_corrections{N}{CB} =0.757;

    # for P
    $$deuteration_corrections{P}{CA} =0.716;
    $$deuteration_corrections{P}{CB} =1.027;
    $$deuteration_corrections{P}{CG} =1.158;
    $$deuteration_corrections{P}{CD} =0.952;

    # for Q
    $$deuteration_corrections{Q}{CA} =0.716;
    $$deuteration_corrections{Q}{CB} =0.951;
    $$deuteration_corrections{Q}{CG} =0.915;

    # for R
    $$deuteration_corrections{R}{CA} =0.716;
    $$deuteration_corrections{R}{CB} =1.067;
    $$deuteration_corrections{R}{CG} =1.155;
    $$deuteration_corrections{R}{CD} =0.990;
    $$deuteration_corrections{R}{CZ} =0.070;

    # for S
    $$deuteration_corrections{S}{CA} =0.632;
    $$deuteration_corrections{S}{CB} =0.837;

    # for T
    $$deuteration_corrections{T}{CA} =0.638;
    $$deuteration_corrections{T}{CB} =0.867;
    $$deuteration_corrections{T}{CG} =0.950;

    # for V
    $$deuteration_corrections{V}{CA} =0.764;
    $$deuteration_corrections{V}{CB} =1.057;
    $$deuteration_corrections{V}{CG1} =0.950;
    $$deuteration_corrections{V}{CG2} =0.950;

    # for W
    $$deuteration_corrections{W}{CA} =0.632;
    $$deuteration_corrections{W}{CB} =0.832;
    $$deuteration_corrections{W}{CG} =0.365;
    $$deuteration_corrections{W}{CD1} =0.350;
    $$deuteration_corrections{W}{CD2} =0.213;
    $$deuteration_corrections{W}{CE2} =0.167;
    $$deuteration_corrections{W}{CE3} =0.429;
    $$deuteration_corrections{W}{CZ2} =0.429;
    $$deuteration_corrections{W}{CZ3} =0.511;
    $$deuteration_corrections{W}{CH2} =0.511;

    # for Y
    $$deuteration_corrections{Y}{CA} =0.632;
    $$deuteration_corrections{Y}{CB} =0.919;
    $$deuteration_corrections{Y}{CG} =0.299;
    $$deuteration_corrections{Y}{CD1} =0.418;
    $$deuteration_corrections{Y}{CD2} =0.418;
    $$deuteration_corrections{Y}{CE1} =0.390;
    $$deuteration_corrections{Y}{CE2} =0.390;
    $$deuteration_corrections{Y}{CZ} =0.220;


    # make the corrections
    foreach my $aa (keys %{$deuteration_corrections})
      {
      foreach my $atom (keys %{$$deuteration_corrections{$aa}})
	{ $$mean{$aa}{$atom} -= ($$deuteration_corrections{$aa}{$atom} * $deuteration); }
      }
    }


  return ($count, $mean, $variance, $restricted_type_lists, $similarity);  
  }
